@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
@@ -14,6 +14,9 @@
  * Copyright (c) 2007      Voltaire All rights reserved.
  * Copyright (c) 2006-2010 University of Houston.  All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2012      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
+ * Copyright (c) 2012      Oak Ridge National Labs.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -24,7 +27,6 @@
 #include "ompi_config.h"
 
 #include "opal/dss/dss.h"
-#include "orte/types.h"
 #include "ompi/proc/proc.h" 
 #include "ompi/communicator/communicator.h"
 #include "ompi/op/op.h"
@@ -32,13 +34,11 @@
 #include "opal/class/opal_pointer_array.h"
 #include "opal/class/opal_list.h"
 #include "ompi/mca/pml/pml.h"
+#include "ompi/mca/rte/rte.h"
 #include "ompi/mca/coll/base/base.h"
 #include "ompi/request/request.h"
 #include "ompi/runtime/ompi_module_exchange.h" 
 #include "ompi/runtime/mpiruntime.h"
-#include "ompi/mca/dpm/dpm.h"
-
-#include "orte/mca/rml/rml.h"
 
 BEGIN_C_DECLS
 
@@ -120,6 +120,7 @@ static int ompi_comm_world_thread_level_mult=0;
 
 int ompi_comm_cid_init (void)
 {
+#if OMPI_ENABLE_THREAD_MULTIPLE
     ompi_proc_t **procs, *thisproc;
     uint8_t thread_level;
     void *tlpointer;
@@ -152,7 +153,10 @@ int ompi_comm_cid_init (void)
             return ret;
         }
     }
-
+    free(procs);
+#else
+    ompi_comm_world_thread_level_mult = 0; // silence compiler warning if not used
+#endif
     return OMPI_SUCCESS;
 }
 
@@ -164,6 +168,7 @@ int ompi_comm_nextcid ( ompi_communicator_t* newcomm,
                         void* remote_leader,
                         int mode, int send_first )
 {
+    int ret;
     int nextcid;
     bool flag;
     int nextlocal_cid;
@@ -180,7 +185,7 @@ int ompi_comm_nextcid ( ompi_communicator_t* newcomm,
      */
 
     switch (mode) 
-    {
+        {
         case OMPI_COMM_CID_INTRA: 
             allredfnct=(ompi_comm_cid_allredfct*)ompi_comm_allreduce_intra;
             break;
@@ -196,85 +201,89 @@ int ompi_comm_nextcid ( ompi_communicator_t* newcomm,
         default: 
             return MPI_UNDEFINED;
             break;
-    }
+        }
     
     do {
-	/* Only one communicator function allowed in same time on the
-	 * same communicator.
-	 */
-	OPAL_THREAD_LOCK(&ompi_cid_lock);
-	response = ompi_comm_register_cid (comm->c_contextid);
-	OPAL_THREAD_UNLOCK(&ompi_cid_lock);
+        /* Only one communicator function allowed in same time on the
+         * same communicator.
+         */
+        OPAL_THREAD_LOCK(&ompi_cid_lock);
+        response = ompi_comm_register_cid (comm->c_contextid);
+        OPAL_THREAD_UNLOCK(&ompi_cid_lock);
     } while (OMPI_SUCCESS != response );
     start = ompi_mpi_communicators.lowest_free;
     
     while (!done) {
-	/**
-	 * This is the real algorithm described in the doc 
-	 */
-	
-	OPAL_THREAD_LOCK(&ompi_cid_lock);
-	if (comm->c_contextid != ompi_comm_lowest_cid() ) {
-	    /* if not lowest cid, we do not continue, but sleep and try again */
-	    OPAL_THREAD_UNLOCK(&ompi_cid_lock);
-	    continue;
-	}
-	OPAL_THREAD_UNLOCK(&ompi_cid_lock);
-        
-        
-	for (i=start; i < mca_pml.pml_max_contextid ; i++) {
-	    flag=opal_pointer_array_test_and_set_item(&ompi_mpi_communicators, 
-						      i, comm);
-	    if (true == flag) {
-		nextlocal_cid = i;
-		break;
-	    }
-	}
-	
-	(allredfnct)(&nextlocal_cid, &nextcid, 1, MPI_MAX, comm, bridgecomm,
-		     local_leader, remote_leader, send_first );
-	if (nextcid == nextlocal_cid) {
-	    response = 1; /* fine with me */
-	}
-	else {
-	    opal_pointer_array_set_item(&ompi_mpi_communicators, 
-					nextlocal_cid, NULL);
-	    
-	    flag = opal_pointer_array_test_and_set_item(&ompi_mpi_communicators, 
-							nextcid, comm );
-	    if (true == flag) {
-		response = 1; /* works as well */
-	    }
-	    else {
-		response = 0; /* nope, not acceptable */
-	    }
-	}
-        
-	(allredfnct)(&response, &glresponse, 1, MPI_MIN, comm, bridgecomm,
-		     local_leader, remote_leader, send_first );
-	if (1 == glresponse) {
-	    done = 1;             /* we are done */
-	    break;
-	}
-	else if ( 0 == glresponse ) {
-	    if ( 1 == response ) {
-		/* we could use that, but other don't agree */
-		opal_pointer_array_set_item(&ompi_mpi_communicators, 
-					    nextcid, NULL);
-	    }
-	    start = nextcid+1; /* that's where we can start the next round */
-	}
+        /**
+         * This is the real algorithm described in the doc 
+         */
+        OPAL_THREAD_LOCK(&ompi_cid_lock);
+        if (comm->c_contextid != ompi_comm_lowest_cid() ) {
+            /* if not lowest cid, we do not continue, but sleep and try again */
+            OPAL_THREAD_UNLOCK(&ompi_cid_lock);
+            continue;
+        }
+        OPAL_THREAD_UNLOCK(&ompi_cid_lock);
+
+        for (i=start; i < mca_pml.pml_max_contextid ; i++) {
+            flag=opal_pointer_array_test_and_set_item(&ompi_mpi_communicators, 
+                                                      i, comm);
+            if (true == flag) {
+                nextlocal_cid = i;
+                break;
+            }
+        }
+
+        ret = (allredfnct)(&nextlocal_cid, &nextcid, 1, MPI_MAX, comm, bridgecomm,
+                           local_leader, remote_leader, send_first );
+        if( OMPI_SUCCESS != ret ) {
+            return ret;
+        }
+        if (nextcid == nextlocal_cid) {
+            response = 1; /* fine with me */
+        }
+        else {
+            opal_pointer_array_set_item(&ompi_mpi_communicators, 
+                                        nextlocal_cid, NULL);
+
+            flag = opal_pointer_array_test_and_set_item(&ompi_mpi_communicators, 
+                                                        nextcid, comm );
+            if (true == flag) {
+                response = 1; /* works as well */
+            }
+            else {
+                response = 0; /* nope, not acceptable */
+            }
+        }
+
+        ret = (allredfnct)(&response, &glresponse, 1, MPI_MIN, comm, bridgecomm,
+                           local_leader, remote_leader, send_first );
+        if( OMPI_SUCCESS != ret ) {
+            return ret;
+        }
+        if (1 == glresponse) {
+            done = 1;             /* we are done */
+            break;
+        }
+        else if ( 0 == glresponse ) {
+            if ( 1 == response ) {
+                /* we could use that, but other don't agree */
+                opal_pointer_array_set_item(&ompi_mpi_communicators, 
+                                            nextcid, NULL);
+            }
+            start = nextcid+1; /* that's where we can start the next round */
+        }
     }
-    
+
     /* set the according values to the newcomm */
     newcomm->c_contextid = nextcid;
     newcomm->c_f_to_c_index = newcomm->c_contextid;
     opal_pointer_array_set_item (&ompi_mpi_communicators, nextcid, newcomm);
-    
+
     OPAL_THREAD_LOCK(&ompi_cid_lock);
     ompi_comm_unregister_cid (comm->c_contextid);
     OPAL_THREAD_UNLOCK(&ompi_cid_lock);
-    
+
     return (MPI_SUCCESS);
 }
 
@@ -318,7 +327,7 @@ static int ompi_comm_register_cid (uint32_t cid )
             if ( regcom->cid > cid ) {
                 break;
             }
-#if OPAL_ENABLE_MPI_THREADS
+#if OMPI_ENABLE_THREAD_MULTIPLE
             if( regcom->cid == cid ) {
                 /**
                  * The MPI standard state that is the user responsability to
@@ -331,7 +340,7 @@ static int ompi_comm_register_cid (uint32_t cid )
                 OBJ_RELEASE(newentry);
                 return OMPI_ERROR;
             }
-#endif  /* OPAL_ENABLE_MPI_THREADS */
+#endif  /* OMPI_ENABLE_THREAD_MULTIPLE */
         }
         opal_list_insert_pos (&ompi_registered_comms, item, 
                               (opal_list_item_t *)newentry);
@@ -403,7 +412,7 @@ int ompi_comm_activate ( ompi_communicator_t** newcomm,
      * send messages over the new communicator
      */
     switch (mode)
-    {
+        {
         case OMPI_COMM_CID_INTRA:
             allredfnct=(ompi_comm_cid_allredfct*)ompi_comm_allreduce_intra;
             break;
@@ -419,20 +428,23 @@ int ompi_comm_activate ( ompi_communicator_t** newcomm,
         default:
             return MPI_UNDEFINED;
             break;
-    }
+        }
 
     if (MPI_UNDEFINED != (*newcomm)->c_local_group->grp_my_rank) {
 
-	/* Initialize the PML stuff in the newcomm  */
-	if ( OMPI_SUCCESS != (ret = MCA_PML_CALL(add_comm(*newcomm))) ) {
-	    goto bail_on_error;
-	}
-	OMPI_COMM_SET_PML_ADDED(*newcomm);
+        /* Initialize the PML stuff in the newcomm  */
+        if ( OMPI_SUCCESS != (ret = MCA_PML_CALL(add_comm(*newcomm))) ) {
+            goto bail_on_error;
+        }
+        OMPI_COMM_SET_PML_ADDED(*newcomm);
     }
 
 
-    (allredfnct)(&ok, &gok, 1, MPI_MIN, comm, bridgecomm,
-                 local_leader, remote_leader, send_first );
+    ret = (allredfnct)(&ok, &gok, 1, MPI_MIN, comm, bridgecomm,
+                       local_leader, remote_leader, send_first );
+    if( OMPI_SUCCESS != ret ) {
+        goto bail_on_error;
+    }
 
 
 
@@ -467,7 +479,7 @@ int ompi_comm_activate ( ompi_communicator_t** newcomm,
     /* Let the collectives components fight over who will do
        collective on this new comm.  */
     if (OMPI_SUCCESS != (ret = mca_coll_base_comm_select(*newcomm))) {
-	goto bail_on_error;
+        goto bail_on_error;
     }
 
     /* For an inter communicator, we have to deal with the potential
@@ -566,7 +578,7 @@ static int ompi_comm_allreduce_inter ( int *inbuf, int *outbuf,
         rc = OMPI_ERR_OUT_OF_RESOURCE;
         goto exit;
     }
-    
+
     /* Execute the inter-allreduce: the result of our group will
        be in the buffer of the remote group */
     rc = intercomm->c_coll.coll_allreduce ( inbuf, tmpbuf, count, MPI_INT,
@@ -604,12 +616,16 @@ static int ompi_comm_allreduce_inter ( int *inbuf, int *outbuf,
 
         if ( &ompi_mpi_op_max.op == op ) {
             for ( i = 0 ; i < count; i++ ) {
-                if (tmpbuf[i] > outbuf[i]) outbuf[i] = tmpbuf[i];
+                if (tmpbuf[i] > outbuf[i]) {
+                    outbuf[i] = tmpbuf[i];
+                }
             }
         }
         else if ( &ompi_mpi_op_min.op == op ) {
             for ( i = 0 ; i < count; i++ ) {
-                if (tmpbuf[i] < outbuf[i]) outbuf[i] = tmpbuf[i];
+                if (tmpbuf[i] < outbuf[i]) {
+                    outbuf[i] = tmpbuf[i];
+                }
             }
         }
         else if ( &ompi_mpi_op_sum.op == op ) {
@@ -623,7 +639,7 @@ static int ompi_comm_allreduce_inter ( int *inbuf, int *outbuf,
             }
         }
     }
- 
+
     /* distribute the overall result to all processes in the other group.
        Instead of using bcast, we are using here allgatherv, to avoid the
        possible deadlock. Else, we need an algorithm to determine, 
@@ -636,7 +652,7 @@ static int ompi_comm_allreduce_inter ( int *inbuf, int *outbuf,
                                             rcounts, rdisps, MPI_INT, 
                                             intercomm,
                                             intercomm->c_coll.coll_allgatherv_module);
-    
+
  exit:
     if ( NULL != tmpbuf ) {
         free ( tmpbuf );
@@ -666,7 +682,7 @@ static int ompi_comm_allreduce_intra_bridge (int *inbuf, int *outbuf,
     int i;
     int rc;
     int local_leader, remote_leader;
-    
+
     local_leader  = (*((int*)lleader));
     remote_leader = (*((int*)rleader));
 
@@ -693,14 +709,14 @@ static int ompi_comm_allreduce_intra_bridge (int *inbuf, int *outbuf,
         MPI_Request req;
         
         rc = MCA_PML_CALL(irecv ( outbuf, count, MPI_INT, remote_leader,
-                                 OMPI_COMM_ALLREDUCE_TAG, 
-                                 bcomm, &req));
+                                  OMPI_COMM_ALLREDUCE_TAG, 
+                                  bcomm, &req));
         if ( OMPI_SUCCESS != rc ) {
             goto exit;       
         }
         rc = MCA_PML_CALL(send (tmpbuf, count, MPI_INT, remote_leader, 
-                               OMPI_COMM_ALLREDUCE_TAG,
-                               MCA_PML_BASE_SEND_STANDARD,  bcomm));
+                                OMPI_COMM_ALLREDUCE_TAG,
+                                MCA_PML_BASE_SEND_STANDARD,  bcomm));
         if ( OMPI_SUCCESS != rc ) {
             goto exit;
         }
@@ -711,12 +727,16 @@ static int ompi_comm_allreduce_intra_bridge (int *inbuf, int *outbuf,
 
         if ( &ompi_mpi_op_max.op == op ) {
             for ( i = 0 ; i < count; i++ ) {
-                if (tmpbuf[i] > outbuf[i]) outbuf[i] = tmpbuf[i];
+                if (tmpbuf[i] > outbuf[i]) {
+                    outbuf[i] = tmpbuf[i];
+                }
             }
         }
         else if ( &ompi_mpi_op_min.op == op ) {
             for ( i = 0 ; i < count; i++ ) {
-                if (tmpbuf[i] < outbuf[i]) outbuf[i] = tmpbuf[i];
+                if (tmpbuf[i] < outbuf[i]) {
+                    outbuf[i] = tmpbuf[i];
+                }
             }
         }
         else if ( &ompi_mpi_op_sum.op == op ) {
@@ -729,9 +749,8 @@ static int ompi_comm_allreduce_intra_bridge (int *inbuf, int *outbuf,
                 outbuf[i] *= tmpbuf[i];
             }
         }
-        
     }
-    
+
     rc = comm->c_coll.coll_bcast ( outbuf, count, MPI_INT, local_leader, 
                                    comm, comm->c_coll.coll_bcast_module );
 
@@ -761,19 +780,19 @@ static int ompi_comm_allreduce_intra_oob (int *inbuf, int *outbuf,
     int i;
     int rc;
     int local_leader, local_rank;
-    orte_process_name_t *remote_leader=NULL;
-    orte_std_cntr_t size_count;
-    
+    ompi_process_name_t *remote_leader=NULL;
+    int32_t size_count;
+
     local_leader  = (*((int*)lleader));
-    remote_leader = (orte_process_name_t*)rleader;
+    remote_leader = (ompi_process_name_t*)rleader;
     size_count = count;
 
     if ( &ompi_mpi_op_sum.op != op && &ompi_mpi_op_prod.op != op &&
          &ompi_mpi_op_max.op != op && &ompi_mpi_op_min.op  != op ) {
         return MPI_ERR_OP;
     }
-    
-    
+
+
     local_rank = ompi_comm_rank ( comm );
     tmpbuf     = (int *) malloc ( count * sizeof(int));
     if ( NULL == tmpbuf ) {
@@ -795,28 +814,28 @@ static int ompi_comm_allreduce_intra_oob (int *inbuf, int *outbuf,
         sbuf = OBJ_NEW(opal_buffer_t);
         rbuf = OBJ_NEW(opal_buffer_t);
         
-        if (ORTE_SUCCESS != (rc = opal_dss.pack(sbuf, tmpbuf, (orte_std_cntr_t)count, OPAL_INT))) {
+        if (OPAL_SUCCESS != (rc = opal_dss.pack(sbuf, tmpbuf, (int32_t)count, OPAL_INT))) {
             goto exit;
         }
 
         if ( send_first ) {
-            if (0 > (rc = orte_rml.send_buffer(remote_leader, sbuf, OMPI_RML_TAG_COMM_CID_INTRA, 0))) {
+            if (0 > (rc = ompi_rte_send_buffer(remote_leader, sbuf, OMPI_RML_TAG_COMM_CID_INTRA, 0))) {
                 goto exit;
             }
-            if (0 > (rc = orte_rml.recv_buffer(remote_leader, rbuf, OMPI_RML_TAG_COMM_CID_INTRA, 0))) {
+            if (0 > (rc = ompi_rte_recv_buffer(remote_leader, rbuf, OMPI_RML_TAG_COMM_CID_INTRA, 0))) {
                 goto exit;
             }
         }
         else {
-            if (0 > (rc = orte_rml.recv_buffer(remote_leader, rbuf, OMPI_RML_TAG_COMM_CID_INTRA, 0))) {
+            if (0 > (rc = ompi_rte_recv_buffer(remote_leader, rbuf, OMPI_RML_TAG_COMM_CID_INTRA, 0))) {
                 goto exit;
             }
-            if (0 > (rc = orte_rml.send_buffer(remote_leader, sbuf, OMPI_RML_TAG_COMM_CID_INTRA, 0))) {
+            if (0 > (rc = ompi_rte_send_buffer(remote_leader, sbuf, OMPI_RML_TAG_COMM_CID_INTRA, 0))) {
                 goto exit;
             }
         }
 
-        if (ORTE_SUCCESS != (rc = opal_dss.unpack(rbuf, outbuf, &size_count, OPAL_INT))) {
+        if (OPAL_SUCCESS != (rc = opal_dss.unpack(rbuf, outbuf, &size_count, OPAL_INT))) {
             goto exit;
         }
         OBJ_RELEASE(sbuf);
@@ -825,12 +844,16 @@ static int ompi_comm_allreduce_intra_oob (int *inbuf, int *outbuf,
 
         if ( &ompi_mpi_op_max.op == op ) {
             for ( i = 0 ; i < count; i++ ) {
-                if (tmpbuf[i] > outbuf[i]) outbuf[i] = tmpbuf[i];
+                if (tmpbuf[i] > outbuf[i]) {
+                    outbuf[i] = tmpbuf[i];
+                }
             }
         }
         else if ( &ompi_mpi_op_min.op == op ) {
             for ( i = 0 ; i < count; i++ ) {
-                if (tmpbuf[i] < outbuf[i]) outbuf[i] = tmpbuf[i];
+                if (tmpbuf[i] < outbuf[i]) {
+                    outbuf[i] = tmpbuf[i];
+                }
             }
         }
         else if ( &ompi_mpi_op_sum.op == op ) {
@@ -843,9 +866,8 @@ static int ompi_comm_allreduce_intra_oob (int *inbuf, int *outbuf,
                 outbuf[i] *= tmpbuf[i];
             }
         }
-        
     }
-    
+
     rc = comm->c_coll.coll_bcast (outbuf, count, MPI_INT, 
                                   local_leader, comm,
                                   comm->c_coll.coll_bcast_module);
@@ -859,4 +881,3 @@ static int ompi_comm_allreduce_intra_oob (int *inbuf, int *outbuf,
 }
 
 END_C_DECLS
-
