@@ -3,16 +3,18 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2006-2009 University of Houston.  All rights reserved.
+ * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2010 University of Houston.  All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc. All rights reserved.
+ * Copyright (c) 2011-2013 INRIA.  All rights reserved.
+ * Copyright (c) 2011-2013 Universite Bordeaux 1
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -23,6 +25,7 @@
 #ifndef OMPI_COMMUNICATOR_H
 #define OMPI_COMMUNICATOR_H
 
+#include "ompi_config.h"
 #include "opal/class/opal_object.h"
 #include "ompi/errhandler/errhandler.h"
 #include "opal/threads/mutex.h"
@@ -30,43 +33,47 @@
 #include "mpi.h"
 #include "ompi/group/group.h"
 #include "ompi/mca/coll/coll.h"
-#include "orte/mca/rml/rml_types.h"
 #include "ompi/proc/proc.h"
 
 BEGIN_C_DECLS
 
 OMPI_DECLSPEC OBJ_CLASS_DECLARATION(ompi_communicator_t);
 
-#define OMPI_COMM_INTER      0x00000001
-#define OMPI_COMM_CART       0x00000002
-#define OMPI_COMM_GRAPH      0x00000004
-#define OMPI_COMM_NAMEISSET  0x00000008
-#define OMPI_COMM_ISFREED    0x00000010
-#define OMPI_COMM_INTRINSIC  0x00000020
-#define OMPI_COMM_DYNAMIC    0x00000040
-#define OMPI_COMM_INVALID    0x00000080
-#define OMPI_COMM_PML_ADDED  0x00000100
+#define OMPI_COMM_INTER        0x00000001
+#define OMPI_COMM_NAMEISSET    0x00000002
+#define OMPI_COMM_INTRINSIC    0x00000004
+#define OMPI_COMM_DYNAMIC      0x00000008
+#define OMPI_COMM_ISFREED      0x00000010
+#define OMPI_COMM_INVALID      0x00000020
+#define OMPI_COMM_CART         0x00000100
+#define OMPI_COMM_GRAPH        0x00000200
+#define OMPI_COMM_DIST_GRAPH   0x00000400
+#define OMPI_COMM_PML_ADDED    0x00001000
+#define OMPI_COMM_EXTRA_RETAIN 0x00004000
 
 /* some utility #defines */
 #define OMPI_COMM_IS_INTER(comm) ((comm)->c_flags & OMPI_COMM_INTER)
 #define OMPI_COMM_IS_INTRA(comm) (!((comm)->c_flags & OMPI_COMM_INTER))
 #define OMPI_COMM_IS_CART(comm) ((comm)->c_flags & OMPI_COMM_CART)
 #define OMPI_COMM_IS_GRAPH(comm) ((comm)->c_flags & OMPI_COMM_GRAPH)
+#define OMPI_COMM_IS_DIST_GRAPH(comm) ((comm)->c_flags & OMPI_COMM_DIST_GRAPH)
 #define OMPI_COMM_IS_INTRINSIC(comm) ((comm)->c_flags & OMPI_COMM_INTRINSIC)
 #define OMPI_COMM_IS_FREED(comm) ((comm)->c_flags & OMPI_COMM_ISFREED)
 #define OMPI_COMM_IS_DYNAMIC(comm) ((comm)->c_flags & OMPI_COMM_DYNAMIC)
 #define OMPI_COMM_IS_INVALID(comm) ((comm)->c_flags & OMPI_COMM_INVALID)
 #define OMPI_COMM_IS_PML_ADDED(comm) ((comm)->c_flags & OMPI_COMM_PML_ADDED)
+#define OMPI_COMM_IS_EXTRA_RETAIN(comm) ((comm)->c_flags & OMPI_COMM_EXTRA_RETAIN)
+
 
 #define OMPI_COMM_SET_DYNAMIC(comm) ((comm)->c_flags |= OMPI_COMM_DYNAMIC)
 #define OMPI_COMM_SET_INVALID(comm) ((comm)->c_flags |= OMPI_COMM_INVALID)
 
 #define OMPI_COMM_SET_PML_ADDED(comm) ((comm)->c_flags |= OMPI_COMM_PML_ADDED)
+#define OMPI_COMM_SET_EXTRA_RETAIN(comm) ((comm)->c_flags |= OMPI_COMM_EXTRA_RETAIN)
 
 /* a set of special tags: */
 
 /*  to recognize an MPI_Comm_join in the comm_connect_accept routine. */
-
 #define OMPI_COMM_ALLGATHER_TAG -31078
 #define OMPI_COMM_BARRIER_TAG   -31079
 #define OMPI_COMM_ALLREDUCE_TAG -31080
@@ -90,6 +97,10 @@ OMPI_DECLSPEC OBJ_CLASS_DECLARATION(ompi_communicator_t);
 #define OMPI_COMM_BLOCK_WORLD      16
 #define OMPI_COMM_BLOCK_OTHERS     8
 
+/* A macro comparing two CIDs */
+#define OMPI_COMM_CID_IS_LOWER(comm1,comm2) ( ((comm1)->c_contextid < (comm2)->c_contextid)? 1:0)
+
+
 OMPI_DECLSPEC extern opal_pointer_array_t ompi_mpi_communicators;
 
 struct ompi_communicator_t {
@@ -110,9 +121,10 @@ struct ompi_communicator_t {
     ompi_group_t        *c_local_group;
     ompi_group_t       *c_remote_group;
 
-    struct ompi_communicator_t *c_local_comm; /* a duplicate of the local 
-                                                 communicator in case the comm  
-                                                 is an inter-comm*/
+    struct ompi_communicator_t *c_local_comm; /* a duplicate of the
+                                                 local communicator in
+                                                 case the comm is an
+                                                 inter-comm*/
                      
     /* Attributes */
     struct opal_hash_table_t       *c_keyhash;
@@ -120,20 +132,11 @@ struct ompi_communicator_t {
     /**< inscribing cube dimension */
     int c_cube_dim;
 
-    /* Hooks for topo module to hang things */
-    mca_base_component_t *c_topo_component;
-
-    const struct mca_topo_base_module_1_0_0_t* c_topo; 
-    /**< structure of function pointers */
-
-    struct mca_topo_base_comm_1_0_0_t* c_topo_comm; 
-    /**< structure containing basic information about the topology */
-
-    struct mca_topo_base_module_comm_t *c_topo_module;
-    /**< module specific data */
+    /* Standard information about the selected topology module (or NULL
+       if this is not a cart, graph or dist graph communicator) */
+    struct mca_topo_base_module_t* c_topo;
 
     /* index in Fortran <-> C translation array */
-
     int c_f_to_c_index;
 
 #ifdef OMPI_WANT_PERUSE
@@ -228,8 +231,18 @@ struct ompi_predefined_communicator_t {
 typedef struct ompi_predefined_communicator_t ompi_predefined_communicator_t;
 
 OMPI_DECLSPEC extern ompi_communicator_t *ompi_mpi_comm_parent;
+OMPI_DECLSPEC extern ompi_predefined_communicator_t ompi_mpi_comm_world;
+OMPI_DECLSPEC extern ompi_predefined_communicator_t ompi_mpi_comm_self;
 OMPI_DECLSPEC extern ompi_predefined_communicator_t ompi_mpi_comm_null;
 
+/*
+ * These variables are for the MPI F03 bindings (F03 must bind Fortran
+ * varaiables to symbols; it cannot bind Fortran variables to the
+ * address of a C variable).
+ */
+OMPI_DECLSPEC extern ompi_predefined_communicator_t *ompi_mpi_comm_world_addr;
+OMPI_DECLSPEC extern ompi_predefined_communicator_t *ompi_mpi_comm_self_addr;
+OMPI_DECLSPEC extern ompi_predefined_communicator_t *ompi_mpi_comm_null_addr;
 
 
 /**
@@ -308,9 +321,9 @@ static inline ompi_communicator_t *ompi_comm_lookup(uint32_t cid)
 
 static inline struct ompi_proc_t* ompi_comm_peer_lookup(ompi_communicator_t* comm, int peer_id)
 {
-#if OMPI_ENABLE_DEBUG
+#if OPAL_ENABLE_DEBUG
     if(peer_id >= comm->c_remote_group->grp_proc_count) {
-        opal_output(0, "ompi_comm_lookup_peer: invalid peer index (%d)", peer_id);
+        opal_output(0, "ompi_comm_peer_lookup: invalid peer index (%d)", peer_id);
         return (struct ompi_proc_t *) NULL;
     }
 #endif
@@ -344,17 +357,25 @@ OMPI_DECLSPEC int ompi_comm_group (ompi_communicator_t *comm, ompi_group_t **gro
 int ompi_comm_create (ompi_communicator_t* comm, ompi_group_t *group,
                       ompi_communicator_t** newcomm);
 
+/**
+ * Take an almost complete communicator and reserve the CID as well
+ * as activate it (initialize the collective and the topologies).
+ */
+int ompi_comm_enable(ompi_communicator_t *old_comm,
+                     ompi_communicator_t *new_comm,
+                     int new_rank,
+                     int num_procs,
+                     ompi_proc_t** topo_procs);
 
 /**
- * create a cartesian communicator
+ * Back end of MPI_DIST_GRAPH_CREATE_ADJACENT
  */
-int ompi_topo_create (ompi_communicator_t *old_comm,
-                      int ndims_or_nnodes,
-                      int *dims_or_index,
-                      int *periods_or_edges,
-                      bool reorder,
-                      ompi_communicator_t **comm_cart,
-                      int cart_or_graph);
+int ompi_topo_dist_graph_create_adjacent(ompi_communicator_t *old_comm, 
+                                         int indegree, int sources[],
+                                         int sourceweights[], int outdegree,
+                                         int destinations[], int destweights[],
+                                         MPI_Info info, int reorder,
+                                         MPI_Comm *comm_dist_graph);
 
 /**
  * split a communicator based on color and key. Parameters
@@ -369,6 +390,21 @@ int ompi_topo_create (ompi_communicator_t *old_comm,
 OMPI_DECLSPEC int ompi_comm_split (ompi_communicator_t *comm, int color, int key,
                                    ompi_communicator_t** newcomm, bool pass_on_topo);
 
+/**
+ * split a communicator based on type and key. Parameters
+ * are identical to the MPI-counterpart of the function.
+ *
+ * @param comm: input communicator
+ * @param color
+ * @param key
+ *
+ * @
+ */
+OMPI_DECLSPEC int ompi_comm_split_type(ompi_communicator_t *comm, 
+                                       int split_type, int key,
+                                       struct ompi_info_t *info,
+                                       ompi_communicator_t** newcomm);
+
 /**
  * dup a communicator. Parameter are identical to the MPI-counterpart
  * of the function. It has been extracted, since we need to be able
@@ -448,7 +484,7 @@ OMPI_DECLSPEC int ompi_comm_set ( ompi_communicator_t** newcomm,
                                   int *remote_ranks,
                                   opal_hash_table_t *attr,
                                   ompi_errhandler_t *errh,
-                                  mca_base_component_t *topocomponent,
+                                  bool copy_topocomponent,
                                   ompi_group_t *local_group,
                                   ompi_group_t *remote_group   );
 /**
@@ -460,7 +496,7 @@ struct ompi_proc_t **ompi_comm_get_rprocs ( ompi_communicator_t *local_comm,
                                             ompi_communicator_t *bridge_comm,
                                             int local_leader,
                                             int remote_leader,
-                                            orte_rml_tag_t tag,
+                                            int tag,
                                             int rsize);
 
 /**
@@ -479,7 +515,14 @@ int ompi_comm_determine_first ( ompi_communicator_t *intercomm,
                                 int high );
 
 
-OMPI_DECLSPEC int ompi_comm_activate ( ompi_communicator_t** newcomm, int do_coll_select );
+OMPI_DECLSPEC int ompi_comm_activate ( ompi_communicator_t** newcomm, 
+				       ompi_communicator_t* comm,
+				       ompi_communicator_t* bridgecomm,
+				       void* local_leader,
+				       void* remote_leader,
+				       int mode,
+				       int send_first );
+				       
 
 /**
  * a simple function to dump the structure
@@ -501,10 +544,12 @@ void ompi_comm_reg_finalize(void);
 extern int ompi_comm_num_dyncomm;
 
 
-/* check in the communicator destructor whether a block if cid's 
-   can be reused.
+/* check whether any of the processes has requested support for 
+   MPI_THREAD_MULTIPLE. Note, that this produces global
+   information across MPI_COMM_WORLD, in contrary to the local
+   flag ompi_mpi_thread_provided 
 */
-OMPI_DECLSPEC void  ompi_comm_checkfor_blockreset ( ompi_communicator_t *comm );
+OMPI_DECLSPEC int ompi_comm_cid_init ( void );
 
 
 END_C_DECLS
