@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
- * Copyright (c) 2009      The University of Tennessee and The University
+ * Copyright (c) 2009-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
@@ -32,6 +32,9 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif
 
 #include "ompi/constants.h"
 #include "opal/class/opal_pointer_array.h"
@@ -56,7 +59,7 @@ BEGIN_C_DECLS
 #define OMPI_DATATYPE_FLAG_DATA_FORTRAN  0xC000
 #define OMPI_DATATYPE_FLAG_DATA_LANGUAGE 0xC000
 
-#define OMPI_DATATYPE_MAX_PREDEFINED 45
+#define OMPI_DATATYPE_MAX_PREDEFINED 47
 
 #if OMPI_DATATYPE_MAX_PREDEFINED > OPAL_DATATYPE_MAX_SUPPORTED
 #error Need to increase the number of supported dataypes by OPAL (value OPAL_DATATYPE_MAX_SUPPORTED).
@@ -159,18 +162,7 @@ ompi_datatype_commit( ompi_datatype_t ** type )
 }
 
 
-static inline int32_t
-ompi_datatype_destroy( ompi_datatype_t** type)
-{
-    ompi_datatype_t* pData = *type;
-
-    if( ompi_datatype_is_predefined(pData) && (pData->super.super.obj_reference_count <= 1) )
-        return OMPI_ERROR;
-
-    OBJ_RELEASE( pData );
-    *type = NULL;
-    return OMPI_SUCCESS;
-}
+OMPI_DECLSPEC int32_t ompi_datatype_destroy( ompi_datatype_t** type);
 
 
 /*
@@ -193,9 +185,11 @@ ompi_datatype_duplicate( const ompi_datatype_t* oldType, ompi_datatype_t** newTy
     if( NULL == new_ompi_datatype ) {
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
-    opal_datatype_clone ( &oldType->super, &new_ompi_datatype->super);
+    opal_datatype_clone( &oldType->super, &new_ompi_datatype->super);
     /* Strip the predefined flag at the OMPI level. */
     new_ompi_datatype->super.flags &= ~OMPI_DATATYPE_FLAG_PREDEFINED;
+    /* By default maintain the relationships related to the old data (such as ops) */
+    new_ompi_datatype->id = oldType->id;
 
     /* Set the keyhash to NULL -- copying attributes is *only* done at
        the top level (specifically, MPI_TYPE_DUP). */
@@ -218,8 +212,17 @@ OMPI_DECLSPEC int32_t ompi_datatype_create_hindexed( int count, const int* pBloc
                                                      const ompi_datatype_t* oldType, ompi_datatype_t** newType );
 OMPI_DECLSPEC int32_t ompi_datatype_create_indexed_block( int count, int bLength, const int* pDisp,
                                                           const ompi_datatype_t* oldType, ompi_datatype_t** newType );
+OMPI_DECLSPEC int32_t ompi_datatype_create_hindexed_block( int count, int bLength, const OPAL_PTRDIFF_TYPE* pDisp,
+                                                           const ompi_datatype_t* oldType, ompi_datatype_t** newType );
 OMPI_DECLSPEC int32_t ompi_datatype_create_struct( int count, const int* pBlockLength, const OPAL_PTRDIFF_TYPE* pDisp,
                                                    ompi_datatype_t* const* pTypes, ompi_datatype_t** newType );
+OMPI_DECLSPEC int32_t ompi_datatype_create_darray( int size, int rank, int ndims, int const* gsize_array,
+                                                   int const* distrib_array, int const* darg_array,
+                                                   int const* psize_array, int order, const ompi_datatype_t* oldtype,
+                                                   ompi_datatype_t** newtype);
+OMPI_DECLSPEC int32_t ompi_datatype_create_subarray(int ndims, int const* size_array, int const* subsize_array,
+                                                    int const* start_array, int order,
+                                                    const ompi_datatype_t* oldtype, ompi_datatype_t** newtype);
 static inline int32_t
 ompi_datatype_create_resized( const ompi_datatype_t* oldType, OPAL_PTRDIFF_TYPE lb, OPAL_PTRDIFF_TYPE extent, ompi_datatype_t** newType )
 {
@@ -269,23 +272,37 @@ ompi_datatype_get_true_extent( const ompi_datatype_t* type, OPAL_PTRDIFF_TYPE* t
     return opal_datatype_get_true_extent( &type->super, true_lb, true_extent);
 }
 
-static inline int32_t
+static inline ssize_t
 ompi_datatype_get_element_count( const ompi_datatype_t* type, size_t iSize )
 {
     return opal_datatype_get_element_count( &type->super, iSize );
 }
 
 static inline int32_t
-ompi_datatype_set_element_count( const ompi_datatype_t* type, uint32_t count, size_t* length )
+ompi_datatype_set_element_count( const ompi_datatype_t* type, size_t count, size_t* length )
 {
     return opal_datatype_set_element_count( &type->super, count, length );
 }
 
 static inline int32_t
-ompi_datatype_copy_content_same_ddt( const ompi_datatype_t* type, int32_t count,
+ompi_datatype_copy_content_same_ddt( const ompi_datatype_t* type, size_t count,
                                      char* pDestBuf, char* pSrcBuf )
 {
-    return opal_datatype_copy_content_same_ddt( &type->super, count, pDestBuf, pSrcBuf );
+    int32_t length, rc;
+    OPAL_PTRDIFF_TYPE extent;
+
+    ompi_datatype_type_extent( type, &extent );
+    while( 0 != count ) {
+        length = INT_MAX;
+        if( ((size_t)length) > count ) length = (int32_t)count;
+        rc = opal_datatype_copy_content_same_ddt( &type->super, length,
+                                                  pDestBuf, pSrcBuf );
+        if( 0 != rc ) return rc;
+        pDestBuf += ((ptrdiff_t)length) * extent;
+        pSrcBuf  += ((ptrdiff_t)length) * extent;
+        count -= (size_t)length;
+    }
+    return 0;
 }
 
 OMPI_DECLSPEC const ompi_datatype_t* ompi_datatype_match_size( int size, uint16_t datakind, uint16_t datalang );
@@ -312,26 +329,53 @@ OMPI_DECLSPEC int32_t ompi_datatype_copy_args( const ompi_datatype_t* source_dat
 OMPI_DECLSPEC int32_t ompi_datatype_release_args( ompi_datatype_t* pData );
 OMPI_DECLSPEC ompi_datatype_t* ompi_datatype_get_single_predefined_type_from_args( ompi_datatype_t* type );
 
-/*
- *
+/**
+ * Return the amount of buffer necessary to pack the entire description of
+ * the datatype. This value is computed once per datatype, and it is stored
+ * in the datatype structure together with the packed description. As a side
+ * note special care is taken to align the amount of data on void* for
+ * architectures that require it.
  */
-OMPI_DECLSPEC size_t ompi_datatype_pack_description_length( const ompi_datatype_t* datatype );
+OMPI_DECLSPEC size_t ompi_datatype_pack_description_length( ompi_datatype_t* datatype );
 
-/*
- *
+/**
+ * Return a pointer to the constant packed representation of the datatype.
+ * The length can be retrieved with the ompi_datatype_pack_description_length,
+ * and it is quarantee this is exactly the amount to be copied and not an
+ * upper bound. Additionally, the packed representation is slightly optimized
+ * compared with the get_content function, as all combiner_dup have been replaced
+ * directly with the target type.
  */
 OMPI_DECLSPEC int ompi_datatype_get_pack_description( ompi_datatype_t* datatype,
                                                       const void** packed_buffer );
 
-/*
- *
+/**
+ * Extract a fully-fledged datatype from the packed representation. This datatype
+ * is ready to be used in communications (it is automatically committed). However,
+ * this datatype does not have an internal representation, so it might not be
+ * repacked. Read the comment for the ompi_datatype_get_pack_description function
+ * for extra information.
  */
 struct ompi_proc_t;
-OMPI_DECLSPEC ompi_datatype_t* ompi_datatype_create_from_packed_description( void** packed_buffer,
-                                                                             struct ompi_proc_t* remote_processor );
+OMPI_DECLSPEC ompi_datatype_t*
+ompi_datatype_create_from_packed_description( void** packed_buffer,
+                                              struct ompi_proc_t* remote_processor );
 
+/**
+ * Auxiliary function providing a pretty print for the packed data description.
+ */
 OMPI_DECLSPEC int32_t ompi_datatype_print_args( const ompi_datatype_t* pData );
 
+
+/**
+ * Get the number of basic elements of the datatype in ucount
+ *
+ * @returns OMPI_SUCCESS if the count is valid
+ * @returns OMPI_ERR_VALUE_OUT_OF_BOUNDS if no
+ */
+OMPI_DECLSPEC int ompi_datatype_get_elements (ompi_datatype_t *datatype, size_t ucount,
+                                              size_t *count);
+
 #if OPAL_ENABLE_DEBUG
 /*
  * Set a breakpoint to this function in your favorite debugger
