@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
@@ -123,13 +123,10 @@ int32_t ompi_datatype_set_args( ompi_datatype_t* pData,
     pArgs = (ompi_datatype_args_t*)pData->args;
     pArgs->create_type = type;
 
-    switch(type){
+    switch(type) {
 
     case MPI_COMBINER_DUP:
-        /* Recompute the data description packed size based on the optimization
-         * for MPI_COMBINER_DUP.
-         */
-        pArgs->total_pack_size = 2 * sizeof(int);
+        pArgs->total_pack_size = 0;  /* store no extra data */
         break;
 
     case MPI_COMBINER_CONTIGUOUS:
@@ -216,6 +213,12 @@ int32_t ompi_datatype_set_args( ompi_datatype_t* pData,
     case MPI_COMBINER_RESIZED:
         break;
 
+    case MPI_COMBINER_HINDEXED_BLOCK:
+        pArgs->i[0] = i[0][0];
+        pArgs->i[1] = i[1][0];
+        memcpy( pArgs->i + 2, i[2], i[0][0] * sizeof(int) );
+        break;
+
     default:
         break;
     }
@@ -245,6 +248,8 @@ int32_t ompi_datatype_set_args( ompi_datatype_t* pData,
             assert( pArgs->total_pack_size ==
                     OPAL_ALIGN(pArgs->total_pack_size, sizeof(OPAL_PTRDIFF_TYPE), int) );
 #endif  /* OPAL_ALIGN_WORD_SIZE_INTEGERS */
+        } else {
+            pArgs->total_pack_size += 2 * sizeof(int);  /* _NAMED + predefined id */
         }
     }
 
@@ -419,16 +424,6 @@ int32_t ompi_datatype_release_args( ompi_datatype_t* pData )
 }
 
 
-size_t ompi_datatype_pack_description_length( const ompi_datatype_t* datatype )
-{
-    if( ompi_datatype_is_predefined(datatype) ) {
-        return sizeof(int) * 2;
-    }
-    assert( NULL != (ompi_datatype_args_t*)datatype->args );
-    return ((ompi_datatype_args_t*)datatype->args)->total_pack_size;
-}
-
-
 static inline int __ompi_datatype_pack_description( ompi_datatype_t* datatype,
                                                     void** packed_buffer, int* next_index )
 {
@@ -437,15 +432,18 @@ static inline int __ompi_datatype_pack_description( ompi_datatype_t* datatype,
     char* next_packed = (char*)*packed_buffer;
 
     if( ompi_datatype_is_predefined(datatype) ) {
-        position[0] = MPI_COMBINER_DUP;
+        position[0] = MPI_COMBINER_NAMED;
         position[1] = datatype->id;   /* On the OMPI - layer, copy the ompi_datatype.id */
+        next_packed += (2 * sizeof(int));
+        *packed_buffer = next_packed;
         return OMPI_SUCCESS;
     }
     /* For duplicated datatype we don't have to store all the information */
     if( MPI_COMBINER_DUP == args->create_type ) {
-        position[0] = args->create_type;
-        position[1] = args->d[0]->id; /* On the OMPI - layer, copy the ompi_datatype.id */
-        return OMPI_SUCCESS;
+        ompi_datatype_t* temp_data = args->d[0];
+        return __ompi_datatype_pack_description(temp_data,
+                                                packed_buffer,
+                                                next_index );
     }
     position[0] = args->create_type;
     position[1] = args->ci;
@@ -469,6 +467,8 @@ static inline int __ompi_datatype_pack_description( ompi_datatype_t* datatype,
     position = (int*)next_packed;
     next_packed += sizeof(int) * args->cd;
 
+    /* description of next datatype should be 64 bits aligned */
+    OMPI_DATATYPE_ALIGN_PTR(next_packed, char*);
     /* copy the aray of counts (32 bits aligned) */
     memcpy( next_packed, args->i, sizeof(int) * args->ci );
     next_packed += args->ci * sizeof(int);
@@ -504,15 +504,37 @@ int ompi_datatype_get_pack_description( ompi_datatype_t* datatype,
         } else if( NULL == args ) {
             return OMPI_ERROR;
         } else {
-            datatype->packed_description = malloc( args->total_pack_size );
+            datatype->packed_description = malloc(args->total_pack_size);
         }
         recursive_buffer = datatype->packed_description;
         __ompi_datatype_pack_description( datatype, &recursive_buffer, &next_index );
+        if( !ompi_datatype_is_predefined(datatype) ) {
+            args->total_pack_size = (uintptr_t)((char*)recursive_buffer - (char*)datatype->packed_description);
+            OMPI_DATATYPE_ALIGN_PTR(args->total_pack_size, char*);
+        }
     }
+
     *packed_buffer = (const void*)datatype->packed_description;
     return OMPI_SUCCESS;
 }
 
+size_t ompi_datatype_pack_description_length( ompi_datatype_t* datatype )
+{
+    if( ompi_datatype_is_predefined(datatype) ) {
+        return 2 * sizeof(int);
+    }
+    if( NULL == datatype->packed_description ) {
+        const void* buf;
+        int rc;
+
+        rc = ompi_datatype_get_pack_description(datatype, &buf);
+        if( OMPI_SUCCESS != rc )
+            return 0;
+    }
+    assert( NULL != (ompi_datatype_args_t*)datatype->args );
+    assert( NULL != (ompi_datatype_args_t*)datatype->packed_description );
+    return ((ompi_datatype_args_t*)datatype->args)->total_pack_size;
+}
 
 static ompi_datatype_t* __ompi_datatype_create_from_packed_description( void** packed_buffer,
                                                                         const struct ompi_proc_t* remote_processor )
@@ -546,7 +568,7 @@ static ompi_datatype_t* __ompi_datatype_create_from_packed_description( void** p
         create_type = opal_swap_bytes4(create_type);
     }
 #endif
-    if( MPI_COMBINER_DUP == create_type ) {
+    if( MPI_COMBINER_NAMED == create_type ) {
         /* there we have a simple predefined datatype */
         data_id = position[1];
 #if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
@@ -595,7 +617,7 @@ static ompi_datatype_t* __ompi_datatype_create_from_packed_description( void** p
         }
         array_of_datatype[i] =
             __ompi_datatype_create_from_packed_description( (void**)&next_buffer,
-                                                       remote_processor );
+                                                            remote_processor );
         if( NULL == array_of_datatype[i] ) {
             /* don't cleanup more than required. We can now modify these
              * values as we already know we have failed to rebuild the
@@ -624,7 +646,7 @@ static ompi_datatype_t* __ompi_datatype_create_from_packed_description( void** p
     }
 #endif
     datatype = __ompi_datatype_create_from_args( array_of_length, array_of_disp,
-                                            array_of_datatype, create_type );
+                                                 array_of_datatype, create_type );
     *packed_buffer = next_buffer;
  cleanup_and_exit:
     for( i = 0; i < number_of_datatype; i++ ) {
@@ -636,7 +658,6 @@ static ompi_datatype_t* __ompi_datatype_create_from_packed_description( void** p
     return datatype;
 }
 
-
 static ompi_datatype_t* __ompi_datatype_create_from_args( int32_t* i, MPI_Aint* a,
                                                           ompi_datatype_t** d, int32_t type )
 {
@@ -645,68 +666,88 @@ static ompi_datatype_t* __ompi_datatype_create_from_args( int32_t* i, MPI_Aint*
     switch(type){
         /******************************************************************/
     case MPI_COMBINER_DUP:
+        /* should we duplicate d[0]? */
+        /* ompi_datatype_set_args( datatype, 0, NULL, 0, NULL, 1, d[0], MPI_COMBINER_DUP ); */
+        assert(0);  /* shouldn't happen */
         break;
         /******************************************************************/
     case MPI_COMBINER_CONTIGUOUS:
         ompi_datatype_create_contiguous( i[0], d[0], &datatype );
+        ompi_datatype_set_args( datatype, 1, &i, 0, NULL, 1, d, MPI_COMBINER_CONTIGUOUS );
         break;
         /******************************************************************/
     case MPI_COMBINER_VECTOR:
         ompi_datatype_create_vector( i[0], i[1], i[2], d[0], &datatype );
+        {
+            int* a_i[3]; a_i[0] = &i[0]; a_i[1] = &i[1]; a_i[2] = &i[2];
+            ompi_datatype_set_args( datatype, 3, a_i, 0, NULL, 1, d, MPI_COMBINER_VECTOR );
+        }
         break;
         /******************************************************************/
     case MPI_COMBINER_HVECTOR_INTEGER:
     case MPI_COMBINER_HVECTOR:
         ompi_datatype_create_hvector( i[0], i[1], a[0], d[0], &datatype );
+        {
+            int* a_i[2]; a_i[0] = &i[0]; a_i[1] = &i[1];
+            ompi_datatype_set_args( datatype, 2, a_i, 1, a, 1, d, MPI_COMBINER_HVECTOR );
+        }
         break;
         /******************************************************************/
     case MPI_COMBINER_INDEXED:  /* TO CHECK */
         ompi_datatype_create_indexed( i[0], &(i[1]), &(i[1+i[0]]), d[0], &datatype );
+        {
+            int* a_i[3]; a_i[0] = &i[0]; a_i[1] = &i[1]; a_i[2] = &(i[1+i[0]]);
+            ompi_datatype_set_args( datatype, 2 * i[0] + 1, a_i, 0, NULL, 1, d, MPI_COMBINER_INDEXED );
+        }
         break;
         /******************************************************************/
     case MPI_COMBINER_HINDEXED_INTEGER:
     case MPI_COMBINER_HINDEXED:
         ompi_datatype_create_hindexed( i[0], &(i[1]), a, d[0], &datatype );
+        {
+            int* a_i[2]; a_i[0] = &i[0]; a_i[1] = &i[1];
+            ompi_datatype_set_args( datatype, i[0] + 1, a_i, i[0], a, 1, d, MPI_COMBINER_HINDEXED );
+        }
         break;
         /******************************************************************/
     case MPI_COMBINER_INDEXED_BLOCK:
         ompi_datatype_create_indexed_block( i[0], i[1], &(i[2]), d[0], &datatype );
+        {
+            int* a_i[3]; a_i[0] = &i[0]; a_i[1] = &i[1]; a_i[2] = &i[2];
+            ompi_datatype_set_args( datatype, 2 * i[0], a_i, 0, NULL, 1, d, MPI_COMBINER_INDEXED_BLOCK );
+        }
         break;
         /******************************************************************/
     case MPI_COMBINER_STRUCT_INTEGER:
     case MPI_COMBINER_STRUCT:
         ompi_datatype_create_struct( i[0], &(i[1]), a, d, &datatype );
+        {
+            int* a_i[2]; a_i[0] = &i[0]; a_i[1] = &i[1];
+            ompi_datatype_set_args( datatype, i[0] + 1, a_i, i[0], a, i[0], d, MPI_COMBINER_STRUCT );
+        }
         break;
         /******************************************************************/
     case MPI_COMBINER_SUBARRAY:
-        /*pos = 1;
-          pArgs->i[0] = i[0][0];
-          memcpy( pArgs->i + pos, i[1], pArgs->i[0] * sizeof(int) );
-          pos += pArgs->i[0];
-          memcpy( pArgs->i + pos, i[2], pArgs->i[0] * sizeof(int) );
-          pos += pArgs->i[0];
-          memcpy( pArgs->i + pos, i[3], pArgs->i[0] * sizeof(int) );
-          pos += pArgs->i[0];
-          pArgs->i[pos] = i[4][0];
-        */
+        ompi_datatype_create_subarray( i[0], &i[1 + 0 * i[0]], &i[1 + 1 * i[0]],
+                                       &i[1 + 2 * i[0]], i[1 + 3 * i[0]],
+                                       d[0], &datatype );
+        {
+            int* a_i[5]; a_i[0] = &i[0]; a_i[1] = &i[1 + 0 * i[0]]; a_i[2] = &i[1 + 1 * i[0]];  a_i[3] = &i[1 + 2 * i[0]]; a_i[4] = &i[1 + 3 * i[0]];
+            ompi_datatype_set_args( datatype, 3 * i[0] + 2, a_i, 0, NULL, 1, d, MPI_COMBINER_SUBARRAY);
+        }
         break;
         /******************************************************************/
     case MPI_COMBINER_DARRAY:
-        /*pos = 3;
-          pArgs->i[0] = i[0][0];
-          pArgs->i[1] = i[1][0];
-          pArgs->i[2] = i[2][0];
-
-          memcpy( pArgs->i + pos, i[3], i[2][0] * sizeof(int) );
-          pos += i[2][0];
-          memcpy( pArgs->i + pos, i[4], i[2][0] * sizeof(int) );
-          pos += i[2][0];
-          memcpy( pArgs->i + pos, i[5], i[2][0] * sizeof(int) );
-          pos += i[2][0];
-          memcpy( pArgs->i + pos, i[6], i[2][0] * sizeof(int) );
-          pos += i[2][0];
-          pArgs->i[pos] = i[7][0];
-        */
+        ompi_datatype_create_darray( i[0] /* size */, i[1] /* rank */, i[2] /* ndims */,
+                                     &i[3 + 0 * i[0]], &i[3 + 1 * i[0]],
+                                     &i[3 + 2 * i[0]], &i[3 + 3 * i[0]],
+                                     i[3 + 4 * i[0]], d[0], &datatype );
+        {
+            int* a_i[8]; a_i[0] = &i[0]; a_i[1] = &i[1]; a_i[2] = &i[2];
+            a_i[3] = &i[3 + 0 * i[0]]; a_i[4] = &i[3 + 1 * i[0]];  a_i[5] = &i[3 + 2 * i[0]];
+            a_i[6] = &i[3 + 3 * i[0]]; a_i[7] = &i[3 + 4 * i[0]];
+            ompi_datatype_set_args( datatype, 4 * i[0] + 4,a_i, 0, NULL, 1, d, MPI_COMBINER_DARRAY);
+        }
         break;
         /******************************************************************/
     case MPI_COMBINER_F90_REAL:
@@ -721,23 +762,31 @@ static ompi_datatype_t* __ompi_datatype_create_from_args( int32_t* i, MPI_Aint*
         break;
         /******************************************************************/
     case MPI_COMBINER_RESIZED:
+        /*ompi_datatype_set_args( datatype, 0, NULL, 2, a, 1, d, MPI_COMBINER_RESIZED );*/
         break;
         /******************************************************************/
-    default:
+    case MPI_COMBINER_HINDEXED_BLOCK:
+        ompi_datatype_create_hindexed_block( i[0], i[1], a, d[0], &datatype );
+        {
+            int* a_i[2]; a_i[0] = &i[0]; a_i[1] = &i[1];
+            ompi_datatype_set_args( datatype, 2 + i[0], a_i, i[0], a, 1, d, MPI_COMBINER_HINDEXED_BLOCK );
+        }
+        break;
+        /******************************************************************/
+     default:
         break;
     }
 
     return datatype;
 }
 
-
 ompi_datatype_t* ompi_datatype_create_from_packed_description( void** packed_buffer,
                                                                struct ompi_proc_t* remote_processor )
 {
     ompi_datatype_t* datatype;
 
     datatype = __ompi_datatype_create_from_packed_description( packed_buffer,
-                                                          remote_processor );
+                                                               remote_processor );
     /* Keep the pointer aligned to MPI_Aint */
     OMPI_DATATYPE_ALIGN_PTR(*packed_buffer, void*);
     if( NULL == datatype ) {
@@ -746,3 +795,43 @@ ompi_datatype_t* ompi_datatype_create_from_packed_description( void** packed_buf
     ompi_datatype_commit( &datatype );
     return datatype;
 }
+
+/**
+ * Parse the datatype description from the args and find if the
+ * datatype is created from a single predefined type. If yes,
+ * return the type, otherwise return NULL.
+ */
+ompi_datatype_t* ompi_datatype_get_single_predefined_type_from_args( ompi_datatype_t* type )
+{
+    ompi_datatype_t *predef = NULL, *current_type, *current_predef;
+    ompi_datatype_args_t* args = (ompi_datatype_args_t*)type->args;
+    int i;
+    
+    if( ompi_datatype_is_predefined(type) )
+        return type;
+
+    for( i = 0; i < args->cd; i++ ) {
+        current_type = args->d[i];
+        if( ompi_datatype_is_predefined(current_type) ) {
+            current_predef = current_type;
+        } else {
+            current_predef = ompi_datatype_get_single_predefined_type_from_args(current_type);
+            if( NULL == current_predef ) { /* No single predefined datatype */
+                return NULL;
+            }
+        }
+        if( NULL == predef ) {  /* This is the first iteration */
+            predef = current_predef;
+        } else {
+            /**
+             *  What exactly should we consider as identical types? If they are
+             * the same MPI level type, or if they map to the same OPAL datatype?
+             * In other words, MPI_FLOAT and MPI_REAL4 are they identical?
+             */
+            if( predef != current_predef ) {
+                return NULL;
+            }
+        }
+    }
+    return predef;
+}
