@@ -1,18 +1,20 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2010 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2010 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -27,25 +29,14 @@
 
 #include "ompi/constants.h"
 #include "opal/datatype/opal_convertor_internal.h"
-#include "opal/datatype/opal_datatype_internal.h"
 #include "opal/util/output.h"
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/datatype/ompi_datatype_internal.h"
 
 #include "mpi.h"
 
-#if OPAL_ENABLE_DEBUG
-#include "opal/mca/base/mca_base_param.h"
-/* These are used within test/datatype/position.c -- and therefore not static */
-int ompi_unpack_debug   = 0;
-int ompi_pack_debug     = 0;
-int ompi_copy_debug     = 0;
-int ompi_position_debug = 0;
-#endif  /* OPAL_ENABLE_DEBUG */
-
 /* by default the debuging is turned off */
 int ompi_datatype_dfd = -1;
-OMPI_DECLSPEC union dt_elem_desc ompi_datatype_predefined_elem_desc[2 * OMPI_DATATYPE_MPI_MAX_PREDEFINED];
 
 /**
  * This is the number of predefined datatypes. It is different than the MAX_PREDEFINED
@@ -64,14 +55,12 @@ int32_t ompi_datatype_number_of_predefined_data = 0;
  * The Macros in the OMPI Layer differ in that:
  *   Additionally to OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE, we have a OMPI_DATATYPE_INIT_PREDEFINED,
  *   for all available types (getting rid of duplication of the name.
- * Of course initialization of ompi_mpi_datatype_null includes the string-name NULL,
- * which CPP converts to ((void*)0), which we do not really want...
  */
 ompi_predefined_datatype_t ompi_mpi_datatype_null =
     {
         {
-            OPAL_DATATYPE_INITIALIZER_NULL(OMPI_DATATYPE_FLAG_PREDEFINED),
-            OMPI_DATATYPE_EMPTY_DATA(NULL),
+            OPAL_DATATYPE_INITIALIZER_EMPTY(OMPI_DATATYPE_FLAG_PREDEFINED),
+            OMPI_DATATYPE_EMPTY_DATA(EMPTY),
         },
         {0, } /* padding */
     };
@@ -114,15 +103,63 @@ ompi_predefined_datatype_t ompi_mpi_packed =         OMPI_DATATYPE_INIT_PREDEFIN
 /*
  * C++ / C99 datatypes
  */
+ompi_predefined_datatype_t ompi_mpi_c_bool = OMPI_DATATYPE_INIT_PREDEFINED (BOOL, OMPI_DATATYPE_FLAG_DATA_C);
 ompi_predefined_datatype_t ompi_mpi_cxx_bool =       OMPI_DATATYPE_INIT_PREDEFINED (BOOL, OMPI_DATATYPE_FLAG_DATA_CPP);
-ompi_predefined_datatype_t ompi_mpi_cxx_cplex =      OMPI_DATATYPE_INIT_DEFER (COMPLEX, OMPI_DATATYPE_FLAG_DATA_CPP | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
-ompi_predefined_datatype_t ompi_mpi_cxx_dblcplex =   OMPI_DATATYPE_INIT_DEFER (DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_CPP | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+
+/*
+ * Complex datatypes for C (base types), C++, and fortran
+ */
+ompi_predefined_datatype_t ompi_mpi_c_float_complex =       OMPI_DATATYPE_INIT_PREDEFINED (C_FLOAT_COMPLEX, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+ompi_predefined_datatype_t ompi_mpi_c_complex =             OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE (C_FLOAT_COMPLEX, C_COMPLEX, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+ompi_predefined_datatype_t ompi_mpi_c_double_complex =      OMPI_DATATYPE_INIT_PREDEFINED (C_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
 #if HAVE_LONG_DOUBLE
-ompi_predefined_datatype_t ompi_mpi_cxx_ldblcplex =  OMPI_DATATYPE_INIT_DEFER (LONG_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_CPP | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+ompi_predefined_datatype_t ompi_mpi_c_long_double_complex = OMPI_DATATYPE_INIT_PREDEFINED (C_LONG_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
 #else
-ompi_predefined_datatype_t ompi_mpi_cxx_ldblcplex =  OMPI_DATATYPE_INIT_UNAVAILABLE (LONG_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_CPP | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+ompi_predefined_datatype_t ompi_mpi_c_long_double_complex = OMPI_DATATYPE_INIT_UNAVAILABLE (C_LONG_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
 #endif  /* HAVE_LONG_DOUBLE */
 
+/* The C++ complex datatypes are the same as the C datatypes */
+ompi_predefined_datatype_t ompi_mpi_cxx_cplex =      OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE (C_FLOAT_COMPLEX, C_FLOAT_COMPLEX, OMPI_DATATYPE_FLAG_DATA_CPP | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+ompi_predefined_datatype_t ompi_mpi_cxx_dblcplex =   OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE (C_DOUBLE_COMPLEX, C_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_CPP | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+ompi_predefined_datatype_t ompi_mpi_cxx_ldblcplex =  OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE (C_LONG_DOUBLE_COMPLEX, C_LONG_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_CPP | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+
+#if OMPI_HAVE_FORTRAN_COMPLEX
+ompi_predefined_datatype_t ompi_mpi_cplex =          OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE (OMPI_KIND_FORTRAN_COMPLEX, COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+#else
+ompi_predefined_datatype_t ompi_mpi_cplex =          OMPI_DATATYPE_INIT_UNAVAILABLE (COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+#endif
+
+#if OMPI_HAVE_FORTRAN_DOUBLE_COMPLEX
+ompi_predefined_datatype_t ompi_mpi_dblcplex =       OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE (OMPI_KIND_FORTRAN_DOUBLE_COMPLEX, DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+#else
+ompi_predefined_datatype_t ompi_mpi_dblcplex =       OMPI_DATATYPE_INIT_UNAVAILABLE (DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+#endif
+
+/* In Fortran, there does not exist a type LONG DOUBLE COMPLEX, but DOUBLE COMPLEX(KIND=8) may require this */
+#if HAVE_LONG_DOUBLE && OMPI_HAVE_FORTRAN_DOUBLE_COMPLEX
+ompi_predefined_datatype_t ompi_mpi_ldblcplex =      OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE (C_LONG_DOUBLE_COMPLEX, LONG_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+#else
+ompi_predefined_datatype_t ompi_mpi_ldblcplex =      OMPI_DATATYPE_INIT_UNAVAILABLE (LONG_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+#endif
+
+#if OMPI_HAVE_FORTRAN_COMPLEX8
+ompi_predefined_datatype_t ompi_mpi_complex8 =       OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE (OMPI_KIND_FORTRAN_COMPLEX8, COMPLEX8, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+#else
+ompi_predefined_datatype_t ompi_mpi_complex8 =       OMPI_DATATYPE_INIT_UNAVAILABLE (COMPLEX8, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX);
+#endif
+
+#if OMPI_HAVE_FORTRAN_COMPLEX16
+ompi_predefined_datatype_t ompi_mpi_complex16 =      OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE (OMPI_KIND_FORTRAN_COMPLEX16, COMPLEX16, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+#else
+ompi_predefined_datatype_t ompi_mpi_complex16 =      OMPI_DATATYPE_INIT_UNAVAILABLE (COMPLEX16, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX);
+#endif
+
+#if OMPI_HAVE_FORTRAN_COMPLEX32
+ompi_predefined_datatype_t ompi_mpi_complex32 =      OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE (OMPI_KIND_FORTRAN_COMPLEX32, COMPLEX32, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
+#else
+ompi_predefined_datatype_t ompi_mpi_complex32 =      OMPI_DATATYPE_INIT_UNAVAILABLE (COMPLEX32, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX);
+#endif
+
 /*
  * Fortran datatypes
  */
@@ -131,21 +168,7 @@ ompi_predefined_datatype_t ompi_mpi_character =      OMPI_DATATYPE_INIT_PREDEFIN
 ompi_predefined_datatype_t ompi_mpi_integer =        OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE_FORTRAN (INT, INTEGER, OMPI_SIZEOF_FORTRAN_INTEGER, OMPI_ALIGNMENT_FORTRAN_INTEGER, OMPI_DATATYPE_FLAG_DATA_INT );
 ompi_predefined_datatype_t ompi_mpi_real =           OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE_FORTRAN (FLOAT, REAL, OMPI_SIZEOF_FORTRAN_REAL, OMPI_ALIGNMENT_FORTRAN_REAL, OMPI_DATATYPE_FLAG_DATA_FLOAT );
 ompi_predefined_datatype_t ompi_mpi_dblprec =        OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE_FORTRAN (FLOAT, DOUBLE_PRECISION, OMPI_SIZEOF_FORTRAN_DOUBLE_PRECISION, OMPI_ALIGNMENT_FORTRAN_DOUBLE_PRECISION, OMPI_DATATYPE_FLAG_DATA_FLOAT );
-ompi_predefined_datatype_t ompi_mpi_cplex =          OMPI_DATATYPE_INIT_DEFER (COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
-#if OMPI_HAVE_F90_DOUBLE_COMPLEX || OMPI_HAVE_FORTRAN_DOUBLE_COMPLEX
-/* We do not configure-check for alignment of F90 types ... Alignment of F77 OMPI_ALIGNMENT_FORTRAN_COMPLEX has to suffice */
-ompi_predefined_datatype_t ompi_mpi_dblcplex =       OMPI_DATATYPE_INIT_DEFER (DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
-#else
-ompi_predefined_datatype_t ompi_mpi_dblcplex =       OMPI_DATATYPE_INIT_UNAVAILABLE (DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
-#endif
 
-/* In Fortran, there does not exist a type LONG DOUBLE COMPLEX, but DOUBLE COMPLEX(KIND=8) may require this */
-#if HAVE_LONG_DOUBLE && ( OMPI_HAVE_F90_DOUBLE_COMPLEX || OMPI_HAVE_FORTRAN_DOUBLE_COMPLEX )
-/* We do not configure-check for alignment of F90 types ... Alignment of F77 OMPI_ALIGNMENT_FORTRAN_COMPLEX has to suffice */
-ompi_predefined_datatype_t ompi_mpi_ldblcplex =      OMPI_DATATYPE_INIT_DEFER (LONG_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
-#else
-ompi_predefined_datatype_t ompi_mpi_ldblcplex =      OMPI_DATATYPE_INIT_UNAVAILABLE (LONG_DOUBLE_COMPLEX, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX );
-#endif
 
 /* Aggregate struct datatypes are not const */
 ompi_predefined_datatype_t ompi_mpi_float_int =      OMPI_DATATYPE_INIT_DEFER (FLOAT_INT, OMPI_DATATYPE_FLAG_DATA_C );
@@ -235,20 +258,43 @@ ompi_predefined_datatype_t ompi_mpi_integer16 =      OMPI_DATATYPE_INIT_PREDEFIN
 ompi_predefined_datatype_t ompi_mpi_integer16 =      OMPI_DATATYPE_INIT_UNAVAILABLE (INTEGER8, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_INT);
 #endif
 
-#if OMPI_HAVE_FORTRAN_COMPLEX8
-ompi_predefined_datatype_t ompi_mpi_complex8 =       OMPI_DATATYPE_INIT_DEFER (COMPLEX8, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX);
+/*
+ * MPI 2.2 Datatypes
+ */
+ompi_predefined_datatype_t ompi_mpi_int8_t   = OMPI_DATATYPE_INIT_PREDEFINED(  INT8_T, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+ompi_predefined_datatype_t ompi_mpi_uint8_t  = OMPI_DATATYPE_INIT_PREDEFINED( UINT8_T, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+ompi_predefined_datatype_t ompi_mpi_int16_t  = OMPI_DATATYPE_INIT_PREDEFINED( INT16_T, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+ompi_predefined_datatype_t ompi_mpi_uint16_t = OMPI_DATATYPE_INIT_PREDEFINED(UINT16_T, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+ompi_predefined_datatype_t ompi_mpi_int32_t  = OMPI_DATATYPE_INIT_PREDEFINED( INT32_T, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+ompi_predefined_datatype_t ompi_mpi_uint32_t = OMPI_DATATYPE_INIT_PREDEFINED(UINT32_T, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+ompi_predefined_datatype_t ompi_mpi_int64_t  = OMPI_DATATYPE_INIT_PREDEFINED( INT64_T, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+ompi_predefined_datatype_t ompi_mpi_uint64_t = OMPI_DATATYPE_INIT_PREDEFINED(UINT64_T, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+
+#if SIZEOF_PTRDIFF_T == 4
+ompi_predefined_datatype_t ompi_mpi_aint = OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE(INT32_T, AINT, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+#elif SIZEOF_PTRDIFF_T == 8
+ompi_predefined_datatype_t ompi_mpi_aint = OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE(INT64_T, AINT, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
 #else
-ompi_predefined_datatype_t ompi_mpi_complex8 =       OMPI_DATATYPE_INIT_UNAVAILABLE (COMPLEX8, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX);
-#endif
-#if OMPI_HAVE_FORTRAN_COMPLEX16
-ompi_predefined_datatype_t ompi_mpi_complex16 =      OMPI_DATATYPE_INIT_DEFER (COMPLEX16, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX);
+ompi_predefined_datatype_t ompi_mpi_aint = OMPI_DATATYPE_INIT_UNAVAILABLE_BASIC_TYPE(INT64_T, AINT, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+#endif  /* SIZEOF_PTRDIFF_T == SIZEOF_LONG */
+
+#if OMPI_MPI_OFFSET_SIZE == 4
+ompi_predefined_datatype_t ompi_mpi_offset = OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE(UINT32_T, OFFSET, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+#elif OMPI_MPI_OFFSET_SIZE == 8
+ompi_predefined_datatype_t ompi_mpi_offset = OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE(UINT64_T, OFFSET, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
 #else
-ompi_predefined_datatype_t ompi_mpi_complex16 =      OMPI_DATATYPE_INIT_UNAVAILABLE (COMPLEX16, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX);
-#endif
-#if OMPI_HAVE_FORTRAN_COMPLEX32
-ompi_predefined_datatype_t ompi_mpi_complex32 =      OMPI_DATATYPE_INIT_DEFER (COMPLEX32, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX);
+ompi_predefined_datatype_t ompi_mpi_offset = OMPI_DATATYPE_INIT_UNAVAILABLE_BASIC_TYPE(UINT64_T, OFFSET, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+#endif  /* OMPI_MPI_OFFSET_SIZE == SIZEOF_INT */
+
+/*
+ * MPI 3.0 Datatypes
+ */
+#if OMPI_MPI_COUNT_SIZE == 4
+ompi_predefined_datatype_t ompi_mpi_count = OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE(INT32_T, COUNT, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
+#elif OMPI_MPI_COUNT_SIZE == 8
+ompi_predefined_datatype_t ompi_mpi_count = OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE(INT64_T, COUNT, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
 #else
-ompi_predefined_datatype_t ompi_mpi_complex32 =      OMPI_DATATYPE_INIT_UNAVAILABLE (COMPLEX32, OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_COMPLEX);
+ompi_predefined_datatype_t ompi_mpi_count = OMPI_DATATYPE_INIT_UNAVAILABLE_BASIC_TYPE(INT64_T, COUNT, OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT);
 #endif
 
 
@@ -258,55 +304,66 @@ ompi_predefined_datatype_t ompi_mpi_complex32 =      OMPI_DATATYPE_INIT_UNAVAILA
  * Everything referring to types/ids should be ORDERED as in ompi_datatype_basicDatatypes array.
  */
 const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX_PREDEFINED] = {
-    &ompi_mpi_lb.dt,                   /*  0 */
-    &ompi_mpi_ub.dt,
-    &ompi_mpi_char.dt,
-    &ompi_mpi_signed_char.dt,
-    &ompi_mpi_unsigned_char.dt,
-    &ompi_mpi_byte.dt,                 /*  5 */
-    &ompi_mpi_short.dt,
-    &ompi_mpi_unsigned_short.dt,
-    &ompi_mpi_int.dt,
-    &ompi_mpi_unsigned.dt,
-    &ompi_mpi_long.dt,                 /* 10 */
-    &ompi_mpi_unsigned_long.dt,
-    &ompi_mpi_long_long_int.dt,
-    &ompi_mpi_unsigned_long_long.dt,
-    &ompi_mpi_float.dt,
-    &ompi_mpi_double.dt,               /* 15 */
-    &ompi_mpi_long_double.dt,
-    &ompi_mpi_complex8.dt,
-    &ompi_mpi_complex16.dt,
-    &ompi_mpi_complex32.dt,
-    &ompi_mpi_wchar.dt,                /* 20 */
-    &ompi_mpi_packed.dt,
+    [OMPI_DATATYPE_MPI_EMPTY] = &ompi_mpi_datatype_null.dt,
+    [OMPI_DATATYPE_MPI_INT8_T] = &ompi_mpi_int8_t.dt,
+    [OMPI_DATATYPE_MPI_UINT8_T] = &ompi_mpi_uint8_t.dt,
+    [OMPI_DATATYPE_MPI_INT16_T] = &ompi_mpi_int16_t.dt,
+    [OMPI_DATATYPE_MPI_UINT16_T] = &ompi_mpi_uint16_t.dt,
+    [OMPI_DATATYPE_MPI_INT32_T] = &ompi_mpi_int32_t.dt,
+    [OMPI_DATATYPE_MPI_UINT32_T] = &ompi_mpi_uint32_t.dt,
+    [OMPI_DATATYPE_MPI_INT64_T] = &ompi_mpi_int64_t.dt,
+    [OMPI_DATATYPE_MPI_UINT64_T] = &ompi_mpi_uint64_t.dt,
+    [OMPI_DATATYPE_MPI_FLOAT] = &ompi_mpi_float.dt,
+    [OMPI_DATATYPE_MPI_DOUBLE] = &ompi_mpi_double.dt,
+    [OMPI_DATATYPE_MPI_LONG_DOUBLE] = &ompi_mpi_long_double.dt,
+    [OMPI_DATATYPE_MPI_COMPLEX8] = &ompi_mpi_complex8.dt,
+    [OMPI_DATATYPE_MPI_COMPLEX16] = &ompi_mpi_complex16.dt,
+    [OMPI_DATATYPE_MPI_COMPLEX32] = &ompi_mpi_complex32.dt,
+    [OMPI_DATATYPE_MPI_WCHAR] = &ompi_mpi_wchar.dt,
+    [OMPI_DATATYPE_MPI_PACKED] = &ompi_mpi_packed.dt,
 
     /* C++ / C99 datatypes */
-    &ompi_mpi_cxx_bool.dt,
+    [OMPI_DATATYPE_MPI_BOOL] = &ompi_mpi_cxx_bool.dt,
 
     /* Fortran datatypes */
-    &ompi_mpi_logical.dt,
-    &ompi_mpi_character.dt,
-    &ompi_mpi_integer.dt,              /* 25 */
-    &ompi_mpi_real.dt,
-    &ompi_mpi_dblprec.dt,
-    &ompi_mpi_cplex.dt,
-    &ompi_mpi_dblcplex.dt,
-    &ompi_mpi_ldblcplex.dt,            /* 30 */
+    [OMPI_DATATYPE_MPI_LOGICAL] = &ompi_mpi_logical.dt,
+    [OMPI_DATATYPE_MPI_CHARACTER] = &ompi_mpi_character.dt,
+    [OMPI_DATATYPE_MPI_INTEGER] = &ompi_mpi_integer.dt,
+    [OMPI_DATATYPE_MPI_REAL] = &ompi_mpi_real.dt,
+    [OMPI_DATATYPE_MPI_DOUBLE_PRECISION] = &ompi_mpi_dblprec.dt,
 
-    /* Structure types, based on two basic types */
-    &ompi_mpi_2int.dt,
-    &ompi_mpi_2integer.dt,
-    &ompi_mpi_2real.dt,
-    &ompi_mpi_2dblprec.dt,
-    &ompi_mpi_2cplex.dt,               /* 35 */
-    &ompi_mpi_2dblcplex.dt,
-    &ompi_mpi_float_int.dt,
-    &ompi_mpi_double_int.dt,
-    &ompi_mpi_longdbl_int.dt,
-    &ompi_mpi_long_int.dt,             /* 40 */
-    &ompi_mpi_short_int.dt,
-    &ompi_mpi_unavailable.dt
+    [OMPI_DATATYPE_MPI_COMPLEX] = &ompi_mpi_cplex.dt,
+    [OMPI_DATATYPE_MPI_DOUBLE_COMPLEX] = &ompi_mpi_dblcplex.dt,
+    [OMPI_DATATYPE_MPI_LONG_DOUBLE_COMPLEX] = &ompi_mpi_ldblcplex.dt,
+    [OMPI_DATATYPE_MPI_2INT] = &ompi_mpi_2int.dt,
+    [OMPI_DATATYPE_MPI_2INTEGER] = &ompi_mpi_2integer.dt,
+    [OMPI_DATATYPE_MPI_2REAL] = &ompi_mpi_2real.dt,
+    [OMPI_DATATYPE_MPI_2DBLPREC] = &ompi_mpi_2dblprec.dt,
+    [OMPI_DATATYPE_MPI_2COMPLEX] = &ompi_mpi_2cplex.dt,
+    [OMPI_DATATYPE_MPI_2DOUBLE_COMPLEX] = &ompi_mpi_2dblcplex.dt,
+
+    [OMPI_DATATYPE_MPI_FLOAT_INT] = &ompi_mpi_float_int.dt,
+    [OMPI_DATATYPE_MPI_DOUBLE_INT] = &ompi_mpi_double_int.dt,
+    [OMPI_DATATYPE_MPI_LONG_DOUBLE_INT] = &ompi_mpi_longdbl_int.dt,
+    [OMPI_DATATYPE_MPI_LONG_INT] = &ompi_mpi_long_int.dt,
+    [OMPI_DATATYPE_MPI_SHORT_INT] = &ompi_mpi_short_int.dt,
+
+    /* MPI 2.2 types */
+    [OMPI_DATATYPE_MPI_AINT] = &ompi_mpi_aint.dt,
+    [OMPI_DATATYPE_MPI_OFFSET] = &ompi_mpi_offset.dt,
+    [OMPI_DATATYPE_MPI_C_BOOL] = &ompi_mpi_c_bool.dt,
+    [OMPI_DATATYPE_MPI_C_COMPLEX] = &ompi_mpi_c_complex.dt,
+    [OMPI_DATATYPE_MPI_C_FLOAT_COMPLEX] = &ompi_mpi_c_float_complex.dt,
+    [OMPI_DATATYPE_MPI_C_DOUBLE_COMPLEX] = &ompi_mpi_c_double_complex.dt,
+    [OMPI_DATATYPE_MPI_C_LONG_DOUBLE_COMPLEX] = &ompi_mpi_c_long_double_complex.dt,
+
+    [OMPI_DATATYPE_MPI_LB] = &ompi_mpi_lb.dt,
+    [OMPI_DATATYPE_MPI_UB] = &ompi_mpi_ub.dt,
+
+    /* MPI 3.0 types */
+    [OMPI_DATATYPE_MPI_COUNT] = &ompi_mpi_count.dt,
+
+    [OMPI_DATATYPE_MPI_UNAVAILABLE] = &ompi_mpi_unavailable.dt,
 };
 
 opal_pointer_array_t ompi_datatype_f_to_c_table;
@@ -395,29 +452,6 @@ int32_t ompi_datatype_init( void )
 {
     int32_t i;
 
-    for( i = OMPI_DATATYPE_FIRST_TYPE; i < OMPI_DATATYPE_MPI_MAX_PREDEFINED; i++ ) {
-        const ompi_datatype_t* datatype = (ompi_datatype_t*)ompi_datatype_basicDatatypes[i];
-
-        datatype->super.desc.desc[0].elem.common.flags = OPAL_DATATYPE_FLAG_PREDEFINED |
-                                                         OPAL_DATATYPE_FLAG_DATA |
-                                                         OPAL_DATATYPE_FLAG_CONTIGUOUS;
-        datatype->super.desc.desc[0].elem.common.type  = i;
-        datatype->super.desc.desc[0].elem.count        = 1;
-        datatype->super.desc.desc[0].elem.disp         = 0;
-        datatype->super.desc.desc[0].elem.extent       = datatype->super.size;
-
-        datatype->super.desc.desc[1].end_loop.common.flags    = 0;
-        datatype->super.desc.desc[1].end_loop.common.type     = OPAL_DATATYPE_END_LOOP;
-        datatype->super.desc.desc[1].end_loop.items           = 1;
-        datatype->super.desc.desc[1].end_loop.first_elem_disp = datatype->super.desc.desc[0].elem.disp;
-        datatype->super.desc.desc[1].end_loop.size            = datatype->super.size;
-
-        /* Check if the data contain gaps */
-        if( (datatype->super.ub - datatype->super.lb) == (OPAL_PTRDIFF_TYPE)datatype->super.size ) {
-            datatype->super.desc.desc[0].elem.common.flags |= OPAL_DATATYPE_FLAG_NO_GAPS;
-        }
-    }
-
     /* Create the f2c translation table */
     OBJ_CONSTRUCT(&ompi_datatype_f_to_c_table, opal_pointer_array_t);
     if( OPAL_SUCCESS != opal_pointer_array_init(&ompi_datatype_f_to_c_table,
@@ -430,25 +464,13 @@ int32_t ompi_datatype_init( void )
      * in mpif.h
      */
 
-    /* the complex datatypes (float, double and long double) */
-    DECLARE_MPI2_COMPOSED_STRUCT_DDT( &ompi_mpi_cplex.dt, OMPI_DATATYPE_COMPLEX, "MPI_COMPLEX",
-                                      float, float, OMPI_DATATYPE_MPI_FLOAT, OMPI_DATATYPE_MPI_FLOAT,
-                                      OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_FLOAT );
-    DECLARE_MPI2_COMPOSED_STRUCT_DDT( &ompi_mpi_dblcplex.dt, OMPI_DATATYPE_DOUBLE_COMPLEX, "MPI_DOUBLE_COMPLEX",
-                                      double, double, OMPI_DATATYPE_MPI_DOUBLE, OMPI_DATATYPE_MPI_DOUBLE,
-                                      OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_FLOAT );
-#if HAVE_LONG_DOUBLE
-    DECLARE_MPI2_COMPOSED_STRUCT_DDT( &ompi_mpi_ldblcplex.dt, OMPI_DATATYPE_LONG_DOUBLE_COMPLEX, "MPI_LONG_DOUBLE_COMPLEX",
-                                      long double, long double, OMPI_DATATYPE_MPI_LONG_DOUBLE, OMPI_DATATYPE_MPI_LONG_DOUBLE,
-                                      OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_FLOAT );
-#endif  /* HAVE_LONG_DOUBLE */
 
     /* Now the predefined MPI2 datatypes (they should last forever!) */
     DECLARE_MPI2_COMPOSED_BLOCK_DDT( &ompi_mpi_2int.dt, OMPI_DATATYPE_2INT, "MPI_2INT",
                                      OMPI_DATATYPE_MPI_INT,
                                      OMPI_DATATYPE_FLAG_DATA_C | OMPI_DATATYPE_FLAG_DATA_INT );
     DECLARE_MPI2_COMPOSED_BLOCK_DDT( &ompi_mpi_2integer.dt, OMPI_DATATYPE_2INTEGER, "MPI_2INTEGER",
-                                     OMPI_DATATYPE_MPI_INT,
+                                     OMPI_DATATYPE_MPI_INTEGER,
                                      OMPI_DATATYPE_FLAG_DATA_FORTRAN | OMPI_DATATYPE_FLAG_DATA_INT);
     DECLARE_MPI2_COMPOSED_BLOCK_DDT( &ompi_mpi_2real.dt, OMPI_DATATYPE_2REAL, "MPI_2REAL",
                                      OMPI_DATATYPE_MPI_FLOAT,
@@ -488,16 +510,6 @@ int32_t ompi_datatype_init( void )
     /* Copy the desc pointer from the <OMPI_DATATYPE_MPI_MAX_PREDEFINED datatypes to
        the synonym types */
 
-    /* C++ complex types */
-    DECLARE_MPI_SYNONYM_DDT( &ompi_mpi_cxx_cplex.dt, "MPI_CXX_COMPLEX", &ompi_mpi_cplex.dt );
-    ompi_mpi_cxx_cplex.dt.super.flags |= OMPI_DATATYPE_FLAG_DATA_CPP | OMPI_DATATYPE_FLAG_DATA_COMPLEX;
-    DECLARE_MPI_SYNONYM_DDT( &ompi_mpi_cxx_dblcplex.dt, "MPI_CXX_DOUBLE_COMPLEX", &ompi_mpi_dblcplex.dt );
-    ompi_mpi_cxx_dblcplex.dt.super.flags |= OMPI_DATATYPE_FLAG_DATA_CPP | OMPI_DATATYPE_FLAG_DATA_COMPLEX;
-#if HAVE_LONG_DOUBLE
-    DECLARE_MPI_SYNONYM_DDT( &ompi_mpi_cxx_ldblcplex.dt, "MPI_CXX_LONG_DOUBLE_COMPLEX", &ompi_mpi_ldblcplex.dt );
-    ompi_mpi_cxx_ldblcplex.dt.super.flags |= OMPI_DATATYPE_FLAG_DATA_CPP | OMPI_DATATYPE_FLAG_DATA_COMPLEX;
-#endif  /* HAVE_LONG_DOUBLE */
-
 
     /* Start to populate the f2c index translation table */
 
@@ -509,86 +521,109 @@ int32_t ompi_datatype_init( void )
     /* This macro makes everything significantly easier to read below.
        All hail the moog!  :-) */
 
-#define MOOG(name)                                                                   \
+#define MOOG(name, index)                                                            \
     {                                                                                \
         ompi_mpi_##name.dt.d_f_to_c_index =                                          \
             opal_pointer_array_add(&ompi_datatype_f_to_c_table, &ompi_mpi_##name);   \
         if( ompi_datatype_number_of_predefined_data < (ompi_mpi_##name).dt.d_f_to_c_index ) \
             ompi_datatype_number_of_predefined_data = (ompi_mpi_##name).dt.d_f_to_c_index; \
+        assert( (index) == ompi_mpi_##name.dt.d_f_to_c_index );                      \
     }
 
     /*
      * This MUST match the order of ompi/include/mpif-common.h
      * Any change will break binary compatibility of Fortran programs.
      */
-    MOOG(datatype_null);
-    MOOG(byte);
-    MOOG(packed);
-    MOOG(ub);
-    MOOG(lb);
-    MOOG(character);
-    MOOG(logical);
-    MOOG(integer);
-    MOOG(integer1);
-    MOOG(integer2);
-    MOOG(integer4);
-    MOOG(integer8);
-    MOOG(integer16);
-    MOOG(real);
-    MOOG(real4);
-    MOOG(real8);
-    MOOG(real16);
-    MOOG(dblprec);
-    MOOG(cplex);
-    MOOG(complex8);
-    MOOG(complex16);
-    MOOG(complex32);
-    MOOG(dblcplex);
-    MOOG(2real);
-    MOOG(2dblprec);
-    MOOG(2integer);
-    MOOG(2cplex);
-    MOOG(2dblcplex);
-    MOOG(real2);
-    MOOG(logical1);
-    MOOG(logical2);
-    MOOG(logical4);
-    MOOG(logical8);
+    MOOG(datatype_null, 0);
+    MOOG(byte, 1);
+    MOOG(packed, 2);
+    MOOG(ub, 3);
+    MOOG(lb, 4);
+    MOOG(character, 5);
+    MOOG(logical, 6);
+    MOOG(integer, 7);
+    MOOG(integer1, 8);
+    MOOG(integer2, 9);
+    MOOG(integer4, 10);
+    MOOG(integer8, 11);
+    MOOG(integer16, 12);
+    MOOG(real, 13);
+    MOOG(real4, 14);
+    MOOG(real8, 15);
+    MOOG(real16, 16);
+    MOOG(dblprec, 17);
+    MOOG(cplex, 18);
+    MOOG(complex8, 19);
+    MOOG(complex16, 20);
+    MOOG(complex32, 21);
+    MOOG(dblcplex, 22);
+    MOOG(2real, 23);
+    MOOG(2dblprec, 24);
+    MOOG(2integer, 25);
+    MOOG(2cplex, 26);
+    MOOG(2dblcplex, 27);
+    MOOG(real2, 28);
+    MOOG(logical1, 29);
+    MOOG(logical2, 30);
+    MOOG(logical4, 31);
+    MOOG(logical8, 32);
 
     /* Now the C types */
 
-    MOOG(wchar);
-    MOOG(char);
-    MOOG(unsigned_char);
-    MOOG(signed_char);
-    MOOG(short);
-    MOOG(unsigned_short);
-    MOOG(int);
-    MOOG(unsigned);
-    MOOG(long);
-    MOOG(unsigned_long);
-    MOOG(long_long_int);
-    MOOG(unsigned_long_long);
+    MOOG(wchar, 33);
+    MOOG(char, 34);
+    MOOG(unsigned_char, 35);
+    MOOG(signed_char, 36);
+    MOOG(short, 37);
+    MOOG(unsigned_short, 38);
+    MOOG(int, 39);
+    MOOG(unsigned, 40);
+    MOOG(long, 41);
+    MOOG(unsigned_long, 42);
+    MOOG(long_long_int, 43);
+    MOOG(unsigned_long_long, 44);
 
-    MOOG(float);
-    MOOG(double);
-    MOOG(long_double);
+    MOOG(float, 45);
+    MOOG(double, 46);
+    MOOG(long_double, 47);
 
-    MOOG(float_int);
-    MOOG(double_int);
-    MOOG(longdbl_int);
-    MOOG(long_int);
-    MOOG(2int);
-    MOOG(short_int);
+    MOOG(float_int, 48);
+    MOOG(double_int, 49);
+    MOOG(longdbl_int, 50);
+    MOOG(long_int, 51);
+    MOOG(2int, 52);
+    MOOG(short_int, 53);
 
     /* C++ types */
 
-    MOOG(cxx_bool);
-    MOOG(cxx_cplex);
-    MOOG(cxx_dblcplex);
-    MOOG(cxx_ldblcplex);
+    MOOG(cxx_bool, 54);
+    MOOG(cxx_cplex, 55);
+    MOOG(cxx_dblcplex, 56);
+    MOOG(cxx_ldblcplex, 57);
 
-    for( i = 0; i < ompi_mpi_cxx_ldblcplex.dt.d_f_to_c_index; i++ ) {
+    /* MPI 2.2 types */
+    MOOG(int8_t, 58);
+    MOOG(uint8_t, 59);
+    MOOG(int16_t, 60);
+    MOOG(uint16_t, 61);
+    MOOG(int32_t, 62);
+    MOOG(uint32_t, 63);
+    MOOG(int64_t, 64);
+    MOOG(uint64_t, 65);
+    MOOG(aint, 66);
+    MOOG(offset, 67);
+    MOOG(c_complex, 68);
+    MOOG(c_float_complex, 69);
+    MOOG(c_double_complex, 70);
+    MOOG(c_long_double_complex, 71);
+
+    /* MPI 3.0 types */
+    MOOG(count, 72);
+
+    /**
+     * Now make sure all non-contiguous types are marked as such.
+     */
+    for( i = 0; i < ompi_mpi_offset.dt.d_f_to_c_index; i++ ) {
         opal_datatype_t* datatype = (opal_datatype_t*)opal_pointer_array_get_item(&ompi_datatype_f_to_c_table, i );
 
         if( (datatype->ub - datatype->lb) == (OPAL_PTRDIFF_TYPE)datatype->size ) {
@@ -712,9 +747,9 @@ void ompi_datatype_dump( const ompi_datatype_t* pData )
                      (long)pData->super.lb, (long)pData->super.ub, (long)(pData->super.ub - pData->super.lb),
                      (int)pData->super.nbElems, (int)pData->super.btypes[OPAL_DATATYPE_LOOP], (int)pData->super.flags );
     /* dump the flags */
-    if( ompi_datatype_is_predefined(pData) )
+    if( ompi_datatype_is_predefined(pData) ) {
         index += snprintf( buffer + index, length - index, "predefined " );
-    else {
+    } else {
         if( pData->super.flags & OPAL_DATATYPE_FLAG_COMMITED ) index += snprintf( buffer + index, length - index, "commited " );
         if( pData->super.flags & OPAL_DATATYPE_FLAG_CONTIGUOUS) index += snprintf( buffer + index, length - index, "contiguous " );
     }
