@@ -3,14 +3,15 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2007 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -20,9 +21,12 @@
 
 #include "ompi_config.h"
 
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
 #include "ompi/communicator/communicator.h"
 #include "ompi/win/win.h"
-#include "ompi/file/file.h"
 #include "ompi/errhandler/errhandler.h"
 #include "ompi/errhandler/errhandler_predefined.h"
 #include "opal/class/opal_pointer_array.h"
@@ -48,10 +52,21 @@ OBJ_CLASS_INSTANCE(ompi_errhandler_t, opal_object_t, ompi_errhandler_construct,
                    ompi_errhandler_destruct);
 
 
-ompi_errhandler_t ompi_mpi_errhandler_null;
-ompi_errhandler_t ompi_mpi_errors_are_fatal;
-ompi_errhandler_t ompi_mpi_errors_return;
-ompi_errhandler_t ompi_mpi_errors_throw_exceptions;
+/*
+ * _addr flavors are for F03 bindings
+ */
+ompi_predefined_errhandler_t ompi_mpi_errhandler_null;
+ompi_predefined_errhandler_t *ompi_mpi_errhandler_null_addr = 
+    &ompi_mpi_errhandler_null;
+ompi_predefined_errhandler_t ompi_mpi_errors_are_fatal;
+ompi_predefined_errhandler_t *ompi_mpi_errors_are_fatal_addr = 
+    &ompi_mpi_errors_are_fatal;
+ompi_predefined_errhandler_t ompi_mpi_errors_return;
+ompi_predefined_errhandler_t *ompi_mpi_errors_return_addr = 
+    &ompi_mpi_errors_return;
+ompi_predefined_errhandler_t ompi_mpi_errors_throw_exceptions;
+ompi_predefined_errhandler_t *ompi_mpi_errors_throw_exceptions_addr = 
+    &ompi_mpi_errors_throw_exceptions;
 
 /*
  * Local state to know when the three intrinsics have been freed; see
@@ -77,41 +92,41 @@ int ompi_errhandler_init(void)
   }
 
   /* Initialize the predefined error handlers */
-  OBJ_CONSTRUCT( &ompi_mpi_errhandler_null, ompi_errhandler_t );
-  if( ompi_mpi_errhandler_null.eh_f_to_c_index != OMPI_ERRHANDLER_NULL_FORTRAN )
+  OBJ_CONSTRUCT( &ompi_mpi_errhandler_null.eh, ompi_errhandler_t );
+  if( ompi_mpi_errhandler_null.eh.eh_f_to_c_index != OMPI_ERRHANDLER_NULL_FORTRAN )
       return OMPI_ERROR;
-  ompi_mpi_errhandler_null.eh_mpi_object_type = OMPI_ERRHANDLER_TYPE_PREDEFINED;
-  ompi_mpi_errhandler_null.eh_lang = OMPI_ERRHANDLER_LANG_C;
-  ompi_mpi_errhandler_null.eh_comm_fn = NULL;
-  ompi_mpi_errhandler_null.eh_file_fn = NULL;
-  ompi_mpi_errhandler_null.eh_win_fn  = NULL ;
-  ompi_mpi_errhandler_null.eh_fort_fn = NULL;
-  strncpy (ompi_mpi_errhandler_null.eh_name, "MPI_ERRHANDLER_NULL", 
+  ompi_mpi_errhandler_null.eh.eh_mpi_object_type = OMPI_ERRHANDLER_TYPE_PREDEFINED;
+  ompi_mpi_errhandler_null.eh.eh_lang = OMPI_ERRHANDLER_LANG_C;
+  ompi_mpi_errhandler_null.eh.eh_comm_fn = NULL;
+  ompi_mpi_errhandler_null.eh.eh_file_fn = NULL;
+  ompi_mpi_errhandler_null.eh.eh_win_fn  = NULL ;
+  ompi_mpi_errhandler_null.eh.eh_fort_fn = NULL;
+  strncpy (ompi_mpi_errhandler_null.eh.eh_name, "MPI_ERRHANDLER_NULL", 
 	   strlen("MPI_ERRHANDLER_NULL")+1 );
 
 
-  OBJ_CONSTRUCT( &ompi_mpi_errors_are_fatal, ompi_errhandler_t );
-  if( ompi_mpi_errors_are_fatal.eh_f_to_c_index != OMPI_ERRORS_ARE_FATAL_FORTRAN )
+  OBJ_CONSTRUCT( &ompi_mpi_errors_are_fatal.eh, ompi_errhandler_t );
+  if( ompi_mpi_errors_are_fatal.eh.eh_f_to_c_index != OMPI_ERRORS_ARE_FATAL_FORTRAN )
       return OMPI_ERROR;
-  ompi_mpi_errors_are_fatal.eh_mpi_object_type = OMPI_ERRHANDLER_TYPE_PREDEFINED;
-  ompi_mpi_errors_are_fatal.eh_lang = OMPI_ERRHANDLER_LANG_C;
-  ompi_mpi_errors_are_fatal.eh_comm_fn = ompi_mpi_errors_are_fatal_comm_handler;
-  ompi_mpi_errors_are_fatal.eh_file_fn = ompi_mpi_errors_are_fatal_file_handler;
-  ompi_mpi_errors_are_fatal.eh_win_fn  = ompi_mpi_errors_are_fatal_win_handler ;
-  ompi_mpi_errors_are_fatal.eh_fort_fn = NULL;
-  strncpy (ompi_mpi_errors_are_fatal.eh_name, "MPI_ERRORS_ARE_FATAL", 
+  ompi_mpi_errors_are_fatal.eh.eh_mpi_object_type = OMPI_ERRHANDLER_TYPE_PREDEFINED;
+  ompi_mpi_errors_are_fatal.eh.eh_lang = OMPI_ERRHANDLER_LANG_C;
+  ompi_mpi_errors_are_fatal.eh.eh_comm_fn = ompi_mpi_errors_are_fatal_comm_handler;
+  ompi_mpi_errors_are_fatal.eh.eh_file_fn = ompi_mpi_errors_are_fatal_file_handler;
+  ompi_mpi_errors_are_fatal.eh.eh_win_fn  = ompi_mpi_errors_are_fatal_win_handler ;
+  ompi_mpi_errors_are_fatal.eh.eh_fort_fn = NULL;
+  strncpy (ompi_mpi_errors_are_fatal.eh.eh_name, "MPI_ERRORS_ARE_FATAL", 
 	   strlen("MPI_ERRORS_ARE_FATAL")+1 );
   
-  OBJ_CONSTRUCT( &ompi_mpi_errors_return, ompi_errhandler_t );
-  if( ompi_mpi_errors_return.eh_f_to_c_index != OMPI_ERRORS_RETURN_FORTRAN )
+  OBJ_CONSTRUCT( &ompi_mpi_errors_return.eh, ompi_errhandler_t );
+  if( ompi_mpi_errors_return.eh.eh_f_to_c_index != OMPI_ERRORS_RETURN_FORTRAN )
       return OMPI_ERROR;
-  ompi_mpi_errors_return.eh_mpi_object_type  = OMPI_ERRHANDLER_TYPE_PREDEFINED;
-  ompi_mpi_errors_return.eh_lang = OMPI_ERRHANDLER_LANG_C;
-  ompi_mpi_errors_return.eh_comm_fn = ompi_mpi_errors_return_comm_handler;
-  ompi_mpi_errors_return.eh_file_fn = ompi_mpi_errors_return_file_handler;
-  ompi_mpi_errors_return.eh_win_fn  = ompi_mpi_errors_return_win_handler;
-  ompi_mpi_errors_return.eh_fort_fn = NULL;
-  strncpy (ompi_mpi_errors_return.eh_name, "MPI_ERRORS_RETURN", 
+  ompi_mpi_errors_return.eh.eh_mpi_object_type  = OMPI_ERRHANDLER_TYPE_PREDEFINED;
+  ompi_mpi_errors_return.eh.eh_lang = OMPI_ERRHANDLER_LANG_C;
+  ompi_mpi_errors_return.eh.eh_comm_fn = ompi_mpi_errors_return_comm_handler;
+  ompi_mpi_errors_return.eh.eh_file_fn = ompi_mpi_errors_return_file_handler;
+  ompi_mpi_errors_return.eh.eh_win_fn  = ompi_mpi_errors_return_win_handler;
+  ompi_mpi_errors_return.eh.eh_fort_fn = NULL;
+  strncpy (ompi_mpi_errors_return.eh.eh_name, "MPI_ERRORS_RETURN", 
 	   strlen("MPI_ERRORS_RETURN")+1 );
 
   /* If we're going to use C++, functions will be fixed up during
@@ -119,19 +134,18 @@ int ompi_errhandler_init(void)
      the dispatch function is in C (although in libmpi_cxx); the
      conversion from C handles to C++ handles happens in that dispatch
      function -- not the errhandler_invoke.c stuff here in libmpi. */
-  OBJ_CONSTRUCT( &ompi_mpi_errors_throw_exceptions, ompi_errhandler_t );
-  ompi_mpi_errors_throw_exceptions.eh_mpi_object_type = 
+  OBJ_CONSTRUCT( &ompi_mpi_errors_throw_exceptions.eh, ompi_errhandler_t );
+  ompi_mpi_errors_throw_exceptions.eh.eh_mpi_object_type = 
       OMPI_ERRHANDLER_TYPE_PREDEFINED;
-  ompi_mpi_errors_throw_exceptions.eh_lang = OMPI_ERRHANDLER_LANG_C;
-  ompi_mpi_errors_throw_exceptions.eh_comm_fn = 
+  ompi_mpi_errors_throw_exceptions.eh.eh_lang = OMPI_ERRHANDLER_LANG_C;
+  ompi_mpi_errors_throw_exceptions.eh.eh_comm_fn = 
       ompi_mpi_errors_are_fatal_comm_handler;
-  ompi_mpi_errors_throw_exceptions.eh_file_fn = 
+  ompi_mpi_errors_throw_exceptions.eh.eh_file_fn = 
       ompi_mpi_errors_are_fatal_file_handler;
-  ompi_mpi_errors_throw_exceptions.eh_win_fn  = 
+  ompi_mpi_errors_throw_exceptions.eh.eh_win_fn  = 
       ompi_mpi_errors_are_fatal_win_handler ;
-  ompi_mpi_errors_throw_exceptions.eh_fort_fn = NULL;
-  strncpy (ompi_mpi_errors_throw_exceptions.eh_name,
-           "MPI_ERRORS_THROW_EXCEPTIONS", 
+  ompi_mpi_errors_throw_exceptions.eh.eh_fort_fn = NULL;
+  strncpy (ompi_mpi_errors_throw_exceptions.eh.eh_name, "MPI_ERRORS_THROW_EXCEPTIONS", 
 	   strlen("MPI_ERRORS_THROW_EXCEPTIONS")+1 );
 
   /* All done */
@@ -154,16 +168,16 @@ int ompi_errhandler_finalize(void)
        freed. */
 
     while (!null_freed) {
-        OBJ_DESTRUCT(&ompi_mpi_errhandler_null);
+        OBJ_DESTRUCT(&ompi_mpi_errhandler_null.eh);
     }
     while (!fatal_freed) {
-        OBJ_DESTRUCT(&ompi_mpi_errors_are_fatal);
+        OBJ_DESTRUCT(&ompi_mpi_errors_are_fatal.eh);
     }
     while (!return_freed) {
-        OBJ_DESTRUCT(&ompi_mpi_errors_return);
+        OBJ_DESTRUCT(&ompi_mpi_errors_return.eh);
     }
     while (!throw_freed) {
-        OBJ_DESTRUCT(&ompi_mpi_errors_throw_exceptions);
+        OBJ_DESTRUCT(&ompi_mpi_errors_throw_exceptions.eh);
     }
   
     /* JMS Add stuff here checking for unreleased errorhandlers,
@@ -196,7 +210,7 @@ ompi_errhandler_t *ompi_errhandler_create(ompi_errhandler_type_t object_type,
 
   new_errhandler = OBJ_NEW(ompi_errhandler_t);
   if (NULL != new_errhandler) {
-    if (OMPI_ERROR == new_errhandler->eh_f_to_c_index) {
+    if (0 > new_errhandler->eh_f_to_c_index) {
       OBJ_RELEASE(new_errhandler);
       new_errhandler = NULL;
     } else {
@@ -210,13 +224,13 @@ ompi_errhandler_t *ompi_errhandler_create(ompi_errhandler_type_t object_type,
       new_errhandler->eh_lang = lang;
       switch (object_type ) {
 	  case (OMPI_ERRHANDLER_TYPE_COMM):
-	      new_errhandler->eh_comm_fn = (MPI_Comm_errhandler_fn *)func;
+	      new_errhandler->eh_comm_fn = (MPI_Comm_errhandler_function *)func;
 	      break;
 	  case (OMPI_ERRHANDLER_TYPE_FILE):
 	      new_errhandler->eh_file_fn = (ompi_file_errhandler_fn *)func;
 	      break;
 	  case (OMPI_ERRHANDLER_TYPE_WIN):
-	      new_errhandler->eh_win_fn = (MPI_Win_errhandler_fn *)func;
+	      new_errhandler->eh_win_fn = (MPI_Win_errhandler_function *)func;
 	      break;
 	  default:
 	      break;
@@ -231,6 +245,12 @@ ompi_errhandler_t *ompi_errhandler_create(ompi_errhandler_type_t object_type,
   return new_errhandler;
 }
 
+/**
+ * Runtime errhandler callback
+ */
+void ompi_errhandler_runtime_callback(opal_pointer_array_t *procs) {
+    ompi_mpi_abort(MPI_COMM_WORLD, 1, false);
+}
 
 /**************************************************************************
  *
@@ -281,13 +301,13 @@ static void ompi_errhandler_destruct(ompi_errhandler_t *errhandler)
   /* Reset the static state if we're releasing one of the
      intrinsics */
 
-  if (&ompi_mpi_errhandler_null == errhandler) {
+  if (&ompi_mpi_errhandler_null.eh == errhandler) {
       null_freed = true;
-  } else if (&ompi_mpi_errors_are_fatal == errhandler) {
+  } else if (&ompi_mpi_errors_are_fatal.eh == errhandler) {
       fatal_freed = true;
-  } else if (&ompi_mpi_errors_return == errhandler) {
+  } else if (&ompi_mpi_errors_return.eh == errhandler) {
       return_freed = true;
-  } else if (&ompi_mpi_errors_throw_exceptions == errhandler) {
+  } else if (&ompi_mpi_errors_throw_exceptions.eh == errhandler) {
       throw_freed = true;
   }
 }
