@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -9,8 +10,10 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2006-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
+ * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2012      NVIDIA Corporation.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -28,8 +31,8 @@
  * During library initialization, all available BTL components are
  * loaded and opened via their mca_base_open_component_fn_t
  * function. The BTL open function should register any mca parameters
- * used to tune/adjust the behaviour of the BTL (mca_base_param_register_int(),
- * mca_base_param_register_string()). Note that the open function may fail
+ * used to tune/adjust the behaviour of the BTL (mca_base_var_register()
+ * mca_base_component_var_register()). Note that the open function may fail
  * if the resources (e.g. shared libraries, etc) required by the network
  * transport are not available.
  *
@@ -106,14 +109,17 @@
  *
  */
 
-#include "opal/mca/mca.h"
-
 #ifndef MCA_BTL_H
 #define MCA_BTL_H
 
-#include "ompi/types.h"
+#include "ompi_config.h"
+#include "opal/mca/mca.h"
+#include "opal/class/opal_bitmap.h"
+#include "opal/datatype/opal_convertor.h"
 #include "opal/prefetch.h" /* For OPAL_LIKELY */
 #include "ompi/mca/mpool/mpool.h"
+#include "ompi/types.h"
+#include "opal/types.h"
 
 #include "opal/mca/crs/crs.h"
 #include "opal/mca/crs/base/base.h"
@@ -187,6 +193,15 @@ typedef uint8_t mca_btl_base_tag_t;
  /* btl can do heterogeneous rdma operations on byte buffers */
 #define MCA_BTL_FLAGS_HETEROGENEOUS_RDMA 0x0100
 
+/* btl can support failover if enabled */
+#define MCA_BTL_FLAGS_FAILOVER_SUPPORT 0x0200
+
+#define MCA_BTL_FLAGS_CUDA_PUT        0x0400
+#define MCA_BTL_FLAGS_CUDA_GET        0x0800
+#define MCA_BTL_FLAGS_CUDA_RDMA (MCA_BTL_FLAGS_CUDA_GET|MCA_BTL_FLAGS_CUDA_PUT)
+#define MCA_BTL_FLAGS_CUDA_COPY_ASYNC_SEND 0x1000
+#define MCA_BTL_FLAGS_CUDA_COPY_ASYNC_RECV 0x2000
+
 /* Default exclusivity levels */
 #define MCA_BTL_EXCLUSIVITY_HIGH     (64*1024) /* internal loopback */
 #define MCA_BTL_EXCLUSIVITY_DEFAULT  1024      /* GM/IB/etc. */
@@ -194,6 +209,7 @@ typedef uint8_t mca_btl_base_tag_t;
 
 /* error callback flags */
 #define MCA_BTL_ERROR_FLAGS_FATAL 0x1
+#define MCA_BTL_ERROR_FLAGS_NONFATAL 0x2
 
 /**
  * Asynchronous callback function on completion of an operation.
@@ -216,23 +232,26 @@ typedef void (*mca_btl_base_completion_fn_t)(
 /**
  * Describes a region/segment of memory that is addressable 
  * by an BTL.
+ *
+ * Note: In many cases the alloc and prepare methods of BTLs
+ * do not return a mca_btl_base_segment_t but instead return a
+ * subclass. Extreme care should be used when modifying
+ * BTL segments to prevent overwriting internal BTL data.
+ *
+ * All BTLs MUST use base segments when calling registered
+ * Callbacks.
+ *
+ * BTL MUST use mca_btl_base_segment_t or a subclass and
+ * MUST store their segment length in btl_seg_size. BTLs
+ * MIST specify a segment no larger than MCA_BTL_SEG_MAX_SIZE.
+ *
  */
 
 struct mca_btl_base_segment_t {
     /** Address of the memory */
     ompi_ptr_t seg_addr;        
      /** Length in bytes */
-    uint32_t   seg_len;           
-#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
-    /** Heterogeneous padding */
-    uint8_t    seg_padding[4];     
-#endif
-    /** Memory segment key required by some RDMA networks */
-    union {
-        uint32_t key32[2];
-        uint64_t key64;
-        uint8_t  key8[8];
-    } seg_key;     
+    uint64_t   seg_len;
 };
 typedef struct mca_btl_base_segment_t mca_btl_base_segment_t;
 
@@ -281,11 +300,26 @@ OMPI_DECLSPEC OBJ_CLASS_DECLARATION(mca_btl_base_descriptor_t);
  */
 #define MCA_BTL_DES_SEND_ALWAYS_CALLBACK    0x0004
 
+/* Tell the PML that the copy is being done asynchronously
+ */
+#define MCA_BTL_DES_FLAGS_CUDA_COPY_ASYNC   0x0008
+
+/* Type of transfer that will be done with this frag.
+ */
+#define MCA_BTL_DES_FLAGS_PUT               0x0010
+#define MCA_BTL_DES_FLAGS_GET               0x0020
+
 /**
  * Maximum number of allowed segments in src/dst fields of a descriptor.
  */
 #define MCA_BTL_DES_MAX_SEGMENTS 16
 
+/**
+ * Maximum size of a BTL segment (NTH: does it really save us anything
+ * to hardcode this?)
+ */
+#define MCA_BTL_SEG_MAX_SIZE 256
+
 /* 
  *  BTL base header, stores the tag at a minimum 
  */ 
@@ -352,6 +386,8 @@ typedef int (*mca_btl_base_component_progress_fn_t)(void);
  * completion function, this implies that all data payload in the 
  * mca_btl_base_descriptor_t must be copied out within this callback or 
  * forfeited back to the BTL.
+ * Note also that descriptor segments (des_dst, des_src) must be base
+ * segments for all callbacks.
  * 
  * @param[IN] btl        BTL module
  * @param[IN] tag        The active message receive callback tag value 
@@ -451,7 +487,7 @@ typedef int (*mca_btl_base_module_add_procs_fn_t)(
     size_t nprocs,
     struct ompi_proc_t** procs, 
     struct mca_btl_base_endpoint_t** endpoints,
-    struct ompi_bitmap_t* reachable
+    struct opal_bitmap_t* reachable
 );
 
 /**
@@ -500,13 +536,17 @@ typedef int (*mca_btl_base_module_register_fn_t)(
  * Callback function that is called asynchronously on receipt
  * of an error from the transport layer 
  *
- * @param[IN] btl    BTL module
- * @param[IN] flags  type of error 
+ * @param[IN] btl     BTL module
+ * @param[IN] flags   type of error 
+ * @param[IN] errproc process that had an error
+ * @param[IN] btlinfo descriptive string from the BTL
  */
 
 typedef void (*mca_btl_base_module_error_cb_fn_t)(
         struct mca_btl_base_module_t* btl,
-        int32_t flags                             
+        int32_t flags,
+        struct ompi_proc_t* errproc,
+        char* btlinfo
 );
 
 
@@ -599,7 +639,7 @@ typedef struct mca_btl_base_descriptor_t* (*mca_btl_base_module_prepare_fn_t)(
     struct mca_btl_base_module_t* btl,
     struct mca_btl_base_endpoint_t* endpoint,
     mca_mpool_base_registration_t* registration,
-    struct ompi_convertor_t* convertor,
+    struct opal_convertor_t* convertor,
     uint8_t order,
     size_t reserve,
     size_t* size,
@@ -646,6 +686,7 @@ typedef int (*mca_btl_base_module_send_fn_t)(
  * @param header_size (IN)     Size of header.
  * @param payload_size (IN)    Size of payload (from convertor).
  * @param order (IN)           The ordering tag (may be MCA_BTL_NO_ORDER)
+ * @param flags (IN)           Flags.
  * @param tag (IN)             The tag value used to notify the peer.
  * @param descriptor (OUT)     The descriptor to be returned unable to be sent immediately
 
@@ -660,7 +701,7 @@ typedef int (*mca_btl_base_module_send_fn_t)(
 typedef int (*mca_btl_base_module_sendi_fn_t)(
     struct mca_btl_base_module_t* btl,
     struct mca_btl_base_endpoint_t* endpoint,
-    struct ompi_convertor_t* convertor,
+    struct opal_convertor_t* convertor,
     void* header,
     size_t header_size,
     size_t payload_size,
@@ -763,6 +804,7 @@ struct mca_btl_base_module_t {
     uint32_t    btl_latency;          /**< relative ranking of latency used to prioritize btls */
     uint32_t    btl_bandwidth;        /**< bandwidth (Mbytes/sec) supported by each endpoint */
     uint32_t    btl_flags;            /**< flags (put/get...) */
+    size_t      btl_seg_size;         /**< size of a btl segment */
 
     /* BTL function table */
     mca_btl_base_module_add_procs_fn_t      btl_add_procs;
