@@ -3,20 +3,21 @@
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2010 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006-2009 Mellanox Technologies. All rights reserved.
- * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2006-2012 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
  * Copyright (c) 2008-2012 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2009      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2013      Intel, Inc. All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -26,13 +27,14 @@
 
 #include "ompi_config.h"
 #include <string.h>
+#ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
-#include "orte/util/show_help.h"
-#include "orte/runtime/orte_globals.h"
-#include "orte/mca/errmgr/base/base.h"
+#endif
 #include "opal/class/opal_bitmap.h"
 #include "opal/util/output.h"
 #include "opal/util/arch.h"
+#include "opal/include/opal_stdint.h"
+#include "opal/util/show_help.h"
 
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/btl/base/btl_base_error.h"
@@ -51,15 +53,19 @@
 #include "opal/datatype/opal_convertor.h"
 #include "ompi/mca/mpool/base/base.h"
 #include "ompi/mca/mpool/mpool.h"
-#include "ompi/mca/mpool/rdma/mpool_rdma.h"
-#include "orte/util/proc_info.h"
+#include "ompi/mca/mpool/grdma/mpool_grdma.h"
+
+#if OMPI_CUDA_SUPPORT
+#include "opal/datatype/opal_datatype_cuda.h"
+#include "ompi/mca/common/cuda/common_cuda.h"
+#endif /* OMPI_CUDA_SUPPORT */
+
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string.h>
 #include <math.h>
-#include <inttypes.h>
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -69,7 +75,9 @@
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #ifdef OPAL_HAVE_HWLOC
 #include "opal/mca/hwloc/hwloc.h"
 #endif
@@ -91,6 +99,7 @@ mca_btl_openib_module_t mca_btl_openib_module = {
         0, /* latency */
         0, /* bandwidth */
         0, /* TODO this should be PUT btl flags */
+        0, /* segment size */
         mca_btl_openib_add_procs,
         mca_btl_openib_del_procs,
         NULL,
@@ -125,7 +134,9 @@ void mca_btl_openib_show_init_error(const char *file, int line,
 {
     if (ENOMEM == errno) {
         int ret;
+#ifndef __WINDOWS__
         struct rlimit limit;
+#endif
         char *str_limit = NULL;
 
 #if HAVE_DECL_RLIMIT_MEMLOCK
@@ -133,6 +144,7 @@ void mca_btl_openib_show_init_error(const char *file, int line,
 #else
         ret = -1;
 #endif
+#ifndef __WINDOWS__
         if (0 != ret) {
             asprintf(&str_limit, "Unknown");
         } else if (limit.rlim_cur == RLIM_INFINITY) {
@@ -140,20 +152,21 @@ void mca_btl_openib_show_init_error(const char *file, int line,
         } else {
             asprintf(&str_limit, "%ld", (long)limit.rlim_cur);
         }
+#endif
 
-        orte_show_help("help-mpi-btl-openib.txt", "init-fail-no-mem",
-                       true, orte_process_info.nodename,
+        opal_show_help("help-mpi-btl-openib.txt", "init-fail-no-mem",
+                       true, ompi_process_info.nodename,
                        file, line, func, dev, str_limit);
 
         if (NULL != str_limit) free(str_limit);
     } else {
-        orte_show_help("help-mpi-btl-openib.txt", "init-fail-create-q",
-                       true, orte_process_info.nodename,
+        opal_show_help("help-mpi-btl-openib.txt", "init-fail-create-q",
+                       true, ompi_process_info.nodename,
                        file, line, func, strerror(errno), errno, dev);
     }
 }
 
-static inline struct ibv_cq *ibv_create_cq_compat(struct ibv_context *context,
+static inline struct ibv_cq *create_cq_compat(struct ibv_context *context,
         int cqe, void *cq_context, struct ibv_comp_channel *channel,
         int comp_vector)
 {
@@ -178,8 +191,8 @@ static int adjust_cq(mca_btl_openib_device_t *device, const int cq)
         cq_size = device->ib_dev_attr.max_cqe;
 
     if(NULL == device->ib_cq[cq]) {
-        device->ib_cq[cq] = ibv_create_cq_compat(device->ib_dev_context, cq_size,
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+        device->ib_cq[cq] = create_cq_compat(device->ib_dev_context, cq_size,
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
                 device, device->ib_channel,
 #else
                 NULL, NULL,
@@ -192,7 +205,7 @@ static int adjust_cq(mca_btl_openib_device_t *device, const int cq)
             return OMPI_ERROR;
         }
 
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
         if(ibv_req_notify_cq(device->ib_cq[cq], 0)) {
             mca_btl_openib_show_init_error(__FILE__, __LINE__,
                                            "ibv_req_notify_cq",
@@ -390,7 +403,11 @@ static int mca_btl_openib_size_queues(struct mca_btl_openib_module_t* openib_btl
 
     /* figure out reasonable sizes for completion queues */
     for(qp = 0; qp < mca_btl_openib_component.num_qps; qp++) {
-        if(BTL_OPENIB_QP_TYPE_SRQ(qp)) {
+        if(BTL_OPENIB_QP_TYPE_SRQ(qp)
+#if HAVE_XRC
+           || BTL_OPENIB_QP_TYPE_XRC(qp)
+#endif
+         ) {
             send_cqes = mca_btl_openib_component.qp_infos[qp].u.srq_qp.sd_max;
             recv_cqes = mca_btl_openib_component.qp_infos[qp].rd_num;
         } else {
@@ -412,8 +429,8 @@ static int mca_btl_openib_size_queues(struct mca_btl_openib_module_t* openib_btl
         goto out;
     }
 
-    if (0 == openib_btl->num_peers && 
-            (mca_btl_openib_component.num_srq_qps > 0 || 
+    if (0 == openib_btl->num_peers &&
+            (mca_btl_openib_component.num_srq_qps > 0 ||
              mca_btl_openib_component.num_xrc_qps > 0)) {
         rc = create_srq(openib_btl);
     }
@@ -425,7 +442,7 @@ out:
 
 mca_btl_openib_transport_type_t mca_btl_openib_get_transport_type(mca_btl_openib_module_t* openib_btl)
 {
-/* If we have a driver with RDMAoE supporting as the device struct contains the same type (IB) for 
+/* If we have a driver with RDMAoE supporting as the device struct contains the same type (IB) for
    IBV_LINK_LAYER_INFINIBAND and IBV_LINK_LAYER_ETHERNET link layers and the single way
    to detect this fact is to check their link_layer fields in a port_attr struct.
    If our driver doesn't support this feature => the checking of transport type in device struct will be enough.
@@ -435,7 +452,7 @@ mca_btl_openib_transport_type_t mca_btl_openib_get_transport_type(mca_btl_openib
 #ifdef HAVE_STRUCT_IBV_DEVICE_TRANSPORT_TYPE
     switch(openib_btl->device->ib_dev->transport_type) {
         case IBV_TRANSPORT_IB:
-#ifdef OMPI_HAVE_RDMAOE
+#if HAVE_DECL_IBV_LINK_LAYER_ETHERNET
             switch(openib_btl->ib_port_attr.link_layer) {
                 case IBV_LINK_LAYER_ETHERNET:
                     return MCA_BTL_OPENIB_TRANSPORT_RDMAOE;
@@ -454,7 +471,7 @@ mca_btl_openib_transport_type_t mca_btl_openib_get_transport_type(mca_btl_openib
         case IBV_TRANSPORT_IWARP:
             return MCA_BTL_OPENIB_TRANSPORT_IWARP;
 
-        case IBV_TRANSPORT_UNKNOWN:		 
+        case IBV_TRANSPORT_UNKNOWN:
         default:
             return MCA_BTL_OPENIB_TRANSPORT_UNKNOWN;
     }
@@ -463,7 +480,7 @@ mca_btl_openib_transport_type_t mca_btl_openib_get_transport_type(mca_btl_openib
 #endif
 }
 
-static int mca_btl_openib_tune_endpoint(mca_btl_openib_module_t* openib_btl, 
+static int mca_btl_openib_tune_endpoint(mca_btl_openib_module_t* openib_btl,
                                             mca_btl_base_endpoint_t* endpoint)
 {
     int ret = OMPI_SUCCESS;
@@ -473,18 +490,19 @@ static int mca_btl_openib_tune_endpoint(mca_btl_openib_module_t* openib_btl,
     ompi_btl_openib_ini_values_t values;
 
     if(mca_btl_openib_get_transport_type(openib_btl) != endpoint->rem_info.rem_transport_type) {
-        orte_show_help("help-mpi-btl-openib.txt",
-                "conflicting transport types", true,
-                orte_process_info.nodename,
-                        ibv_get_device_name(openib_btl->device->ib_dev),
-                        (openib_btl->device->ib_dev_attr).vendor_id,
-                        (openib_btl->device->ib_dev_attr).vendor_part_id,
-                        mca_btl_openib_transport_name_strings[mca_btl_openib_get_transport_type(openib_btl)],
-                        endpoint->endpoint_proc->proc_ompi->proc_hostname,
-                        endpoint->rem_info.rem_vendor_id,
-                        endpoint->rem_info.rem_vendor_part_id,
-                        mca_btl_openib_transport_name_strings[endpoint->rem_info.rem_transport_type]);
-    
+        opal_show_help("help-mpi-btl-openib.txt",
+                       "conflicting transport types", true,
+                       ompi_process_info.nodename,
+                       ibv_get_device_name(openib_btl->device->ib_dev),
+                       (openib_btl->device->ib_dev_attr).vendor_id,
+                       (openib_btl->device->ib_dev_attr).vendor_part_id,
+                       mca_btl_openib_transport_name_strings[mca_btl_openib_get_transport_type(openib_btl)],
+                       (NULL == endpoint->endpoint_proc->proc_ompi->proc_hostname) ?
+                       "unknown" : endpoint->endpoint_proc->proc_ompi->proc_hostname,
+                       endpoint->rem_info.rem_vendor_id,
+                       endpoint->rem_info.rem_vendor_part_id,
+                       mca_btl_openib_transport_name_strings[endpoint->rem_info.rem_transport_type]);
+
         return OMPI_ERROR;
     }
 
@@ -492,16 +510,17 @@ static int mca_btl_openib_tune_endpoint(mca_btl_openib_module_t* openib_btl,
     ret = ompi_btl_openib_ini_query(endpoint->rem_info.rem_vendor_id,
                           endpoint->rem_info.rem_vendor_part_id, &values);
 
-    if (OMPI_SUCCESS != ret && OMPI_ERR_NOT_FOUND != ret) {
-        orte_show_help("help-mpi-btl-openib.txt",
+    if (OMPI_SUCCESS != ret &&
+        OMPI_ERR_NOT_FOUND != ret) {
+        opal_show_help("help-mpi-btl-openib.txt",
                        "error in device init", true,
-                       orte_process_info.nodename,
+                       ompi_process_info.nodename,
                        ibv_get_device_name(openib_btl->device->ib_dev));
         return ret;
     }
 
     if(openib_btl->device->mtu < endpoint->rem_info.rem_mtu) {
-        endpoint->rem_info.rem_mtu = openib_btl->device->mtu; 
+        endpoint->rem_info.rem_mtu = openib_btl->device->mtu;
     }
 
     endpoint->use_eager_rdma = openib_btl->device->use_eager_rdma &
@@ -518,11 +537,11 @@ static int mca_btl_openib_tune_endpoint(mca_btl_openib_module_t* openib_btl,
         case BTL_OPENIB_RQ_SOURCE_MAX:
             break;
 
-        /* If the queues configuration was set from command line 
+        /* If the queues configuration was set from command line
            (with --mca btl_openib_receive_queues parameter) => both sides have a same configuration */
 
         /* In this case the local queues configuration was gotten from INI file =>
-           not possible that remote side got its queues configuration from command line => 
+           not possible that remote side got its queues configuration from command line =>
            (by prio) the configuration was set from INI file or (if not configure)
            by default queues configuration */
         case BTL_OPENIB_RQ_SOURCE_DEVICE_INI:
@@ -534,14 +553,15 @@ static int mca_btl_openib_tune_endpoint(mca_btl_openib_module_t* openib_btl,
 
             if(0 != strcmp(mca_btl_openib_component.receive_queues,
                                                          recv_qps)) {
-                orte_show_help("help-mpi-btl-openib.txt",
+                opal_show_help("help-mpi-btl-openib.txt",
                                "unsupported queues configuration", true,
-                               orte_process_info.nodename,
+                               ompi_process_info.nodename,
                                ibv_get_device_name(openib_btl->device->ib_dev),
                                (openib_btl->device->ib_dev_attr).vendor_id,
                                (openib_btl->device->ib_dev_attr).vendor_part_id,
                                mca_btl_openib_component.receive_queues,
-                               endpoint->endpoint_proc->proc_ompi->proc_hostname,
+                               (NULL == endpoint->endpoint_proc->proc_ompi->proc_hostname) ?
+                               "unknown": endpoint->endpoint_proc->proc_ompi->proc_hostname,
                                endpoint->rem_info.rem_vendor_id,
                                endpoint->rem_info.rem_vendor_part_id,
                                recv_qps);
@@ -550,20 +570,21 @@ static int mca_btl_openib_tune_endpoint(mca_btl_openib_module_t* openib_btl,
             }
             break;
 
-        /* If the local queues configuration was set 
+        /* If the local queues configuration was set
            by default queues => check all possible cases for remote side and compare */
         case  BTL_OPENIB_RQ_SOURCE_DEFAULT:
             if(NULL != values.receive_queues) {
                 if(0 != strcmp(mca_btl_openib_component.receive_queues,
                                                 values.receive_queues)) {
-                     orte_show_help("help-mpi-btl-openib.txt",
+                     opal_show_help("help-mpi-btl-openib.txt",
                                "unsupported queues configuration", true,
-                               orte_process_info.nodename,
+                               ompi_process_info.nodename,
                                ibv_get_device_name(openib_btl->device->ib_dev),
                                (openib_btl->device->ib_dev_attr).vendor_id,
                                (openib_btl->device->ib_dev_attr).vendor_part_id,
                                mca_btl_openib_component.receive_queues,
-                               endpoint->endpoint_proc->proc_ompi->proc_hostname,
+                               (NULL == endpoint->endpoint_proc->proc_ompi->proc_hostname) ?
+                               "unknown": endpoint->endpoint_proc->proc_ompi->proc_hostname,
                                endpoint->rem_info.rem_vendor_id,
                                endpoint->rem_info.rem_vendor_part_id,
                                values.receive_queues);
@@ -625,7 +646,11 @@ static uint64_t calculate_max_reg (void)
 
     mem_total = calculate_total_mem ();
 
-    if (0 == stat("/sys/module/mlx4_core/parameters", &statinfo)) {
+    if (0 == stat("/sys/module/mlx5_core", &statinfo)) {
+        /* mlx5 means that we have ofed 2.0 and it can always register 2xmem_total for any mlx hca */
+        max_reg = 2 * mem_total;
+    }
+    else if (0 == stat("/sys/module/mlx4_core/parameters", &statinfo)) {
         mtts_per_seg = 1 << read_module_param("/sys/module/mlx4_core/parameters/log_mtts_per_seg", 1);
         num_mtt = 1 << read_module_param("/sys/module/mlx4_core/parameters/log_num_mtt", 20);
         if (1 == num_mtt) {
@@ -641,32 +666,18 @@ static uint64_t calculate_max_reg (void)
 
         max_reg = (num_mtt - reserved_mtt) * getpagesize () * mtts_per_seg;
     } else {
-        /* Need to update to determine the registration limit for this
-           configuration */
+        /* need to update to determine the registration limit for this configuration */
         max_reg = mem_total;
     }
 
-    /* Print a warning if we can't register more than 75% of physical
-       memory.  Abort if the abort_not_enough_reg_mem MCA param was
-       set. */
+    /* NTH: print a warning if we can't register more than 75% of physical memory */
     if (max_reg < mem_total * 3 / 4) {
-        char *action;
-
-        if (mca_btl_openib_component.abort_not_enough_reg_mem) {
-            action = "Your MPI job will now abort.";
-        } else {
-            action = "Your MPI job will continue, but may be behave poorly and/or hang.";
-        }
-        orte_show_help("help-mpi-btl-openib.txt", "reg mem limit low", true,
-                       orte_process_info.nodename, (unsigned long)(max_reg >> 20),
-                       (unsigned long)(mem_total >> 20), action);
-        if (mca_btl_openib_component.abort_not_enough_reg_mem) {
-            orte_errmgr.abort(1, NULL);
-        }
+        opal_show_help("help-mpi-btl-openib.txt", "reg mem limit low", true,
+                       ompi_process_info.nodename, (unsigned long)(max_reg >> 20),
+                       (unsigned long)(mem_total >> 20));
     }
 
-    /* Limit us to 87.5% of the registered memory (some fluff for QPs,
-       file systems, etc) */
+    /* limit us to 87.5% of the registered memory (some fluff for QPs, file systems, etc) */
     return (max_reg * 7) >> 3;
 }
 
@@ -725,9 +736,9 @@ int mca_btl_openib_add_procs(
         }
 
         /* OOB, XOOB, and RDMACM do not support SELF comunication, so
-         * mark the proc as unreachable by openib btl  */
-        if (OPAL_EQUAL == orte_util_compare_name_fields
-                (ORTE_NS_CMP_ALL, ORTE_PROC_MY_NAME, &ompi_proc->proc_name)) {
+         * mark the prco as unreachable by openib btl  */
+        if (OPAL_EQUAL == ompi_rte_compare_name_fields
+                (OMPI_RTE_CMP_ALL, OMPI_PROC_MY_NAME, &ompi_proc->proc_name)) {
             continue;
         }
 #if defined(HAVE_STRUCT_IBV_DEVICE_TRANSPORT_TYPE)
@@ -801,7 +812,7 @@ int mca_btl_openib_add_procs(
            on the peer has a matching CPC. */
         assert(btl_rank <= ib_proc->proc_port_count);
         assert(remote_matching_port != -1);
-        if (OMPI_SUCCESS != 
+        if (OMPI_SUCCESS !=
             ompi_btl_openib_connect_base_find_match(openib_btl,
                                                     &(ib_proc->proc_ports[remote_matching_port]),
                                                     &local_cpc,
@@ -847,8 +858,8 @@ int mca_btl_openib_add_procs(
             }
         }
 #endif
-        mca_btl_openib_endpoint_init(openib_btl, endpoint, 
-                                     local_cpc, 
+        mca_btl_openib_endpoint_init(openib_btl, endpoint,
+                                     local_cpc,
                                      &(ib_proc->proc_ports[remote_matching_port]),
                                      remote_cpc_data);
 
@@ -913,7 +924,7 @@ int mca_btl_openib_del_procs(struct mca_btl_base_module_t* btl,
         for(ep_index=0;
             ep_index < opal_pointer_array_get_size(openib_btl->device->endpoints);
             ep_index++) {
-            endpoint =
+            endpoint = (mca_btl_openib_endpoint_t *)
                 opal_pointer_array_get_item(openib_btl->device->endpoints,
                         ep_index);
             if(!endpoint || endpoint->endpoint_btl != openib_btl) {
@@ -951,12 +962,12 @@ static inline mca_btl_base_descriptor_t *
 ib_frag_alloc(mca_btl_openib_module_t *btl, size_t size, uint8_t order,
         uint32_t flags)
 {
-    int qp, rc;
+    int qp;
     ompi_free_list_item_t* item = NULL;
 
     for(qp = 0; qp < mca_btl_openib_component.num_qps; qp++) {
          if(mca_btl_openib_component.qp_infos[qp].size >= size) {
-             OMPI_FREE_LIST_GET(&btl->device->qps[qp].send_free, item, rc);
+             OMPI_FREE_LIST_GET_MT(&btl->device->qps[qp].send_free, item);
              if(item)
                  break;
          }
@@ -965,7 +976,7 @@ ib_frag_alloc(mca_btl_openib_module_t *btl, size_t size, uint8_t order,
         return NULL;
 
     /* not all upper layer users set this */
-    to_base_frag(item)->segment.seg_len = size;
+    to_base_frag(item)->segment.base.seg_len = size;
     to_base_frag(item)->base.order = order;
     to_base_frag(item)->base.des_flags = flags;
 
@@ -995,7 +1006,7 @@ static mca_btl_openib_send_frag_t *check_coalescing(opal_list_t *frag_list,
         }
 
         total_length = size + frag->coalesced_length +
-            to_base_frag(frag)->segment.seg_len +
+            to_base_frag(frag)->segment.base.seg_len +
             sizeof(mca_btl_openib_header_coalesced_t);
 
         qp = to_base_frag(frag)->base.order;
@@ -1073,8 +1084,8 @@ mca_btl_base_descriptor_t* mca_btl_openib_alloc(
         sfrag->hdr->tag = MCA_BTL_TAG_BTL;
         ctrl_hdr->type = MCA_BTL_OPENIB_CONTROL_COALESCED;
         clsc_hdr->tag = org_tag;
-        clsc_hdr->size = to_base_frag(sfrag)->segment.seg_len;
-        clsc_hdr->alloc_size = to_base_frag(sfrag)->segment.seg_len;
+        clsc_hdr->size = to_base_frag(sfrag)->segment.base.seg_len;
+        clsc_hdr->alloc_size = to_base_frag(sfrag)->segment.base.seg_len;
         if(ep->nbo)
             BTL_OPENIB_HEADER_COALESCED_HTON(*clsc_hdr);
         sfrag->coalesced_length = sizeof(mca_btl_openib_control_header_t) +
@@ -1084,13 +1095,13 @@ mca_btl_base_descriptor_t* mca_btl_openib_alloc(
 
     cfrag->hdr = (mca_btl_openib_header_coalesced_t*)((unsigned char*)(sfrag->hdr + 1) + 
                   sfrag->coalesced_length +
-                  to_base_frag(sfrag)->segment.seg_len);
-    cfrag->hdr = (mca_btl_openib_header_coalesced_t*)(unsigned char*)(BTL_OPENIB_ALIGN_COALESCE_HDR((unsigned char*)(cfrag->hdr)));
+                  to_base_frag(sfrag)->segment.base.seg_len);
+    cfrag->hdr = (mca_btl_openib_header_coalesced_t*)BTL_OPENIB_ALIGN_COALESCE_HDR(cfrag->hdr);
     cfrag->hdr->alloc_size = size;
 
     /* point coalesced frag pointer into a data buffer */
-    to_base_frag(cfrag)->segment.seg_addr.pval = cfrag->hdr + 1;
-    to_base_frag(cfrag)->segment.seg_len = size;
+    to_base_frag(cfrag)->segment.base.seg_addr.pval = cfrag->hdr + 1;
+    to_base_frag(cfrag)->segment.base.seg_len = size;
 
     /* save coalesced fragment on a main fragment; we will need it after send
      * completion to free it and to call upper layer callback */
@@ -1140,6 +1151,8 @@ int mca_btl_openib_free(
             to_com_frag(des)->sg_entry.addr =
                 (uint64_t)(uintptr_t)to_send_frag(des)->hdr;
             to_send_frag(des)->coalesced_length = 0;
+	    to_base_frag(des)->segment.base.seg_addr.pval =
+		to_send_frag(des)->hdr + 1;
             assert(!opal_list_get_size(&to_send_frag(des)->coalesced_frags));
             /* fall throug */
         case MCA_BTL_OPENIB_FRAG_SEND_USER:
@@ -1193,6 +1206,7 @@ mca_btl_base_descriptor_t* mca_btl_openib_prepare_src(
     struct iovec iov;
     uint32_t iov_count = 1;
     size_t max_data = *size;
+    void *ptr;
     int rc;
 
     openib_btl = (mca_btl_openib_module_t*)btl;
@@ -1231,18 +1245,14 @@ mca_btl_base_descriptor_t* mca_btl_openib_prepare_src(
 
             to_base_frag(frag)->base.order = order;
             to_base_frag(frag)->base.des_flags = flags;
-            to_base_frag(frag)->segment.seg_len = max_data;
-            to_base_frag(frag)->segment.seg_addr.pval = iov.iov_base;
-            to_base_frag(frag)->segment.seg_key.key32[0] =
-                (uint32_t)frag->sg_entry.lkey;
+            to_base_frag(frag)->segment.base.seg_len = max_data;
+            to_base_frag(frag)->segment.base.seg_addr.lval = (uint64_t)(uintptr_t) iov.iov_base;
+            to_base_frag(frag)->segment.key = frag->sg_entry.lkey;
 
             assert(MCA_BTL_NO_ORDER == order);
 
-            BTL_VERBOSE(("frag->sg_entry.lkey = %" PRIu32 " .addr = %" PRIx64
-                         " frag->segment.seg_key.key32[0] = %" PRIu32,
-                         frag->sg_entry.lkey, 
-                         frag->sg_entry.addr,
-                         frag->sg_entry.lkey));
+            BTL_VERBOSE(("frag->sg_entry.lkey = %" PRIu32 " .addr = %" PRIx64,
+                         frag->sg_entry.lkey, frag->sg_entry.addr));
 
             return &to_base_frag(frag)->base;
         }
@@ -1254,23 +1264,43 @@ mca_btl_base_descriptor_t* mca_btl_openib_prepare_src(
         max_data = btl->btl_max_send_size - reserve;
     }
 
-    frag = (mca_btl_openib_com_frag_t*)(reserve ?
-            mca_btl_openib_alloc(btl, endpoint, order, max_data + reserve,
-                flags) :
-            ib_frag_alloc(openib_btl, max_data, order, flags));
+    if (OPAL_UNLIKELY(0 == reserve)) {
+	frag = (mca_btl_openib_com_frag_t *) ib_frag_alloc(openib_btl, max_data, order, flags);
+	if(NULL == frag)
+	    return NULL;
 
-    if(NULL == frag)
-        return NULL;
+	/* NTH: this frag will be ue used for either a get or put so we need to set the lval to be
+	   consistent with the usage in get and put. the pval will be restored in mca_btl_openib_free */
+	ptr = to_base_frag(frag)->segment.base.seg_addr.pval;
+	to_base_frag(frag)->segment.base.seg_addr.lval =
+	    (uint64_t)(uintptr_t) ptr;
+    } else {
+	frag =
+	    (mca_btl_openib_com_frag_t *) mca_btl_openib_alloc(btl, endpoint, order,
+							       max_data + reserve, flags);
+	if(NULL == frag)
+	    return NULL;
+
+	ptr = to_base_frag(frag)->segment.base.seg_addr.pval;
+    }
 
     iov.iov_len = max_data;
-    iov.iov_base = (unsigned char*)to_base_frag(frag)->segment.seg_addr.pval +
-        reserve;
+    iov.iov_base = (IOVBASE_TYPE *) ( (unsigned char*) ptr + reserve );
     rc = opal_convertor_pack(convertor, &iov, &iov_count, &max_data);
 
+#if OMPI_CUDA_SUPPORT /* CUDA_ASYNC_SEND */
+    /* If the convertor is copying the data asynchronously, then record an event
+     * that will trigger the callback when it completes.  Mark descriptor as async.*/
+    if (convertor->flags & CONVERTOR_CUDA_ASYNC) {
+        mca_common_cuda_record_dtoh_event("btl_openib", (mca_btl_base_descriptor_t *)frag);
+        to_base_frag(frag)->base.des_flags = flags | MCA_BTL_DES_FLAGS_CUDA_COPY_ASYNC;
+    }
+#endif /* OMPI_CUDA_SUPPORT */
+
     *size = max_data;
 
     /* not all upper layer users set this */
-    to_base_frag(frag)->segment.seg_len = max_data + reserve;
+    to_base_frag(frag)->segment.base.seg_len = max_data + reserve;
 
     return &to_base_frag(frag)->base;
 }
@@ -1300,18 +1330,38 @@ mca_btl_base_descriptor_t* mca_btl_openib_prepare_dst(
     uint32_t flags)
 {
     mca_btl_openib_module_t *openib_btl;
+    mca_btl_openib_component_t *openib_component;
     mca_btl_openib_com_frag_t *frag;
     mca_btl_openib_reg_t *openib_reg;
+    uint32_t max_msg_sz;
     int rc;
     void *buffer;
 
     openib_btl = (mca_btl_openib_module_t*)btl;
+    openib_component = (mca_btl_openib_component_t*)btl->btl_component;
 
     frag = alloc_recv_user_frag();
     if(NULL == frag) {
         return NULL;
     }
 
+    /* max_msg_sz is the maximum message size of the HCA (hw limitation)
+       set the minimum between local max_msg_sz and the remote */
+    max_msg_sz = MIN(openib_btl->ib_port_attr.max_msg_sz,
+                     endpoint->endpoint_btl->ib_port_attr.max_msg_sz);
+
+    /* check if user has explicitly limited the max message size */
+    if (openib_component->max_hw_msg_size > 0 &&
+        max_msg_sz > (size_t)openib_component->max_hw_msg_size) {
+        max_msg_sz = openib_component->max_hw_msg_size;
+    }
+
+    /* limit the message so to max_msg_sz */
+    if (*size > (size_t)max_msg_sz) {
+        *size = (size_t)max_msg_sz;
+        BTL_VERBOSE(("message size limited to %" PRIsize_t "\n", *size));
+    }
+
     opal_convertor_get_current_pointer(convertor, &buffer);
 
     if(NULL == registration){
@@ -1333,16 +1383,14 @@ mca_btl_base_descriptor_t* mca_btl_openib_prepare_dst(
     frag->sg_entry.lkey = openib_reg->mr->lkey;
     frag->sg_entry.addr = (uint64_t)(uintptr_t)buffer;
 
-    to_base_frag(frag)->segment.seg_addr.pval = buffer;
-    to_base_frag(frag)->segment.seg_len = *size;
-    to_base_frag(frag)->segment.seg_key.key32[0] = openib_reg->mr->rkey;
+    to_base_frag(frag)->segment.base.seg_addr.lval = (uint64_t)(uintptr_t) buffer;
+    to_base_frag(frag)->segment.base.seg_len = *size;
+    to_base_frag(frag)->segment.key = openib_reg->mr->rkey;
     to_base_frag(frag)->base.order = order;
     to_base_frag(frag)->base.des_flags = flags;
 
     BTL_VERBOSE(("frag->sg_entry.lkey = %" PRIu32 " .addr = %" PRIx64 " "
-                 "frag->segment.seg_key.key32[0] = %" PRIu32,
-                 frag->sg_entry.lkey, 
-                 frag->sg_entry.addr,
+                 "rkey = %" PRIu32, frag->sg_entry.lkey, frag->sg_entry.addr,
                  openib_reg->mr->rkey));
 
     return &to_base_frag(frag)->base;
@@ -1362,25 +1410,27 @@ static int mca_btl_openib_finalize_resources(struct mca_btl_base_module_t* btl)
     }
 
     /* Release all QPs */
-    for (ep_index=0;
-         ep_index < opal_pointer_array_get_size(openib_btl->device->endpoints);
-         ep_index++) {
-        endpoint=opal_pointer_array_get_item(openib_btl->device->endpoints,
-                                             ep_index);
-        if(!endpoint) {
-            BTL_VERBOSE(("In finalize, got another null endpoint"));
-            continue;
-        }
-        if(endpoint->endpoint_btl != openib_btl) {
-            continue;
-        }
-        for(i = 0; i < openib_btl->device->eager_rdma_buffers_count; i++) {
-            if(openib_btl->device->eager_rdma_buffers[i] == endpoint) {
-                openib_btl->device->eager_rdma_buffers[i] = NULL;
-                OBJ_RELEASE(endpoint);
+    if (NULL != openib_btl->device->endpoints) {
+        for (ep_index=0;
+             ep_index < opal_pointer_array_get_size(openib_btl->device->endpoints);
+             ep_index++) {
+            endpoint=(mca_btl_openib_endpoint_t *)opal_pointer_array_get_item(openib_btl->device->endpoints,
+                                                                              ep_index);
+            if(!endpoint) {
+                BTL_VERBOSE(("In finalize, got another null endpoint"));
+                continue;
             }
+            if(endpoint->endpoint_btl != openib_btl) {
+                continue;
+            }
+            for(i = 0; i < openib_btl->device->eager_rdma_buffers_count; i++) {
+                if(openib_btl->device->eager_rdma_buffers[i] == endpoint) {
+                    openib_btl->device->eager_rdma_buffers[i] = NULL;
+                    OBJ_RELEASE(endpoint);
+                }
+            }
+            OBJ_RELEASE(endpoint);
         }
-        OBJ_RELEASE(endpoint);
     }
 
     /* Release SRQ resources */
@@ -1398,7 +1448,7 @@ static int mca_btl_openib_finalize_resources(struct mca_btl_base_module_t* btl)
                 opal_hash_table_t *srq_addr_table =
                             &mca_btl_openib_component.srq_manager.srq_addr_table;
 
-                opal_mutex_lock(lock);		
+                opal_mutex_lock(lock);
                 if (OPAL_SUCCESS !=
                         opal_hash_table_remove_value_ptr(srq_addr_table,
                                     &openib_btl->qps[qp].u.srq_qp.srq,
@@ -1434,7 +1484,7 @@ static int mca_btl_openib_finalize_resources(struct mca_btl_base_module_t* btl)
     }
 
     if (NULL != openib_btl->qps) {
-        free(openib_btl->qps); 
+        free(openib_btl->qps);
     }
 
     return rc;
@@ -1480,7 +1530,7 @@ int mca_btl_openib_finalize(struct mca_btl_base_module_t* btl)
 
 /*
  *  Send immediate - Minimum function calls minimum checks, send the data ASAP.
- *  If BTL can't to send the messages imidiate, it creates messages descriptor 
+ *  If BTL can't to send the messages imidiate, it creates messages descriptor
  *  returns it to PML.
  */
 int mca_btl_openib_sendi( struct mca_btl_base_module_t* btl,
@@ -1492,20 +1542,20 @@ int mca_btl_openib_sendi( struct mca_btl_base_module_t* btl,
         uint8_t order,
         uint32_t flags,
         mca_btl_base_tag_t tag,
-        mca_btl_base_descriptor_t** descriptor) 
+        mca_btl_base_descriptor_t** descriptor)
 {
     mca_btl_openib_module_t *obtl = (mca_btl_openib_module_t*)btl;
     size_t size = payload_size + header_size;
     size_t eager_limit;
-    int rc,  
-        qp = frag_size_to_order(obtl, size), 
-        prio = !(flags & MCA_BTL_DES_FLAGS_PRIORITY), 
+    int qp = frag_size_to_order(obtl, size),
+        prio = !(flags & MCA_BTL_DES_FLAGS_PRIORITY),
         ib_rc;
     int32_t cm_return;
     bool do_rdma = false;
     ompi_free_list_item_t* item = NULL;
     mca_btl_openib_frag_t *frag;
     mca_btl_openib_header_t *hdr;
+    int send_signaled;
 
     OPAL_THREAD_LOCK(&ep->endpoint_lock);
 
@@ -1552,21 +1602,21 @@ int mca_btl_openib_sendi( struct mca_btl_base_module_t* btl,
     }
 
     /* Allocate fragment */
-    OMPI_FREE_LIST_GET(&obtl->device->qps[qp].send_free, item, rc);
+    OMPI_FREE_LIST_GET_MT(&obtl->device->qps[qp].send_free, item);
     if(OPAL_UNLIKELY(NULL == item)) {
         /* we don't return NULL because maybe later we will try to coalesce */
         goto no_frags;
     }
     frag = to_base_frag(item);
     hdr = to_send_frag(item)->hdr;
-    frag->segment.seg_len = size;
+    frag->segment.base.seg_len = size;
     frag->base.order = qp;
     frag->base.des_flags = flags;
     hdr->tag = tag;
     to_com_frag(item)->endpoint = ep;
 
     /* put match header */
-    memcpy(frag->segment.seg_addr.pval, header, header_size);
+    memcpy(frag->segment.base.seg_addr.pval, header, header_size);
 
     /* Pack data */
     if(payload_size) {
@@ -1574,7 +1624,7 @@ int mca_btl_openib_sendi( struct mca_btl_base_module_t* btl,
         struct iovec iov;
         uint32_t iov_count;
         /* pack the data into the supplied buffer */
-        iov.iov_base = (IOVBASE_TYPE*)((unsigned char*)frag->segment.seg_addr.pval + header_size);
+        iov.iov_base = (IOVBASE_TYPE*)((unsigned char*)frag->segment.base.seg_addr.pval + header_size);
         iov.iov_len  = max_data = payload_size;
         iov_count    = 1;
 
@@ -1606,12 +1656,22 @@ int mca_btl_openib_sendi( struct mca_btl_base_module_t* btl,
         hdr->cm_seen = cm_return;
     }
 
-    ib_rc = post_send(ep, to_send_frag(item), do_rdma);
+#if BTL_OPENIB_FAILOVER_ENABLED
+    send_signaled = 1;
+#else
+    send_signaled = qp_need_signal(ep, qp, payload_size + header_size, do_rdma);
+#endif
+    ib_rc = post_send(ep, to_send_frag(item), do_rdma, send_signaled);
 
     if(!ib_rc) {
+        if (0 == send_signaled) {
+            MCA_BTL_IB_FRAG_RETURN(frag);
+        }
 #if BTL_OPENIB_FAILOVER_ENABLED
-        /* Return up in case needed for failover */
-        *descriptor = (struct mca_btl_base_descriptor_t *) frag;
+        else {
+            /* Return up in case needed for failover */
+            *descriptor = (struct mca_btl_base_descriptor_t *) frag;
+        }
 #endif
         OPAL_THREAD_UNLOCK(&ep->endpoint_lock);
         return OMPI_SUCCESS;
@@ -1688,11 +1748,13 @@ int mca_btl_openib_put( mca_btl_base_module_t* btl,
                     mca_btl_base_endpoint_t* ep,
                     mca_btl_base_descriptor_t* descriptor)
 {
+    mca_btl_openib_segment_t *src_seg = (mca_btl_openib_segment_t *) descriptor->des_src;
+    mca_btl_openib_segment_t *dst_seg = (mca_btl_openib_segment_t *) descriptor->des_dst;
     struct ibv_send_wr* bad_wr;
     mca_btl_openib_out_frag_t* frag = to_out_frag(descriptor);
     int qp = descriptor->order;
-    uint64_t rem_addr = descriptor->des_dst->seg_addr.lval;
-    uint32_t rkey = descriptor->des_dst->seg_key.key32[0];
+    uint64_t rem_addr = dst_seg->base.seg_addr.lval;
+    uint32_t rkey = dst_seg->key;
 
     assert(openib_frag_type(frag) == MCA_BTL_OPENIB_FRAG_SEND_USER ||
             openib_frag_type(frag) == MCA_BTL_OPENIB_FRAG_SEND);
@@ -1732,9 +1794,8 @@ int mca_btl_openib_put( mca_btl_base_module_t* btl,
     frag->sr_desc.wr.rdma.remote_addr = rem_addr;
     frag->sr_desc.wr.rdma.rkey = rkey;
 
-    to_com_frag(frag)->sg_entry.addr =
-        (uint64_t)(uintptr_t)descriptor->des_src->seg_addr.pval;
-    to_com_frag(frag)->sg_entry.length = descriptor->des_src->seg_len;
+    to_com_frag(frag)->sg_entry.addr = src_seg->base.seg_addr.lval;
+    to_com_frag(frag)->sg_entry.length = src_seg->base.seg_len;
     to_com_frag(frag)->endpoint = ep;
 #if HAVE_XRC
     if (MCA_BTL_XRC_ENABLED && BTL_OPENIB_QP_TYPE_XRC(qp))
@@ -1745,7 +1806,11 @@ int mca_btl_openib_put( mca_btl_base_module_t* btl,
     /* Setting opcode on a frag constructor isn't enough since prepare_src
      * may return send_frag instead of put_frag */
     frag->sr_desc.opcode = IBV_WR_RDMA_WRITE;
-    frag->sr_desc.send_flags = ib_send_flags(descriptor->des_src->seg_len, &(ep->qps[qp]));
+
+    frag->sr_desc.send_flags = ib_send_flags(src_seg->base.seg_len, &(ep->qps[qp]), 1);
+    qp_inflight_wqe_to_frag(ep, qp, to_com_frag(frag));
+    qp_reset_signal_count(ep, qp);
+
     if(ibv_post_send(ep->qps[qp].qp->lcl_qp, &frag->sr_desc, &bad_wr))
         return OMPI_ERROR;
 
@@ -1760,11 +1825,13 @@ int mca_btl_openib_get(mca_btl_base_module_t* btl,
                     mca_btl_base_endpoint_t* ep,
                     mca_btl_base_descriptor_t* descriptor)
 {
+    mca_btl_openib_segment_t *src_seg = (mca_btl_openib_segment_t *) descriptor->des_src;
+    mca_btl_openib_segment_t *dst_seg = (mca_btl_openib_segment_t *) descriptor->des_dst;
     struct ibv_send_wr* bad_wr;
     mca_btl_openib_get_frag_t* frag = to_get_frag(descriptor);
     int qp = descriptor->order;
-    uint64_t rem_addr = descriptor->des_src->seg_addr.lval;
-    uint32_t rkey = descriptor->des_src->seg_key.key32[0];
+    uint64_t rem_addr = src_seg->base.seg_addr.lval;
+    uint32_t rkey = src_seg->key;
 
     assert(openib_frag_type(frag) == MCA_BTL_OPENIB_FRAG_RECV_USER);
 
@@ -1813,9 +1880,8 @@ int mca_btl_openib_get(mca_btl_base_module_t* btl,
     frag->sr_desc.wr.rdma.remote_addr = rem_addr;
     frag->sr_desc.wr.rdma.rkey = rkey;
 
-    to_com_frag(frag)->sg_entry.addr =
-        (uint64_t)(uintptr_t)descriptor->des_dst->seg_addr.pval;
-    to_com_frag(frag)->sg_entry.length  = descriptor->des_dst->seg_len;
+    to_com_frag(frag)->sg_entry.addr = dst_seg->base.seg_addr.lval;
+    to_com_frag(frag)->sg_entry.length = dst_seg->base.seg_len;
     to_com_frag(frag)->endpoint = ep;
 
 #if HAVE_XRC
@@ -1823,6 +1889,10 @@ int mca_btl_openib_get(mca_btl_base_module_t* btl,
         frag->sr_desc.xrc_remote_srq_num=ep->rem_info.rem_srqs[qp].rem_srq_num;
 #endif
     descriptor->order = qp;
+
+    qp_inflight_wqe_to_frag(ep, qp, to_com_frag(frag));
+    qp_reset_signal_count(ep, qp);
+
     if(ibv_post_send(ep->qps[qp].qp->lcl_qp, &frag->sr_desc, &bad_wr))
         return OMPI_ERROR;
 
@@ -1840,7 +1910,7 @@ int mca_btl_openib_ft_event(int state) {
     if(OPAL_CRS_CHECKPOINT == state) {
         /* Continue must reconstruct the routes (including modex), since we
          * have to tear down the devices completely. */
-        ompi_cr_continue_like_restart = true;
+        orte_cr_continue_like_restart = true;
 
         /*
          * To keep the node from crashing we need to call ibv_close_device
