@@ -38,7 +38,7 @@
 #include "opal/class/opal_pointer_array.h"
 #include "opal/class/opal_hash_table.h"
 #include "opal/util/output.h"
-#include "opal/event/event.h"
+#include "opal/mca/event/event.h"
 #include "opal/threads/threads.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/mpool/mpool.h"
@@ -57,6 +57,7 @@ BEGIN_C_DECLS
 #define MCA_BTL_IB_LEAVE_PINNED 1
 #define IB_DEFAULT_GID_PREFIX 0xfe80000000000000ll
 #define MCA_BTL_IB_PKEY_MASK 0x7fff
+#define MCA_BTL_OPENIB_CQ_POLL_BATCH_DEFAULT (256)
 
 
 /*--------------------------------------------------------------------*/
@@ -142,7 +143,7 @@ typedef enum {
 /* The structer for manage all BTL SRQs */
 typedef struct mca_btl_openib_srq_manager_t {
     opal_mutex_t lock;
-    /* The keys of this hash table are addresses of 
+    /* The keys of this hash table are addresses of
        SRQs structures, and the elements are BTL modules
        pointers that associated with these SRQs */
     opal_hash_table_t srq_addr_table;
@@ -199,46 +200,50 @@ struct mca_btl_openib_component_t {
 
     size_t eager_limit;      /**< Eager send limit of first fragment, in Bytes */
     size_t max_send_size;    /**< Maximum send size, in Bytes */
+    uint32_t max_hw_msg_size;/**< Maximum message size for RDMA protocols in Bytes */
     uint32_t reg_mru_len;    /**< Length of the registration cache most recently used list */
     uint32_t use_srq;        /**< Use the Shared Receive Queue (SRQ mode) */
 
     uint32_t ib_cq_size[2];  /**< Max outstanding CQE on the CQ */
 
-    int32_t ib_max_inline_data; /**< Max size of inline data */
-    uint32_t ib_pkey_val;
-    uint32_t ib_psn;
-    uint32_t ib_qp_ous_rd_atom;
+    int      ib_max_inline_data; /**< Max size of inline data */
+    unsigned int ib_pkey_val;
+    unsigned int ib_psn;
+    unsigned int ib_qp_ous_rd_atom;
     uint32_t ib_mtu;
-    uint32_t ib_min_rnr_timer;
-    uint32_t ib_timeout;
-    uint32_t ib_retry_count;
-    uint32_t ib_rnr_retry;
-    uint32_t ib_max_rdma_dst_ops;
-    uint32_t ib_service_level;
+    unsigned int ib_min_rnr_timer;
+    unsigned int ib_timeout;
+    unsigned int ib_retry_count;
+    unsigned int ib_rnr_retry;
+    unsigned int ib_max_rdma_dst_ops;
+    unsigned int ib_service_level;
 #if (ENABLE_DYNAMIC_SL)
-    uint32_t ib_path_record_service_level;
+    unsigned int ib_path_record_service_level;
 #endif
-    int32_t use_eager_rdma;
-    int32_t eager_rdma_threshold; /**< After this number of msg, use RDMA for short messages, always */
-    int32_t eager_rdma_num;
+    int     use_eager_rdma;
+    int     eager_rdma_threshold; /**< After this number of msg, use RDMA for short messages, always */
+    int     eager_rdma_num;
     int32_t max_eager_rdma;
-    uint32_t btls_per_lid;
-    uint32_t max_lmc;
-    int32_t apm_lmc;
-    int32_t apm_ports;
-    uint32_t buffer_alignment;    /**< Preferred communication buffer alignment in Bytes (must be power of two) */
+    unsigned int btls_per_lid;
+    unsigned int max_lmc;
+    int     apm_lmc;
+    int     apm_ports;
+    unsigned int buffer_alignment;    /**< Preferred communication buffer alignment in Bytes (must be power of two) */
 #if OPAL_HAVE_THREADS
     int32_t error_counter;           /**< Counts number on error events that we got on all devices */
     int async_pipe[2];               /**< Pipe for comunication with async event thread */
     int async_comp_pipe[2];          /**< Pipe for async thread comunication with main thread */
     pthread_t   async_thread;        /**< Async thread that will handle fatal errors */
-    uint32_t use_async_event_thread; /**< Use the async event handler */
+    bool use_async_event_thread;     /**< Use the async event handler */
     mca_btl_openib_srq_manager_t srq_manager;     /**< Hash table for all BTL SRQs */
 #if BTL_OPENIB_FAILOVER_ENABLED
-    uint32_t port_error_failover;    /**< Report port errors to speed up failover */
+    bool port_error_failover;        /**< Report port errors to speed up failover */
 #endif
 #endif
-    btl_openib_device_type_t device_type;
+    /* declare as an int instead of btl_openib_device_type_t since there is no
+       guarantee about the size of an enum. this value will be registered as an
+       integer with the MCA variable system */
+    int device_type;
     char *if_include;
     char **if_include_list;
     char *if_exclude;
@@ -266,18 +271,15 @@ struct mca_btl_openib_component_t {
     /** Whether we want a warning if the user specifies a non-existent
         device and/or port via btl_openib_if_[in|ex]clude MCA params */
     bool warn_nonexistent_if;
-    /** Whether we want to abort if there's not enough registered
-        memory available */
-    bool abort_not_enough_reg_mem;
-
     /** Dummy argv-style list; a copy of names from the
         if_[in|ex]clude list that we use for error checking (to ensure
         that they all exist) */
     char **if_list;
     bool use_message_coalescing;
-    uint32_t cq_poll_ratio;
-    uint32_t cq_poll_progress;
-    uint32_t eager_rdma_poll_ratio;
+    unsigned int cq_poll_ratio;
+    unsigned int cq_poll_progress;
+    unsigned int cq_poll_batch;
+    unsigned int eager_rdma_poll_ratio;
 #ifdef HAVE_IBV_FORK_INIT
     /** Whether we want fork support or not */
     int want_fork_support;
@@ -305,6 +307,10 @@ struct mca_btl_openib_component_t {
     size_t memalign_threshold;
     void* (*previous_malloc_hook)(size_t __size, const void*);
 #endif
+#if OMPI_CUDA_SUPPORT /* CUDA_ASYNC_SEND */
+    bool cuda_async_send;
+    bool cuda_async_recv;
+#endif /* OMPI_CUDA_SUPPORT */
 }; typedef struct mca_btl_openib_component_t mca_btl_openib_component_t;
 
 OMPI_MODULE_DECLSPEC extern mca_btl_openib_component_t mca_btl_openib_component;
@@ -354,7 +360,7 @@ struct mca_btl_base_endpoint_t;
 typedef struct mca_btl_openib_device_t {
     opal_object_t super;
     struct ibv_device *ib_dev;  /* the ib device */
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
     struct ibv_comp_channel *ib_channel; /* Channel event for the device */
     opal_thread_t thread;                /* Progress thread */
     volatile bool progress;              /* Progress status */
@@ -415,12 +421,12 @@ struct mca_btl_openib_module_srq_qp_t {
     /** We post additional WQEs only if a number of WQEs (in specific SRQ) is less of this value.
          The value increased together with rd_curr_num. The value is unique for every SRQ. */
     int32_t rd_low_local;
-    /** The flag points if we want to get the 
+    /** The flag points if we want to get the
          IBV_EVENT_SRQ_LIMIT_REACHED events for dynamically resizing SRQ */
     bool srq_limit_event_flag;
     /**< In difference of the "--mca enable_srq_resize" parameter that says, if we want(or no)
          to start with small num of pre-posted receive buffers (rd_curr_num) and to increase this number by needs
-         (the max of this value is rd_num  the whole size of SRQ), the "srq_limit_event_flag" says if we want to get limit event
+         (the max of this value is rd_num * the whole size of SRQ), the "srq_limit_event_flag" says if we want to get limit event
          from device if the defined srq limit was reached (signal to the main thread) and we put off this flag if the rd_curr_num
          was increased up to rd_num.
          In order to prevent lock/unlock operation in the critical path we prefer only put-on
@@ -485,7 +491,7 @@ struct mca_btl_openib_reg_t {
 };
 typedef struct mca_btl_openib_reg_t mca_btl_openib_reg_t;
 
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
 extern void* mca_btl_openib_progress_thread(opal_object_t*);
 #endif
 
@@ -592,7 +598,7 @@ extern int mca_btl_openib_sendi( struct mca_btl_base_module_t* btl,
     uint32_t flags,
     mca_btl_base_tag_t tag,
     mca_btl_base_descriptor_t** descriptor
-); 
+);
 
 /**
  * PML->BTL Initiate a put of the specified size.
