@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -14,6 +14,7 @@
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
  * Copyright (c) 2007-2009 Mellanox Technologies.  All rights reserved.
+ * Copyright (c) 2010-2012 Oracle and/or its affiliates.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,7 +26,7 @@
 #define MCA_BTL_IB_ENDPOINT_H
 
 #include "opal/class/opal_list.h"
-#include "opal/event/event.h"
+#include "opal/mca/event/event.h"
 #include "opal/util/output.h"
 #include "ompi/mca/btl/btl.h"
 #include "btl_openib.h"
@@ -36,6 +37,8 @@
 #include "ompi/mca/btl/base/btl_base_error.h"
 #include "connect/base.h"
 
+#define QP_TX_BATCH_COUNT 64
+
 BEGIN_C_DECLS
 
 struct mca_btl_openib_frag_t;
@@ -132,6 +135,8 @@ typedef struct mca_btl_openib_qp_t {
     struct ibv_qp *lcl_qp;
     uint32_t lcl_psn;
     int32_t  sd_wqe;      /**< number of available send wqe entries */
+    int32_t  sd_wqe_inflight;
+    int wqe_count;
     int users;
     opal_mutex_t lock;
 } mca_btl_openib_qp_t;
@@ -207,7 +212,7 @@ struct mca_btl_base_endpoint_t {
     /** list of pending rget ops */
     opal_list_t                 pending_get_frags;
     /** list of pending rput ops */
-    opal_list_t                 pending_put_frags; 
+    opal_list_t                 pending_put_frags;
 
     /** number of available get tokens */
     int32_t                     get_tokens;
@@ -269,6 +274,55 @@ static inline int32_t qp_put_wqe(mca_btl_openib_endpoint_t *ep, const int qp)
     return OPAL_THREAD_ADD32(&ep->qps[qp].qp->sd_wqe, 1);
 }
 
+
+static inline int32_t qp_inc_inflight_wqe(mca_btl_openib_endpoint_t *ep, const int qp, mca_btl_openib_com_frag_t *frag)
+{
+    frag->n_wqes_inflight = 0;
+    return OPAL_THREAD_ADD32(&ep->qps[qp].qp->sd_wqe_inflight, 1);
+}
+
+static inline void qp_inflight_wqe_to_frag(mca_btl_openib_endpoint_t *ep, const int qp, mca_btl_openib_com_frag_t *frag)
+{
+
+    frag->n_wqes_inflight = ep->qps[qp].qp->sd_wqe_inflight;
+    ep->qps[qp].qp->sd_wqe_inflight = 0;
+}
+
+static inline int qp_frag_to_wqe(mca_btl_openib_endpoint_t *ep, const int qp, mca_btl_openib_com_frag_t *frag)
+{
+    int n;
+    n = frag->n_wqes_inflight;
+    OPAL_THREAD_ADD32(&ep->qps[qp].qp->sd_wqe, n);
+    frag->n_wqes_inflight = 0;
+
+    return n;
+}
+
+static inline int qp_need_signal(mca_btl_openib_endpoint_t *ep, const int qp, size_t size, int rdma)
+{
+
+    /* note that size here is payload only */
+    if (ep->qps[qp].qp->sd_wqe <= 0  || 
+            size + sizeof(mca_btl_openib_header_t) + (rdma ? sizeof(mca_btl_openib_footer_t) : 0) > ep->qps[qp].ib_inline_max ||
+             (!BTL_OPENIB_QP_TYPE_PP(qp) && ep->endpoint_btl->qps[qp].u.srq_qp.sd_credits <= 0)) {
+        ep->qps[qp].qp->wqe_count = QP_TX_BATCH_COUNT;
+        return 1;
+    }
+
+    if (0 < --ep->qps[qp].qp->wqe_count) {
+        return 0;
+    }
+
+    ep->qps[qp].qp->wqe_count = QP_TX_BATCH_COUNT;
+    return 1;
+}
+
+static inline void qp_reset_signal_count(mca_btl_openib_endpoint_t *ep, const int qp)
+{
+    ep->qps[qp].qp->wqe_count = QP_TX_BATCH_COUNT;
+}
+
+
 int mca_btl_openib_endpoint_send(mca_btl_base_endpoint_t*,
         mca_btl_openib_send_frag_t*);
 int mca_btl_openib_endpoint_post_send(mca_btl_openib_endpoint_t*,
@@ -303,9 +357,8 @@ static inline int post_recvs(mca_btl_base_endpoint_t *ep, const int qp,
         return OMPI_SUCCESS;
 
     for(i = 0; i < num_post; i++) {
-        int rc;
         ompi_free_list_item_t* item;
-        OMPI_FREE_LIST_WAIT(&openib_btl->device->qps[qp].recv_free, item, rc);
+        OMPI_FREE_LIST_WAIT_MT(&openib_btl->device->qps[qp].recv_free, item);
         to_base_frag(item)->base.order = qp;
         to_com_frag(item)->endpoint = ep;
         if(NULL == wr)
@@ -456,10 +509,14 @@ static inline int check_endpoint_state(mca_btl_openib_endpoint_t *ep,
 }
 
 static inline __opal_attribute_always_inline__ int
-ib_send_flags(uint32_t size, mca_btl_openib_endpoint_qp_t *qp)
+ib_send_flags(uint32_t size, mca_btl_openib_endpoint_qp_t *qp, int do_signal)
 {
-    return IBV_SEND_SIGNALED |
-        ((size <= qp->ib_inline_max) ? IBV_SEND_INLINE : 0);
+    if (do_signal) {
+        return IBV_SEND_SIGNALED |
+            ((size <= qp->ib_inline_max) ? IBV_SEND_INLINE : 0);
+    } else {
+        return   ((size <= qp->ib_inline_max) ? IBV_SEND_INLINE : 0);
+    }
 }
 
 static inline int
@@ -474,48 +531,57 @@ acquire_eager_rdma_send_credit(mca_btl_openib_endpoint_t *endpoint)
 }
 
 static inline int post_send(mca_btl_openib_endpoint_t *ep,
-        mca_btl_openib_send_frag_t *frag, const bool rdma)
+        mca_btl_openib_send_frag_t *frag, const bool rdma, int do_signal)
 {
     mca_btl_openib_module_t *openib_btl = ep->endpoint_btl;
-    mca_btl_base_segment_t *seg = &to_base_frag(frag)->segment;
+    mca_btl_openib_segment_t *seg = &to_base_frag(frag)->segment;
     struct ibv_sge *sg = &to_com_frag(frag)->sg_entry;
     struct ibv_send_wr *sr_desc = &to_out_frag(frag)->sr_desc;
     struct ibv_send_wr *bad_wr;
     int qp = to_base_frag(frag)->base.order;
 
-    sg->length = seg->seg_len + sizeof(mca_btl_openib_header_t) +
+    sg->length = seg->base.seg_len + sizeof(mca_btl_openib_header_t) +
         (rdma ? sizeof(mca_btl_openib_footer_t) : 0) + frag->coalesced_length;
 
-    sr_desc->send_flags = ib_send_flags(sg->length, &(ep->qps[qp]));
+    sr_desc->send_flags = ib_send_flags(sg->length, &(ep->qps[qp]), do_signal);
 
     if(ep->nbo)
         BTL_OPENIB_HEADER_HTON(*frag->hdr);
 
     if(rdma) {
-        int32_t head;
+        int32_t head;   
         mca_btl_openib_footer_t* ftr =
-            (mca_btl_openib_footer_t*)(((char*)frag->hdr) + sg->length -
-                    sizeof(mca_btl_openib_footer_t));
+            (mca_btl_openib_footer_t*)(((char*)frag->hdr) + sg->length +
+                    BTL_OPENIB_FTR_PADDING(sg->length) - sizeof(mca_btl_openib_footer_t));
         sr_desc->opcode = IBV_WR_RDMA_WRITE;
         MCA_BTL_OPENIB_RDMA_FRAG_SET_SIZE(ftr, sg->length);
         MCA_BTL_OPENIB_RDMA_MAKE_LOCAL(ftr);
 #if OPAL_ENABLE_DEBUG
-	do {
-	  ftr->seq = ep->eager_rdma_remote.seq;
-	} while (!OPAL_ATOMIC_CMPSET_32((int32_t*)&ep->eager_rdma_remote.seq, (int32_t)ftr->seq, (int32_t)ftr->seq+1));
+        do {
+          ftr->seq = ep->eager_rdma_remote.seq;
+        } while (!OPAL_ATOMIC_CMPSET_32((int32_t*) &ep->eager_rdma_remote.seq,
+                                        (int32_t) ftr->seq,
+                                        (int32_t) (ftr->seq+1)));
 #endif
         if(ep->nbo)
             BTL_OPENIB_FOOTER_HTON(*ftr);
 
         sr_desc->wr.rdma.rkey = ep->eager_rdma_remote.rkey;
         MCA_BTL_OPENIB_RDMA_MOVE_INDEX(ep->eager_rdma_remote.head, head);
+#if BTL_OPENIB_FAILOVER_ENABLED
+        /* frag->ftr is unused on the sending fragment, so use it
+         * to indicate it is an eager fragment.  A non-zero value
+         * indicates it is eager, and the value indicates the
+         * location in the eager RDMA array that it lives. */
+        frag->ftr = (mca_btl_openib_footer_t*)(long)(1 + head);
+#endif
         sr_desc->wr.rdma.remote_addr =
             ep->eager_rdma_remote.base.lval +
             head * openib_btl->eager_rdma_frag_size +
             sizeof(mca_btl_openib_header_t) +
             mca_btl_openib_component.eager_limit +
             sizeof(mca_btl_openib_footer_t);
-        sr_desc->wr.rdma.remote_addr -= sg->length;
+        sr_desc->wr.rdma.remote_addr -= sg->length + BTL_OPENIB_FTR_PADDING(sg->length);
     } else {
         if(BTL_OPENIB_QP_TYPE_PP(qp)) {
             sr_desc->opcode = IBV_WR_SEND;
@@ -535,6 +601,12 @@ static inline int post_send(mca_btl_openib_endpoint_t *ep,
 #endif
     assert(sg->addr == (uint64_t)(uintptr_t)frag->hdr);
 
+    if (sr_desc->send_flags & IBV_SEND_SIGNALED) {
+        qp_inflight_wqe_to_frag(ep, qp, to_com_frag(frag));
+    } else {
+        qp_inc_inflight_wqe(ep, qp, to_com_frag(frag));
+    }
+
     return ibv_post_send(ep->qps[qp].qp->lcl_qp, sr_desc, &bad_wr);
 }
 
