@@ -5,15 +5,18 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2009 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008      Mellanox Technologies. All rights reserved.
+ * Copyright (c) 2012-2013 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -23,9 +26,11 @@
 #include <ctype.h>
 #include <stdlib.h>
 
-#include "opal/util/output.h"
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include "opal/util/show_help.h"
-#include "opal/mca/base/mca_base_param.h"
 
 #include "btl_openib.h"
 #include "btl_openib_lex.h"
@@ -33,7 +38,7 @@
 
 static const char *ini_filename = NULL;
 static bool initialized = false;
-static opal_list_t hcas;
+static opal_list_t devices;
 static char *key_buffer = NULL;
 static size_t key_buffer_len = 0;
 
@@ -47,7 +52,7 @@ typedef struct parsed_section_values_t {
 
     uint32_t *vendor_ids;
     int vendor_ids_len;
-    
+
     uint32_t *vendor_part_ids;
     int vendor_part_ids_len;
 
@@ -63,7 +68,7 @@ typedef struct parsed_section_values_t {
  * - There is a super of opal_list_item_t so that we can have a list
  *   of these
  */
-typedef struct hca_values_t {
+typedef struct device_values_t {
     opal_list_item_t super;
 
     char *section_name;
@@ -71,15 +76,15 @@ typedef struct hca_values_t {
     uint32_t vendor_part_id;
 
     ompi_btl_openib_ini_values_t values;
-} hca_values_t;
+} device_values_t;
 
-static void hca_values_constructor(hca_values_t *s);
-static void hca_values_destructor(hca_values_t *s);
+static void device_values_constructor(device_values_t *s);
+static void device_values_destructor(device_values_t *s);
 
-OBJ_CLASS_INSTANCE(hca_values_t,
+OBJ_CLASS_INSTANCE(device_values_t,
                    opal_list_item_t,
-                   hca_values_constructor,
-                   hca_values_destructor);
+                   device_values_constructor,
+                   device_values_destructor);
 
 
 /*
@@ -94,7 +99,7 @@ static inline void show_help(const char *topic);
 
 
 /*
- * Read the INI files for HCA-specific values and save them in
+ * Read the INI files for device-specific values and save them in
  * internal data structures for later lookup.
  */
 int ompi_btl_openib_ini_init(void)
@@ -102,16 +107,24 @@ int ompi_btl_openib_ini_init(void)
     int ret = OMPI_ERR_NOT_FOUND;
     char *colon;
 
-    OBJ_CONSTRUCT(&hcas, opal_list_t);
+#ifndef __WINDOWS__
+    char separator = ':';
+#else
+    /* ':' is part of the path on Windows,
+       so use ';' instead. */
+    char separator = ';';
+#endif
 
-    colon = strchr(mca_btl_openib_component.hca_params_file_names, ':');
+    OBJ_CONSTRUCT(&devices, opal_list_t);
+
+    colon = strchr(mca_btl_openib_component.device_params_file_names, separator);
     if (NULL == colon) {
         /* If we've only got 1 file (i.e., no colons found), parse it
            and be done */
-        ret = parse_file(mca_btl_openib_component.hca_params_file_names);
+        ret = parse_file(mca_btl_openib_component.device_params_file_names);
     } else {
         /* Otherwise, loop over all the files and parse them */
-        char *orig = strdup(mca_btl_openib_component.hca_params_file_names);
+        char *orig = strdup(mca_btl_openib_component.device_params_file_names);
         char *str = orig;
 
         while (NULL != (colon = strchr(str, ':'))) {
@@ -143,14 +156,14 @@ int ompi_btl_openib_ini_init(void)
 
 
 /*
- * The component found an HCA and is querying to see if an INI file
+ * The component found a device and is querying to see if an INI file
  * specified any parameters for it.
  */
 int ompi_btl_openib_ini_query(uint32_t vendor_id, uint32_t vendor_part_id,
                               ompi_btl_openib_ini_values_t *values)
 {
     int ret;
-    hca_values_t *h;
+    device_values_t *h;
     opal_list_item_t *item;
 
     if (!initialized) {
@@ -160,17 +173,17 @@ int ompi_btl_openib_ini_query(uint32_t vendor_id, uint32_t vendor_part_id,
     }
 
     if (mca_btl_openib_component.verbose) {
-        BTL_OUTPUT(("Querying INI files for vendor 0x%04x, part ID %d", 
+        BTL_OUTPUT(("Querying INI files for vendor 0x%04x, part ID %d",
                     vendor_id, vendor_part_id));
     }
 
     reset_values(values);
 
-    /* Iterate over all the saved hcas */
-    for (item = opal_list_get_first(&hcas); 
-         item != opal_list_get_end(&hcas);
+    /* Iterate over all the saved devices */
+    for (item = opal_list_get_first(&devices);
+         item != opal_list_get_end(&devices);
          item = opal_list_get_next(item)) {
-        h = (hca_values_t*) item;
+        h = (device_values_t*) item;
         if (vendor_id == h->vendor_id &&
             vendor_part_id == h->vendor_part_id) {
             /* Found it! */
@@ -202,12 +215,12 @@ int ompi_btl_openib_ini_finalize(void)
     opal_list_item_t *item;
 
     if (initialized) {
-        for (item = opal_list_remove_first(&hcas);
+        for (item = opal_list_remove_first(&devices);
              NULL != item;
-             item = opal_list_remove_first(&hcas)) {
+             item = opal_list_remove_first(&devices)) {
             OBJ_RELEASE(item);
         }
-        OBJ_DESTRUCT(&hcas);
+        OBJ_DESTRUCT(&devices);
         initialized = true;
     }
 
@@ -291,6 +304,7 @@ static int parse_file(char *filename)
     }
     save_section(&section);
     fclose(btl_openib_ini_yyin);
+    btl_openib_ini_yylex_destroy ();
 
 cleanup:
     reset_section(true, &section);
@@ -316,7 +330,7 @@ static int parse_line(parsed_section_values_t *sv)
     if (key_buffer_len < strlen(btl_openib_ini_yytext) + 1) {
         char *tmp;
         key_buffer_len = strlen(btl_openib_ini_yytext) + 1;
-        tmp = realloc(key_buffer, key_buffer_len);
+        tmp = (char *) realloc(key_buffer, key_buffer_len);
         if (NULL == tmp) {
             free(key_buffer);
             key_buffer_len = 0;
@@ -362,7 +376,7 @@ static int parse_line(parsed_section_values_t *sv)
        all whitespace at the beginning and ending of the value. */
 
     if (0 == strcasecmp(key_buffer, "vendor_id")) {
-        if (OMPI_SUCCESS != (ret = ompi_btl_openib_ini_intify_list(value, &sv->vendor_ids, 
+        if (OMPI_SUCCESS != (ret = ompi_btl_openib_ini_intify_list(value, &sv->vendor_ids,
                                                &sv->vendor_ids_len))) {
             return ret;
         }
@@ -387,11 +401,36 @@ static int parse_line(parsed_section_values_t *sv)
         sv->values.use_eager_rdma_set = true;
     }
 
+    else if (0 == strcasecmp(key_buffer, "receive_queues")) {
+        /* Single value (already strdup'ed) */
+        sv->values.receive_queues = value;
+        value = NULL;
+    }
+
+    else if (0 == strcasecmp(key_buffer, "max_inline_data")) {
+        /* Single value */
+        sv->values.max_inline_data = (int32_t) ompi_btl_openib_ini_intify(value);
+        sv->values.max_inline_data_set = true;
+    }
+
+    else if (0 == strcasecmp(key_buffer, "rdmacm_reject_causes_connect_error")) {
+        /* Single value */
+        sv->values.rdmacm_reject_causes_connect_error =
+            (bool) ompi_btl_openib_ini_intify(value);
+        sv->values.rdmacm_reject_causes_connect_error_set = true;
+    }
+
+    else if (0 == strcasecmp(key_buffer, "ignore_device")) {
+        /* Single value */
+        sv->values.ignore_device = (bool) ompi_btl_openib_ini_intify(value);
+        sv->values.ignore_device_set = true;
+    }
+
     else {
         /* Have no idea what this parameter is.  Not an error -- just
            ignore it */
         if (!showed_unknown_field_warning) {
-            opal_show_help("help-mpi-btl-openib.txt", 
+            opal_show_help("help-mpi-btl-openib.txt",
                            "ini file:unknown field", true,
                            ini_filename, btl_openib_ini_yynewlines,
                            key_buffer);
@@ -409,9 +448,9 @@ static int parse_line(parsed_section_values_t *sv)
 
 
 /*
- * Construct an hca_values_t and set all of its values to known states
+ * Construct an device_values_t and set all of its values to known states
  */
-static void hca_values_constructor(hca_values_t *s)
+static void device_values_constructor(device_values_t *s)
 {
     s->section_name = NULL;
     s->vendor_id = 0;
@@ -421,13 +460,16 @@ static void hca_values_constructor(hca_values_t *s)
 
 
 /*
- * Destruct an hca_values_t and free any memory that it has
+ * Destruct an device_values_t and free any memory that it has
  */
-static void hca_values_destructor(hca_values_t *s)
+static void device_values_destructor(device_values_t *s)
 {
     if (NULL != s->section_name) {
         free(s->section_name);
     }
+    if (NULL != s->values.receive_queues) {
+        free(s->values.receive_queues);
+    }
 }
 
 
@@ -468,6 +510,14 @@ static void reset_values(ompi_btl_openib_ini_values_t *v)
 
     v->use_eager_rdma = 0;
     v->use_eager_rdma_set = false;
+
+    v->receive_queues = NULL;
+
+    v->max_inline_data = 0;
+    v->max_inline_data_set = false;
+
+    v->rdmacm_reject_causes_connect_error = false;
+    v->rdmacm_reject_causes_connect_error_set = false;
 }
 
 
@@ -481,11 +531,11 @@ static int save_section(parsed_section_values_t *s)
 {
     int i, j;
     opal_list_item_t *item;
-    hca_values_t *h;
+    device_values_t *h;
     bool found;
 
     /* Is the parsed section valid? */
-    if (NULL == s->name || 0 == s->vendor_ids_len || 
+    if (NULL == s->name || 0 == s->vendor_ids_len ||
         0 == s->vendor_part_ids_len) {
         return OMPI_ERR_BAD_PARAM;
     }
@@ -496,11 +546,11 @@ static int save_section(parsed_section_values_t *s)
         for (j = 0; j < s->vendor_part_ids_len; ++j) {
             found = false;
 
-            /* Iterate over all the saved hcas */
-            for (item = opal_list_get_first(&hcas); 
-                 item != opal_list_get_end(&hcas);
+            /* Iterate over all the saved devices */
+            for (item = opal_list_get_first(&devices);
+                 item != opal_list_get_end(&devices);
                  item = opal_list_get_next(item)) {
-                h = (hca_values_t*) item;
+                h = (device_values_t*) item;
                 if (s->vendor_ids[i] == h->vendor_id &&
                     s->vendor_part_ids[j] == h->vendor_part_id) {
                     /* Found a match.  Update any newly-set values. */
@@ -513,7 +563,29 @@ static int save_section(parsed_section_values_t *s)
                         h->values.use_eager_rdma = s->values.use_eager_rdma;
                         h->values.use_eager_rdma_set = true;
                     }
-                    
+
+                    if (NULL != s->values.receive_queues) {
+                        h->values.receive_queues =
+                            strdup(s->values.receive_queues);
+                    }
+
+                    if (s->values.max_inline_data_set) {
+                        h->values.max_inline_data = s->values.max_inline_data;
+                        h->values.max_inline_data_set = true;
+                    }
+
+                    if (s->values.rdmacm_reject_causes_connect_error_set) {
+                        h->values.rdmacm_reject_causes_connect_error =
+                            s->values.rdmacm_reject_causes_connect_error;
+                        h->values.rdmacm_reject_causes_connect_error_set =
+                            true;
+                    }
+
+                    if (s->values.ignore_device_set) {
+                        h->values.ignore_device = s->values.ignore_device;
+                        h->values.ignore_device_set = true;
+                    }
+
                     found = true;
                     break;
                 }
@@ -522,7 +594,7 @@ static int save_section(parsed_section_values_t *s)
             /* Did we find/update it in the exising list?  If not,
                create a new one. */
             if (!found) {
-                h = OBJ_NEW(hca_values_t);
+                h = OBJ_NEW(device_values_t);
                 h->section_name = strdup(s->name);
                 h->vendor_id = s->vendor_ids[i];
                 h->vendor_part_id = s->vendor_part_ids[j];
@@ -531,7 +603,11 @@ static int save_section(parsed_section_values_t *s)
                    containing bool members by value.  So do a memcpy
                    here instead. */
                 memcpy(&h->values, &s->values, sizeof(s->values));
-                opal_list_append(&hcas, &h->super);
+                /* Need to strdup the string, though */
+                if (NULL != h->values.receive_queues) {
+                    h->values.receive_queues = strdup(s->values.receive_queues);
+                }
+                opal_list_append(&devices, &h->super);
             }
         }
     }
@@ -572,27 +648,39 @@ int ompi_btl_openib_ini_intify_list(char *value, uint32_t **values, int *len)
     char *str = value;
 
     *len = 0;
-    
+
     /* Comma-delimited list of values */
     comma = strchr(str, ',');
     if (NULL == comma) {
         /* If we only got one value (i.e., no comma found), then
            just make an array of one value and save it */
-        *values = malloc(sizeof(uint32_t));
+        *values = (uint32_t *) malloc(sizeof(uint32_t));
         if (NULL == *values) {
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
         *values[0] = (uint32_t) ompi_btl_openib_ini_intify(str);
         *len = 1;
     } else {
-        /* If we found a comma, loop over all the values.  Be a
-           little clever in that we alwasy alloc enough space for
-           an extra value so that when we exit the loop, we don't
-           have to realloc again to get space for the last item. */
+        int newsize = 1;
+
+        /* Count how many values there are and allocate enough space
+           for them */
+        while (NULL != comma) {
+            ++newsize;
+            str = comma + 1;
+            comma = strchr(str, ',');
+        }
+        *values = (uint32_t *) malloc(sizeof(uint32_t) * newsize);
+        if (NULL == *values) {
+            return OMPI_ERR_OUT_OF_RESOURCE;
+        }
+
+        /* Iterate over the values and save them */
+        str = value;
+        comma = strchr(str, ',');
         do {
             *comma = '\0';
-            *values = realloc(*values, sizeof(uint32_t) * (*len + 2));
-            (*values)[*len] = (int32_t) ompi_btl_openib_ini_intify(str);
+            (*values)[*len] = (uint32_t) ompi_btl_openib_ini_intify(str);
             ++(*len);
             str = comma + 1;
             comma = strchr(str, ',');
