@@ -2,24 +2,24 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2009 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2006      Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ * Copyright (c) 2006-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2012 Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * Copyright (c) 2008-2011 Mellanox Technologies.  All rights reserved.
- * Copyright (c) 2009      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2009-2011 IBM Corporation.  All rights reserved.
  * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved
  *
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -27,28 +27,24 @@
 
 #include "opal/dss/dss.h"
 #include "opal_stdint.h"
-#include "orte/util/show_help.h"
 #include "opal/util/error.h"
 #include "opal/util/output.h"
-#include "orte/mca/rml/rml.h"
-#include "orte/mca/rml/rml_types.h"
-#include "orte/mca/errmgr/errmgr.h"
-#include "orte/util/name_fns.h"
-#include "orte/runtime/orte_globals.h"
-#include "ompi/mca/dpm/dpm.h"
+#include "opal/util/show_help.h"
 
+#include "ompi/mca/rte/rte.h"
 #include "btl_openib.h"
-#include "btl_openib_endpoint.h" 
+#include "btl_openib_endpoint.h"
 #include "btl_openib_proc.h"
 #include "connect/connect.h"
-#include "orte/util/show_help.h"
 
 #if (ENABLE_DYNAMIC_SL)
 #include "connect/btl_openib_connect_sl.h"
 #endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+
 typedef enum {
     ENDPOINT_CONNECT_REQUEST,
     ENDPOINT_CONNECT_RESPONSE,
@@ -59,7 +55,7 @@ static int oob_priority = 50;
 static bool rml_recv_posted = false;
 
 static void oob_component_register(void);
-static int oob_component_query(mca_btl_openib_module_t *openib_btl, 
+static int oob_component_query(mca_btl_openib_module_t *openib_btl,
                                ompi_btl_openib_connect_base_module_t **cpc);
 static int oob_component_finalize(void);
 
@@ -73,14 +69,14 @@ static int qp_connect_all(mca_btl_base_endpoint_t* endpoint);
 static int qp_create_all(mca_btl_base_endpoint_t* endpoint);
 static int qp_create_one(mca_btl_base_endpoint_t* endpoint, int qp,
         struct ibv_srq *srq, uint32_t max_recv_wr, uint32_t max_send_wr);
-static int send_connect_data(mca_btl_base_endpoint_t* endpoint, 
+static int send_connect_data(mca_btl_base_endpoint_t* endpoint,
                              uint8_t message_type);
 
-static void rml_send_cb(int status, orte_process_name_t* endpoint, 
-                        opal_buffer_t* buffer, orte_rml_tag_t tag, 
+static void rml_send_cb(int status, ompi_process_name_t* endpoint,
+                        opal_buffer_t* buffer, ompi_rml_tag_t tag,
                         void* cbdata);
-static void rml_recv_cb(int status, orte_process_name_t* process_name, 
-                        opal_buffer_t* buffer, orte_rml_tag_t tag, 
+static void rml_recv_cb(int status, ompi_process_name_t* process_name,
+                        opal_buffer_t* buffer, ompi_rml_tag_t tag,
                         void* cbdata);
 
 /*
@@ -102,10 +98,14 @@ ompi_btl_openib_connect_base_component_t ompi_btl_openib_connect_oob = {
 /* Open - this functions sets up any oob specific commandline params */
 static void oob_component_register(void)
 {
-    mca_base_param_reg_int(&mca_btl_openib_component.super.btl_version,
-                           "connect_oob_priority",
-                           "The selection method priority for oob",
-                           false, false, oob_priority, &oob_priority);
+    oob_priority = 50;
+    (void) mca_base_component_var_register(&mca_btl_openib_component.super.btl_version,
+                                           "connect_oob_priority",
+                                           "The selection method priority for oob",
+                                           MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                           OPAL_INFO_LVL_9,
+                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           &oob_priority);
 
     if (oob_priority > 100) {
         oob_priority = 100;
@@ -118,7 +118,7 @@ static void oob_component_register(void)
  * Init function.  Post non-blocking RML receive to accept incoming
  * connection requests.
  */
-static int oob_component_query(mca_btl_openib_module_t *btl, 
+static int oob_component_query(mca_btl_openib_module_t *btl,
                                ompi_btl_openib_connect_base_module_t **cpc)
 {
     int rc;
@@ -126,10 +126,10 @@ static int oob_component_query(mca_btl_openib_module_t *btl,
     /* If we have the transport_type member, check to ensure we're on
        IB (this CPC will not work with iWarp).  If we do not have the
        transport_type member, then we must be < OFED v1.2, and
-       therefore we must be IB. */   
-#if defined(HAVE_STRUCT_IBV_DEVICE_TRANSPORT_TYPE)
+       therefore we must be IB. */
+#if defined(HAVE_STRUCT_IBV_DEVICE_TRANSPORT_TYPE) && HAVE_DECL_IBV_LINK_LAYER_ETHERNET
     if (BTL_OPENIB_CONNECT_BASE_CHECK_IF_NOT_IB(btl)) {
-        opal_output_verbose(5, mca_btl_base_output,
+        opal_output_verbose(5, ompi_btl_base_framework.framework_output,
                             "openib BTL: oob CPC only supported on InfiniBand; skipped on  %s:%d",
                             ibv_get_device_name(btl->device->ib_dev),
                             btl->port_num);
@@ -138,7 +138,7 @@ static int oob_component_query(mca_btl_openib_module_t *btl,
 #endif
 
     if (mca_btl_openib_component.num_xrc_qps > 0) {
-        opal_output_verbose(5, mca_btl_base_output,
+        opal_output_verbose(5, ompi_btl_base_framework.framework_output,
                             "openib BTL: oob CPC not supported with XRC receive queues, please try xoob CPC; skipped on %s:%d",
                             ibv_get_device_name(btl->device->ib_dev),
                             btl->port_num);
@@ -148,13 +148,13 @@ static int oob_component_query(mca_btl_openib_module_t *btl,
        ensure to only post it *once*, because another btl may have
        come in before this and already posted it. */
     if (!rml_recv_posted) {
-        rc = orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, 
+        rc = ompi_rte_recv_buffer_nb(OMPI_NAME_WILDCARD,
                                      OMPI_RML_TAG_OPENIB,
-                                     ORTE_RML_PERSISTENT,
+                                     OMPI_RML_PERSISTENT,
                                      rml_recv_cb,
                                      NULL);
-        if (ORTE_SUCCESS != rc) {
-            opal_output_verbose(5, mca_btl_base_output,
+        if (OMPI_SUCCESS != rc) {
+            opal_output_verbose(5, ompi_btl_base_framework.framework_output,
                                 "openib BTL: oob CPC system error %d (%s)",
                                 rc, opal_strerror(rc));
             return rc;
@@ -162,14 +162,21 @@ static int oob_component_query(mca_btl_openib_module_t *btl,
         rml_recv_posted = true;
     }
 
-    *cpc = malloc(sizeof(ompi_btl_openib_connect_base_module_t));
+    *cpc = (ompi_btl_openib_connect_base_module_t *) malloc(sizeof(ompi_btl_openib_connect_base_module_t));
     if (NULL == *cpc) {
-        orte_rml.recv_cancel(ORTE_NAME_WILDCARD, OMPI_RML_TAG_OPENIB);
+        ompi_rte_recv_cancel(OMPI_NAME_WILDCARD, OMPI_RML_TAG_OPENIB);
         rml_recv_posted = false;
-        opal_output_verbose(5, mca_btl_base_output,
+        opal_output_verbose(5, ompi_btl_base_framework.framework_output,
                             "openib BTL: oob CPC system error (malloc failed)");
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
+
+    if (oob_priority > 100) {
+        oob_priority = 100;
+    } else if (oob_priority < -1) {
+        oob_priority = -1;
+    }
+
     (*cpc)->data.cbm_component = &ompi_btl_openib_connect_oob;
     (*cpc)->data.cbm_priority = oob_priority;
     (*cpc)->data.cbm_modex_message = NULL;
@@ -181,7 +188,7 @@ static int oob_component_query(mca_btl_openib_module_t *btl,
     (*cpc)->cbm_finalize = NULL;
     (*cpc)->cbm_uses_cts = false;
 
-    opal_output_verbose(5, mca_btl_base_output,
+    opal_output_verbose(5, ompi_btl_base_framework.framework_output,
                         "openib BTL: oob CPC available for use on %s:%d",
                         ibv_get_device_name(btl->device->ib_dev),
                         btl->port_num);
@@ -207,7 +214,7 @@ static int oob_module_start_connect(ompi_btl_openib_connect_base_module_t *cpc,
     endpoint->endpoint_state = MCA_BTL_IB_CONNECTING;
     if (OMPI_SUCCESS !=
         (rc = send_connect_data(endpoint, ENDPOINT_CONNECT_REQUEST))) {
-        BTL_ERROR(("error sending connect request, error code %d", rc)); 
+        BTL_ERROR(("error sending connect request, error code %d", rc));
         return rc;
     }
 
@@ -220,7 +227,7 @@ static int oob_module_start_connect(ompi_btl_openib_connect_base_module_t *cpc,
 static int oob_component_finalize(void)
 {
     if (rml_recv_posted) {
-        orte_rml.recv_cancel(ORTE_NAME_WILDCARD, OMPI_RML_TAG_OPENIB);
+        ompi_rte_recv_cancel(OMPI_NAME_WILDCARD, OMPI_RML_TAG_OPENIB);
         rml_recv_posted = false;
     }
 #if (ENABLE_DYNAMIC_SL)
@@ -249,7 +256,7 @@ static int reply_start_connect(mca_btl_openib_endpoint_t *endpoint,
 
     /* Set the remote side info */
     set_remote_info(endpoint, rem_info);
-    
+
     /* Connect to remote endpoint qp's */
     if (OMPI_SUCCESS != (rc = qp_connect_all(endpoint))) {
         return rc;
@@ -275,12 +282,11 @@ static int set_remote_info(mca_btl_base_endpoint_t* endpoint,
     free(endpoint->rem_info.rem_qps);
 
     /* copy the rem_info stuff */
-    memcpy(&((mca_btl_openib_endpoint_t*) endpoint)->rem_info, 
-           rem_info, sizeof(mca_btl_openib_rem_info_t)); 
-    
-    BTL_VERBOSE(("Setting QP info,  LID = %d", endpoint->rem_info.rem_lid));
-    return ORTE_SUCCESS;
+    memcpy(&((mca_btl_openib_endpoint_t*) endpoint)->rem_info,
+           rem_info, sizeof(mca_btl_openib_rem_info_t));
 
+    BTL_VERBOSE(("Setting QP info,  LID = %d", endpoint->rem_info.rem_lid));
+    return OMPI_SUCCESS;
 }
 
 
@@ -296,8 +302,8 @@ static int qp_connect_all(mca_btl_openib_endpoint_t *endpoint)
     for (i = 0; i < mca_btl_openib_component.num_qps; i++) {
         struct ibv_qp_attr attr;
         struct ibv_qp* qp = endpoint->qps[i].qp->lcl_qp;
-        enum ibv_mtu mtu = (openib_btl->device->mtu < endpoint->rem_info.rem_mtu) ?
-            openib_btl->device->mtu : endpoint->rem_info.rem_mtu;
+        enum ibv_mtu mtu = (enum ibv_mtu) ((openib_btl->device->mtu < endpoint->rem_info.rem_mtu) ?
+            openib_btl->device->mtu : endpoint->rem_info.rem_mtu) ;
 
         memset(&attr, 0, sizeof(attr));
         attr.qp_state           = IBV_QPS_RTR;
@@ -349,7 +355,7 @@ static int qp_connect_all(mca_btl_openib_endpoint_t *endpoint)
                           IBV_QP_MIN_RNR_TIMER)) {
             BTL_ERROR(("error modifing QP to RTR errno says %s",
                        strerror(errno)));
-            return OMPI_ERROR; 
+            return OMPI_ERROR;
         }
         attr.qp_state       = IBV_QPS_RTS;
         attr.timeout        = mca_btl_openib_component.ib_timeout;
@@ -398,7 +404,7 @@ static int qp_create_all(mca_btl_base_endpoint_t* endpoint)
     if(0 == pp_qp_num && true == endpoint->use_eager_rdma)
         pp_qp_num = 1;
 
-    for (qp = 0; qp < mca_btl_openib_component.num_qps; ++qp) { 
+    for (qp = 0; qp < mca_btl_openib_component.num_qps; ++qp) {
         struct ibv_srq *srq = NULL;
         uint32_t max_recv_wr, max_send_wr;
         int32_t rd_rsv, rd_num_credits;
@@ -456,7 +462,7 @@ static uint32_t max_inline_size(int qp, mca_btl_openib_device_t *device)
  * Create the local side of one qp.  The remote side will be connected
  * later.
  */
-static int qp_create_one(mca_btl_base_endpoint_t* endpoint, int qp, 
+static int qp_create_one(mca_btl_base_endpoint_t* endpoint, int qp,
         struct ibv_srq *srq, uint32_t max_recv_wr, uint32_t max_send_wr)
 {
     mca_btl_openib_module_t *openib_btl = endpoint->endpoint_btl;
@@ -472,7 +478,7 @@ static int qp_create_one(mca_btl_base_endpoint_t* endpoint, int qp,
     init_attr.send_cq = openib_btl->device->ib_cq[BTL_OPENIB_LP_CQ];
     init_attr.recv_cq = openib_btl->device->ib_cq[qp_cq_prio(qp)];
     init_attr.srq     = srq;
-    init_attr.cap.max_inline_data = req_inline = 
+    init_attr.cap.max_inline_data = req_inline =
         max_inline_size(qp, openib_btl->device);
     init_attr.cap.max_send_sge = 1;
     init_attr.cap.max_recv_sge = 1; /* we do not use SG list */
@@ -483,39 +489,43 @@ static int qp_create_one(mca_btl_base_endpoint_t* endpoint, int qp,
     }
     init_attr.cap.max_send_wr  = max_send_wr;
 
-    my_qp = ibv_create_qp(openib_btl->device->ib_pd, &init_attr); 
-    
-    if (NULL == my_qp) { 
-        BTL_ERROR(("error creating qp errno says %s", strerror(errno))); 
-        return OMPI_ERROR; 
+    my_qp = ibv_create_qp(openib_btl->device->ib_pd, &init_attr);
+
+    if (NULL == my_qp) {
+        opal_show_help("help-mpi-btl-openib-cpc-base.txt",
+                       "ibv_create_qp failed", true,
+                       ompi_process_info.nodename,
+                       ibv_get_device_name(openib_btl->device->ib_dev),
+                       "Reliable connected (RC)");
+        return OMPI_ERROR;
     }
     endpoint->qps[qp].qp->lcl_qp = my_qp;
 
     if (init_attr.cap.max_inline_data < req_inline) {
         endpoint->qps[qp].ib_inline_max = init_attr.cap.max_inline_data;
-        orte_show_help("help-mpi-btl-openib-cpc-base.txt",
-                       "inline truncated", true, orte_process_info.nodename,
+        opal_show_help("help-mpi-btl-openib-cpc-base.txt",
+                       "inline truncated", true, ompi_process_info.nodename,
                        ibv_get_device_name(openib_btl->device->ib_dev),
                        openib_btl->port_num,
                        req_inline, init_attr.cap.max_inline_data);
     } else {
         endpoint->qps[qp].ib_inline_max = req_inline;
     }
-    
+
     attr.qp_state        = IBV_QPS_INIT;
     attr.pkey_index      = openib_btl->pkey_index;
     attr.port_num        = openib_btl->port_num;
     attr.qp_access_flags = IBV_ACCESS_REMOTE_WRITE | IBV_ACCESS_REMOTE_READ;
 
-    if (ibv_modify_qp(endpoint->qps[qp].qp->lcl_qp, 
-                      &attr, 
-                      IBV_QP_STATE | 
-                      IBV_QP_PKEY_INDEX | 
-                      IBV_QP_PORT | 
-                      IBV_QP_ACCESS_FLAGS )) { 
-        BTL_ERROR(("error modifying qp to INIT errno says %s", strerror(errno))); 
-        return OMPI_ERROR; 
-    } 
+    if (ibv_modify_qp(endpoint->qps[qp].qp->lcl_qp,
+                      &attr,
+                      IBV_QP_STATE |
+                      IBV_QP_PKEY_INDEX |
+                      IBV_QP_PORT |
+                      IBV_QP_ACCESS_FLAGS )) {
+        BTL_ERROR(("error modifying qp to INIT errno says %s", strerror(errno)));
+        return OMPI_ERROR;
+    }
 
     /* Setup meta data on the endpoint */
     endpoint->qps[qp].qp->lcl_psn = lrand48() & 0xffffff;
@@ -528,29 +538,29 @@ static int qp_create_one(mca_btl_base_endpoint_t* endpoint, int qp,
 /*
  * RML send connect information to remote endpoint
  */
-static int send_connect_data(mca_btl_base_endpoint_t* endpoint, 
+static int send_connect_data(mca_btl_base_endpoint_t* endpoint,
                              uint8_t message_type)
 {
     opal_buffer_t* buffer = OBJ_NEW(opal_buffer_t);
     int rc;
-    
+
     if (NULL == buffer) {
-         ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-         return ORTE_ERR_OUT_OF_RESOURCE;
+         OMPI_ERROR_LOG(OMPI_ERR_OUT_OF_RESOURCE);
+         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
-    /* pack the info in the send buffer */ 
+    /* pack the info in the send buffer */
     BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT8));
     rc = opal_dss.pack(buffer, &message_type, 1, OPAL_UINT8);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         return rc;
     }
 
     BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT64));
     rc = opal_dss.pack(buffer, &endpoint->subnet_id, 1, OPAL_UINT64);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         return rc;
     }
 
@@ -560,14 +570,14 @@ static int send_connect_data(mca_btl_base_endpoint_t* endpoint,
         rc = opal_dss.pack(buffer,
                            &endpoint->rem_info.rem_qps[0].rem_qp_num, 1,
                            OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
         BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT16));
         rc = opal_dss.pack(buffer, &endpoint->rem_info.rem_lid, 1, OPAL_UINT16);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
     }
@@ -575,54 +585,54 @@ static int send_connect_data(mca_btl_base_endpoint_t* endpoint,
     if (message_type != ENDPOINT_CONNECT_ACK) {
         int qp;
         /* stuff all the QP info into the buffer */
-        for (qp = 0; qp < mca_btl_openib_component.num_qps; qp++) { 
+        for (qp = 0; qp < mca_btl_openib_component.num_qps; qp++) {
             BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT32));
             rc = opal_dss.pack(buffer, &endpoint->qps[qp].qp->lcl_qp->qp_num,
                                1, OPAL_UINT32);
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
+            if (OPAL_SUCCESS != rc) {
+                OMPI_ERROR_LOG(rc);
                 return rc;
             }
             BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT32));
             rc = opal_dss.pack(buffer, &endpoint->qps[qp].qp->lcl_psn, 1,
-                               OPAL_UINT32); 
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
+                               OPAL_UINT32);
+            if (OPAL_SUCCESS != rc) {
+                OMPI_ERROR_LOG(rc);
                 return rc;
             }
         }
-        
+
         BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT16));
         rc = opal_dss.pack(buffer, &endpoint->endpoint_btl->lid, 1, OPAL_UINT16);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
         BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT32));
         rc = opal_dss.pack(buffer, &endpoint->endpoint_btl->device->mtu, 1,
                 OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
         BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT32));
         rc = opal_dss.pack(buffer, &endpoint->index, 1, OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
     }
 
     /* send to remote endpoint */
-    rc = orte_rml.send_buffer_nb(&endpoint->endpoint_proc->proc_guid, 
+    rc = ompi_rte_send_buffer_nb(&endpoint->endpoint_proc->proc_ompi->proc_name,
                                  buffer, OMPI_RML_TAG_OPENIB, 0,
                                  rml_send_cb, NULL);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OMPI_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         return rc;
     }
     BTL_VERBOSE(("Sent QP Info, LID = %d, SUBNET = %" PRIx64 "\n",
-                 endpoint->endpoint_btl->lid, 
+                 endpoint->endpoint_btl->lid,
                  endpoint->subnet_id));
 
     return OMPI_SUCCESS;
@@ -633,8 +643,8 @@ static int send_connect_data(mca_btl_base_endpoint_t* endpoint,
  * Callback when we have finished RML sending the connect data to a
  * remote peer
  */
-static void rml_send_cb(int status, orte_process_name_t* endpoint, 
-                        opal_buffer_t* buffer, orte_rml_tag_t tag, 
+static void rml_send_cb(int status, ompi_process_name_t* endpoint,
+                        opal_buffer_t* buffer, ompi_rml_tag_t tag,
                         void* cbdata)
 {
     OBJ_RELEASE(buffer);
@@ -646,8 +656,8 @@ static void rml_send_cb(int status, orte_process_name_t* endpoint,
  * and if this endpoint is trying to connect, reply with our QP info,
  * otherwise try to modify QP's and establish reliable connection
  */
-static void rml_recv_cb(int status, orte_process_name_t* process_name, 
-                        opal_buffer_t* buffer, orte_rml_tag_t tag, 
+static void rml_recv_cb(int status, ompi_process_name_t* process_name,
+                        opal_buffer_t* buffer, ompi_rml_tag_t tag,
                         void* cbdata)
 {
     mca_btl_openib_proc_t *ib_proc;
@@ -666,97 +676,97 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
        the pointers */
     memset(&rem_info,0, sizeof(rem_info));
 
-    /* start by unpacking data first so we know who is knocking at 
-       our door */ 
+   /* start by unpacking data first so we know who is knocking at
+       our door */
     BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT8));
     rc = opal_dss.unpack(buffer, &message_type, &cnt, OPAL_UINT8);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OMPI_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         mca_btl_openib_endpoint_invoke_error(NULL);
         return;
     }
-    
+
     BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT64));
     rc = opal_dss.unpack(buffer, &rem_info.rem_subnet_id, &cnt, OPAL_UINT64);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OMPI_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         mca_btl_openib_endpoint_invoke_error(NULL);
         return;
     }
-    
+
     if (ENDPOINT_CONNECT_REQUEST != message_type) {
         BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT32));
         rc = opal_dss.unpack(buffer, &lcl_qp, &cnt, OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OMPI_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             mca_btl_openib_endpoint_invoke_error(NULL);
             return;
         }
         BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT16));
         rc = opal_dss.unpack(buffer, &lcl_lid, &cnt, OPAL_UINT16);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OMPI_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             mca_btl_openib_endpoint_invoke_error(NULL);
             return;
         }
     }
     if (ENDPOINT_CONNECT_ACK != message_type) {
-        int qp; 
+        int qp;
         /* get ready for the data */
-        rem_info.rem_qps = 
-            (mca_btl_openib_rem_qp_info_t*) malloc(sizeof(mca_btl_openib_rem_qp_info_t) * 
+        rem_info.rem_qps =
+            (mca_btl_openib_rem_qp_info_t*) malloc(sizeof(mca_btl_openib_rem_qp_info_t) *
                                                    mca_btl_openib_component.num_qps);
-        
+
         /* unpack all the qp info */
-        for (qp = 0; qp < mca_btl_openib_component.num_qps; ++qp) { 
+        for (qp = 0; qp < mca_btl_openib_component.num_qps; ++qp) {
             BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT32));
             rc = opal_dss.unpack(buffer, &rem_info.rem_qps[qp].rem_qp_num, &cnt,
                                  OPAL_UINT32);
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
+            if (OMPI_SUCCESS != rc) {
+                OMPI_ERROR_LOG(rc);
                 mca_btl_openib_endpoint_invoke_error(NULL);
                 return;
             }
             BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT32));
             rc = opal_dss.unpack(buffer, &rem_info.rem_qps[qp].rem_psn, &cnt,
                                  OPAL_UINT32);
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
+            if (OMPI_SUCCESS != rc) {
+                OMPI_ERROR_LOG(rc);
                 mca_btl_openib_endpoint_invoke_error(NULL);
                 return;
             }
         }
-        
+
         BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT16));
         rc = opal_dss.unpack(buffer, &rem_info.rem_lid, &cnt, OPAL_UINT16);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OMPI_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             mca_btl_openib_endpoint_invoke_error(NULL);
             return;
         }
         BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT32));
         rc = opal_dss.unpack(buffer, &rem_info.rem_mtu, &cnt, OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OMPI_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             mca_btl_openib_endpoint_invoke_error(NULL);
             return;
         }
         BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT32));
         rc = opal_dss.unpack(buffer, &rem_info.rem_index, &cnt, OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OMPI_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             mca_btl_openib_endpoint_invoke_error(NULL);
             return;
         }
     }
-    
+
     BTL_VERBOSE(("Received QP Info,  LID = %d, SUBNET = %" PRIx64 "\n",
-                 rem_info.rem_lid, 
+                 rem_info.rem_lid,
                  rem_info.rem_subnet_id));
-    
-    master = orte_util_compare_name_fields(ORTE_NS_CMP_ALL, ORTE_PROC_MY_NAME,
+
+    master = ompi_rte_compare_name_fields(OMPI_RTE_CMP_ALL, OMPI_PROC_MY_NAME,
                                     process_name) > 0 ? true : false;
-    
+
     /* Need to protect the ib_procs list */
     OPAL_THREAD_LOCK(&mca_btl_openib_component.ib_lock);
 
@@ -766,16 +776,16 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
             opal_list_get_end(&mca_btl_openib_component.ib_procs);
         ib_proc  = (mca_btl_openib_proc_t*)opal_list_get_next(ib_proc)) {
         bool found = false;
-        
-        if (orte_util_compare_name_fields(ORTE_NS_CMP_ALL,
-                                   &ib_proc->proc_guid, process_name) != OPAL_EQUAL) {
+
+        if (OPAL_EQUAL != ompi_rte_compare_name_fields(OMPI_RTE_CMP_ALL,
+                                   &ib_proc->proc_ompi->proc_name, process_name)) {
             continue;
         }
-        
+
         if (ENDPOINT_CONNECT_REQUEST != message_type) {
             /* This is a reply message. Try to get the endpoint
                instance the reply belongs to */
-            for (i = 0; i < ib_proc->proc_endpoint_count; i++) { 
+            for (i = 0; i < ib_proc->proc_endpoint_count; i++) {
                 ib_endpoint = ib_proc->proc_endpoints[i];
                 if (ib_endpoint->qps[0].qp->lcl_qp != NULL &&
                     lcl_lid == ib_endpoint->endpoint_btl->lid &&
@@ -793,7 +803,7 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
             mca_btl_openib_endpoint_t *ib_endpoint_found = NULL;
             int master_first_closed = -1;
 
-            for (i = 0; i < ib_proc->proc_endpoint_count; i++) { 
+            for (i = 0; i < ib_proc->proc_endpoint_count; i++) {
                 ib_endpoint = ib_proc->proc_endpoints[i];
                 if (ib_endpoint->subnet_id != rem_info.rem_subnet_id ||
                    (ib_endpoint->endpoint_state != MCA_BTL_IB_CONNECTING
@@ -815,7 +825,7 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
                     break; /* Found one. No point to continue */
             }
             ib_endpoint = ib_endpoint_found;
-            
+
             if (found && master &&
                 MCA_BTL_IB_CLOSED == ib_endpoint->endpoint_state ) {
                 /* since this is master and no endpoints found in
@@ -833,17 +843,17 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
                 return;
             }
         }
-        
+
         if (!found) {
-            BTL_ERROR(("can't find suitable endpoint for this peer\n")); 
+            BTL_ERROR(("can't find suitable endpoint for this peer\n"));
             mca_btl_openib_endpoint_invoke_error(NULL);
             OPAL_THREAD_UNLOCK(&mca_btl_openib_component.ib_lock);
-            return; 
+            return;
         }
-        
+
         OPAL_THREAD_LOCK(&ib_endpoint->endpoint_lock);
         endpoint_state = ib_endpoint->endpoint_state;
-        
+
         /* Update status */
         switch (endpoint_state) {
         case MCA_BTL_IB_CLOSED :
@@ -855,17 +865,17 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
                 assert(rem_info.rem_qps != NULL);
                 rc = reply_start_connect(ib_endpoint, &rem_info);
             } else {
-                rc = oob_module_start_connect(ib_endpoint->endpoint_local_cpc, 
+                rc = oob_module_start_connect(ib_endpoint->endpoint_local_cpc,
                                               ib_endpoint);
             }
-            
+
             if (OMPI_SUCCESS != rc) {
                 BTL_ERROR(("error in endpoint reply start connect"));
                 mca_btl_openib_endpoint_invoke_error(ib_endpoint);
                 OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
                 break;
             }
-            
+
             /* As long as we expect a message from the peer (in order
                to setup the connection) let the event engine pool the
                RML events. Note: we increment it once peer active
@@ -873,17 +883,17 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
             opal_progress_event_users_increment();
             OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
             break;
-             
+
         case MCA_BTL_IB_CONNECTING :
             assert(rem_info.rem_qps != NULL);
             set_remote_info(ib_endpoint, &rem_info);
             if (OMPI_SUCCESS != (rc = qp_connect_all(ib_endpoint))) {
-                BTL_ERROR(("endpoint connect error: %d", rc)); 
+                BTL_ERROR(("endpoint connect error: %d", rc));
                 mca_btl_openib_endpoint_invoke_error(ib_endpoint);
                 OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
                 break;
             }
-           
+
             if (master) {
                 ib_endpoint->endpoint_state = MCA_BTL_IB_WAITING_ACK;
 
@@ -897,24 +907,30 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
                 /* cpc complete unlock the endpoint */
              }
             break;
-            
+
         case MCA_BTL_IB_WAITING_ACK:
             /* Tell main BTL that we're done */
             mca_btl_openib_endpoint_cpc_complete(ib_endpoint);
             /* cpc complete unlock the endpoint */
             break;
-            
+
         case MCA_BTL_IB_CONNECT_ACK:
             send_connect_data(ib_endpoint, ENDPOINT_CONNECT_ACK);
             /* Tell main BTL that we're done */
             mca_btl_openib_endpoint_cpc_complete(ib_endpoint);
             /* cpc complete unlock the endpoint */
             break;
-            
+
         case MCA_BTL_IB_CONNECTED:
             OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
             break;
 
+        case MCA_BTL_IB_FAILED:
+            /* This connection has been put in the failed state
+             * so just ignore the connection message. */
+            OPAL_THREAD_UNLOCK(&ib_endpoint->endpoint_lock);
+            break;
+
         default :
             BTL_ERROR(("Invalid endpoint state %d", endpoint_state));
             mca_btl_openib_endpoint_invoke_error(ib_endpoint);
