@@ -2,6 +2,11 @@
  * Copyright (c) 2007-2011 Mellanox Technologies.  All rights reserved.
  * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2009      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2010-2011 The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
+ * Copyright (c) 2012-2013 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  *
  * $COPYRIGHT$
  *
@@ -16,12 +21,9 @@
 #include "opal/dss/dss.h"
 #include "opal/util/error.h"
 #include "opal/util/output.h"
-#include "orte/util/show_help.h"
-#include "orte/util/name_fns.h"
-#include "orte/mca/rml/rml.h"
-#include "orte/mca/rml/rml_types.h"
-#include "orte/mca/errmgr/errmgr.h"
-#include "ompi/mca/dpm/dpm.h"
+#include "opal/util/show_help.h"
+
+#include "ompi/mca/rte/rte.h"
 
 #include "btl_openib.h"
 #include "btl_openib_endpoint.h"
@@ -29,13 +31,12 @@
 #include "btl_openib_xrc.h"
 #include "btl_openib_async.h"
 #include "connect/connect.h"
-#include "orte/util/show_help.h"
 #if (ENABLE_DYNAMIC_SL)
 #include "connect/btl_openib_connect_sl.h"
 #endif
 
 static void xoob_component_register(void);
-static int xoob_component_query(mca_btl_openib_module_t *openib_btl, 
+static int xoob_component_query(mca_btl_openib_module_t *openib_btl,
                                 ompi_btl_openib_connect_base_module_t **cpc);
 static int xoob_component_finalize(void);
 
@@ -89,8 +90,8 @@ static int xoob_priority = 60;
  * Callback when we have finished RML sending the connect data to a
  * remote peer
  */
-static void xoob_rml_send_cb(int status, orte_process_name_t* endpoint,
-                        opal_buffer_t* buffer, orte_rml_tag_t tag,
+static void xoob_rml_send_cb(int status, ompi_process_name_t* endpoint,
+                        opal_buffer_t* buffer, ompi_rml_tag_t tag,
                         void* cbdata)
 {
     OBJ_RELEASE(buffer);
@@ -105,24 +106,24 @@ static int xoob_receive_connect_data(mca_btl_openib_rem_info_t *info, uint16_t *
     /* Recv standart header */
     BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT8));
     rc = opal_dss.unpack(buffer, message_type, &cnt, OPAL_UINT8);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         return OMPI_ERROR;
     }
     BTL_VERBOSE(("Recv unpack Message type  = %d\n", *message_type));
 
     BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT64));
     rc = opal_dss.unpack(buffer, &info->rem_subnet_id, &cnt, OPAL_UINT64);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         return OMPI_ERROR;
     }
     BTL_VERBOSE(("Recv unpack sid  = %" PRIx64 "\n", info->rem_subnet_id));
 
     BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT16));
     rc = opal_dss.unpack(buffer, &info->rem_lid, &cnt, OPAL_UINT16);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         return OMPI_ERROR;
     }
     BTL_VERBOSE(("Recv unpack lid  = %d", info->rem_lid));
@@ -135,8 +136,8 @@ static int xoob_receive_connect_data(mca_btl_openib_rem_info_t *info, uint16_t *
         BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT32));
         rc = opal_dss.unpack(buffer, &info->rem_qps->rem_qp_num, &cnt,
                 OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return OMPI_ERROR;
         }
         BTL_VERBOSE(("Recv unpack remote qp  = %x", info->rem_qps->rem_qp_num));
@@ -144,16 +145,16 @@ static int xoob_receive_connect_data(mca_btl_openib_rem_info_t *info, uint16_t *
         BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT32));
         rc = opal_dss.unpack(buffer, &info->rem_qps->rem_psn, &cnt,
                 OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return OMPI_ERROR;
         }
         BTL_VERBOSE(("Recv unpack remote psn = %d", info->rem_qps->rem_psn));
 
         BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT32));
         rc = opal_dss.unpack(buffer, &info->rem_mtu, &cnt, OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return OMPI_ERROR;
         }
         BTL_VERBOSE(("Recv unpack remote mtu = %d", info->rem_mtu));
@@ -164,8 +165,8 @@ static int xoob_receive_connect_data(mca_btl_openib_rem_info_t *info, uint16_t *
         /* unpack requested lid info */
         BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT16));
         rc = opal_dss.unpack(buffer, lid, &cnt, OPAL_UINT16);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return OMPI_ERROR;
         }
         BTL_VERBOSE(("Recv unpack requested lid = %d", *lid));
@@ -177,8 +178,8 @@ static int xoob_receive_connect_data(mca_btl_openib_rem_info_t *info, uint16_t *
         /* In XRC request case we will use rem_qp_num as container for requested qp number */
         rc = opal_dss.unpack(buffer, &info->rem_qps->rem_qp_num, &cnt,
                 OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
         BTL_VERBOSE(("Recv unpack requested qp = %x", info->rem_qps->rem_qp_num));
@@ -188,8 +189,8 @@ static int xoob_receive_connect_data(mca_btl_openib_rem_info_t *info, uint16_t *
             ENDPOINT_XOOB_CONNECT_XRC_RESPONSE == *message_type) {
         BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT32));
         rc = opal_dss.unpack(buffer, &info->rem_index, &cnt, OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return OMPI_ERROR;
         }
         BTL_VERBOSE(("Recv unpack remote index = %d", info->rem_index));
@@ -197,8 +198,8 @@ static int xoob_receive_connect_data(mca_btl_openib_rem_info_t *info, uint16_t *
         for (srq = 0; srq < mca_btl_openib_component.num_xrc_qps; srq++) {
             BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT32));
             rc = opal_dss.unpack(buffer, &info->rem_srqs[srq].rem_srq_num, &cnt, OPAL_UINT32);
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
+            if (OPAL_SUCCESS != rc) {
+                OMPI_ERROR_LOG(rc);
                 return OMPI_ERROR;
             }
             BTL_VERBOSE(("Recv unpack remote index srq num[%d]= %d", srq, info->rem_srqs[srq].rem_srq_num));
@@ -217,8 +218,8 @@ static int xoob_send_connect_data(mca_btl_base_endpoint_t* endpoint,
     int rc, srq;
 
     if (NULL == buffer) {
-        ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-        return ORTE_ERR_OUT_OF_RESOURCE;
+        OMPI_ERROR_LOG(OMPI_ERR_OUT_OF_RESOURCE);
+        return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
     /* Bulding standart header that we use in all messages:
@@ -230,24 +231,24 @@ static int xoob_send_connect_data(mca_btl_base_endpoint_t* endpoint,
     BTL_VERBOSE(("Send pack Message type = %d", message_type));
     BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT8));
     rc = opal_dss.pack(buffer, &message_type, 1, OPAL_UINT8);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         return rc;
     }
 
     BTL_VERBOSE(("Send pack sid = %" PRIx64 "\n", endpoint->subnet_id));
     BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT64));
     rc = opal_dss.pack(buffer, &endpoint->subnet_id, 1, OPAL_UINT64);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         return rc;
     }
 
     BTL_VERBOSE(("Send pack lid = %d", endpoint->endpoint_btl->lid));
     BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT16));
     rc = opal_dss.pack(buffer, &endpoint->endpoint_btl->lid, 1, OPAL_UINT16);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         return rc;
     }
 
@@ -273,15 +274,15 @@ static int xoob_send_connect_data(mca_btl_base_endpoint_t* endpoint,
         BTL_VERBOSE(("Send pack qp num = %x", qp_num));
         BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT32));
         rc = opal_dss.pack(buffer, &qp_num, 1, OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
         BTL_VERBOSE(("Send pack lpsn = %d", psn));
         BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT32));
         rc = opal_dss.pack(buffer, &psn, 1, OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
 
@@ -289,8 +290,8 @@ static int xoob_send_connect_data(mca_btl_base_endpoint_t* endpoint,
         BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT32));
         rc = opal_dss.pack(buffer, &endpoint->endpoint_btl->device->mtu, 1,
                 OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
     }
@@ -306,8 +307,8 @@ static int xoob_send_connect_data(mca_btl_base_endpoint_t* endpoint,
         BTL_VERBOSE(("Send pack remote lid = %d", endpoint->ib_addr->lid));
         BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT16));
         rc = opal_dss.pack(buffer, &endpoint->ib_addr->lid, 1, OPAL_UINT16);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
     }
@@ -319,8 +320,8 @@ static int xoob_send_connect_data(mca_btl_base_endpoint_t* endpoint,
         BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT32));
         rc = opal_dss.pack(buffer, &endpoint->ib_addr->remote_xrc_rcv_qp_num,
                 1, OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
     }
@@ -335,8 +336,8 @@ static int xoob_send_connect_data(mca_btl_base_endpoint_t* endpoint,
         BTL_VERBOSE(("Send pack index = %d", endpoint->index));
         BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT32));
         rc = opal_dss.pack(buffer, &endpoint->index, 1, OPAL_UINT32);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+        if (OPAL_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
             return rc;
         }
         /* on response we add all SRQ numbers */
@@ -345,19 +346,19 @@ static int xoob_send_connect_data(mca_btl_base_endpoint_t* endpoint,
             BTL_VERBOSE(("packing %d of %d\n", 1, OPAL_UINT32));
             rc = opal_dss.pack(buffer, &endpoint->endpoint_btl->qps[srq].u.srq_qp.srq->xrc_srq_num,
                     1, OPAL_UINT32);
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
+            if (OPAL_SUCCESS != rc) {
+                OMPI_ERROR_LOG(rc);
                 return rc;
             }
         }
     }
 
     /* send to remote endpoint */
-    rc = orte_rml.send_buffer_nb(&endpoint->endpoint_proc->proc_guid,
+    rc = ompi_rte_send_buffer_nb(&endpoint->endpoint_proc->proc_ompi->proc_name,
             buffer, OMPI_RML_TAG_XOPENIB, 0,
             xoob_rml_send_cb, NULL);
-    if (ORTE_SUCCESS != rc) {
-        ORTE_ERROR_LOG(rc);
+    if (OMPI_SUCCESS != rc) {
+        OMPI_ERROR_LOG(rc);
         return rc;
     }
 
@@ -409,14 +410,18 @@ static int xoob_send_qp_create (mca_btl_base_endpoint_t* endpoint)
     qp_init_attr.xrc_domain = openib_btl->device->xrc_domain;
     *qp = ibv_create_qp(openib_btl->device->ib_pd, &qp_init_attr);
     if (NULL == *qp) {
-        BTL_ERROR(("Error creating QP, errno says: %s", strerror(errno)));
+	opal_show_help("help-mpi-btl-openib-cpc-base.txt",
+		       "ibv_create_qp failed", true,
+		       ompi_process_info.nodename,
+		       ibv_get_device_name(openib_btl->device->ib_dev),
+		       "Reliable connected (XRC)");
         return OMPI_ERROR;
     }
 
     if (qp_init_attr.cap.max_inline_data < req_inline) {
         endpoint->qps[0].ib_inline_max = qp_init_attr.cap.max_inline_data;
-        orte_show_help("help-mpi-btl-openib-cpc-base.txt",
-                       "inline truncated", orte_process_info.nodename,
+        opal_show_help("help-mpi-btl-openib-cpc-base.txt",
+                       "inline truncated", ompi_process_info.nodename,
                        ibv_get_device_name(openib_btl->device->ib_dev),
                        openib_btl->port_num,
                        req_inline, qp_init_attr.cap.max_inline_data);
@@ -680,7 +685,7 @@ static int xoob_reply_first_connect(mca_btl_openib_endpoint_t *endpoint,
 }
 
 /* Find endpoint for specific subnet/lid/message */
-static mca_btl_openib_endpoint_t* xoob_find_endpoint(orte_process_name_t* process_name,
+static mca_btl_openib_endpoint_t* xoob_find_endpoint(ompi_process_name_t* process_name,
         uint64_t subnet_id, uint16_t lid, uint8_t message_type)
 {
     size_t i;
@@ -689,8 +694,12 @@ static mca_btl_openib_endpoint_t* xoob_find_endpoint(orte_process_name_t* proces
     bool found = false;
 
     BTL_VERBOSE(("Searching for ep and proc with follow parameters:"
-                "jobid %d, vpid %d, sid %" PRIx64 ", lid %d",
-                process_name->jobid, process_name->vpid, subnet_id, lid));
+                "jobid %d, vpid %d, "
+                "sid %" PRIx64 ", lid %d",
+                process_name->jobid, process_name->vpid,
+                subnet_id, lid));
+
+
     /* find ibproc */
     OPAL_THREAD_LOCK(&mca_btl_openib_component.ib_lock);
     for (ib_proc = (mca_btl_openib_proc_t*)
@@ -698,8 +707,8 @@ static mca_btl_openib_endpoint_t* xoob_find_endpoint(orte_process_name_t* proces
             ib_proc != (mca_btl_openib_proc_t*)
             opal_list_get_end(&mca_btl_openib_component.ib_procs);
             ib_proc  = (mca_btl_openib_proc_t*)opal_list_get_next(ib_proc)) {
-        if (orte_util_compare_name_fields(ORTE_NS_CMP_ALL,
-                    &ib_proc->proc_guid, process_name) == OPAL_EQUAL) {
+        if (OPAL_EQUAL == ompi_rte_compare_name_fields(OMPI_RTE_CMP_ALL,
+                    &ib_proc->proc_ompi->proc_name, process_name)) {
             found = true;
             break;
         }
@@ -809,8 +818,8 @@ static void free_rem_info(mca_btl_openib_rem_info_t *rem_info)
  * and if this endpoint is trying to connect, reply with our QP info,
  * otherwise try to modify QP's and establish reliable connection
  */
-static void xoob_rml_recv_cb(int status, orte_process_name_t* process_name,
-                        opal_buffer_t* buffer, orte_rml_tag_t tag,
+static void xoob_rml_recv_cb(int status, ompi_process_name_t* process_name,
+                        opal_buffer_t* buffer, ompi_rml_tag_t tag,
                         void* cbdata)
 {
     int rc;
@@ -841,7 +850,7 @@ static void xoob_rml_recv_cb(int status, orte_process_name_t* process_name,
                     requested_lid, message_type);
             if ( NULL == ib_endpoint) {
                 BTL_ERROR(("Got ENDPOINT_XOOB_CONNECT_REQUEST."
-                           " Failed to find endpoint with subnet %" PRIx64 
+                           " Failed to find endpoint with subnet %" PRIx64
                            " and LID %d",
                            rem_info.rem_subnet_id,requested_lid));
                 mca_btl_openib_endpoint_invoke_error(NULL);
@@ -928,7 +937,7 @@ static void xoob_rml_recv_cb(int status, orte_process_name_t* process_name,
             /* update ib_addr with remote qp number */
             ib_endpoint->ib_addr->remote_xrc_rcv_qp_num =
                 ib_endpoint->rem_info.rem_qps->rem_qp_num;
-            BTL_VERBOSE(("rem_info: lid %d, sid %" PRIx64 
+            BTL_VERBOSE(("rem_info: lid %d, sid %" PRIx64
                          " ep %d %" PRIx64 "\n",
                          rem_info.rem_lid,
                          rem_info.rem_subnet_id,
@@ -991,13 +1000,13 @@ static void xoob_rml_recv_cb(int status, orte_process_name_t* process_name,
  */
 
 /* Quere for the XOOB priority - will be highest in XRC case */
-static int xoob_component_query(mca_btl_openib_module_t *openib_btl, 
+static int xoob_component_query(mca_btl_openib_module_t *openib_btl,
         ompi_btl_openib_connect_base_module_t **cpc)
 {
     int rc;
 
     if (mca_btl_openib_component.num_xrc_qps <= 0) {
-        opal_output_verbose(5, mca_btl_base_output,
+        opal_output_verbose(5, ompi_btl_base_framework.framework_output,
                             "openib BTL: xoob CPC only supported with XRC receive queues; skipped on %s:%d",
                             ibv_get_device_name(openib_btl->device->ib_dev),
                             openib_btl->port_num);
@@ -1006,7 +1015,7 @@ static int xoob_component_query(mca_btl_openib_module_t *openib_btl,
 
     *cpc = malloc(sizeof(ompi_btl_openib_connect_base_module_t));
     if (NULL == *cpc) {
-        opal_output_verbose(5, mca_btl_base_output,
+        opal_output_verbose(5, ompi_btl_base_framework.framework_output,
                             "openib BTL: xoob CPC system error (malloc failed)");
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
@@ -1015,20 +1024,26 @@ static int xoob_component_query(mca_btl_openib_module_t *openib_btl,
        ensure to only post it *once*, because another btl may have
        come in before this and already posted it. */
     if (!rml_recv_posted) {
-        rc = orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, 
+        rc = ompi_rte_recv_buffer_nb(OMPI_NAME_WILDCARD,
                                      OMPI_RML_TAG_XOPENIB,
-                                     ORTE_RML_PERSISTENT,
+                                     OMPI_RML_PERSISTENT,
                                      xoob_rml_recv_cb,
                                      NULL);
-        if (ORTE_SUCCESS != rc) {
-            opal_output_verbose(5, mca_btl_base_output,
+        if (OMPI_SUCCESS != rc) {
+            opal_output_verbose(5, ompi_btl_base_framework.framework_output,
                                 "openib BTL: xoob CPC system error %d (%s)",
                                 rc, opal_strerror(rc));
             return rc;
         }
         rml_recv_posted = true;
     }
-        
+
+    if (xoob_priority > 100) {
+        xoob_priority = 100;
+    } else if (xoob_priority < -1) {
+        xoob_priority = -1;
+    }
+
     (*cpc)->data.cbm_component = &ompi_btl_openib_connect_xoob;
     (*cpc)->data.cbm_priority = xoob_priority;
     (*cpc)->data.cbm_modex_message = NULL;
@@ -1040,7 +1055,7 @@ static int xoob_component_query(mca_btl_openib_module_t *openib_btl,
     (*cpc)->cbm_finalize = NULL;
     (*cpc)->cbm_uses_cts = false;
 
-    opal_output_verbose(5, mca_btl_base_output,
+    opal_output_verbose(5, ompi_btl_base_framework.framework_output,
                         "openib BTL: xoob CPC available for use on %s:%d",
                         ibv_get_device_name(openib_btl->device->ib_dev),
                         openib_btl->port_num);
@@ -1050,10 +1065,14 @@ static int xoob_component_query(mca_btl_openib_module_t *openib_btl,
 /* Open - this functions sets up any xoob specific commandline params */
 static void xoob_component_register(void)
 {
-    mca_base_param_reg_int(&mca_btl_openib_component.super.btl_version,
-                           "connect_xoob_priority",
-                           "The selection method priority for xoob",
-                           false, false, xoob_priority, &xoob_priority);
+    xoob_priority = 60;
+    (void) mca_base_component_var_register(&mca_btl_openib_component.super.btl_version,
+                                           "connect_xoob_priority",
+                                           "The selection method priority for xoob",
+                                           MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                           OPAL_INFO_LVL_9,
+                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           &xoob_priority);
 
     if (xoob_priority > 100) {
         xoob_priority = 100;
@@ -1128,7 +1147,7 @@ static int xoob_module_start_connect(ompi_btl_openib_connect_base_module_t *cpc,
 static int xoob_component_finalize(void)
 {
     if (rml_recv_posted) {
-        orte_rml.recv_cancel(ORTE_NAME_WILDCARD, OMPI_RML_TAG_XOPENIB);
+        ompi_rte_recv_cancel(OMPI_NAME_WILDCARD, OMPI_RML_TAG_XOPENIB);
         rml_recv_posted = false;
     }
 #if (ENABLE_DYNAMIC_SL)
