@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2011 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2009 The University of Tennessee and The University
@@ -10,9 +10,11 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Voltaire. All rights reserved.
- * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011      Los Alamos National Security, LLC.
+ * Copyright (c) 2009-2010 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
+ * Copyright (c) 2011      NVIDIA Corporation.  All rights reserved.
+ * Copyright (c) 2010-2012 IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -41,13 +43,10 @@
 #endif  /* HAVE_SYS_STAT_H */
 
 #include "ompi/constants.h"
-#include "opal/event/event.h"
+#include "opal/mca/event/event.h"
+#include "opal/util/bit_ops.h"
 #include "opal/util/output.h"
-#include "orte/util/proc_info.h"
-#include "orte/util/show_help.h"
-#include "orte/runtime/orte_globals.h"
 
-#include "opal/mca/base/mca_base_param.h"
 #include "ompi/mca/mpool/base/base.h"
 #include "ompi/mca/common/sm/common_sm.h"
 #include "ompi/mca/btl/base/btl_base_error.h"
@@ -60,6 +59,16 @@
 #include "btl_sm_frag.h"
 #include "btl_sm_fifo.h"
 
+static int mca_btl_sm_component_open(void);
+static int mca_btl_sm_component_close(void);
+static int sm_register(void);
+static mca_btl_base_module_t** mca_btl_sm_component_init(
+    int *num_btls,
+    bool enable_progress_threads,
+    bool enable_mpi_threads
+);
+
+
 /*
  * Shared Memory (SM) component instance.
  */
@@ -75,7 +84,9 @@ mca_btl_sm_component_t mca_btl_sm_component = {
             OMPI_MINOR_VERSION,  /* MCA component minor version */
             OMPI_RELEASE_VERSION,  /* MCA component release version */
             mca_btl_sm_component_open,  /* component open */
-            mca_btl_sm_component_close  /* component close */
+            mca_btl_sm_component_close,  /* component close */
+            NULL,
+            sm_register,
         },
         {
             /* The component is checkpoint ready */
@@ -92,112 +103,116 @@ mca_btl_sm_component_t mca_btl_sm_component = {
  * utility routines for parameter registration
  */
 
-static inline char* mca_btl_sm_param_register_string(
-    const char* param_name,
-    const char* default_value)
-{
-    char *param_value;
-    int id = mca_base_param_register_string("btl","sm",param_name,NULL,default_value);
-    mca_base_param_lookup_string(id, &param_value);
-    return param_value;
-}
-
 static inline int mca_btl_sm_param_register_int(
     const char* param_name,
-    int default_value)
+    int default_value,
+    int level,
+    int *storage)
 {
-    int id = mca_base_param_register_int("btl","sm",param_name,NULL,default_value);
-    int param_value = default_value;
-    mca_base_param_lookup_int(id,&param_value);
-    return param_value;
+    *storage = default_value;
+    (void) mca_base_component_var_register (&mca_btl_sm_component.super.btl_version,
+                                            param_name, NULL, MCA_BASE_VAR_TYPE_INT,
+                                            NULL, 0, 0, level,
+                                            MCA_BASE_VAR_SCOPE_READONLY, storage);
+    return *storage;
 }
 
-
-/*
- *  Called by MCA framework to open the component, registers
- *  component parameters.
- */
-
-int mca_btl_sm_component_open(void)
+static inline unsigned int mca_btl_sm_param_register_uint(
+    const char* param_name,
+    unsigned int default_value,
+    int level,
+    unsigned int *storage)
 {
-    int i;
+    *storage = default_value;
+    (void) mca_base_component_var_register (&mca_btl_sm_component.super.btl_version,
+                                            param_name, NULL, MCA_BASE_VAR_TYPE_UNSIGNED_INT,
+                                            NULL, 0, 0, level,
+                                            MCA_BASE_VAR_SCOPE_READONLY, storage);
+    return *storage;
+}
 
-    /* Register an MCA param to indicate whether we have knem support
-       or not */
-    mca_base_param_reg_int(&mca_btl_sm_component.super.btl_version,
-                           "have_knem_support", "Whether this component supports the knem Linux kernel module or not",
-                           false, true, OMPI_BTL_SM_HAVE_KNEM, NULL);
-
-    if (OMPI_BTL_SM_HAVE_KNEM) {
-        i = -1;
-    } else {
-        i = 0;
+static int mca_btl_sm_component_verify(void) {
+#if OMPI_BTL_SM_HAVE_KNEM || OMPI_BTL_SM_HAVE_CMA
+    if (mca_btl_sm_component.use_knem || mca_btl_sm_component.use_cma) {
+        mca_btl_sm.super.btl_flags |= MCA_BTL_FLAGS_GET;
     }
-    mca_base_param_reg_int(&mca_btl_sm_component.super.btl_version,
-                           "use_knem",
-                           "Whether knem support is desired or not "
-                           "(negative = try to enable knem support, but continue even if it is not available, 0 = do not enable knem support, positive = try to enable knem support and fail if it is not available)",
-                           false, false, i, &i);
+
+    if (mca_btl_sm_component.use_knem && mca_btl_sm_component.use_cma) {
+        /* Disable CMA if knem is runtime enabled */
+        opal_output(0, "CMA disabled because knem is enabled");
+        mca_btl_sm_component.use_cma = 0;
+    }
+
+#endif /* OMPI_BTL_SM_HAVE_KNEM || OMPI_BTL_SM_HAVE_CMA */
+
+    return mca_btl_base_param_verify(&mca_btl_sm.super);
+}
+
+static int sm_register(void)
+{
+    mca_base_var_flag_t var_flags;
+
     if (OMPI_BTL_SM_HAVE_KNEM) {
-        mca_btl_sm_component.use_knem = i;
+        var_flags = 0;
+        mca_btl_sm_component.use_knem = -1;
     } else {
-        if (i > 0) {
-            orte_show_help("help-mpi-btl-sm.txt",
-                           "knem requested but not supported", true,
-                           orte_process_info.nodename);
-            return OMPI_ERROR;
-        }
+        var_flags = MCA_BASE_VAR_FLAG_DEFAULT_ONLY;
         mca_btl_sm_component.use_knem = 0;
     }
+    (void) mca_base_component_var_register(&mca_btl_sm_component.super.btl_version,
+                                           "use_knem", "Whether knem support is desired or not "
+                                           "(negative = try to enable knem support, but continue "
+                                           "even if it is not available, 0 = do not enable knem "
+                                           "support, positive = try to enable knem support and "
+                                           "fail if it is not available)", MCA_BASE_VAR_TYPE_INT,
+                                           NULL, 0, var_flags, OPAL_INFO_LVL_9,
+                                           MCA_BASE_VAR_SCOPE_READONLY, &mca_btl_sm_component.use_knem);
+
     /* Currently disabling DMA mode by default; it's not clear that
        this is useful in all applications and architectures. */
-    mca_base_param_reg_int(&mca_btl_sm_component.super.btl_version,
-                           "knem_dma_min",
-                           "Minimum message size (in bytes) to use the knem DMA mode; ignored if knem does not support DMA mode (0 = do not use the knem DMA mode)",
-                           false, false, 0, &i);
-    mca_btl_sm_component.knem_dma_min = (uint32_t) i;
+    mca_btl_sm_component.knem_dma_min = 0;
+    (void) mca_base_component_var_register(&mca_btl_sm_component.super.btl_version,
+                                           "knem_dma_min",
+                                           "Minimum message size (in bytes) to use the knem DMA mode; "
+                                           "ignored if knem does not support DMA mode (0 = do not use the "
+                                           "knem DMA mode)", MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0,
+                                           0, OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                           &mca_btl_sm_component.knem_dma_min);
 
-    mca_base_param_reg_int(&mca_btl_sm_component.super.btl_version,
-                           "knem_max_simultaneous",
-                           "Max number of simultaneous ongoing knem operations to support (0 = do everything synchronously, which probably gives the best large message latency; >0 means to do all operations asynchronously, which supports better overlap for simultaneous large message sends)",
-                           false, false, 0,
-                           &mca_btl_sm_component.knem_max_simultaneous);
+    mca_btl_sm_component.knem_max_simultaneous = 0;
+    (void) mca_base_component_var_register(&mca_btl_sm_component.super.btl_version,
+                                           "knem_max_simultaneous",
+                                           "Max number of simultaneous ongoing knem operations to support "
+                                           "(0 = do everything synchronously, which probably gives the "
+                                           "best large message latency; >0 means to do all operations "
+                                           "asynchronously, which supports better overlap for simultaneous "
+                                           "large message sends)", MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0,
+                                           0, OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY, 
+                                           &mca_btl_sm_component.knem_max_simultaneous);
+
+    /* CMA parameters */
+    mca_btl_sm_component.use_cma = 0;
+    (void) mca_base_component_var_register(&mca_btl_sm_component.super.btl_version,
+                                           "use_cma", "Whether or not to enable CMA",
+                                           MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0, 0,
+                                           OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                           &mca_btl_sm_component.use_cma);
 
-    mca_btl_sm_component.sm_max_btls = 1;
     /* register SM component parameters */
-    mca_btl_sm_component.sm_free_list_num =
-        mca_btl_sm_param_register_int("free_list_num", 8);
-    mca_btl_sm_component.sm_free_list_max =
-        mca_btl_sm_param_register_int("free_list_max", -1);
-    mca_btl_sm_component.sm_free_list_inc =
-        mca_btl_sm_param_register_int("free_list_inc", 64);
-    mca_btl_sm_component.sm_max_procs =
-        mca_btl_sm_param_register_int("max_procs", -1);
-    mca_btl_sm_component.sm_mpool_name =
-        mca_btl_sm_param_register_string("mpool", "sm");
-    mca_btl_sm_component.fifo_size =
-        mca_btl_sm_param_register_int("fifo_size", 4096);
-    mca_btl_sm_component.nfifos =
-        mca_btl_sm_param_register_int("num_fifos", 1);
-    /* make sure the number of fifos is a power of 2 */
-    {
-        int i = 1;
-        while ( i < mca_btl_sm_component.nfifos )
-            i <<= 1;
-        mca_btl_sm_component.nfifos = i;
-    }
-    mca_btl_sm_component.fifo_lazy_free =
-        mca_btl_sm_param_register_int("fifo_lazy_free", 120);
+    mca_btl_sm_param_register_int("free_list_num", 8, OPAL_INFO_LVL_5, &mca_btl_sm_component.sm_free_list_num);
+    mca_btl_sm_param_register_int("free_list_max", -1, OPAL_INFO_LVL_5, &mca_btl_sm_component.sm_free_list_max);
+    mca_btl_sm_param_register_int("free_list_inc", 64, OPAL_INFO_LVL_5, &mca_btl_sm_component.sm_free_list_inc);
+    mca_btl_sm_param_register_int("max_procs", -1, OPAL_INFO_LVL_5, &mca_btl_sm_component.sm_max_procs);
+    /* there is no practical use for the mpool name parameter since mpool resources differ
+       between components */
+    mca_btl_sm_component.sm_mpool_name = "sm";
+    mca_btl_sm_param_register_uint("fifo_size", 4096, OPAL_INFO_LVL_4, &mca_btl_sm_component.fifo_size);
+    mca_btl_sm_param_register_int("num_fifos", 1, OPAL_INFO_LVL_4, &mca_btl_sm_component.nfifos);
 
-    /* make sure that queue size and lazy free parameter are compatible */
-    if (mca_btl_sm_component.fifo_lazy_free >= (mca_btl_sm_component.fifo_size >> 1) )
-        mca_btl_sm_component.fifo_lazy_free  = (mca_btl_sm_component.fifo_size >> 1);
-    if (mca_btl_sm_component.fifo_lazy_free <= 0)
-        mca_btl_sm_component.fifo_lazy_free  = 1;
+    mca_btl_sm_param_register_uint("fifo_lazy_free", 120, OPAL_INFO_LVL_5, &mca_btl_sm_component.fifo_lazy_free);
 
     /* default number of extra procs to allow for future growth */
-    mca_btl_sm_component.sm_extra_procs =
-        mca_btl_sm_param_register_int("sm_extra_procs", 0);
+    mca_btl_sm_param_register_int("sm_extra_procs", 0, OPAL_INFO_LVL_9, &mca_btl_sm_component.sm_extra_procs);
 
     mca_btl_sm.super.btl_exclusivity = MCA_BTL_EXCLUSIVITY_HIGH-1;
     mca_btl_sm.super.btl_eager_limit = 4*1024;
@@ -207,16 +222,39 @@ int mca_btl_sm_component_open(void)
     mca_btl_sm.super.btl_rdma_pipeline_frag_size = 64*1024;
     mca_btl_sm.super.btl_min_rdma_pipeline_size = 64*1024;
     mca_btl_sm.super.btl_flags = MCA_BTL_FLAGS_SEND;
-#if OMPI_BTL_SM_HAVE_KNEM
-    if (mca_btl_sm_component.use_knem) {
-        mca_btl_sm.super.btl_flags |= MCA_BTL_FLAGS_GET;
-    }
-#endif
+    mca_btl_sm.super.btl_seg_size = sizeof (mca_btl_sm_segment_t);
     mca_btl_sm.super.btl_bandwidth = 9000;  /* Mbs */
     mca_btl_sm.super.btl_latency   = 1;     /* Microsecs */
 
+    /* Call the BTL based to register its MCA params */
     mca_btl_base_param_register(&mca_btl_sm_component.super.btl_version,
                                 &mca_btl_sm.super);
+
+    return mca_btl_sm_component_verify();
+}
+
+/*
+ *  Called by MCA framework to open the component, registers
+ *  component parameters.
+ */
+
+static int mca_btl_sm_component_open(void)
+{
+    if (OMPI_SUCCESS != mca_btl_sm_component_verify()) {
+        return OMPI_ERROR;
+    }
+
+    mca_btl_sm_component.sm_max_btls = 1;
+
+    /* make sure the number of fifos is a power of 2 */
+    mca_btl_sm_component.nfifos = opal_next_poweroftwo_inclusive (mca_btl_sm_component.nfifos);
+
+    /* make sure that queue size and lazy free parameter are compatible */
+    if (mca_btl_sm_component.fifo_lazy_free >= (mca_btl_sm_component.fifo_size >> 1) )
+        mca_btl_sm_component.fifo_lazy_free  = (mca_btl_sm_component.fifo_size >> 1);
+    if (mca_btl_sm_component.fifo_lazy_free <= 0)
+        mca_btl_sm_component.fifo_lazy_free  = 1;
+
     mca_btl_sm_component.max_frag_size = mca_btl_sm.super.btl_max_send_size;
     mca_btl_sm_component.eager_limit = mca_btl_sm.super.btl_eager_limit;
 
@@ -234,7 +272,7 @@ int mca_btl_sm_component_open(void)
  * component cleanup - sanity checking of queue lengths
  */
 
-int mca_btl_sm_component_close(void)
+static int mca_btl_sm_component_close(void)
 {
     int return_value = OMPI_SUCCESS;
 
@@ -257,7 +295,7 @@ int mca_btl_sm_component_close(void)
     OBJ_DESTRUCT(&mca_btl_sm_component.sm_lock);
     /**
      * We don't have to destroy the fragment lists. They are allocated
-     * directly into the mmapped file, they will auto-magically dissapear
+     * directly into the mmapped file, they will auto-magically disappear
      * when the file get unmapped.
      */
     /*OBJ_DESTRUCT(&mca_btl_sm_component.sm_frags_eager);*/
@@ -290,7 +328,7 @@ int mca_btl_sm_component_close(void)
         OBJ_RELEASE(mca_btl_sm_component.sm_seg);
     }
 
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
     /* close/cleanup fifo create for event notification */
     if(mca_btl_sm_component.sm_fifo_fd > 0) {
         /* write a done message down the pipe */
@@ -306,10 +344,6 @@ int mca_btl_sm_component_close(void)
     }
 #endif
 
-    if (NULL != mca_btl_sm_component.sm_mpool_name) {
-        free(mca_btl_sm_component.sm_mpool_name);
-    }
-
 CLEANUP:
 
     /* return */
@@ -319,7 +353,7 @@ CLEANUP:
 /*
  *  SM component initialization
  */
-mca_btl_base_module_t** mca_btl_sm_component_init(
+static mca_btl_base_module_t** mca_btl_sm_component_init(
     int *num_btls,
     bool enable_progress_threads,
     bool enable_mpi_threads)
@@ -330,16 +364,23 @@ mca_btl_base_module_t** mca_btl_sm_component_init(
 #endif
 
     *num_btls = 0;
-
     /* lookup/create shared memory pool only when used */
     mca_btl_sm_component.sm_mpool = NULL;
     mca_btl_sm_component.sm_mpool_base = NULL;
 
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+    /* if no session directory was created, then we cannot be used */
+    if (NULL == ompi_process_info.job_session_dir) {
+    /* SKG - this isn't true anymore. Some backing facilities don't require a
+     * file-backed store. Extend shmem to provide this info one day. Especially
+     * when we use a proper modex for init. */
+        return NULL;
+    }
+
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
     /* create a named pipe to receive events  */
     sprintf( mca_btl_sm_component.sm_fifo_path,
-             "%s"OPAL_PATH_SEP"sm_fifo.%lu", orte_process_info.job_session_dir,
-             (unsigned long)ORTE_PROC_MY_NAME->vpid );
+             "%s"OPAL_PATH_SEP"sm_fifo.%lu", ompi_process_info.job_session_dir,
+             (unsigned long)OMPI_PROC_MY_NAME->vpid );
     if(mkfifo(mca_btl_sm_component.sm_fifo_path, 0660) < 0) {
         opal_output(0, "mca_btl_sm_component_init: mkfifo failed with errno=%d\n",errno);
         return NULL;
@@ -381,96 +422,106 @@ mca_btl_base_module_t** mca_btl_sm_component_init(
     mca_btl_sm.btl_inited = false;
 
 #if OMPI_BTL_SM_HAVE_KNEM
-    /* Set knem_status_num_used outside the check for use_knem so that
-       we can only have to check one thing (knem_status_num_used) in
-       the progress loop. */
-    mca_btl_sm.knem_fd = -1;
-    mca_btl_sm.knem_status_array = NULL;
-    mca_btl_sm.knem_frag_array = NULL;
-    mca_btl_sm.knem_status_num_used = 0;
-    mca_btl_sm.knem_status_first_avail = 0;
-    mca_btl_sm.knem_status_first_used = 0;
+    if (mca_btl_sm_component.use_knem) {
+        /* Set knem_status_num_used outside the check for use_knem so that
+           we can only have to check one thing (knem_status_num_used) in
+           the progress loop. */
+        mca_btl_sm.knem_fd = -1;
+        mca_btl_sm.knem_status_array = NULL;
+        mca_btl_sm.knem_frag_array = NULL;
+        mca_btl_sm.knem_status_num_used = 0;
+        mca_btl_sm.knem_status_first_avail = 0;
+        mca_btl_sm.knem_status_first_used = 0;
 
-    if (0 != mca_btl_sm_component.use_knem) {
-        /* Open the knem device.  Try to print a helpful message if we
-           fail to open it. */
-        mca_btl_sm.knem_fd = open("/dev/knem", O_RDWR);
-        if (mca_btl_sm.knem_fd < 0) {
-            if (EACCES == errno) {
-                struct stat sbuf;
-                if (0 != stat("/dev/knem", &sbuf)) {
-                    sbuf.st_mode = 0;
+        if (0 != mca_btl_sm_component.use_knem) {
+            /* Open the knem device.  Try to print a helpful message if we
+               fail to open it. */
+            mca_btl_sm.knem_fd = open("/dev/knem", O_RDWR);
+            if (mca_btl_sm.knem_fd < 0) {
+                if (EACCES == errno) {
+                    struct stat sbuf;
+                    if (0 != stat("/dev/knem", &sbuf)) {
+                        sbuf.st_mode = 0;
+                    }
+                    opal_show_help("help-mpi-btl-sm.txt", "knem permission denied",
+                                   true, ompi_process_info.nodename, sbuf.st_mode);
+                } else {
+                    opal_show_help("help-mpi-btl-sm.txt", "knem fail open",
+                                   true, ompi_process_info.nodename, errno,
+                                   strerror(errno));
                 }
-                orte_show_help("help-mpi-btl-sm.txt", "knem permission denied",
-                               true, orte_process_info.nodename, sbuf.st_mode);
-            } else {
-                orte_show_help("help-mpi-btl-sm.txt", "knem fail open",
-                               true, orte_process_info.nodename, errno,
-                               strerror(errno));
+                goto no_knem;
             }
-            goto no_knem;
-        }
 
-        /* Check that the ABI if the kernel module running is the same
-           as what we were compiled against */
-        rc = ioctl(mca_btl_sm.knem_fd, KNEM_CMD_GET_INFO, 
-                   &mca_btl_sm_component.knem_info);
-        if (rc < 0) {
-            orte_show_help("help-mpi-btl-sm.txt", "knem get ABI fail",
-                           true, orte_process_info.nodename, errno,
-                           strerror(errno));
-            goto no_knem;
-        }
-        if (KNEM_ABI_VERSION != mca_btl_sm_component.knem_info.abi) {
-            orte_show_help("help-mpi-btl-sm.txt", "knem ABI mismatch",
-                           true, orte_process_info.nodename, KNEM_ABI_VERSION,
-                           mca_btl_sm_component.knem_info.abi);
-            goto no_knem;
-        }
+            /* Check that the ABI if the kernel module running is the same
+               as what we were compiled against */
+            rc = ioctl(mca_btl_sm.knem_fd, KNEM_CMD_GET_INFO,
+                       &mca_btl_sm_component.knem_info);
+            if (rc < 0) {
+                opal_show_help("help-mpi-btl-sm.txt", "knem get ABI fail",
+                               true, ompi_process_info.nodename, errno,
+                               strerror(errno));
+                goto no_knem;
+            }
+            if (KNEM_ABI_VERSION != mca_btl_sm_component.knem_info.abi) {
+                opal_show_help("help-mpi-btl-sm.txt", "knem ABI mismatch",
+                               true, ompi_process_info.nodename, KNEM_ABI_VERSION,
+                               mca_btl_sm_component.knem_info.abi);
+                goto no_knem;
+            }
 
-        /* If we want DMA mode and DMA mode is supported, then set
-           knem_dma_flag to KNEM_FLAG_DMA. */
-        mca_btl_sm_component.knem_dma_flag = 0;
-        if (mca_btl_sm_component.knem_dma_min > 0 && 
-            (mca_btl_sm_component.knem_info.features & KNEM_FEATURE_DMA)) {
-            mca_btl_sm_component.knem_dma_flag = KNEM_FLAG_DMA;
-        }
+            /* If we want DMA mode and DMA mode is supported, then set
+               knem_dma_flag to KNEM_FLAG_DMA. */
+            mca_btl_sm_component.knem_dma_flag = 0;
+            if (mca_btl_sm_component.knem_dma_min > 0 &&
+                (mca_btl_sm_component.knem_info.features & KNEM_FEATURE_DMA)) {
+                mca_btl_sm_component.knem_dma_flag = KNEM_FLAG_DMA;
+            }
 
-        /* Get the array of statuses from knem if max_simultaneous > 0 */
+            /* Get the array of statuses from knem if max_simultaneous > 0 */
+            if (mca_btl_sm_component.knem_max_simultaneous > 0) {
+                mca_btl_sm.knem_status_array = mmap(NULL,
+                                                    mca_btl_sm_component.knem_max_simultaneous,
+                                                    (PROT_READ | PROT_WRITE),
+                                                    MAP_SHARED, mca_btl_sm.knem_fd,
+                                                    KNEM_STATUS_ARRAY_FILE_OFFSET);
+                if (MAP_FAILED == mca_btl_sm.knem_status_array) {
+                    opal_show_help("help-mpi-btl-sm.txt", "knem mmap fail",
+                                   true, ompi_process_info.nodename, errno,
+                                   strerror(errno));
+                    goto no_knem;
+                }
+
+                /* The first available status index is 0.  Make an empty frag
+                   array. */
+                mca_btl_sm.knem_frag_array = (mca_btl_sm_frag_t **)
+                    malloc(sizeof(mca_btl_sm_frag_t *) *
+                           mca_btl_sm_component.knem_max_simultaneous);
+                if (NULL == mca_btl_sm.knem_frag_array) {
+                    opal_show_help("help-mpi-btl-sm.txt", "knem init fail",
+                                   true, ompi_process_info.nodename, "malloc",
+                                   errno, strerror(errno));
+                    goto no_knem;
+                }
+            }
+        }
+        /* Set the BTL get function pointer if we're supporting KNEM;
+           choose between synchronous and asynchronous. */
         if (mca_btl_sm_component.knem_max_simultaneous > 0) {
-            mca_btl_sm.knem_status_array = mmap(NULL, 
-                                                mca_btl_sm_component.knem_max_simultaneous,
-                                                (PROT_READ | PROT_WRITE), 
-                                                MAP_SHARED, mca_btl_sm.knem_fd, 
-                                                KNEM_STATUS_ARRAY_FILE_OFFSET);
-            if (MAP_FAILED == mca_btl_sm.knem_status_array) {
-                orte_show_help("help-mpi-btl-sm.txt", "knem mmap fail",
-                               true, orte_process_info.nodename, errno,
-                               strerror(errno));
-                goto no_knem;
-            }
-
-            /* The first available status index is 0.  Make an empty frag
-               array. */
-            mca_btl_sm.knem_frag_array = (mca_btl_sm_frag_t **)
-                malloc(sizeof(mca_btl_sm_frag_t *) *
-                       mca_btl_sm_component.knem_max_simultaneous);
-            if (NULL == mca_btl_sm.knem_frag_array) {
-                orte_show_help("help-mpi-btl-sm.txt", "knem init fail",
-                               true, orte_process_info.nodename, "malloc",
-                               errno, strerror(errno));
-                goto no_knem;
-            }
+            mca_btl_sm.super.btl_get = mca_btl_sm_get_async;
+        } else {
+            mca_btl_sm.super.btl_get = mca_btl_sm_get_sync;
         }
     }
-    /* Set the BTL get function pointer if we're supporting KNEM;
-       choose between synchronous and asynchronous. */
-    if (mca_btl_sm_component.knem_max_simultaneous > 0) {
-        mca_btl_sm.super.btl_get = mca_btl_sm_get_async;
-    } else {
+#endif /* OMPI_BTL_SM_HAVE_KNEM */
+
+#if OMPI_BTL_SM_HAVE_CMA
+    if (mca_btl_sm_component.use_cma) {
+        /* Will only ever have either cma or knem enabled at runtime
+           so no problems with accidentally overwriting this set earlier */
         mca_btl_sm.super.btl_get = mca_btl_sm_get_sync;
     }
-#endif
+#endif /* OMPI_BTL_SM_HAVE_CMA */
 
     return btls;
 
@@ -510,7 +561,7 @@ mca_btl_base_module_t** mca_btl_sm_component_init(
  *  SM component progress.
  */
 
-#if OPAL_ENABLE_PROGRESS_THREADS == 1
+#if OMPI_ENABLE_PROGRESS_THREADS == 1
 void mca_btl_sm_component_event_thread(opal_object_t* thread)
 {
     while(1) {
@@ -559,12 +610,13 @@ void btl_sm_process_pending_sends(struct mca_btl_base_endpoint_t *ep)
 int mca_btl_sm_component_progress(void)
 {
     /* local variables */
+    mca_btl_base_segment_t seg;
     mca_btl_sm_frag_t *frag;
     mca_btl_sm_frag_t Frag;
     sm_fifo_t *fifo = NULL;
     mca_btl_sm_hdr_t *hdr;
     int my_smp_rank = mca_btl_sm_component.my_smp_rank;
-    int peer_smp_rank, j, rc = 0, nevents = 0;
+    int peer_smp_rank, j, rc, nevents = 0;
 
     /* first, deal with any pending sends */
     /* This check should be fast since we only need to check one variable. */
@@ -621,17 +673,17 @@ int mca_btl_sm_component_progress(void)
 #endif
                 /* recv upcall */
                 reg = mca_btl_base_active_message_trigger + hdr->tag;
-                Frag.segment.seg_addr.pval = ((char*)hdr) +
-                    sizeof(mca_btl_sm_hdr_t);
-                Frag.segment.seg_len = hdr->len;
+                seg.seg_addr.pval = ((char *)hdr) + sizeof(mca_btl_sm_hdr_t);
+                seg.seg_len = hdr->len;
                 Frag.base.des_dst_cnt = 1;
-                Frag.base.des_dst = &(Frag.segment);
+                Frag.base.des_dst = &seg;
                 reg->cbfunc(&mca_btl_sm.super, hdr->tag, &(Frag.base),
                             reg->cbdata);
                 /* return the fragment */
                 MCA_BTL_SM_FIFO_WRITE(
                         mca_btl_sm_component.sm_peers[peer_smp_rank],
                         my_smp_rank, peer_smp_rank, hdr->frag, false, true, rc);
+                (void)rc;  // silence compilier warning
                 break;
             }
         case MCA_BTL_SM_FRAG_ACK:
@@ -680,6 +732,7 @@ int mca_btl_sm_component_progress(void)
                 MCA_BTL_SM_FIFO_WRITE(
                         mca_btl_sm_component.sm_peers[peer_smp_rank],
                         my_smp_rank, peer_smp_rank, hdr, false, true, rc);
+                (void)rc;  // silence compilier warning
                 break;
         }
     }
