@@ -2,16 +2,18 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2012 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2011 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Laboratory
+ * Copyright (c) 2012-2013 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -48,16 +50,12 @@
 #include <ctype.h>
 #include <limits.h>
 
-#include "opal/event/event.h"
+#include "opal/mca/event/event.h"
 #include "opal/util/if.h"
 #include "opal/util/output.h"
 #include "opal/util/argv.h"
 #include "opal/util/net.h"
-#include "opal/mca/base/mca_base_param.h"
-
-#include "orte/types.h"
-#include "orte/util/show_help.h"
-#include "orte/mca/ess/ess.h"
+#include "opal/util/show_help.h"
 
 #include "ompi/constants.h"
 #include "ompi/mca/btl/btl.h"
@@ -79,6 +77,7 @@ static int mca_btl_tcp_component_register(void);
 static int mca_btl_tcp_component_open(void);
 static int mca_btl_tcp_component_close(void);
 
+static char *mca_btl_tcp_if_seq_string;
 
 mca_btl_tcp_component_t mca_btl_tcp_component = {
     {
@@ -114,25 +113,46 @@ mca_btl_tcp_component_t mca_btl_tcp_component = {
 static inline char* mca_btl_tcp_param_register_string(
         const char* param_name, 
         const char* help_string,
-        const char* default_value)
+        const char* default_value,
+        int level,
+        char **storage)
 {
-    char *value;
-    mca_base_param_reg_string(&mca_btl_tcp_component.super.btl_version,
-                              param_name, help_string, false, false,
-                              default_value, &value);
-    return value;
+    *storage = (char *) default_value;
+    (void) mca_base_component_var_register(&mca_btl_tcp_component.super.btl_version,
+                                           param_name, help_string, MCA_BASE_VAR_TYPE_STRING,
+                                           NULL, 0, 0, level,
+                                           MCA_BASE_VAR_SCOPE_READONLY, storage);
+    return *storage;
 }
 
 static inline int mca_btl_tcp_param_register_int(
         const char* param_name, 
         const char* help_string,
-        int default_value)
+        int default_value,
+        int level,
+        int *storage)
 {
-    int value;
-    mca_base_param_reg_int(&mca_btl_tcp_component.super.btl_version,
-                           param_name, help_string, false, false, 
-                           default_value, &value);
-    return value;
+    *storage = default_value;
+    (void) mca_base_component_var_register(&mca_btl_tcp_component.super.btl_version,
+                                           param_name, help_string, MCA_BASE_VAR_TYPE_INT,
+                                           NULL, 0, 0, level,
+                                           MCA_BASE_VAR_SCOPE_READONLY, storage);
+    return *storage;
+}
+
+static inline unsigned int mca_btl_tcp_param_register_uint(
+        const char* param_name, 
+        const char* help_string,
+        unsigned int default_value,
+        int level,
+        unsigned int *storage)
+{
+    *storage = default_value;
+    (void) mca_base_component_var_register(&mca_btl_tcp_component.super.btl_version,
+                                           param_name, help_string, MCA_BASE_VAR_TYPE_UNSIGNED_INT,
+                                           NULL, 0, 0, level,
+                                           MCA_BASE_VAR_SCOPE_READONLY, storage);
+    return *storage;
 }
 
 
@@ -173,6 +193,25 @@ OBJ_CLASS_INSTANCE(
 static void mca_btl_tcp_component_recv_handler(int, short, void*);
 static void mca_btl_tcp_component_accept_handler(int, short, void*);
 
+static int mca_btl_tcp_component_verify(void)
+{
+    if( mca_btl_tcp_component.tcp_port_min > USHRT_MAX ) {
+        opal_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
+                       true, "v4", ompi_process_info.nodename,
+                       mca_btl_tcp_component.tcp_port_min );
+        mca_btl_tcp_component.tcp_port_min = 1024;
+    }
+#if OPAL_WANT_IPV6
+    if( mca_btl_tcp_component.tcp6_port_min > USHRT_MAX ) {
+        opal_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
+                       true, "v6", ompi_process_info.nodename,
+                       mca_btl_tcp_component.tcp6_port_min );
+        mca_btl_tcp_component.tcp6_port_min = 1024;
+    }
+#endif
+
+    return OMPI_SUCCESS;
+}
 
 /*
  *  Called by MCA framework to open the component, registers
@@ -184,67 +223,48 @@ static int mca_btl_tcp_component_register(void)
     char* message;
 
     /* register TCP component parameters */
-    mca_btl_tcp_component.tcp_num_links =
-        mca_btl_tcp_param_register_int("links", NULL, 1);
-    mca_btl_tcp_component.tcp_if_include =
-        mca_btl_tcp_param_register_string("if_include", "Comma-delimited list of devices or CIDR notation of networks to use for MPI communication (e.g., \"eth0,eth1\" or \"192.168.0.0/16,10.1.4.0/24\").  Mutually exclusive with btl_tcp_if_exclude.", "");
-    mca_btl_tcp_component.tcp_if_exclude =
-        mca_btl_tcp_param_register_string("if_exclude", "Comma-delimited list of devices or CIDR notation of networks to NOT use for MPI communication -- all devices not matching these specifications will be used (e.g., \"eth0,eth1\" or \"192.168.0.0/16,10.1.4.0/24\").  Mutually exclusive with btl_tcp_if_include.", "lo,sppp");
+    mca_btl_tcp_param_register_uint("links", NULL, 1, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_num_links);
+    mca_btl_tcp_param_register_string("if_include", "Comma-delimited list of devices and/or CIDR notation of networks to use for MPI communication (e.g., \"eth0,192.168.0.0/16\").  Mutually exclusive with btl_tcp_if_exclude.", "", OPAL_INFO_LVL_1, &mca_btl_tcp_component.tcp_if_include);
+    mca_btl_tcp_param_register_string("if_exclude", "Comma-delimited list of devices and/or CIDR notation of networks to NOT use for MPI communication -- all devices not matching these specifications will be used (e.g., \"eth0,192.168.0.0/16\").  If set to a non-default value, it is mutually exclusive with btl_tcp_if_include.", 
+                                      "127.0.0.1/8,sppp",
+                                      OPAL_INFO_LVL_1, &mca_btl_tcp_component.tcp_if_exclude);
+
+    mca_btl_tcp_param_register_int ("free_list_num", NULL, 8, OPAL_INFO_LVL_5,  &mca_btl_tcp_component.tcp_free_list_num);
+    mca_btl_tcp_param_register_int ("free_list_max", NULL, -1, OPAL_INFO_LVL_5,  &mca_btl_tcp_component.tcp_free_list_max);
+    mca_btl_tcp_param_register_int ("free_list_inc", NULL, 32, OPAL_INFO_LVL_5,  &mca_btl_tcp_component.tcp_free_list_inc);
+    mca_btl_tcp_param_register_int ("sndbuf", NULL, 128*1024, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_sndbuf);
+    mca_btl_tcp_param_register_int ("rcvbuf", NULL, 128*1024, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_rcvbuf);
+    mca_btl_tcp_param_register_int ("endpoint_cache",
+        "The size of the internal cache for each TCP connection. This cache is"
+        " used to reduce the number of syscalls, by replacing them with memcpy."
+        " Every read will read the expected data plus the amount of the"
+                                    " endpoint_cache", 30*1024, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_endpoint_cache);
+    mca_btl_tcp_param_register_int ("use_nagle", "Whether to use Nagle's algorithm or not (using Nagle's algorithm may increase short message latency)", 0, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_not_use_nodelay);
+    mca_btl_tcp_param_register_int( "port_min_v4", 
+                                    "The minimum port where the TCP BTL will try to bind (default 1024)",
+                                    1024, OPAL_INFO_LVL_2, &mca_btl_tcp_component.tcp_port_min);
 
-    mca_btl_tcp_component.tcp_free_list_num =
-        mca_btl_tcp_param_register_int ("free_list_num", NULL, 8);
-    mca_btl_tcp_component.tcp_free_list_max =
-        mca_btl_tcp_param_register_int ("free_list_max", NULL, -1);
-    mca_btl_tcp_component.tcp_free_list_inc =
-        mca_btl_tcp_param_register_int ("free_list_inc", NULL, 32);
-    mca_btl_tcp_component.tcp_sndbuf =
-        mca_btl_tcp_param_register_int ("sndbuf", NULL, 128*1024);
-    mca_btl_tcp_component.tcp_rcvbuf =
-        mca_btl_tcp_param_register_int ("rcvbuf", NULL, 128*1024);
-    mca_btl_tcp_component.tcp_endpoint_cache =
-        mca_btl_tcp_param_register_int ("endpoint_cache",
-            "The size of the internal cache for each TCP connection. This cache is"
-            " used to reduce the number of syscalls, by replacing them with memcpy."
-            " Every read will read the expected data plus the amount of the"
-            " endpoint_cache", 30*1024);
-    mca_btl_tcp_component.tcp_use_nodelay =
-        !mca_btl_tcp_param_register_int ("use_nagle", "Whether to use Nagle's algorithm or not (using Nagle's algorithm may increase short message latency)", 0);
-    mca_btl_tcp_component.tcp_port_min =
-        mca_btl_tcp_param_register_int( "port_min_v4",
-            "The minimum port where the TCP BTL will try to bind (default 1024)", 1024 );
-    if( mca_btl_tcp_component.tcp_port_min > USHRT_MAX ) {
-        orte_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
-                       true, "v4", orte_process_info.nodename,
-                       mca_btl_tcp_component.tcp_port_min );
-        mca_btl_tcp_component.tcp_port_min = 1024;
-    }
     asprintf( &message, 
               "The number of ports where the TCP BTL will try to bind (default %d)."
               " This parameter together with the port min, define a range of ports"
               " where Open MPI will open sockets.",
               (0x1 << 16) - mca_btl_tcp_component.tcp_port_min - 1 );
-    mca_btl_tcp_component.tcp_port_range =
-        mca_btl_tcp_param_register_int( "port_range_v4", message,
-                                        (0x1 << 16) - mca_btl_tcp_component.tcp_port_min - 1);
+    mca_btl_tcp_param_register_int( "port_range_v4", message,
+                                    (0x1 << 16) - mca_btl_tcp_component.tcp_port_min - 1,
+                                    OPAL_INFO_LVL_2, &mca_btl_tcp_component.tcp_port_range);
     free(message);
 #if OPAL_WANT_IPV6
-    mca_btl_tcp_component.tcp6_port_min =
-        mca_btl_tcp_param_register_int( "port_min_v6",
-            "The minimum port where the TCP BTL will try to bind (default 1024)", 1024 );
-    if( mca_btl_tcp_component.tcp6_port_min > USHRT_MAX ) {
-        orte_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
-                       true, "v6", orte_process_info.nodename,
-                       mca_btl_tcp_component.tcp6_port_min );
-        mca_btl_tcp_component.tcp6_port_min = 1024;
-    }
+    mca_btl_tcp_param_register_int( "port_min_v6",
+                                    "The minimum port where the TCP BTL will try to bind (default 1024)", 1024,
+                                    OPAL_INFO_LVL_2, & mca_btl_tcp_component.tcp6_port_min );
     asprintf( &message, 
               "The number of ports where the TCP BTL will try to bind (default %d)."
               " This parameter together with the port min, define a range of ports"
               " where Open MPI will open sockets.",
               (0x1 << 16) - mca_btl_tcp_component.tcp6_port_min - 1 );
-    mca_btl_tcp_component.tcp6_port_range =
-        mca_btl_tcp_param_register_int( "port_range_v6", message,
-                                        (0x1 << 16) - mca_btl_tcp_component.tcp6_port_min - 1);
+    mca_btl_tcp_param_register_int( "port_range_v6", message,
+                                    (0x1 << 16) - mca_btl_tcp_component.tcp6_port_min - 1,
+                                    OPAL_INFO_LVL_2, &mca_btl_tcp_component.tcp6_port_range );
     free(message);
 #endif
 
@@ -260,28 +280,29 @@ static int mca_btl_tcp_component_register(void)
                                        MCA_BTL_FLAGS_NEED_CSUM |
                                        MCA_BTL_FLAGS_NEED_ACK |
                                        MCA_BTL_FLAGS_HETEROGENEOUS_RDMA;
+    mca_btl_tcp_module.super.btl_seg_size = sizeof (mca_btl_base_segment_t);
     mca_btl_tcp_module.super.btl_bandwidth = 100;
     mca_btl_tcp_module.super.btl_latency = 100;
 
     mca_btl_base_param_register(&mca_btl_tcp_component.super.btl_version,
                                 &mca_btl_tcp_module.super);
 
-    mca_btl_tcp_component.tcp_disable_family =
-        mca_btl_tcp_param_register_int ("disable_family", NULL, 0);
+    mca_btl_tcp_param_register_int ("disable_family", NULL, 0, OPAL_INFO_LVL_2,  &mca_btl_tcp_component.tcp_disable_family);
 
     /* Register a list of interfaces to use in sequence */
-    message = mca_btl_tcp_param_register_string("if_seq", 
-                                                "If specified, a comma-delimited list of TCP interfaces.  Interfaces will be assigned, one to each MPI process, in a round-robin fashion on each server.  For example, if the list is \"eth0,eth1\" and four MPI processes are run on a single server, then local ranks 0 and 2 will use eth0 and local ranks 1 and 3 will use eth1.", NULL);
+    mca_btl_tcp_param_register_string("if_seq", 
+                                      "If specified, a comma-delimited list of TCP interfaces.  Interfaces will be assigned, one to each MPI process, in a round-robin fashion on each server.  For example, if the list is \"eth0,eth1\" and four MPI processes are run on a single server, then local ranks 0 and 2 will use eth0 and local ranks 1 and 3 will use eth1.", NULL, OPAL_INFO_LVL_9, &mca_btl_tcp_if_seq_string);
+
     mca_btl_tcp_component.tcp_if_seq = NULL;
-    if (NULL != message && '\0' != *message) {
-        char **argv = opal_argv_split(message, ',');
+    if (NULL != mca_btl_tcp_if_seq_string && '\0' != *mca_btl_tcp_if_seq_string) {
+        char **argv = opal_argv_split(mca_btl_tcp_if_seq_string, ',');
 
         if (NULL != argv && '\0' != *(argv[0])) {
             int if_index, rc, count;
-            orte_node_rank_t node_rank;
+            ompi_node_rank_t node_rank;
             char name[256];
 
-            node_rank = orte_ess.get_node_rank(ORTE_PROC_MY_NAME);
+            node_rank = ompi_process_info.my_node_rank;
 
             /* Now that we've got that local rank, take the
                corresponding entry from the tcp_if_seq list (wrapping
@@ -303,31 +324,29 @@ static int mca_btl_tcp_component_register(void)
                 }
             }
             if (if_index < 0) {
-                orte_show_help("help-mpi-btl-tcp.txt", 
+                opal_show_help("help-mpi-btl-tcp.txt", 
                                "invalid if_inexclude",
                                true, "if_seq",
-                               orte_process_info.nodename,
+                               ompi_process_info.nodename,
                                mca_btl_tcp_component.tcp_if_seq,
                                "Interface does not exist");
-                return OMPI_ERR_BAD_PARAM;
+                free(mca_btl_tcp_component.tcp_if_seq);
+                mca_btl_tcp_component.tcp_if_seq = NULL;
+            } else {
+                BTL_VERBOSE(("Node rank %d using TCP interface %s",
+                             node_rank, mca_btl_tcp_component.tcp_if_seq));
             }
-            BTL_VERBOSE(("Node rank %d using TCP interface %s",
-                         node_rank, mca_btl_tcp_component.tcp_if_seq));
         }
     }
 
-    return OMPI_SUCCESS;
+    return mca_btl_tcp_component_verify();
 }
 
 static int mca_btl_tcp_component_open(void)
 {
-#ifdef __WINDOWS__
-    WSADATA win_sock_data;
-    if( WSAStartup(MAKEWORD(2,2), &win_sock_data) != 0 ) {
-        BTL_ERROR(("failed to initialise windows sockets:%d", WSAGetLastError()));
+    if (OMPI_SUCCESS != mca_btl_tcp_component_verify()) {
         return OMPI_ERROR;
     }
-#endif
 
     /* initialize state */
     mca_btl_tcp_component.tcp_listen_sd = -1;
@@ -349,7 +368,7 @@ static int mca_btl_tcp_component_open(void)
 
     /* if_include and if_exclude need to be mutually exclusive */
     if (OPAL_SUCCESS != 
-        mca_base_param_check_exclusive_string(
+        mca_base_var_check_exclusive("ompi",
         mca_btl_tcp_component.super.btl_version.mca_type_name,
         mca_btl_tcp_component.super.btl_version.mca_component_name,
         "if_include",
@@ -374,21 +393,13 @@ static int mca_btl_tcp_component_close(void)
     opal_list_item_t* item;
     opal_list_item_t* next;
 
-    if(NULL != mca_btl_tcp_component.tcp_if_include) {
-        free(mca_btl_tcp_component.tcp_if_include);
-        mca_btl_tcp_component.tcp_if_include = NULL;
-    }
-    if(NULL != mca_btl_tcp_component.tcp_if_exclude) {
-       free(mca_btl_tcp_component.tcp_if_exclude);
-       mca_btl_tcp_component.tcp_if_exclude = NULL;
-    }
     if (NULL != mca_btl_tcp_component.tcp_if_seq) {
         free(mca_btl_tcp_component.tcp_if_seq);
     }
 
     if (NULL != mca_btl_tcp_component.tcp_btls)
         free(mca_btl_tcp_component.tcp_btls);
- 
+  
     if (mca_btl_tcp_component.tcp_listen_sd >= 0) {
         opal_event_del(&mca_btl_tcp_component.tcp_recv_event);
         CLOSE_THE_SOCKET(mca_btl_tcp_component.tcp_listen_sd);
@@ -422,10 +433,6 @@ static int mca_btl_tcp_component_close(void)
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_frag_user);
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_lock);
 
-#ifdef __WINDOWS__
-    WSACleanup();
-#endif
-
     return OMPI_SUCCESS;
 }
 
@@ -458,11 +465,11 @@ static int mca_btl_tcp_create(int if_kindex, const char* if_name)
 
         /* allow user to specify interface bandwidth */
         sprintf(param, "bandwidth_%s", if_name);
-        btl->super.btl_bandwidth = mca_btl_tcp_param_register_int(param, NULL, btl->super.btl_bandwidth);
+        mca_btl_tcp_param_register_uint(param, NULL, btl->super.btl_bandwidth, OPAL_INFO_LVL_5, &btl->super.btl_bandwidth);
 
         /* allow user to override/specify latency ranking */
         sprintf(param, "latency_%s", if_name);
-        btl->super.btl_latency = mca_btl_tcp_param_register_int(param, NULL, btl->super.btl_latency);
+        mca_btl_tcp_param_register_uint(param, NULL, btl->super.btl_latency, OPAL_INFO_LVL_5,  &btl->super.btl_latency);
         if( i > 0 ) {
             btl->super.btl_bandwidth >>= 1;
             btl->super.btl_latency   <<= 1;
@@ -470,11 +477,11 @@ static int mca_btl_tcp_create(int if_kindex, const char* if_name)
 
         /* allow user to specify interface bandwidth */
         sprintf(param, "bandwidth_%s:%d", if_name, i);
-        btl->super.btl_bandwidth = mca_btl_tcp_param_register_int(param, NULL, btl->super.btl_bandwidth);
+        mca_btl_tcp_param_register_uint(param, NULL, btl->super.btl_bandwidth, OPAL_INFO_LVL_5, &btl->super.btl_bandwidth);
 
         /* allow user to override/specify latency ranking */
         sprintf(param, "latency_%s:%d", if_name, i);
-        btl->super.btl_latency = mca_btl_tcp_param_register_int(param, NULL, btl->super.btl_latency);
+        mca_btl_tcp_param_register_uint(param, NULL, btl->super.btl_latency, OPAL_INFO_LVL_5, &btl->super.btl_latency);
 #if 0 && OPAL_ENABLE_DEBUG
         BTL_OUTPUT(("interface %s instance %i: bandwidth %d latency %d\n", if_name, i,
                     btl->super.btl_bandwidth, btl->super.btl_latency));
@@ -517,8 +524,8 @@ static char **split_and_resolve(char **orig_str, char *name)
         tmp = strdup(argv[i]);
         str = strchr(argv[i], '/');
         if (NULL == str) {
-            orte_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
-                           true, name, orte_process_info.nodename, 
+            opal_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
+                           true, name, ompi_process_info.nodename, 
                            tmp, "Invalid specification (missing \"/\")");
             free(argv[i]);
             free(tmp);
@@ -534,13 +541,13 @@ static char **split_and_resolve(char **orig_str, char *name)
         free(argv[i]);
 
         if (1 != ret) {
-            orte_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
-                           true, name, orte_process_info.nodename, tmp,
+            opal_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
+                           true, name, ompi_process_info.nodename, tmp,
                            "Invalid specification (inet_pton() failed)");
             free(tmp);
             continue;
         }
-        opal_output_verbose(20, mca_btl_base_output, 
+        opal_output_verbose(20, ompi_btl_base_framework.framework_output, 
                             "btl: tcp: Searching for %s address+prefix: %s / %u",
                             name,
                             opal_net_get_hostname((struct sockaddr*) &argv_inaddr),
@@ -561,8 +568,8 @@ static char **split_and_resolve(char **orig_str, char *name)
         
         /* If we didn't find a match, keep trying */
         if (if_index < 0) {
-            orte_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
-                           true, name, orte_process_info.nodename, tmp,
+            opal_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
+                           true, name, ompi_process_info.nodename, tmp,
                            "Did not find interface matching this subnet");
             free(tmp);
             continue;
@@ -571,7 +578,7 @@ static char **split_and_resolve(char **orig_str, char *name)
         /* We found a match; get the name and replace it in the
            argv */
         opal_ifindextoname(if_index, if_name, sizeof(if_name));
-        opal_output_verbose(20, mca_btl_base_output, 
+        opal_output_verbose(20, ompi_btl_base_framework.framework_output, 
                             "btl: tcp: Found match: %s (%s)",
                             opal_net_get_hostname((struct sockaddr*) &if_inaddr),
                             if_name);
@@ -705,7 +712,7 @@ static int mca_btl_tcp_component_create_instances(void)
             /* check to see if this interface exists in the exclude list */
             argv = exclude;
             while(argv && *argv) {
-                if(strcmp(*argv,if_name) == 0)
+                if(strncmp(*argv,if_name,strlen(*argv)) == 0)
                     break;
                 argv++;
             }
@@ -888,7 +895,7 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
 
     /* register listen port */
     if (AF_INET == af_family) {
-        opal_event_set( &mca_btl_tcp_component.tcp_recv_event,
+        opal_event_set(opal_event_base, &mca_btl_tcp_component.tcp_recv_event,
                         mca_btl_tcp_component.tcp_listen_sd,
                         OPAL_EV_READ|OPAL_EV_PERSIST,
                         mca_btl_tcp_component_accept_handler,
@@ -897,7 +904,7 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
     }
 #if OPAL_WANT_IPV6
     if (AF_INET6 == af_family) {
-        opal_event_set( &mca_btl_tcp_component.tcp6_recv_event,
+        opal_event_set(opal_event_base, &mca_btl_tcp_component.tcp6_recv_event,
                         mca_btl_tcp_component.tcp6_listen_sd,
                         OPAL_EV_READ|OPAL_EV_PERSIST,
                         mca_btl_tcp_component_accept_handler,
@@ -1054,7 +1061,8 @@ mca_btl_base_module_t** mca_btl_tcp_component_init(int *num_btl_modules,
     }
 #if OPAL_WANT_IPV6
     if((ret = mca_btl_tcp_component_create_listen(AF_INET6)) != OMPI_SUCCESS) {
-        if (!(OMPI_ERR_IN_ERRNO == ret && EAFNOSUPPORT == opal_socket_errno)) {
+        if (!(OMPI_ERR_IN_ERRNO == ret &&
+              EAFNOSUPPORT == opal_socket_errno)) {
             opal_output (0, "mca_btl_tcp_component: IPv6 listening socket failed\n");
             return 0;
         }
@@ -1077,15 +1085,6 @@ mca_btl_base_module_t** mca_btl_tcp_component_init(int *num_btl_modules,
     return btls;
 }
 
-/*
- *  TCP module control
- */
-
-int mca_btl_tcp_component_control(int param, void* value, size_t size)
-{
-    return OMPI_SUCCESS;
-}
-
 
 /**
  * Called by the event engine when the listening socket has
@@ -1119,7 +1118,7 @@ static void mca_btl_tcp_component_accept_handler( int incoming_sd,
         /* wait for receipt of peers process identifier to complete this connection */
          
         event = OBJ_NEW(mca_btl_tcp_event_t);
-        opal_event_set(&event->event, sd, OPAL_EV_READ, mca_btl_tcp_component_recv_handler, event);
+        opal_event_set(opal_event_base, &event->event, sd, OPAL_EV_READ, mca_btl_tcp_component_recv_handler, event);
         opal_event_add(&event->event, 0);
     }
 }
@@ -1133,7 +1132,7 @@ static void mca_btl_tcp_component_accept_handler( int incoming_sd,
  */
 static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
 {
-    orte_process_name_t guid;
+    ompi_process_name_t guid;
     struct sockaddr_storage addr;
     int retval;
     mca_btl_tcp_proc_t* btl_proc;
@@ -1148,7 +1147,7 @@ static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
         CLOSE_THE_SOCKET(sd);
         return;
     }
-    ORTE_PROCESS_NAME_NTOH(guid);
+    OMPI_PROCESS_NAME_NTOH(guid);
 
     /* now set socket up to be non-blocking */
     if((flags = fcntl(sd, F_GETFL, 0)) < 0) {
