@@ -10,6 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2010 Oracle and/or its affiliates.  All rights reserved
+ * Copyright (c) 2013      Intel, Inc. All rights reserved
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -19,12 +20,12 @@
 
 #include "ompi_config.h"
 
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
 
 #include "opal/class/opal_hash_table.h"
 #include "ompi/mca/btl/base/btl_base_error.h"
@@ -76,7 +77,7 @@ void mca_btl_tcp_proc_destruct(mca_btl_tcp_proc_t* tcp_proc)
     /* remove from list of all proc instances */
     OPAL_THREAD_LOCK(&mca_btl_tcp_component.tcp_lock);
     opal_hash_table_remove_value_uint64(&mca_btl_tcp_component.tcp_procs, 
-                                        orte_util_hash_name(&tcp_proc->proc_ompi->proc_name));
+                                        ompi_rte_hash_name(&tcp_proc->proc_ompi->proc_name));
     OPAL_THREAD_UNLOCK(&mca_btl_tcp_component.tcp_lock);
 
     /* release resources */
@@ -99,7 +100,7 @@ mca_btl_tcp_proc_t* mca_btl_tcp_proc_create(ompi_proc_t* ompi_proc)
     int rc;
     size_t size;
     mca_btl_tcp_proc_t* btl_proc;
-    uint64_t hash = orte_util_hash_name(&ompi_proc->proc_name);
+    uint64_t hash = ompi_rte_hash_name(&ompi_proc->proc_name);
 
     OPAL_THREAD_LOCK(&mca_btl_tcp_component.tcp_lock);
     rc = opal_hash_table_get_value_uint64(&mca_btl_tcp_component.tcp_procs, 
@@ -218,15 +219,15 @@ static void visit(int k, int level, int siz, int *a)
 }
 
 
-static void mca_btl_tcp_initialise_interface(mca_btl_tcp_interface_t* interface,
+static void mca_btl_tcp_initialise_interface(mca_btl_tcp_interface_t* tcp_interface,
         int ifk_index, int index)
 {
-    interface->kernel_index = ifk_index;
-    interface->peer_interface = -1;
-    interface->ipv4_address = NULL;
-    interface->ipv6_address =  NULL;
-    interface->index = index;
-    interface->inuse = 0;
+    tcp_interface->kernel_index = ifk_index;
+    tcp_interface->peer_interface = -1;
+    tcp_interface->ipv4_address = NULL;
+    tcp_interface->ipv6_address =  NULL;
+    tcp_interface->index = index;
+    tcp_interface->inuse = 0;
 }
 
 static mca_btl_tcp_interface_t** mca_btl_tcp_retrieve_local_interfaces(void)
@@ -372,10 +373,15 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
                              mca_btl_base_endpoint_t* btl_endpoint )
 {
     struct sockaddr_storage endpoint_addr_ss;
+    const char *proc_hostname;
     unsigned int perm_size;
     int rc, *a = NULL;
     size_t i, j;
 
+    if (NULL == (proc_hostname = btl_proc->proc_ompi->proc_hostname)) {
+        return OMPI_ERR_UNREACH;
+    }
+
 #ifndef WORDS_BIGENDIAN
     /* if we are little endian and our peer is not so lucky, then we
        need to put all information sent to him in big endian (aka
@@ -504,10 +510,20 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
             if(NULL != local_interfaces[i]->ipv4_address &&
                NULL != peer_interfaces[j]->ipv4_address) {
 
-                /*  check for RFC1918 */
-                if(opal_net_addr_isipv4public((struct sockaddr*) local_interfaces[i]->ipv4_address)
-                   && opal_net_addr_isipv4public((struct sockaddr*) 
-                                                 peer_interfaces[j]->ipv4_address)) {
+                /*  check for loopback */
+                if ((opal_net_islocalhost((struct sockaddr *)local_interfaces[i]->ipv4_address)
+                     && !opal_net_islocalhost((struct sockaddr *)peer_interfaces[j]->ipv4_address))
+                    || (opal_net_islocalhost((struct sockaddr *)peer_interfaces[j]->ipv4_address)
+                        && !opal_net_islocalhost((struct sockaddr *)local_interfaces[i]->ipv4_address))
+                    || (opal_net_islocalhost((struct sockaddr *)local_interfaces[i]->ipv4_address)
+                        && !opal_ifislocal(proc_hostname))) {
+
+                    /* No connection is possible on these interfaces */
+                
+                    /*  check for RFC1918 */
+                } else if(opal_net_addr_isipv4public((struct sockaddr*) local_interfaces[i]->ipv4_address)
+                          && opal_net_addr_isipv4public((struct sockaddr*) 
+                                                        peer_interfaces[j]->ipv4_address)) {
                     if(opal_net_samenetwork((struct sockaddr*) local_interfaces[i]->ipv4_address,
                                             (struct sockaddr*) peer_interfaces[j]->ipv4_address,
                                             local_interfaces[i]->ipv4_netmask)) {
@@ -534,9 +550,20 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
              */
             if(NULL != local_interfaces[i]->ipv6_address &&
                NULL != peer_interfaces[j]->ipv6_address) {
-                if(opal_net_samenetwork((struct sockaddr*) local_interfaces[i]->ipv6_address,
-                                        (struct sockaddr*) peer_interfaces[j]->ipv6_address,
-                                        local_interfaces[i]->ipv6_netmask)) {
+
+                /*  check for loopback */
+                if ((opal_net_islocalhost((struct sockaddr *)local_interfaces[i]->ipv6_address)
+                     && !opal_net_islocalhost((struct sockaddr *)peer_interfaces[j]->ipv6_address))
+                    || (opal_net_islocalhost((struct sockaddr *)peer_interfaces[j]->ipv6_address)
+                        && !opal_net_islocalhost((struct sockaddr *)local_interfaces[i]->ipv6_address))
+                    || (opal_net_islocalhost((struct sockaddr *)local_interfaces[i]->ipv6_address)
+                        && !opal_ifislocal(proc_hostname))) {
+
+                    /* No connection is possible on these interfaces */
+
+                } else if(opal_net_samenetwork((struct sockaddr*) local_interfaces[i]->ipv6_address,
+                                               (struct sockaddr*) peer_interfaces[j]->ipv6_address,
+                                               local_interfaces[i]->ipv6_netmask)) {
                     weights[i][j] = CQ_PUBLIC_SAME_NETWORK;
                 } else {
                     weights[i][j] = CQ_PUBLIC_DIFFERENT_NETWORK;
@@ -561,52 +588,54 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
 
     /* Can only find the best set of connections when the number of
      * interfaces is not too big.  When it gets larger, we fall back
-     * to a simpler and faster (and not as optimal) algorithm. */
+     * to a simpler and faster (and not as optimal) algorithm. 
+     * See ticket https://svn.open-mpi.org/trac/ompi/ticket/2031 
+     * for more details about this issue.  */
     if (perm_size <= MAX_PERMUTATION_INTERFACES) {
-       memset(a, 0, perm_size * sizeof(int));
-       max_assignment_cardinality = -1;
-       max_assignment_weight = -1;
-       visit(0, -1, perm_size, a);
+        memset(a, 0, perm_size * sizeof(int));
+        max_assignment_cardinality = -1;
+        max_assignment_weight = -1;
+        visit(0, -1, perm_size, a);
 
-       rc = OMPI_ERR_UNREACH;
-       for(i = 0; i < perm_size; ++i) {
-	   if(best_assignment[i] > num_peer_interfaces
-	      || weights[i][best_assignment[i]] == CQ_NO_CONNECTION
-	      || peer_interfaces[best_assignment[i]]->inuse 
-	      || NULL == peer_interfaces[best_assignment[i]]) {
-	       continue;
-	   } 
-	   peer_interfaces[best_assignment[i]]->inuse++;
-	   btl_endpoint->endpoint_addr = best_addr[i][best_assignment[i]];
-	   btl_endpoint->endpoint_addr->addr_inuse++;
-	   rc = OMPI_SUCCESS;
-	   break;
-       }
+        rc = OMPI_ERR_UNREACH;
+        for(i = 0; i < perm_size; ++i) {
+            if(best_assignment[i] > num_peer_interfaces
+               || weights[i][best_assignment[i]] == CQ_NO_CONNECTION
+               || peer_interfaces[best_assignment[i]]->inuse 
+               || NULL == peer_interfaces[best_assignment[i]]) {
+                continue;
+            } 
+            peer_interfaces[best_assignment[i]]->inuse++;
+            btl_endpoint->endpoint_addr = best_addr[i][best_assignment[i]];
+            btl_endpoint->endpoint_addr->addr_inuse++;
+            rc = OMPI_SUCCESS;
+            break;
+        }
     } else {
-	enum mca_btl_tcp_connection_quality max;
-	int i_max = 0, j_max = 0;
-	/* Find the best connection that is not in use.  Save away
-	 * the indices of the best location. */
-	max = CQ_NO_CONNECTION;
-	for(i=0; i<num_local_interfaces; ++i) {
-	    for(j=0; j<num_peer_interfaces; ++j) {
-		if (!peer_interfaces[j]->inuse) {
-		    if (weights[i][j] > max) {
-			max = weights[i][j];
-			i_max = i;
-			j_max = j;
-		    }
-		}
-	    }
-	}
-	/* Now see if there is a some type of connection available. */
-	rc = OMPI_ERR_UNREACH;
-	if (CQ_NO_CONNECTION != max) {
-	    peer_interfaces[j_max]->inuse++;
-	    btl_endpoint->endpoint_addr = best_addr[i_max][j_max];
-	    btl_endpoint->endpoint_addr->addr_inuse++;
-	    rc = OMPI_SUCCESS;
-	}
+        enum mca_btl_tcp_connection_quality max;
+        int i_max = 0, j_max = 0;
+        /* Find the best connection that is not in use.  Save away
+         * the indices of the best location. */
+        max = CQ_NO_CONNECTION;
+        for(i=0; i<num_local_interfaces; ++i) {
+            for(j=0; j<num_peer_interfaces; ++j) {
+                if (!peer_interfaces[j]->inuse) {
+                    if (weights[i][j] > max) {
+                        max = weights[i][j];
+                        i_max = i;
+                        j_max = j;
+                    }
+                }
+            }
+        }
+        /* Now see if there is a some type of connection available. */
+        rc = OMPI_ERR_UNREACH;
+        if (CQ_NO_CONNECTION != max) {
+            peer_interfaces[j_max]->inuse++;
+            btl_endpoint->endpoint_addr = best_addr[i_max][j_max];
+            btl_endpoint->endpoint_addr->addr_inuse++;
+            rc = OMPI_SUCCESS;
+        }
     }
 
     for(i = 0; i < perm_size; ++i) {
@@ -683,12 +712,12 @@ int mca_btl_tcp_proc_remove(mca_btl_tcp_proc_t* btl_proc, mca_btl_base_endpoint_
  * Look for an existing TCP process instance based on the globally unique
  * process identifier.
  */
-mca_btl_tcp_proc_t* mca_btl_tcp_proc_lookup(const orte_process_name_t *name)
+mca_btl_tcp_proc_t* mca_btl_tcp_proc_lookup(const ompi_process_name_t *name)
 {
     mca_btl_tcp_proc_t* proc = NULL;
     OPAL_THREAD_LOCK(&mca_btl_tcp_component.tcp_lock);
     opal_hash_table_get_value_uint64(&mca_btl_tcp_component.tcp_procs, 
-                                     orte_util_hash_name(name), (void**)&proc);
+                                     ompi_rte_hash_name(name), (void**)&proc);
     OPAL_THREAD_UNLOCK(&mca_btl_tcp_component.tcp_lock);
     return proc;
 }
