@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2007 The University of Tennessee and The University
+ * Copyright (c) 2004-2009 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -21,17 +21,16 @@
 
 #include "mpi.h"
 #include "ompi/constants.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/communicator/communicator.h"
-#include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/coll_tags.h"
 #include "ompi/mca/pml/pml.h"
 #include "coll_tuned_util.h"
 
-int ompi_coll_tuned_sendrecv_actual( void* sendbuf, int scount, 
+int ompi_coll_tuned_sendrecv_actual( void* sendbuf, size_t scount, 
                                      ompi_datatype_t* sdatatype,
                                      int dest, int stag,
-                                     void* recvbuf, int rcount, 
+                                     void* recvbuf, size_t rcount, 
                                      ompi_datatype_t* rdatatype,
                                      int source, int rtag,
                                      struct ompi_communicator_t* comm,
@@ -55,15 +54,39 @@ int ompi_coll_tuned_sendrecv_actual( void* sendbuf, int scount,
     err = ompi_request_wait_all( 2, reqs, statuses );
     if (err != MPI_SUCCESS) { line = __LINE__; goto error_handler; }
 
-    if (MPI_STATUS_IGNORE!=status) {
+    if (MPI_STATUS_IGNORE != status) {
         *status = statuses[0];
     }
     
     return (MPI_SUCCESS);
 
  error_handler:
-    OPAL_OUTPUT ((ompi_coll_tuned_stream, "%s:%d: Error %d occurred\n",
-                  __FILE__,line,err));
+    /* As we use wait_all we will get MPI_ERR_IN_STATUS which is not an error
+     * code that we can propagate up the stack. Instead, look for the real
+     * error code from the MPI_ERROR in the status.
+     */
+    if( MPI_ERR_IN_STATUS == err ) {
+        /* At least we know the error was detected during the wait_all */
+        int err_index = 0;
+        if( MPI_SUCCESS == statuses[0].MPI_ERROR ) {
+            err_index = 1;
+        }
+        if (MPI_STATUS_IGNORE != status) {
+            *status = statuses[err_index];
+        }
+        err = statuses[err_index].MPI_ERROR;
+        OPAL_OUTPUT ((ompi_coll_tuned_stream, "%s:%d: Error %d occurred (req index %d)\n",
+                      __FILE__, line, err, err_index));
+    } else {
+        /* Error discovered during the posting of the irecv or isend,
+         * and no status is available.
+         */
+        OPAL_OUTPUT ((ompi_coll_tuned_stream, "%s:%d: Error %d occurred\n",
+                      __FILE__, line, err));
+        if (MPI_STATUS_IGNORE != status) {
+            status->MPI_ERROR = err;
+        }
+    }
     return (err);
 }
 
@@ -73,10 +96,10 @@ int ompi_coll_tuned_sendrecv_actual( void* sendbuf, int scount,
  * version when available
  */
 
-int ompi_coll_tuned_sendrecv_actual_localcompleted( void* sendbuf, int scount, 
+int ompi_coll_tuned_sendrecv_actual_localcompleted( void* sendbuf, size_t scount, 
                                                     ompi_datatype_t* sdatatype, 
                                                     int dest, int stag,
-                                                    void* recvbuf, int rcount, 
+                                                    void* recvbuf, size_t rcount, 
                                                     ompi_datatype_t* rdatatype, 
                                                     int source, int rtag, 
                                                     struct ompi_communicator_t* comm, 
@@ -85,7 +108,7 @@ int ompi_coll_tuned_sendrecv_actual_localcompleted( void* sendbuf, int scount,
 { /* post receive first, then [local] sync send, then wait... should be fast (I hope) */
     int err, line = 0;
     ompi_request_t* req[2];
-    ompi_status_public_t tmpstatus[2];
+    ompi_status_public_t statuses[2];
 
     /* post new irecv */
     err = MCA_PML_CALL(irecv( recvbuf, rcount, rdatatype, source, rtag, 
@@ -97,17 +120,43 @@ int ompi_coll_tuned_sendrecv_actual_localcompleted( void* sendbuf, int scount,
                               MCA_PML_BASE_SEND_SYNCHRONOUS, comm, &(req[1])));
     if (err != MPI_SUCCESS) { line = __LINE__; goto error_handler; }
 
-    err = ompi_request_wait_all( 2, req, tmpstatus );
+    err = ompi_request_wait_all( 2, req, statuses );
     if (err != MPI_SUCCESS) { line = __LINE__; goto error_handler; }
 
-    if (MPI_STATUS_IGNORE!=status) {
-        *status = tmpstatus[0];
+    if (MPI_STATUS_IGNORE != status) {
+        *status = statuses[0];
     }
 
     return (MPI_SUCCESS);
 
  error_handler:
-    OPAL_OUTPUT ((ompi_coll_tuned_stream, "%s:%d: Error %d occurred\n",__FILE__,line,err));
+    /* As we use wait_all we will get MPI_ERR_IN_STATUS which is not an error
+     * code that we can propagate up the stack. Instead, look for the real
+     * error code from the MPI_ERROR in the status.
+     */
+    if( MPI_ERR_IN_STATUS == err ) { 
+        /* At least we know the error was detected during the wait_all */
+        int err_index = 0;
+        if( MPI_SUCCESS == statuses[0].MPI_ERROR ) {
+            err_index = 1;
+        }
+        if (MPI_STATUS_IGNORE != status) {
+            *status = statuses[err_index];
+        }
+        err = statuses[err_index].MPI_ERROR;
+        OPAL_OUTPUT ((ompi_coll_tuned_stream, "%s:%d: Error %d occurred (req index %d)\n",
+                      __FILE__,line,err, err_index));
+    } else {
+        /* Error discovered during the posting of the irecv or isend,
+         * and no status is available.
+         */
+        OPAL_OUTPUT ((ompi_coll_tuned_stream, "%s:%d: Error %d occurred\n",
+                      __FILE__, line, err));
+        if (MPI_STATUS_IGNORE != status) {
+            status->MPI_ERROR = err;
+        }
+    }
+
     return (err);
 }
 
