@@ -12,6 +12,7 @@
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Mellanox Technologies. All rights reserved.
  * Copyright (c) 2008-2009 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2012      Los Alamos National Security, LLC.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -19,7 +20,7 @@
  * $HEADER$
  */
 
-#define OMPI_DISABLE_ENABLE_MEM_DEBUG 1
+#define OPAL_DISABLE_ENABLE_MEM_DEBUG 1
 #include "ompi_config.h"
 
 #include <stdio.h>
@@ -30,25 +31,23 @@
 
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
-#include "opal/util/output.h"
-#include "orte/util/show_help.h"
-#include "orte/util/name_fns.h"
-#include "orte/util/proc_info.h"
-#include "orte/runtime/orte_globals.h"
-#include "ompi/mca/mpool/mpool.h"
+#include "opal/util/show_help.h"
+
+#include "ompi/mca/rte/rte.h"
 #include "ompi/runtime/params.h"
+
+#include "ompi/mca/mpool/mpool.h"
 #include "ompi/mca/mpool/base/base.h"
+#include "opal/memoryhooks/memory.h"
 #include "mpool_base_mem_cb.h"
 
 
-extern opal_pointer_array_t mca_mpool_base_mem_cb_array; 
-
 mca_mpool_base_component_t* mca_mpool_base_component_lookup(const char* name)
 {
     /* Traverse the list of available modules; call their init functions. */
     opal_list_item_t* item;
-    for (item = opal_list_get_first(&mca_mpool_base_components);
-         item != opal_list_get_end(&mca_mpool_base_components);
+    for (item = opal_list_get_first(&ompi_mpool_base_framework.framework_components);
+         item != opal_list_get_end(&ompi_mpool_base_framework.framework_components);
          item = opal_list_get_next(item)) {
          mca_base_component_list_item_t *cli = 
            (mca_base_component_list_item_t *) item;
@@ -72,8 +71,8 @@ mca_mpool_base_module_t* mca_mpool_base_module_create(
     opal_list_item_t* item;
     mca_mpool_base_selected_module_t *sm;
 
-    for (item = opal_list_get_first(&mca_mpool_base_components);
-         item != opal_list_get_end(&mca_mpool_base_components);
+    for (item = opal_list_get_first(&ompi_mpool_base_framework.framework_components);
+         item != opal_list_get_end(&ompi_mpool_base_framework.framework_components);
          item = opal_list_get_next(item)) {
          mca_base_component_list_item_t *cli = 
            (mca_base_component_list_item_t *) item;
@@ -84,7 +83,7 @@ mca_mpool_base_module_t* mca_mpool_base_module_create(
          }
     }
 
-    if (opal_list_get_end(&mca_mpool_base_components) == item) {
+    if (opal_list_get_end(&ompi_mpool_base_framework.framework_components) == item) {
         return NULL;
     }
     module = component->mpool_init(resources); 
@@ -108,7 +107,7 @@ mca_mpool_base_module_t* mca_mpool_base_module_create(
            leave_pinned variables may have been set by a user MCA
            param or elsewhere in the code base).  Yes, we could have
            coded this more succinctly, but this is more clear. */
-        if (ompi_mpi_leave_pinned || ompi_mpi_leave_pinned_pipeline) {
+        if (ompi_mpi_leave_pinned > 0 || ompi_mpi_leave_pinned_pipeline) {
             use_mem_hooks = 1;
         }
 
@@ -117,11 +116,10 @@ mca_mpool_base_module_t* mca_mpool_base_module_create(
                 ((OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT) & 
                  opal_mem_hooks_support_level())) {
                 opal_mem_hooks_register_release(mca_mpool_base_mem_cb, NULL);
-                OBJ_CONSTRUCT(&mca_mpool_base_mem_cb_array, opal_pointer_array_t);
             } else {
-                orte_show_help("help-mpool-base.txt", "leave pinned failed",
-                               true, name, ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                               orte_process_info.nodename);
+                opal_show_help("help-mpool-base.txt", "leave pinned failed",
+                               true, name, OMPI_NAME_PRINT(OMPI_PROC_MY_NAME),
+                               ompi_process_info.nodename);
                 return NULL;
             }
 
