@@ -29,23 +29,33 @@ typedef struct mca_mtl_mxm_module_t {
     int                   verbose;
     int                   mxm_np;
     mxm_h                 mxm_context;
+    mxm_ep_h              ep;
 #if MXM_API < MXM_VERSION(1,5)
     mxm_context_opts_t    mxm_opts;
 #else
     mxm_context_opts_t   *mxm_opts;
 #endif
-    mxm_ep_h              ep;
+#if MXM_API >= MXM_VERSION(2,0)
+    int                   using_mem_hooks;
+#endif
 } mca_mtl_mxm_module_t;
 
 
+#if MXM_API < MXM_VERSION(2,0)
 typedef struct ompi_mtl_mxm_ep_conn_info_t {
     struct sockaddr_storage  ptl_addr[MXM_PTL_LAST];
 } ompi_mtl_mxm_ep_conn_info_t;
+#endif
 
 extern mca_mtl_mxm_module_t ompi_mtl_mxm;
 
 typedef struct mca_mtl_mxm_component_t {
     mca_mtl_base_component_2_0_0_t super; /**< base MTL component */
+
+#if MXM_API >= MXM_VERSION(1,5)
+    ompi_free_list_t mxm_messages; /* will be used for MPI_Mprobe and MPI_Mrecv calls */
+#endif
+
 } mca_mtl_mxm_component_t;
 
 
@@ -54,7 +64,7 @@ OMPI_DECLSPEC mca_mtl_mxm_component_t mca_mtl_mxm_component;
 
 static inline mxm_conn_h ompi_mtl_mxm_conn_lookup(struct ompi_communicator_t* comm, int rank) {
     ompi_proc_t* ompi_proc = ompi_comm_peer_lookup(comm, rank);
-    mca_mtl_mxm_endpoint_t *endpoint = (mca_mtl_mxm_endpoint_t*) ompi_proc->proc_pml;
+    mca_mtl_mxm_endpoint_t *endpoint = (mca_mtl_mxm_endpoint_t*) ompi_proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_MTL];
 
     return endpoint->mxm_conn;
 }
