@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University.
  *                         All rights reserved.
@@ -7,9 +8,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2007-2012 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
- * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2009-2011 Oracle and/or its affiliates.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -42,7 +43,7 @@
 static inline int32_t
 create_send_tag(ompi_osc_rdma_module_t *module)
 {
-#if OPAL_HAVE_THREAD_SUPPORT && OPAL_HAVE_ATOMIC_CMPSET_32
+#if OPAL_ENABLE_MULTI_THREADS && OPAL_HAVE_ATOMIC_CMPSET_32
     int32_t newval, oldval;
     do {
         oldval = module->m_tag_counter;
@@ -186,15 +187,15 @@ ompi_osc_rdma_sendreq_rdma(ompi_osc_rdma_module_t *module,
 
             assert(NULL != descriptor);
 
-            descriptor->des_dst = sendreq->remote_segs;
+            descriptor->des_dst = (mca_btl_base_segment_t *) sendreq->remote_segs;
             descriptor->des_dst_cnt = 1;
+            memmove (descriptor->des_dst, rdma_btl->peer_seg, sizeof (rdma_btl->peer_seg));
+
             descriptor->des_dst[0].seg_addr.lval = 
                 module->m_peer_info[target].peer_base + 
                 ((unsigned long)sendreq->req_target_disp * module->m_win->w_disp_unit);
             descriptor->des_dst[0].seg_len = 
                 sendreq->req_origin_bytes_packed;
-            descriptor->des_dst[0].seg_key.key64 = 
-                rdma_btl->peer_seg_key;
 #if 0
             opal_output(0, "putting to %d: 0x%lx(%d), %d, %d",
                         target, descriptor->des_dst[0].seg_addr.lval,
@@ -214,15 +215,15 @@ ompi_osc_rdma_sendreq_rdma(ompi_osc_rdma_module_t *module,
 
             assert(NULL != descriptor);
 
-            descriptor->des_src = sendreq->remote_segs;
+            descriptor->des_src = (mca_btl_base_segment_t *) sendreq->remote_segs;
             descriptor->des_src_cnt = 1;
+            memmove (descriptor->des_src, rdma_btl->peer_seg, sizeof (rdma_btl->peer_seg));
+
             descriptor->des_src[0].seg_addr.lval = 
                 module->m_peer_info[target].peer_base + 
                 ((unsigned long)sendreq->req_target_disp * module->m_win->w_disp_unit);
             descriptor->des_src[0].seg_len = 
                 sendreq->req_origin_bytes_packed;
-            descriptor->des_src[0].seg_key.key64 = 
-                rdma_btl->peer_seg_key;
 
             descriptor->des_cbdata = sendreq;
             descriptor->des_cbfunc = rdma_cb;
@@ -253,14 +254,15 @@ ompi_osc_rdma_sendreq_rdma(ompi_osc_rdma_module_t *module,
  * Sending a sendreq to target
  *
  **********************************************************************/
-static void
-ompi_osc_rdma_sendreq_send_long_cb(ompi_osc_rdma_longreq_t *longreq)
+static int
+ompi_osc_rdma_sendreq_send_long_cb(ompi_request_t *request)
 {
-    ompi_osc_rdma_sendreq_t *sendreq = 
-        (ompi_osc_rdma_sendreq_t*) longreq->cbdata;
+    ompi_osc_rdma_longreq_t *longreq = 
+        (ompi_osc_rdma_longreq_t*) request->req_complete_cb_data;
+    ompi_osc_rdma_sendreq_t *sendreq = longreq->req_basereq.req_sendreq;
     int32_t count;
 
-    OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_output,
+    OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "%d completed long sendreq to %d",
                          ompi_comm_rank(sendreq->req_module->m_comm),
                          sendreq->req_target_rank));
@@ -273,6 +275,9 @@ ompi_osc_rdma_sendreq_send_long_cb(ompi_osc_rdma_longreq_t *longreq)
     ompi_osc_rdma_sendreq_free(sendreq);
 
     if (0 == count) opal_condition_broadcast(&sendreq->req_module->m_cond);
+
+    ompi_request_free(&request);
+    return OMPI_SUCCESS;
 }
 
 
@@ -332,28 +337,23 @@ ompi_osc_rdma_sendreq_send_cb(struct mca_btl_base_module_t* btl,
                 ompi_osc_rdma_longreq_t *longreq;
                 ompi_osc_rdma_longreq_alloc(&longreq);
                 
-                longreq->cbfunc = ompi_osc_rdma_sendreq_send_long_cb;
-                longreq->cbdata = sendreq;
-                OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_output,
+                longreq->req_basereq.req_sendreq = sendreq;
+
+                OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                                      "%d starting long sendreq to %d (%d)",
                                      ompi_comm_rank(sendreq->req_module->m_comm),
                                      sendreq->req_target_rank,
                                      header->hdr_origin_tag));
-                        
-                mca_pml.pml_isend(sendreq->req_origin_convertor.pBaseBuf,
-                                  sendreq->req_origin_convertor.count,
-                                  sendreq->req_origin_datatype,
-                                  sendreq->req_target_rank,
-                                  header->hdr_origin_tag,
-                                  MCA_PML_BASE_SEND_STANDARD,
-                                  sendreq->req_module->m_comm,
-                                  &(longreq->request));
 
-                /* put the send request in the waiting list */
-                OPAL_THREAD_LOCK(&mca_osc_rdma_component.c_lock);
-                opal_list_append(&mca_osc_rdma_component.c_pending_requests,
-                                 &(longreq->super.super));
-                OPAL_THREAD_UNLOCK(&mca_osc_rdma_component.c_lock);
+                ompi_osc_rdma_component_isend(sendreq->req_origin_convertor.pBaseBuf,
+                                              sendreq->req_origin_convertor.count,
+                                              sendreq->req_origin_datatype,
+                                              sendreq->req_target_rank,
+                                              header->hdr_origin_tag,
+                                              sendreq->req_module->m_comm,
+                                              &(longreq->request),
+                                              ompi_osc_rdma_sendreq_send_long_cb,
+                                              longreq);
             }
         } else {
             ompi_osc_rdma_sendreq_free(sendreq);
@@ -386,7 +386,7 @@ ompi_osc_rdma_sendreq_send_cb(struct mca_btl_base_module_t* btl,
         int ret, i, len;
 
         len = opal_list_get_size(&module->m_queued_sendreqs);
-        OPAL_OUTPUT_VERBOSE((40, ompi_osc_base_output,
+        OPAL_OUTPUT_VERBOSE((40, ompi_osc_base_framework.framework_output,
                              "%d items in restart queue",
                              len));
         for (i = 0 ; i < len ; ++i) {
@@ -463,7 +463,7 @@ ompi_osc_rdma_sendreq_send(ompi_osc_rdma_module_t *module,
         }
 
         /* get a buffer... */
-        endpoint = (mca_bml_base_endpoint_t*) sendreq->req_target_proc->proc_bml;
+        endpoint = (mca_bml_base_endpoint_t*) sendreq->req_target_proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
         bml_btl = mca_bml_base_btl_array_get_next(&endpoint->btl_eager);
         btl = bml_btl->btl;
         mca_bml_base_alloc(bml_btl, &descriptor, MCA_BTL_NO_ORDER,
@@ -581,7 +581,7 @@ ompi_osc_rdma_sendreq_send(ompi_osc_rdma_module_t *module,
          * bus errors can occur.  Keeping things aligned also may
          * offer some performance improvements on other platforms.
          */
-        offset = OPAL_ALIGN_PAD_AMOUNT(descriptor->des_src[0].seg_len, sizeof(void*));
+        offset = OPAL_ALIGN_PAD_AMOUNT(descriptor->des_src[0].seg_len, sizeof(uint64_t));
         if (0 != offset) {
             header->hdr_base.hdr_flags |= OMPI_OSC_RDMA_HDR_FLAG_ALIGN_MASK & offset;
             descriptor->des_src[0].seg_len += offset;
@@ -618,7 +618,7 @@ ompi_osc_rdma_sendreq_send(ompi_osc_rdma_module_t *module,
 #endif
 
         /* send fragment */
-        OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_output,
+        OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                              "%d sending sendreq to %d",
                              ompi_comm_rank(sendreq->req_module->m_comm),
                              sendreq->req_target_rank));
@@ -647,16 +647,21 @@ ompi_osc_rdma_sendreq_send(ompi_osc_rdma_module_t *module,
  * Sending a replyreq back to origin
  *
  **********************************************************************/
-static void
-ompi_osc_rdma_replyreq_send_long_cb(ompi_osc_rdma_longreq_t *longreq)
+static int
+ompi_osc_rdma_replyreq_send_long_cb(ompi_request_t *request)
 {
-    ompi_osc_rdma_replyreq_t *replyreq = 
-        (ompi_osc_rdma_replyreq_t*) longreq->cbdata;
+    ompi_osc_rdma_longreq_t *longreq = 
+        (ompi_osc_rdma_longreq_t*) request->req_complete_cb_data;
+    ompi_osc_rdma_replyreq_t *replyreq = longreq->req_basereq.req_replyreq;
 
     inmsg_mark_complete(replyreq->rep_module);
 
     ompi_osc_rdma_longreq_free(longreq);
     ompi_osc_rdma_replyreq_free(replyreq);
+
+    ompi_request_free(&request);
+
+    return OMPI_SUCCESS;
 }
 
 
@@ -692,24 +697,17 @@ ompi_osc_rdma_replyreq_send_cb(struct mca_btl_base_module_t* btl,
     } else {
             ompi_osc_rdma_longreq_t *longreq;
             ompi_osc_rdma_longreq_alloc(&longreq);
+            longreq->req_basereq.req_replyreq = replyreq;
 
-            longreq->cbfunc = ompi_osc_rdma_replyreq_send_long_cb;
-            longreq->cbdata = replyreq;
-
-            mca_pml.pml_isend(replyreq->rep_target_convertor.pBaseBuf,
-                              replyreq->rep_target_convertor.count,
-                              replyreq->rep_target_datatype,
-                              replyreq->rep_origin_rank,
-                              header->hdr_target_tag,
-                              MCA_PML_BASE_SEND_STANDARD,
-                              replyreq->rep_module->m_comm,
-                              &(longreq->request));
-
-            /* put the send request in the waiting list */
-            OPAL_THREAD_LOCK(&mca_osc_rdma_component.c_lock);
-            opal_list_append(&mca_osc_rdma_component.c_pending_requests,
-                             &(longreq->super.super));
-            OPAL_THREAD_UNLOCK(&mca_osc_rdma_component.c_lock);
+            ompi_osc_rdma_component_isend(replyreq->rep_target_convertor.pBaseBuf,
+                                          replyreq->rep_target_convertor.count,
+                                          replyreq->rep_target_datatype,
+                                          replyreq->rep_origin_rank,
+                                          header->hdr_target_tag,
+                                          replyreq->rep_module->m_comm,
+                                          &(longreq->request),
+                                          ompi_osc_rdma_replyreq_send_long_cb,
+                                          longreq);
     }
     
     /* release the descriptor and replyreq */
@@ -729,7 +727,7 @@ ompi_osc_rdma_replyreq_send(ompi_osc_rdma_module_t *module,
     size_t written_data = 0;
         
     /* Get a BTL and a fragment to go with it */
-    endpoint = (mca_bml_base_endpoint_t*) replyreq->rep_origin_proc->proc_bml;
+    endpoint = (mca_bml_base_endpoint_t*) replyreq->rep_origin_proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
     bml_btl = mca_bml_base_btl_array_get_next(&endpoint->btl_eager);
     mca_bml_base_alloc(bml_btl, &descriptor, MCA_BTL_NO_ORDER,
             bml_btl->btl->btl_eager_limit, MCA_BTL_DES_FLAGS_PRIORITY | MCA_BTL_DES_SEND_ALWAYS_CALLBACK);
@@ -821,17 +819,24 @@ ompi_osc_rdma_replyreq_send(ompi_osc_rdma_module_t *module,
  * Receive a put on the target side
  *
  **********************************************************************/
-static void
-ompi_osc_rdma_sendreq_recv_put_long_cb(ompi_osc_rdma_longreq_t *longreq)
+static int
+ompi_osc_rdma_sendreq_recv_put_long_cb(ompi_request_t *request)
 {
+    ompi_osc_rdma_longreq_t *longreq = 
+        (ompi_osc_rdma_longreq_t*) request->req_complete_cb_data;
+
     OBJ_RELEASE(longreq->req_datatype);
-    ompi_osc_rdma_longreq_free(longreq);
     
-    OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_output,
+    OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "%d finished receiving long put message",
                          ompi_comm_rank(longreq->req_module->m_comm))); 
 
     inmsg_mark_complete(longreq->req_module);
+    ompi_osc_rdma_longreq_free(longreq);
+
+    ompi_request_free(&request);
+
+    return OMPI_SUCCESS;
 }
 
 
@@ -848,7 +853,7 @@ ompi_osc_rdma_sendreq_recv_put(ompi_osc_rdma_module_t *module,
         ompi_osc_base_datatype_create(proc, inbuf);
 
     if (NULL == datatype) {
-        opal_output(ompi_osc_base_output,
+        opal_output(ompi_osc_base_framework.framework_output,
                     "Error recreating datatype.  Aborting.");
         ompi_mpi_abort(module->m_comm, 1, false);
     }
@@ -889,7 +894,7 @@ ompi_osc_rdma_sendreq_recv_put(ompi_osc_rdma_module_t *module,
         inmsg_mark_complete(module);
         *inbuf = ((char*) *inbuf) + header->hdr_msg_length;
 
-        OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_output,
+        OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                              "%d received put message from %d",
                              ompi_comm_rank(module->m_comm),
                              header->hdr_origin));
@@ -897,31 +902,24 @@ ompi_osc_rdma_sendreq_recv_put(ompi_osc_rdma_module_t *module,
     } else {
         ompi_osc_rdma_longreq_t *longreq;
         ompi_osc_rdma_longreq_alloc(&longreq);
-
-        longreq->cbfunc = ompi_osc_rdma_sendreq_recv_put_long_cb;
-        longreq->cbdata = NULL;
         longreq->req_datatype = datatype;
         longreq->req_module = module;
 
-        ret = mca_pml.pml_irecv(target,
-                                header->hdr_target_count,
-                                datatype,
-                                header->hdr_origin,
-                                header->hdr_origin_tag,
-                                module->m_comm,
-                                &(longreq->request));
+        ompi_osc_rdma_component_irecv(target,
+                                      header->hdr_target_count,
+                                      datatype,
+                                      header->hdr_origin,
+                                      header->hdr_origin_tag,
+                                      module->m_comm,
+                                      &(longreq->request),
+                                      ompi_osc_rdma_sendreq_recv_put_long_cb,
+                                      longreq);
 
-        OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_output,
+        OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                              "%d started long recv put message from %d (%d)",
                              ompi_comm_rank(module->m_comm),
                              header->hdr_origin,
                              header->hdr_origin_tag));
-
-        /* put the send request in the waiting list */
-        OPAL_THREAD_LOCK(&mca_osc_rdma_component.c_lock);
-        opal_list_append(&mca_osc_rdma_component.c_pending_requests,
-                         &(longreq->super.super));
-        OPAL_THREAD_UNLOCK(&mca_osc_rdma_component.c_lock);
     }
 
     return ret;
@@ -937,13 +935,13 @@ ompi_osc_rdma_sendreq_recv_put(ompi_osc_rdma_module_t *module,
  **********************************************************************/
 
 
-static void
-ompi_osc_rdma_sendreq_recv_accum_long_cb(ompi_osc_rdma_longreq_t *longreq)
+static int
+ompi_osc_rdma_sendreq_recv_accum_long_cb(ompi_request_t *request)
 {
-    ompi_osc_rdma_send_header_t *header = 
-        (ompi_osc_rdma_send_header_t*) longreq->cbdata;
+    ompi_osc_rdma_longreq_t *longreq = 
+        (ompi_osc_rdma_longreq_t*) request->req_complete_cb_data;
+    ompi_osc_rdma_send_header_t *header = longreq->req_basereq.req_sendhdr;
     void *payload = (void*) (header + 1);
-    int ret;
     ompi_osc_rdma_module_t *module = longreq->req_module;
     unsigned char *target_buffer =
         (unsigned char*) module->m_win->w_baseptr + 
@@ -987,7 +985,7 @@ ompi_osc_rdma_sendreq_recv_accum_long_cb(ompi_osc_rdma_longreq_t *longreq)
         OBJ_DESTRUCT(&convertor);
     } else {
         /* copy the data from the temporary buffer into the user window */
-        ret = ompi_osc_base_process_op(target_buffer,
+        (void)ompi_osc_base_process_op(target_buffer,
                                        payload,
                                        header->hdr_msg_length,
                                        longreq->req_datatype,
@@ -998,13 +996,13 @@ ompi_osc_rdma_sendreq_recv_accum_long_cb(ompi_osc_rdma_longreq_t *longreq)
     /* unlock the window for accumulates */
     OPAL_THREAD_UNLOCK(&longreq->req_module->m_acc_lock);
     
-    OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_output,
+    OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "%d finished receiving long accum message from %d",
                          ompi_comm_rank(longreq->req_module->m_comm), 
                          header->hdr_origin));
 
     /* free the temp buffer */
-    free(longreq->cbdata);
+    free(longreq->req_basereq.req_sendhdr);
 
     /* Release datatype & op */
     OBJ_RELEASE(longreq->req_datatype);
@@ -1013,6 +1011,10 @@ ompi_osc_rdma_sendreq_recv_accum_long_cb(ompi_osc_rdma_longreq_t *longreq)
     inmsg_mark_complete(longreq->req_module);
 
     ompi_osc_rdma_longreq_free(longreq);
+
+    ompi_request_free(&request);
+
+    return OMPI_SUCCESS;
 }
 
 
@@ -1028,7 +1030,7 @@ ompi_osc_rdma_sendreq_recv_accum(ompi_osc_rdma_module_t *module,
         ompi_osc_base_datatype_create(proc, payload);
 
     if (NULL == datatype) {
-        opal_output(ompi_osc_base_output,
+        opal_output(ompi_osc_base_framework.framework_output,
                     "Error recreating datatype.  Aborting.");
         ompi_mpi_abort(module->m_comm, 1, false);
     }
@@ -1150,7 +1152,7 @@ ompi_osc_rdma_sendreq_recv_accum(ompi_osc_rdma_module_t *module,
 
         inmsg_mark_complete(module);
 
-        OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_output,
+        OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                              "%d received accum message from %d",
                              ompi_comm_rank(module->m_comm),
                              header->hdr_origin));
@@ -1173,39 +1175,34 @@ ompi_osc_rdma_sendreq_recv_accum(ompi_osc_rdma_module_t *module,
         /* get a longreq and fill it in */
         ompi_osc_rdma_longreq_alloc(&longreq);
 
-        longreq->cbfunc = ompi_osc_rdma_sendreq_recv_accum_long_cb;
         longreq->req_datatype = datatype;
         longreq->req_op = op;
         longreq->req_module = module;
 
         /* allocate a buffer to receive into ... */
-        longreq->cbdata = malloc(buflen + sizeof(ompi_osc_rdma_send_header_t));
+        longreq->req_basereq.req_sendhdr = (ompi_osc_rdma_send_header_t *) malloc(buflen + sizeof(ompi_osc_rdma_send_header_t));
         
-        if (NULL == longreq->cbdata) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
+        if (NULL == longreq->req_basereq.req_sendhdr) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
         /* fill in tmp header */
-        memcpy(longreq->cbdata, header,
+        memcpy(longreq->req_basereq.req_sendhdr, header,
                sizeof(ompi_osc_rdma_send_header_t));
-        ((ompi_osc_rdma_send_header_t*) longreq->cbdata)->hdr_msg_length = buflen;
+        longreq->req_basereq.req_sendhdr->hdr_msg_length = buflen;
 
-        ret = mca_pml.pml_irecv(((char*) longreq->cbdata) + sizeof(ompi_osc_rdma_send_header_t),
-                                primitive_count,
-                                primitive_datatype,
-                                header->hdr_origin,
-                                header->hdr_origin_tag,
-                                module->m_comm,
-                                &(longreq->request));
+        ompi_osc_rdma_component_irecv(longreq->req_basereq.req_sendhdr + 1,
+                                      primitive_count,
+                                      primitive_datatype,
+                                      header->hdr_origin,
+                                      header->hdr_origin_tag,
+                                      module->m_comm,
+                                      &(longreq->request),
+                                      ompi_osc_rdma_sendreq_recv_accum_long_cb,
+                                      longreq);
 
-        OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_output,
+        OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                              "%d started long recv accum message from %d (%d)",
                              ompi_comm_rank(module->m_comm),
                              header->hdr_origin,
                              header->hdr_origin_tag));
-
-        /* put the send request in the waiting list */
-        OPAL_THREAD_LOCK(&mca_osc_rdma_component.c_lock);
-        opal_list_append(&mca_osc_rdma_component.c_pending_requests,
-                         &(longreq->super.super));
-        OPAL_THREAD_UNLOCK(&mca_osc_rdma_component.c_lock);
     }
 
     return ret;
@@ -1217,11 +1214,13 @@ ompi_osc_rdma_sendreq_recv_accum(ompi_osc_rdma_module_t *module,
  * Recveive a get on the origin side
  *
  **********************************************************************/
-static void
-ompi_osc_rdma_replyreq_recv_long_cb(ompi_osc_rdma_longreq_t *longreq)
+static int
+ompi_osc_rdma_replyreq_recv_long_cb(ompi_request_t *request)
 {
+    ompi_osc_rdma_longreq_t *longreq = 
+        (ompi_osc_rdma_longreq_t*) request->req_complete_cb_data;
     ompi_osc_rdma_sendreq_t *sendreq =
-        (ompi_osc_rdma_sendreq_t*) longreq->cbdata;
+        (ompi_osc_rdma_sendreq_t*) longreq->req_basereq.req_sendreq;
     int32_t count;
 
     OPAL_THREAD_LOCK(&sendreq->req_module->m_lock);
@@ -1232,6 +1231,10 @@ ompi_osc_rdma_replyreq_recv_long_cb(ompi_osc_rdma_longreq_t *longreq)
     ompi_osc_rdma_sendreq_free(sendreq);
 
     if (0 == count) opal_condition_broadcast(&sendreq->req_module->m_cond);
+
+    ompi_request_free(&request);
+
+    return OMPI_SUCCESS;
 }
 
 
@@ -1277,24 +1280,18 @@ ompi_osc_rdma_replyreq_recv(ompi_osc_rdma_module_t *module,
         ompi_osc_rdma_longreq_t *longreq;
         ompi_osc_rdma_longreq_alloc(&longreq);
 
-        longreq->cbfunc = ompi_osc_rdma_replyreq_recv_long_cb;
-        longreq->cbdata = sendreq;
+        longreq->req_basereq.req_sendreq = sendreq;
         longreq->req_module = module;
 
-        /* BWB - FIX ME -  George is going to kill me for this */
-        ret = mca_pml.pml_irecv(sendreq->req_origin_convertor.pBaseBuf,
-                                sendreq->req_origin_convertor.count,
-                                sendreq->req_origin_datatype,
-                                sendreq->req_target_rank,
-                                header->hdr_target_tag,
-                                module->m_comm,
-                                &(longreq->request));
-
-        /* put the send request in the waiting list */
-        OPAL_THREAD_LOCK(&mca_osc_rdma_component.c_lock);
-        opal_list_append(&mca_osc_rdma_component.c_pending_requests,
-                         &(longreq->super.super));
-        OPAL_THREAD_UNLOCK(&mca_osc_rdma_component.c_lock);
+        ret = ompi_osc_rdma_component_irecv(sendreq->req_origin_convertor.pBaseBuf,
+                                            sendreq->req_origin_convertor.count,
+                                            sendreq->req_origin_datatype,
+                                            sendreq->req_target_rank,
+                                            header->hdr_target_tag,
+                                            module->m_comm,
+                                            &(longreq->request),
+                                            ompi_osc_rdma_replyreq_recv_long_cb,
+                                            longreq);
     }
 
     return ret;
@@ -1312,10 +1309,6 @@ ompi_osc_rdma_control_send_cb(struct mca_btl_base_module_t* btl,
                                struct mca_btl_base_descriptor_t* descriptor,
                                int status)
 {
-    ompi_osc_rdma_control_header_t *header = NULL;
-
-    header = (ompi_osc_rdma_control_header_t*) descriptor->des_src[0].seg_addr.pval;
-
     /* release the descriptor and sendreq */
     btl->btl_free(btl, descriptor);
 }
@@ -1333,7 +1326,7 @@ ompi_osc_rdma_control_send(ompi_osc_rdma_module_t *module,
     ompi_osc_rdma_control_header_t *header = NULL;
         
     /* Get a BTL and a fragment to go with it */
-    endpoint = (mca_bml_base_endpoint_t*) proc->proc_bml;
+    endpoint = (mca_bml_base_endpoint_t*) proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
     bml_btl = mca_bml_base_btl_array_get_next(&endpoint->btl_eager);
     mca_bml_base_alloc(bml_btl, &descriptor, MCA_BTL_NO_ORDER,
             sizeof(ompi_osc_rdma_control_header_t),
