@@ -10,6 +10,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2012      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
+ * Copyright (c) 2013      Intel, Inc. All rights reserved
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -25,16 +28,12 @@
 
 #include "opal/class/opal_list.h"
 #include "opal/util/output.h"
-#include "orte/util/show_help.h"
+#include "opal/util/show_help.h"
 #include "opal/runtime/opal_progress.h"
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
 #include "opal/runtime/opal.h"
 
-#include "orte/mca/errmgr/errmgr.h"
-#include "orte/util/name_fns.h"
-#include "orte/runtime/orte_globals.h"
-
 #include "ompi/constants.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/pml/base/base.h"
@@ -82,10 +81,7 @@ int mca_pml_base_select(bool enable_progress_threads,
     best_component = NULL;
     module = NULL;
     OBJ_CONSTRUCT(&opened, opal_list_t);
-    for (item = opal_list_get_first(&mca_pml_base_components_available);
-         opal_list_get_end(&mca_pml_base_components_available) != item;
-         item = opal_list_get_next(item) ) {
-        cli = (mca_base_component_list_item_t *) item;
+    OPAL_LIST_FOREACH(cli, &ompi_pml_base_framework.framework_components, mca_base_component_list_item_t) {
         component = (mca_pml_base_component_t *) cli->cli_component;
 
         /* if there is an include list - item must be in the list to be included */
@@ -105,7 +101,7 @@ int mca_pml_base_select(bool enable_progress_threads,
         }
 
         if(!found_pml && opal_pointer_array_get_size(&mca_pml_base_pml)) { 
-            opal_output_verbose( 10, mca_pml_base_output,
+            opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                                      "select: component %s not in the include list",
                                      component->pmlm_version.mca_component_name );
                 
@@ -114,7 +110,7 @@ int mca_pml_base_select(bool enable_progress_threads,
 
         /* if there is no init function - ignore it */
         if (NULL == component->pmlm_init) {
-            opal_output_verbose( 10, mca_pml_base_output,
+            opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                                  "select: no init function; ignoring component %s",
                                  component->pmlm_version.mca_component_name );
             continue;
@@ -124,7 +120,7 @@ int mca_pml_base_select(bool enable_progress_threads,
         num_pml++;
 
         /* Init component to get its priority */
-        opal_output_verbose( 10, mca_pml_base_output, 
+        opal_output_verbose( 10, ompi_pml_base_framework.framework_output, 
                              "select: initializing %s component %s",
                              component->pmlm_version.mca_type_name,
                              component->pmlm_version.mca_component_name );
@@ -132,18 +128,18 @@ int mca_pml_base_select(bool enable_progress_threads,
         module = component->pmlm_init(&priority, enable_progress_threads,
                                       enable_mpi_threads);
         if (NULL == module) {
-            opal_output_verbose( 10, mca_pml_base_output,
+            opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                                  "select: init returned failure for component %s",
                                  component->pmlm_version.mca_component_name );
             continue;
         }
 
-        opal_output_verbose( 10, mca_pml_base_output,
+        opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                              "select: init returned priority %d", priority );
 #if OPAL_ENABLE_FT_CR == 1
         /* Determine if this is the wrapper component */
         if( priority <= PML_SELECT_WRAPPER_PRIORITY) {
-            opal_output_verbose( 10, mca_pml_base_output,
+            opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                                  "pml:select: Wrapper Component: Component %s was determined to be a Wrapper PML with priority %d",
                                  component->pmlm_version.mca_component_name, priority );
             wrapper_priority  = priority;
@@ -172,21 +168,21 @@ int mca_pml_base_select(bool enable_progress_threads,
     /* Finished querying all components.  Check for the bozo case. */
     
     if( NULL == best_component ) {
-        orte_show_help("help-mca-base.txt", "find-available:none-found", true, "pml");
+        opal_show_help("help-mca-base.txt", "find-available:none-found", true, "pml");
         for( i = 0; i < opal_pointer_array_get_size(&mca_pml_base_pml); i++) { 
             char * tmp_val = NULL;
             tmp_val = (char *) opal_pointer_array_get_item(&mca_pml_base_pml, i);
             if( NULL == tmp_val) {
                 continue;
             }
-            orte_errmgr.abort(1, "PML %s cannot be selected", tmp_val);
+            ompi_rte_abort(1, "PML %s cannot be selected", tmp_val);
         }
         if(0 == i) { 
-            orte_errmgr.abort(2, "No pml component available.  This shouldn't happen.");
+            ompi_rte_abort(2, "No pml component available.  This shouldn't happen.");
         }
     } 
     
-    opal_output_verbose( 10, mca_pml_base_output,
+    opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                          "selected %s best priority %d\n", 
                          best_component->pmlm_version.mca_component_name, best_priority);
     
@@ -218,7 +214,7 @@ int mca_pml_base_select(bool enable_progress_threads,
                    don't matter anymore) */
                 
                 om->om_component->pmlm_finalize();
-                opal_output_verbose(10, mca_pml_base_output, 
+                opal_output_verbose(10, ompi_pml_base_framework.framework_output, 
                                     "select: component %s not selected / finalized",
                                     om->om_component->pmlm_version.mca_component_name);
             }
@@ -229,18 +225,15 @@ int mca_pml_base_select(bool enable_progress_threads,
     OBJ_DESTRUCT( &opened );
 
 #if OPAL_ENABLE_FT_CR == 1
-    /* Remove the wrapper component from the mca_pml_base_components_available list
+    /* Remove the wrapper component from the ompi_pml_base_framework.framework_components list
      * so we don't unload it prematurely in the next call
      */
     if( NULL != wrapper_component ) {
-        for (item  = opal_list_get_first(&mca_pml_base_components_available);
-             item != opal_list_get_end(&mca_pml_base_components_available);
-             item  = opal_list_get_next(item) ) {
-            cli       = (mca_base_component_list_item_t *) item;
+        OPAL_LIST_FOREACH(cli, &ompi_pml_base_framework.framework_components, mca_base_component_list_item_t) {
             component = (mca_pml_base_component_t *) cli->cli_component;
             
             if( component == wrapper_component ) {
-                opal_list_remove_item(&mca_pml_base_components_available, item);
+                opal_list_remove_item(&ompi_pml_base_framework.framework_components, item);
             }
         }
     }
@@ -250,27 +243,23 @@ int mca_pml_base_select(bool enable_progress_threads,
 
     mca_pml_base_selected_component = *best_component;
     mca_pml = *best_module;
-    opal_output_verbose( 10, mca_pml_base_output, 
+    opal_output_verbose( 10, ompi_pml_base_framework.framework_output, 
                          "select: component %s selected",
                          mca_pml_base_selected_component.pmlm_version.mca_component_name );
 
-    if (opal_profile) {
-        opal_output(0, "pml:%s",  mca_pml_base_selected_component.pmlm_version.mca_component_name );
-    }
-    
     /* This base function closes, unloads, and removes from the
        available list all unselected components.  The available list will
        contain only the selected component. */
 
-    mca_base_components_close(mca_pml_base_output, 
-                              &mca_pml_base_components_available, 
+    mca_base_components_close(ompi_pml_base_framework.framework_output, 
+                              &ompi_pml_base_framework.framework_components, 
                               (mca_base_component_t *) best_component);
     
 #if OPAL_ENABLE_FT_CR == 1
     /* If we have a wrapper then initalize it */
     if( NULL != wrapper_component ) {
         priority = PML_SELECT_WRAPPER_PRIORITY;
-        opal_output_verbose( 10, mca_pml_base_output,
+        opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                              "pml:select: Wrapping: Component %s [%d] is being wrapped by component %s [%d]", 
                              mca_pml_base_selected_component.pmlm_version.mca_component_name,
                              best_priority,
@@ -298,7 +287,7 @@ int mca_pml_base_select(bool enable_progress_threads,
     }
 
     /* register winner in the modex */
-    if (modex_reqd && 0 == ORTE_PROC_MY_NAME->vpid) {
+    if (modex_reqd && 0 == OMPI_PROC_MY_NAME->vpid) {
         mca_pml_base_pml_selected(best_component->pmlm_version.mca_component_name);
     }
 
@@ -339,14 +328,14 @@ mca_pml_base_pml_check_selected(const char *my_pml,
      * we can assume success
      */
     if (!modex_reqd) {
-        opal_output_verbose( 10, mca_pml_base_output,
+        opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                             "check:select: modex not reqd");
         return OMPI_SUCCESS;
     }
     
     /* if we are rank=0, then we can also assume success */
-    if (0 == ORTE_PROC_MY_NAME->vpid) {
-        opal_output_verbose( 10, mca_pml_base_output,
+    if (0 == OMPI_PROC_MY_NAME->vpid) {
+        opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                             "check:select: rank=0");
         return OMPI_SUCCESS;
     }
@@ -356,10 +345,10 @@ mca_pml_base_pml_check_selected(const char *my_pml,
                           procs[0],
                           (void**) &remote_pml, &size);
     
-    /* if modex isn't implemented, then just assume all is well... */
-    if (OMPI_ERR_NOT_IMPLEMENTED == ret) {
-        opal_output_verbose( 10, mca_pml_base_output,
-                            "check:select: modex not implemented");
+    /* if this key wasn't found, then just assume all is well... */
+    if (OMPI_SUCCESS != ret) {
+        opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
+                            "check:select: modex data not found");
         return OMPI_SUCCESS;
     }
 
@@ -368,30 +357,23 @@ mca_pml_base_pml_check_selected(const char *my_pml,
      * is fast...let's be sure
      */
     if (NULL == remote_pml) {
-        opal_output_verbose( 10, mca_pml_base_output,
+        opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                             "check:select: got a NULL pml from rank=0");
         return OMPI_ERR_UNREACH;
     }
     
-    opal_output_verbose( 10, mca_pml_base_output,
+    opal_output_verbose( 10, ompi_pml_base_framework.framework_output,
                         "check:select: checking my pml %s against rank=0 pml %s",
                         my_pml, remote_pml);
     
     /* if that module doesn't match my own, return an error */
     if ((size != strlen(my_pml) + 1) ||
         (0 != strcmp(my_pml, remote_pml))) {
-        if (procs[0]->proc_hostname) {
-            opal_output(0, "%s selected pml %s, but peer %s on %s selected pml %s",
-                        ORTE_NAME_PRINT(&ompi_proc_local()->proc_name),
-                        my_pml, ORTE_NAME_PRINT(&procs[0]->proc_name),
-                        procs[0]->proc_hostname,
-                        remote_pml);
-        } else {
-            opal_output(0, "%s selected pml %s, but peer %s selected pml %s",
-                        ORTE_NAME_PRINT(&ompi_proc_local()->proc_name),
-                        my_pml, ORTE_NAME_PRINT(&procs[0]->proc_name),
-                        remote_pml);
-        }
+        opal_output(0, "%s selected pml %s, but peer %s on %s selected pml %s",
+                    OMPI_NAME_PRINT(&ompi_proc_local()->proc_name),
+                    my_pml, OMPI_NAME_PRINT(&procs[0]->proc_name),
+                    (NULL == procs[0]->proc_hostname) ? "unknown" : procs[0]->proc_hostname,
+                    remote_pml);
         free(remote_pml); /* cleanup before returning */
         return OMPI_ERR_UNREACH;
     }
