@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2004-2008 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2009 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -11,6 +12,10 @@
  *                         All rights reserved.
  * Copyright (c) 2008      UT-Battelle, LLC. All rights reserved.
  * Copyright (c) 2006-2008 University of Houston.  All rights reserved.
+ * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved
+ * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
+ * Copyright (c) 2011-2012 Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -23,12 +28,18 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "ompi/class/ompi_bitmap.h"
+#include "opal/class/opal_bitmap.h"
+#include "opal/util/output.h"
+#include "opal/util/show_help.h"
+
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/pml/base/base.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/pml/base/base.h"
 #include "ompi/mca/btl/base/base.h"
+#include "ompi/mca/bml/base/base.h"
+#include "ompi/runtime/ompi_cr.h"
+
 #include "pml_ob1.h"
 #include "pml_ob1_component.h"
 #include "pml_ob1_comm.h"
@@ -37,14 +48,6 @@
 #include "pml_ob1_sendreq.h"
 #include "pml_ob1_recvreq.h"
 #include "pml_ob1_rdmafrag.h"
-#include "ompi/mca/bml/base/base.h"
-#include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/grpcomm/grpcomm.h"
-#include "orte/runtime/orte_globals.h"
-
-#include "ompi/runtime/ompi_cr.h"
-#include "ompi/runtime/ompi_module_exchange.h"
-#include "orte/mca/rml/rml.h"
 
 mca_pml_ob1_t mca_pml_ob1 = {
     {
@@ -63,6 +66,10 @@ mca_pml_ob1_t mca_pml_ob1 = {
         mca_pml_ob1_iprobe,
         mca_pml_ob1_probe,
         mca_pml_ob1_start,
+        mca_pml_ob1_improbe,
+        mca_pml_ob1_mprobe,
+        mca_pml_ob1_imrecv,
+        mca_pml_ob1_mrecv,
         mca_pml_ob1_dump,
         mca_pml_ob1_ft_event,
         65535,
@@ -72,11 +79,14 @@ mca_pml_ob1_t mca_pml_ob1 = {
 
 
 void mca_pml_ob1_error_handler( struct mca_btl_base_module_t* btl,
-                                int32_t flags );
+                                int32_t flags, ompi_proc_t* errproc,
+                                char* btlinfo );
 
 int mca_pml_ob1_enable(bool enable)
 {
-    if( false == enable ) return OMPI_SUCCESS;
+    if( false == enable ) {
+        return OMPI_SUCCESS;
+    }
 
     OBJ_CONSTRUCT(&mca_pml_ob1.lock, opal_mutex_t);
 
@@ -84,9 +94,9 @@ int mca_pml_ob1_enable(bool enable)
     OBJ_CONSTRUCT(&mca_pml_ob1.rdma_frags, ompi_free_list_t);
     ompi_free_list_init_new( &mca_pml_ob1.rdma_frags,
                          sizeof(mca_pml_ob1_rdma_frag_t),
-                         CACHE_LINE_SIZE,
+                         opal_cache_line_size,
                          OBJ_CLASS(mca_pml_ob1_rdma_frag_t),
-                         0,CACHE_LINE_SIZE,
+                         0,opal_cache_line_size,
                          mca_pml_ob1.free_list_num,
                          mca_pml_ob1.free_list_max,
                          mca_pml_ob1.free_list_inc,
@@ -96,9 +106,9 @@ int mca_pml_ob1_enable(bool enable)
 
     ompi_free_list_init_new( &mca_pml_ob1.recv_frags,
                          sizeof(mca_pml_ob1_recv_frag_t) + mca_pml_ob1.unexpected_limit,
-                         CACHE_LINE_SIZE,
+                         opal_cache_line_size,
                          OBJ_CLASS(mca_pml_ob1_recv_frag_t),
-                         0,CACHE_LINE_SIZE,
+                         0,opal_cache_line_size,
                          mca_pml_ob1.free_list_num,
                          mca_pml_ob1.free_list_max,
                          mca_pml_ob1.free_list_inc,
@@ -107,9 +117,9 @@ int mca_pml_ob1_enable(bool enable)
     OBJ_CONSTRUCT(&mca_pml_ob1.pending_pckts, ompi_free_list_t);
     ompi_free_list_init_new( &mca_pml_ob1.pending_pckts,
                          sizeof(mca_pml_ob1_pckt_pending_t),
-                         CACHE_LINE_SIZE,
+                         opal_cache_line_size,
                          OBJ_CLASS(mca_pml_ob1_pckt_pending_t),
-                         0,CACHE_LINE_SIZE,
+                         0,opal_cache_line_size,
                          mca_pml_ob1.free_list_num,
                          mca_pml_ob1.free_list_max,
                          mca_pml_ob1.free_list_inc,
@@ -121,9 +131,9 @@ int mca_pml_ob1_enable(bool enable)
     ompi_free_list_init_new( &mca_pml_ob1.send_ranges,
                          sizeof(mca_pml_ob1_send_range_t) +
                          (mca_pml_ob1.max_send_per_range - 1) * sizeof(mca_pml_ob1_com_btl_t),
-                         CACHE_LINE_SIZE,
+                         opal_cache_line_size,
                          OBJ_CLASS(mca_pml_ob1_send_range_t),
-                         0,CACHE_LINE_SIZE,
+                         0,opal_cache_line_size,
                          mca_pml_ob1.free_list_num,
                          mca_pml_ob1.free_list_max,
                          mca_pml_ob1.free_list_inc,
@@ -134,6 +144,7 @@ int mca_pml_ob1_enable(bool enable)
     OBJ_CONSTRUCT(&mca_pml_ob1.recv_pending, opal_list_t);
     OBJ_CONSTRUCT(&mca_pml_ob1.pckt_pending, opal_list_t);
     OBJ_CONSTRUCT(&mca_pml_ob1.rdma_pending, opal_list_t);
+
     /* missing communicator pending list */
     OBJ_CONSTRUCT(&mca_pml_ob1.non_existing_communicator_pending, opal_list_t);
 
@@ -146,9 +157,9 @@ int mca_pml_ob1_enable(bool enable)
                          sizeof(mca_pml_ob1_send_request_t) +
                          (mca_pml_ob1.max_rdma_per_request - 1) *
                          sizeof(mca_pml_ob1_com_btl_t),
-                         CACHE_LINE_SIZE,
+                         opal_cache_line_size,
                          OBJ_CLASS(mca_pml_ob1_send_request_t),
-                         0,CACHE_LINE_SIZE,
+                         0,opal_cache_line_size,
                          mca_pml_ob1.free_list_num,
                          mca_pml_ob1.free_list_max,
                          mca_pml_ob1.free_list_inc,
@@ -158,9 +169,9 @@ int mca_pml_ob1_enable(bool enable)
                          sizeof(mca_pml_ob1_recv_request_t) +
                          (mca_pml_ob1.max_rdma_per_request - 1) *
                          sizeof(mca_pml_ob1_com_btl_t),
-                         CACHE_LINE_SIZE,
+                         opal_cache_line_size,
                          OBJ_CLASS(mca_pml_ob1_recv_request_t),
-                         0,CACHE_LINE_SIZE,
+                         0,opal_cache_line_size,
                          mca_pml_ob1.free_list_num,
                          mca_pml_ob1.free_list_max,
                          mca_pml_ob1.free_list_inc,
@@ -185,7 +196,7 @@ int mca_pml_ob1_add_comm(ompi_communicator_t* comm)
     }
 
     /* should never happen, but it was, so check */
-    if (comm->c_contextid > (uint32_t) mca_pml_ob1.super.pml_max_contextid) {
+    if (comm->c_contextid > mca_pml_ob1.super.pml_max_contextid) {
         OBJ_RELEASE(pml_comm);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
@@ -209,11 +220,11 @@ int mca_pml_ob1_add_comm(ompi_communicator_t* comm)
         if( frag->hdr.hdr_match.hdr_ctx != comm->c_contextid )
             continue;
 
-        /* As we now know we work on a fragment for this communicator we should
-         * remove it from the non_existing_communicator_pending list. As a result
-         * after the call item will contain the previous item so the loop will
-         * continue to work as expected. */
-        item = opal_list_remove_item( &mca_pml_ob1.non_existing_communicator_pending, item );
+        /* As we now know we work on a fragment for this communicator
+         * we should remove it from the
+         * non_existing_communicator_pending list. */
+        opal_list_remove_item( &mca_pml_ob1.non_existing_communicator_pending, 
+                               item );
 
       add_fragment_to_unexpected:
 
@@ -266,6 +277,12 @@ int mca_pml_ob1_add_comm(ompi_communicator_t* comm)
 
 int mca_pml_ob1_del_comm(ompi_communicator_t* comm)
 {
+    mca_pml_ob1_comm_t* pml_comm = comm->c_pml_comm;
+    int i;
+
+    for( i = 0; i < comm->c_remote_group->grp_proc_count; i++ ) {
+        OBJ_RELEASE(pml_comm->procs[i].ompi_proc);
+    }
     OBJ_RELEASE(comm->c_pml_comm);
     comm->c_pml_comm = NULL;
     return OMPI_SUCCESS;
@@ -280,21 +297,15 @@ int mca_pml_ob1_del_comm(ompi_communicator_t* comm)
 
 int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
 {
-    ompi_bitmap_t reachable;
+    opal_bitmap_t reachable;
     int rc;
-    size_t i;
+    opal_list_item_t *item;
 
     if(nprocs == 0)
         return OMPI_SUCCESS;
 
-    /* we don't have any endpoint data we need to cache on the
-       ompi_proc_t, so set proc_pml to NULL */
-    for (i = 0 ; i < nprocs ; ++i) {
-        procs[i]->proc_pml = NULL;
-    }
-
-    OBJ_CONSTRUCT(&reachable, ompi_bitmap_t);
-    rc = ompi_bitmap_init(&reachable, (int)nprocs);
+    OBJ_CONSTRUCT(&reachable, opal_bitmap_t);
+    rc = opal_bitmap_init(&reachable, (int)nprocs);
     if(OMPI_SUCCESS != rc)
         return rc;
 
@@ -304,7 +315,7 @@ int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
      * return failure as all processes will return the wrapper PML
      * component in use instead of the wrapped PML component underneath.
      */
-#if OPAL_ENABLE_FT == 0
+#if OPAL_ENABLE_FT_CR == 0
     /* make sure remote procs are using the same PML as us */
     if (OMPI_SUCCESS != (rc = mca_pml_base_pml_check_selected("ob1",
                                                               procs,
@@ -319,6 +330,39 @@ int mca_pml_ob1_add_procs(ompi_proc_t** procs, size_t nprocs)
     if(OMPI_SUCCESS != rc)
         goto cleanup_and_return;
 
+    /* Check that values supplied by all initialized btls will work
+       for us.  Note that this is the list of all initialized BTLs,
+       not the ones used for the just added procs.  This is a little
+       overkill and inaccurate, as we may end up not using the BTL in
+       question and all add_procs calls after the first one are
+       duplicating an already completed check.  But the final
+       initialization of the PML occurs before the final
+       initialization of the BTLs, and iterating through the in-use
+       BTLs requires iterating over the procs, as the BML does not
+       expose all currently in use btls. */
+
+    for (item = opal_list_get_first(&mca_btl_base_modules_initialized) ;
+         item != opal_list_get_end(&mca_btl_base_modules_initialized) ;
+         item = opal_list_get_next(item)) {
+        mca_btl_base_selected_module_t *sm = 
+            (mca_btl_base_selected_module_t*) item;
+        if (sm->btl_module->btl_eager_limit < sizeof(mca_pml_ob1_hdr_t)) {
+            opal_show_help("help-mpi-pml-ob1.txt", "eager_limit_too_small",
+                           true, 
+                           sm->btl_component->btl_version.mca_component_name,
+                           ompi_process_info.nodename,
+                           sm->btl_component->btl_version.mca_component_name,
+                           sm->btl_module->btl_eager_limit,
+                           sm->btl_component->btl_version.mca_component_name,
+                           sizeof(mca_pml_ob1_hdr_t),
+                           sm->btl_component->btl_version.mca_component_name);
+            rc = OMPI_ERR_BAD_PARAM;
+            goto cleanup_and_return;
+        }
+    }
+
+
+    /* TODO: Move these callback registration to another place */
     rc = mca_bml.bml_register( MCA_PML_OB1_HDR_TYPE_MATCH,
                                mca_pml_ob1_recv_frag_callback_match,
                                NULL );
@@ -394,7 +438,7 @@ int mca_pml_ob1_dump(struct ompi_communicator_t* comm, int verbose)
     /* iterate through all procs on communicator */
     for( i = 0; i < (int)pml_comm->num_procs; i++ ) {
         mca_pml_ob1_comm_proc_t* proc = &pml_comm->procs[i];
-        mca_bml_base_endpoint_t* ep = (mca_bml_base_endpoint_t*)proc->ompi_proc->proc_bml;
+        mca_bml_base_endpoint_t* ep = (mca_bml_base_endpoint_t*)proc->ompi_proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
         size_t n;
 
         opal_output(0, "[Rank %d]\n", i);
@@ -438,8 +482,7 @@ int mca_pml_ob1_send_fin( ompi_proc_t* proc,
     int rc;
 
     mca_bml_base_alloc(bml_btl, &fin, order, sizeof(mca_pml_ob1_fin_hdr_t),
-                       MCA_BTL_DES_FLAGS_PRIORITY | MCA_BTL_DES_FLAGS_BTL_OWNERSHIP |
-                       MCA_BTL_DES_SEND_ALWAYS_CALLBACK);
+                       MCA_BTL_DES_FLAGS_PRIORITY | MCA_BTL_DES_FLAGS_BTL_OWNERSHIP);
 
     if(NULL == fin) {
         MCA_PML_OB1_ADD_FIN_TO_PENDING(proc, hdr_des, bml_btl, order, status);
@@ -489,8 +532,10 @@ void mca_pml_ob1_process_pending_packets(mca_bml_base_btl_t* bml_btl)
                 pckt->bml_btl->btl == bml_btl->btl) {
             send_dst = pckt->bml_btl;
         } else {
+            mca_bml_base_endpoint_t* endpoint = 
+                (mca_bml_base_endpoint_t*) pckt->proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
             send_dst = mca_bml_base_btl_array_find(
-                    &pckt->proc->proc_bml->btl_eager, bml_btl->btl);
+                    &endpoint->btl_eager, bml_btl->btl);
         }
         if(NULL == send_dst) {
             OPAL_THREAD_LOCK(&mca_pml_ob1.lock);
@@ -547,8 +592,10 @@ void mca_pml_ob1_process_pending_rdma(void)
         OPAL_THREAD_UNLOCK(&mca_pml_ob1.lock);
         if(NULL == frag)
             break;
+
+        frag->retries++;
+
         if(frag->rdma_state == MCA_PML_OB1_RDMA_PUT) {
-            frag->retries++;
             rc = mca_pml_ob1_send_request_put_frag(frag);
         } else {
             rc = mca_pml_ob1_recv_request_get_frag(frag);
@@ -560,12 +607,12 @@ void mca_pml_ob1_process_pending_rdma(void)
 
 
 void mca_pml_ob1_error_handler(
-        struct mca_btl_base_module_t* btl,
-        int32_t flags) { 
-    orte_errmgr.abort(-1, NULL);
+        struct mca_btl_base_module_t* btl, int32_t flags,
+        ompi_proc_t* errproc, char* btlinfo ) { 
+    ompi_rte_abort(-1, NULL);
 }
 
-#if OPAL_ENABLE_FT    == 0
+#if OPAL_ENABLE_FT_CR    == 0
 int mca_pml_ob1_ft_event( int state ) {
     return OMPI_SUCCESS;
 }
@@ -596,7 +643,7 @@ int mca_pml_ob1_ft_event( int state )
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2P2);
         }
 
-        if( ompi_cr_continue_like_restart && !first_continue_pass ) {
+        if( orte_cr_continue_like_restart && !first_continue_pass ) {
             /*
              * Get a list of processes
              */
@@ -616,6 +663,9 @@ int mca_pml_ob1_ft_event( int state )
                 opal_output(0,
                             "pml:ob1: ft_event(Restart): proc_refresh Failed %d",
                             ret);
+                for(p = 0; p < (int)num_procs; ++p) {
+                    OBJ_RELEASE(procs[p]);
+                }
                 free (procs);
                 return ret;
             }
@@ -652,6 +702,9 @@ int mca_pml_ob1_ft_event( int state )
             opal_output(0,
                         "pml:ob1: ft_event(Restart): proc_refresh Failed %d",
                         ret);
+            for(p = 0; p < (int)num_procs; ++p) {
+                OBJ_RELEASE(procs[p]);
+            }
             free (procs);
             return ret;
         }
@@ -690,7 +743,7 @@ int mca_pml_ob1_ft_event( int state )
             OPAL_CR_SET_TIMER(OPAL_CR_TIMER_P2P3);
         }
 
-        if( ompi_cr_continue_like_restart && !first_continue_pass ) {
+        if( orte_cr_continue_like_restart && !first_continue_pass ) {
             /*
              * Exchange the modex information once again.
              * BTLs will have republished their modex information.
@@ -780,7 +833,7 @@ int mca_pml_ob1_ft_event( int state )
 
     return OMPI_SUCCESS;
 }
-#endif /* OPAL_ENABLE_FT */
+#endif /* OPAL_ENABLE_FT_CR */
 
 int mca_pml_ob1_com_btl_comp(const void *v1, const void *v2)
 {
@@ -794,3 +847,4 @@ int mca_pml_ob1_com_btl_comp(const void *v1, const void *v2)
 
     return 0;
 }
+
