@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
@@ -10,6 +11,11 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      UT-Battelle, LLC. All rights reserved.
+ * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
+ * Copyright (c) 2012-2013 NVIDIA Corporation.  All rights reserved.
+ * Copyright (c) 2011-2012 Los Alamos National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2012      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -29,14 +35,22 @@
 #include "pml_ob1_sendreq.h"
 #include "pml_ob1_rdmafrag.h"
 #include "ompi/mca/bml/base/base.h" 
-#include "orte/mca/errmgr/errmgr.h"
 #include "opal/util/arch.h"
 #include "ompi/memchecker.h"
+#if OMPI_CUDA_SUPPORT
+#include "opal/datatype/opal_datatype_cuda.h"
+#include "ompi/mca/common/cuda/common_cuda.h"
+#endif /* OMPI_CUDA_SUPPORT */
+
+#if OMPI_CUDA_SUPPORT
+int mca_pml_ob1_cuda_need_buffers(mca_pml_ob1_recv_request_t* recvreq,
+                                  mca_btl_base_module_t* btl);
+#endif /* OMPI_CUDA_SUPPORT */
 
 void mca_pml_ob1_recv_request_process_pending(void)
 {
     mca_pml_ob1_recv_request_t* recvreq;
-    int i, s = (int)opal_list_get_size(&mca_pml_ob1.recv_pending);
+    int rc, i, s = (int)opal_list_get_size(&mca_pml_ob1.recv_pending);
 
     for(i = 0; i < s; i++) {
         OPAL_THREAD_LOCK(&mca_pml_ob1.lock);
@@ -46,8 +60,8 @@ void mca_pml_ob1_recv_request_process_pending(void)
         if( OPAL_UNLIKELY(NULL == recvreq) )
             break;
         recvreq->req_pending = false;
-        if(mca_pml_ob1_recv_request_schedule_exclusive(recvreq, NULL) == 
-                OMPI_ERR_OUT_OF_RESOURCE)
+        rc = mca_pml_ob1_recv_request_schedule_exclusive(recvreq, NULL);
+        if(OMPI_ERR_OUT_OF_RESOURCE == rc)
             break;
     }
 }
@@ -87,36 +101,26 @@ static int mca_pml_ob1_recv_request_cancel(struct ompi_request_t* ompi_request,
     mca_pml_ob1_recv_request_t* request = (mca_pml_ob1_recv_request_t*)ompi_request;
     mca_pml_ob1_comm_t* comm = request->req_recv.req_base.req_comm->c_pml_comm;
 
-    if( true == ompi_request->req_complete ) { /* way to late to cancel this one */
-        /*
-         * Receive request completed, make user buffer accessable.
-         */
-        MEMCHECKER(
-            memchecker_call(&opal_memchecker_base_mem_defined,
-                            request->req_recv.req_base.req_addr,
-                            request->req_recv.req_base.req_count,
-                            request->req_recv.req_base.req_datatype);
-        );
+    if( true == request->req_match_received ) { /* way to late to cancel this one */
+        assert( OMPI_ANY_TAG != ompi_request->req_status.MPI_TAG ); /* not matched isn't it */
         return OMPI_SUCCESS;
     }
 
     /* The rest should be protected behind the match logic lock */
     OPAL_THREAD_LOCK(&comm->matching_lock);
-    if( OMPI_ANY_TAG == ompi_request->req_status.MPI_TAG ) { /* the match has not been already done */
-       if( request->req_recv.req_base.req_peer == OMPI_ANY_SOURCE ) {
-          opal_list_remove_item( &comm->wild_receives, (opal_list_item_t*)request );
-       } else {
-          mca_pml_ob1_comm_proc_t* proc = comm->procs + request->req_recv.req_base.req_peer;
-          opal_list_remove_item(&proc->specific_receives, (opal_list_item_t*)request);
-       }
-       PERUSE_TRACE_COMM_EVENT( PERUSE_COMM_REQ_REMOVE_FROM_POSTED_Q,
-                                &(request->req_recv.req_base), PERUSE_RECV );
-       /**
-        * As now the PML is done with this request we have to force the pml_complete
-        * to true. Otherwise, the request will never be freed.
-        */
-       request->req_recv.req_base.req_pml_complete = true;
+    if( request->req_recv.req_base.req_peer == OMPI_ANY_SOURCE ) {
+        opal_list_remove_item( &comm->wild_receives, (opal_list_item_t*)request );
+    } else {
+        mca_pml_ob1_comm_proc_t* proc = comm->procs + request->req_recv.req_base.req_peer;
+        opal_list_remove_item(&proc->specific_receives, (opal_list_item_t*)request);
     }
+    PERUSE_TRACE_COMM_EVENT( PERUSE_COMM_REQ_REMOVE_FROM_POSTED_Q,
+                             &(request->req_recv.req_base), PERUSE_RECV );
+    /**
+     * As now the PML is done with this request we have to force the pml_complete
+     * to true. Otherwise, the request will never be freed.
+     */
+    request->req_recv.req_base.req_pml_complete = true;
     OPAL_THREAD_UNLOCK(&comm->matching_lock);
     
     OPAL_THREAD_LOCK(&ompi_request_lock);
@@ -128,7 +132,7 @@ static int mca_pml_ob1_recv_request_cancel(struct ompi_request_t* ompi_request,
     MCA_PML_OB1_RECV_REQUEST_MPI_COMPLETE(request);
     OPAL_THREAD_UNLOCK(&ompi_request_lock);
     /*
-     * Receive request cancelled, make user buffer accessable.
+     * Receive request cancelled, make user buffer accessible.
      */
     MEMCHECKER(
         memchecker_call(&opal_memchecker_base_mem_defined,
@@ -183,8 +187,9 @@ static void mca_pml_ob1_put_completion( mca_btl_base_module_t* btl,
     size_t bytes_received = 0;
 
     if( OPAL_LIKELY(status == OMPI_SUCCESS) ) {
-        MCA_PML_OB1_COMPUTE_SEGMENT_LENGTH( des->des_dst, des->des_dst_cnt,
-                                            0, bytes_received );
+        bytes_received = mca_pml_ob1_compute_segment_length (btl->btl_seg_size,
+                                                             (void *) des->des_dst,
+                                                             des->des_dst_cnt, 0);
     }
     OPAL_THREAD_ADD_SIZE_T(&recvreq->req_pipeline_depth,-1);
 
@@ -250,7 +255,7 @@ static int mca_pml_ob1_recv_request_ack(
     ompi_proc_t* proc = (ompi_proc_t*)recvreq->req_recv.req_base.req_proc;
     mca_bml_base_endpoint_t* bml_endpoint = NULL;
 
-    bml_endpoint = (mca_bml_base_endpoint_t*) proc->proc_bml; 
+    bml_endpoint = (mca_bml_base_endpoint_t*) proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML]; 
 
     /* by default copy everything */
     recvreq->req_send_offset = bytes_received;
@@ -261,11 +266,11 @@ static int mca_pml_ob1_recv_request_ack(
          * registered. 
          */
 
-        if(ompi_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == 0 &&
+        if(opal_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == 0 &&
            hdr->hdr_match.hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_CONTIG &&
            rdma_num != 0) {
             unsigned char *base;
-            ompi_convertor_get_current_pointer( &recvreq->req_recv.req_base.req_convertor, (void**)&(base) );
+            opal_convertor_get_current_pointer( &recvreq->req_recv.req_base.req_convertor, (void**)&(base) );
            
             if(hdr->hdr_match.hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_PIN)
                 recvreq->req_rdma_cnt = mca_pml_ob1_rdma_btls(bml_endpoint,
@@ -288,7 +293,7 @@ static int mca_pml_ob1_recv_request_ack(
 
                 /* use converter to figure out the rdma offset for this
                  * request */
-                ompi_convertor_set_position(&recvreq->req_recv.req_base.req_convertor,
+                opal_convertor_set_position(&recvreq->req_recv.req_base.req_convertor,
                         &recvreq->req_send_offset);
 
                 recvreq->req_rdma_cnt =
@@ -308,7 +313,6 @@ static int mca_pml_ob1_recv_request_ack(
                                              recvreq->req_send_offset == bytes_received);
 }
 
-
 /**
  * Return resources used by the RDMA
  */
@@ -325,17 +329,19 @@ static void mca_pml_ob1_rget_completion( mca_btl_base_module_t* btl,
     /* check completion status */
     if( OPAL_UNLIKELY(OMPI_SUCCESS != status) ) {
         /* TSW - FIX */
-        ORTE_ERROR_LOG(status);
-        orte_errmgr.abort(-1, NULL);
+        OMPI_ERROR_LOG(status);
+        ompi_rte_abort(-1, NULL);
     }
 
-    mca_pml_ob1_send_fin(recvreq->req_recv.req_base.req_proc,
-                         bml_btl,
-                         frag->rdma_hdr.hdr_rget.hdr_des,
-                         des->order, 0); 
-
     /* is receive request complete */
     OPAL_THREAD_ADD_SIZE_T(&recvreq->req_bytes_received, frag->rdma_length);
+    if (recvreq->req_recv.req_bytes_packed <= recvreq->req_bytes_received) {
+        mca_pml_ob1_send_fin(recvreq->req_recv.req_base.req_proc,
+                             bml_btl,
+                             frag->rdma_hdr.hdr_rget.hdr_des,
+                             des->order, 0);
+    }
+
     recv_request_pml_complete_check(recvreq);
 
     MCA_PML_OB1_RDMA_FRAG_RETURN(frag);
@@ -344,6 +350,58 @@ static void mca_pml_ob1_rget_completion( mca_btl_base_module_t* btl,
 }
 
 
+static int mca_pml_ob1_init_get_fallback (mca_pml_ob1_rdma_frag_t *frag,
+                                          mca_btl_base_descriptor_t *dst) {
+    mca_pml_ob1_recv_request_t *recvreq = (mca_pml_ob1_recv_request_t *) frag->rdma_req;
+    mca_bml_base_btl_t *bml_btl = frag->rdma_bml;
+    mca_btl_base_descriptor_t *ctl;
+    mca_pml_ob1_rdma_hdr_t *hdr;
+    size_t seg_size;
+    int rc;
+
+    seg_size = bml_btl->btl->btl_seg_size * dst->des_dst_cnt;
+
+    /* prepare a descriptor for rdma control message */
+    mca_bml_base_alloc (bml_btl, &ctl, MCA_BTL_NO_ORDER, sizeof (mca_pml_ob1_rdma_hdr_t) + seg_size,
+                        MCA_BTL_DES_FLAGS_PRIORITY | MCA_BTL_DES_FLAGS_BTL_OWNERSHIP |
+                        MCA_BTL_DES_SEND_ALWAYS_CALLBACK);
+    if (OPAL_UNLIKELY(NULL == ctl)) {
+        return OMPI_ERR_OUT_OF_RESOURCE;
+    }
+    ctl->des_cbfunc = mca_pml_ob1_recv_ctl_completion;
+        
+    /* fill in rdma header */
+    hdr = (mca_pml_ob1_rdma_hdr_t *) ctl->des_src->seg_addr.pval;
+    hdr->hdr_common.hdr_type = MCA_PML_OB1_HDR_TYPE_PUT;
+    hdr->hdr_common.hdr_flags =
+        (!recvreq->req_ack_sent) ? MCA_PML_OB1_HDR_TYPE_ACK : 0;
+
+    hdr->hdr_req = frag->rdma_hdr.hdr_rget.hdr_rndv.hdr_src_req;
+    hdr->hdr_rdma_offset = recvreq->req_rdma_offset;
+    hdr->hdr_des.pval = dst;
+    hdr->hdr_recv_req.pval = recvreq;
+
+    hdr->hdr_seg_cnt = dst->des_dst_cnt;
+
+    /* copy segments */
+    memcpy (hdr + 1, dst->des_dst, seg_size);
+
+    dst->des_cbfunc = mca_pml_ob1_put_completion;
+    dst->des_cbdata = recvreq;
+
+    if (!recvreq->req_ack_sent)
+        recvreq->req_ack_sent = true;
+
+    /* send rdma request to peer */
+    rc = mca_bml_base_send (bml_btl, ctl, MCA_PML_OB1_HDR_TYPE_PUT);
+    if (OPAL_UNLIKELY(rc < 0)) {
+        mca_bml_base_free (bml_btl, ctl);
+        return rc;
+    }
+
+    return OMPI_SUCCESS;
+}
+
 /*
  *
  */
@@ -362,17 +420,29 @@ int mca_pml_ob1_recv_request_get_frag( mca_pml_ob1_rdma_frag_t* frag )
                               MCA_BTL_NO_ORDER,
                               0,
                               &frag->rdma_length,
-                              MCA_BTL_DES_FLAGS_BTL_OWNERSHIP | MCA_BTL_DES_SEND_ALWAYS_CALLBACK,
+                              MCA_BTL_DES_FLAGS_BTL_OWNERSHIP | MCA_BTL_DES_SEND_ALWAYS_CALLBACK |
+                              MCA_BTL_DES_FLAGS_GET,
                               &descriptor );
     if( OPAL_UNLIKELY(NULL == descriptor) ) {
-        frag->rdma_length = save_size;
-        OPAL_THREAD_LOCK(&mca_pml_ob1.lock);
-        opal_list_append(&mca_pml_ob1.rdma_pending, (opal_list_item_t*)frag);
-        OPAL_THREAD_UNLOCK(&mca_pml_ob1.lock);
-        return OMPI_ERR_OUT_OF_RESOURCE;
+        if (frag->retries < mca_pml_ob1.rdma_retries_limit) {
+            frag->rdma_length = save_size;
+            OPAL_THREAD_LOCK(&mca_pml_ob1.lock);
+            opal_list_append(&mca_pml_ob1.rdma_pending, (opal_list_item_t*)frag);
+            OPAL_THREAD_UNLOCK(&mca_pml_ob1.lock);
+            return OMPI_ERR_OUT_OF_RESOURCE;
+        } else {
+            ompi_proc_t *proc = (ompi_proc_t *) recvreq->req_recv.req_base.req_proc;
+
+            /* tell peer to fall back on send */
+            recvreq->req_send_offset = 0;
+            rc = mca_pml_ob1_recv_request_ack_send(proc, frag->rdma_hdr.hdr_rget.hdr_rndv.hdr_src_req.lval,
+                                                   recvreq, recvreq->req_send_offset, true);
+            MCA_PML_OB1_RDMA_FRAG_RETURN(frag);
+            return rc;
+        }
     }
 
-    descriptor->des_src = frag->rdma_segs;
+    descriptor->des_src = (mca_btl_base_segment_t *) frag->rdma_segs;
     descriptor->des_src_cnt = frag->rdma_hdr.hdr_rdma.hdr_seg_cnt;
     descriptor->des_cbfunc = mca_pml_ob1_rget_completion;
     descriptor->des_cbdata = frag;
@@ -384,6 +454,11 @@ int mca_pml_ob1_recv_request_get_frag( mca_pml_ob1_rdma_frag_t* frag )
     /* queue up get request */
     rc = mca_bml_base_get(bml_btl,descriptor);
     if( OPAL_UNLIKELY(OMPI_SUCCESS != rc) ) {
+        if (OPAL_UNLIKELY(OMPI_ERR_NOT_AVAILABLE == rc)) {
+            /* get isn't supported for this transfer. tell peer to fallback on put */
+            rc = mca_pml_ob1_init_get_fallback (frag, descriptor);
+         }
+
         if(OMPI_ERR_OUT_OF_RESOURCE == rc) {
             mca_bml_base_free(bml_btl, descriptor);
             OPAL_THREAD_LOCK(&mca_pml_ob1.lock);
@@ -391,9 +466,9 @@ int mca_pml_ob1_recv_request_get_frag( mca_pml_ob1_rdma_frag_t* frag )
                     (opal_list_item_t*)frag);
             OPAL_THREAD_UNLOCK(&mca_pml_ob1.lock);
             return OMPI_ERR_OUT_OF_RESOURCE;
-        } else {
-            ORTE_ERROR_LOG(rc);
-            orte_errmgr.abort(-1, NULL);
+        } else if (OPAL_UNLIKELY(OMPI_SUCCESS != rc)) {
+            OMPI_ERROR_LOG(rc);
+            ompi_rte_abort(-1, NULL);
         }
     }
 
@@ -413,17 +488,15 @@ void mca_pml_ob1_recv_request_progress_frag( mca_pml_ob1_recv_request_t* recvreq
                                              mca_btl_base_segment_t* segments,
                                              size_t num_segments )
 {
-    size_t bytes_received = 0;
-    size_t bytes_delivered = 0;
-    size_t data_offset = 0;
+    size_t bytes_received, data_offset = 0;
+    size_t bytes_delivered __opal_attribute_unused__; /* is being set to zero in MCA_PML_OB1_RECV_REQUEST_UNPACK */
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
 
-    MCA_PML_OB1_COMPUTE_SEGMENT_LENGTH( segments, num_segments,
-                                        0, bytes_received );
-    bytes_received -= sizeof(mca_pml_ob1_frag_hdr_t);
+    bytes_received = mca_pml_ob1_compute_segment_length_base (segments, num_segments,
+                                                              sizeof(mca_pml_ob1_frag_hdr_t));
     data_offset     = hdr->hdr_frag.hdr_frag_offset;
     /*
-     *  Make user buffer accessable(defined) before unpacking.
+     *  Make user buffer accessible(defined) before unpacking.
      */
     MEMCHECKER(
                memchecker_call(&opal_memchecker_base_mem_defined,
@@ -457,6 +530,85 @@ void mca_pml_ob1_recv_request_progress_frag( mca_pml_ob1_recv_request_t* recvreq
     }
 }
 
+#if OMPI_CUDA_SUPPORT /* CUDA_ASYNC_RECV */
+/**
+ * This function is basically the first half of the code in the
+ * mca_pml_ob1_recv_request_progress_frag function.  This fires off
+ * the asynchronous copy and returns.  Unused fields in the descriptor
+ * are used to pass extra information for when the asynchronous copy
+ * completes.  No memchecker support in this function as copies are 
+ * happening asynchronously.
+ */
+void mca_pml_ob1_recv_request_frag_copy_start( mca_pml_ob1_recv_request_t* recvreq,
+                                               mca_btl_base_module_t* btl,
+                                               mca_btl_base_segment_t* segments,
+                                               size_t num_segments,
+                                               mca_btl_base_descriptor_t* des)
+{
+    int result;
+    size_t bytes_received = 0, data_offset = 0;
+    size_t bytes_delivered __opal_attribute_unused__; /* is being set to zero in MCA_PML_OB1_RECV_REQUEST_UNPACK */
+    mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
+
+    OPAL_OUTPUT((-1, "start_frag_copy frag=%p", (void *)des));
+
+    bytes_received = mca_pml_ob1_compute_segment_length_base (segments, num_segments,
+                                                              sizeof(mca_pml_ob1_frag_hdr_t));
+    data_offset     = hdr->hdr_frag.hdr_frag_offset;
+
+    MCA_PML_OB1_RECV_REQUEST_UNPACK( recvreq,
+                                     segments,
+                                     num_segments,
+                                     sizeof(mca_pml_ob1_frag_hdr_t),
+                                     data_offset,
+                                     bytes_received,
+                                     bytes_delivered );
+    /* Store the receive request in unused context pointer. */
+    des->des_context = (void *)recvreq;
+    /* Store the amount of bytes in unused src count value */
+    des->des_src_cnt = bytes_delivered;
+    /* Then record an event that will get triggered by a PML progress call which
+     * checks the stream events.  If we get an error, abort.  Should get message
+     * from CUDA code about what went wrong. */
+    result = mca_common_cuda_record_htod_event("pml", des);
+    if (OMPI_SUCCESS != result) {
+        opal_output(0, "%s:%d FATAL", __FILE__, __LINE__);
+        ompi_rte_abort(-1, NULL);
+    }
+}
+
+/**
+ * This function is basically the second half of the code in the
+ * mca_pml_ob1_recv_request_progress_frag function.  The number of
+ * bytes delivered is updated.  Then a call is made into the BTL so it
+ * can free the fragment that held that data.  This is currently
+ * called directly by the common CUDA code. No memchecker support
+ * in this function as copies are happening asynchronously.
+ */
+void mca_pml_ob1_recv_request_frag_copy_finished( mca_btl_base_module_t* btl,
+                                                  struct mca_btl_base_endpoint_t* ep,
+                                                  struct mca_btl_base_descriptor_t* des,
+                                                  int status )
+{
+    mca_pml_ob1_recv_request_t* recvreq = (mca_pml_ob1_recv_request_t*)des->des_context;
+    size_t bytes_received = des->des_src_cnt;
+
+    OPAL_OUTPUT((-1, "frag_copy_finished (delivered=%d), frag=%p", (int)bytes_received, (void *)des));
+    /* Call into the BTL so it can free the descriptor.  At this point, it is 
+     * known that the data has been copied out of the descriptor. */
+    des->des_cbfunc(NULL, (struct mca_btl_base_endpoint_t *)des->des_cbdata, des, 0);
+
+    OPAL_THREAD_ADD_SIZE_T(&recvreq->req_bytes_received, bytes_received);
+
+    /* check completion status */
+    if(recv_request_pml_complete_check(recvreq) == false &&
+            recvreq->req_rdma_offset < recvreq->req_send_offset) {
+        /* schedule additional rdma operations */
+        mca_pml_ob1_recv_request_schedule(recvreq, NULL);
+    }
+}
+#endif /* OMPI_CUDA_SUPPORT */
+
 /*
  * Update the recv request status to reflect the number of bytes
  * received and actually delivered to the application. 
@@ -467,15 +619,16 @@ void mca_pml_ob1_recv_request_progress_rget( mca_pml_ob1_recv_request_t* recvreq
                                              mca_btl_base_segment_t* segments,
                                              size_t num_segments )
 {
-    size_t bytes_received = 0;
     mca_pml_ob1_rget_hdr_t* hdr = (mca_pml_ob1_rget_hdr_t*)segments->seg_addr.pval;
     mca_bml_base_endpoint_t* bml_endpoint = NULL;
-    mca_pml_ob1_rdma_frag_t* frag;
-    size_t i, size = 0;
+    size_t bytes_remaining, prev_sent, offset;
+    mca_btl_base_segment_t *r_segments;
+    mca_pml_ob1_rdma_frag_t *frag;
+    mca_bml_base_btl_t *rdma_bml;
     int rc;
 
-    MCA_PML_OB1_COMPUTE_SEGMENT_LENGTH( segments, num_segments,
-                                        0, bytes_received );
+    prev_sent = offset = 0;
+    bytes_remaining = hdr->hdr_rndv.hdr_msg_length;
     recvreq->req_recv.req_bytes_packed = hdr->hdr_rndv.hdr_msg_length;
 
     MCA_PML_OB1_RECV_REQUEST_MATCHED(recvreq, &hdr->hdr_rndv.hdr_match);
@@ -484,48 +637,94 @@ void mca_pml_ob1_recv_request_progress_rget( mca_pml_ob1_recv_request_t* recvreq
      * fall back to copy in/out protocol. It is a pity because buffer on the
      * sender side is already registered. We need to be smarter here, perhaps
      * do couple of RDMA reads */
-    if(ompi_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == true) {
-        mca_pml_ob1_recv_request_ack(recvreq, &hdr->hdr_rndv, 0);
-        return;
-    }
-    
-    MCA_PML_OB1_RDMA_FRAG_ALLOC(frag,rc);
-    if( OPAL_UNLIKELY(NULL == frag) ) {
-        /* GLB - FIX */
-         ORTE_ERROR_LOG(rc);
-         orte_errmgr.abort(-1, NULL);
-    }
-
-    /* lookup bml datastructures */
-    bml_endpoint = (mca_bml_base_endpoint_t*)recvreq->req_recv.req_base.req_proc->proc_bml; 
-
-    /* allocate/initialize a fragment */
-    for(i = 0; i < hdr->hdr_seg_cnt; i++) {
-        frag->rdma_segs[i] = hdr->hdr_segs[i];
-#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
-        if ((recvreq->req_recv.req_base.req_proc->proc_arch & OPAL_ARCH_ISBIGENDIAN) !=
-            (ompi_proc_local()->proc_arch & OPAL_ARCH_ISBIGENDIAN)) {
-            size += opal_swap_bytes4(hdr->hdr_segs[i].seg_len);
-        } else 
-#endif
+    if (opal_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == true) {
+#if OMPI_CUDA_SUPPORT
+        if (mca_pml_ob1_cuda_need_buffers(recvreq, btl))
+#endif /* OMPI_CUDA_SUPPORT */
         {
-            size += hdr->hdr_segs[i].seg_len;
+            mca_pml_ob1_recv_request_ack(recvreq, &hdr->hdr_rndv, 0);
+            return;
         }
     }
-    frag->rdma_bml = mca_bml_base_btl_array_find(&bml_endpoint->btl_rdma, btl);
-    if( OPAL_UNLIKELY(NULL == frag->rdma_bml) ) {
+    
+    /* lookup bml datastructures */
+    bml_endpoint = (mca_bml_base_endpoint_t*)recvreq->req_recv.req_base.req_proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_BML];
+    rdma_bml = mca_bml_base_btl_array_find(&bml_endpoint->btl_rdma, btl);
+
+#if OMPI_CUDA_SUPPORT
+    if (OPAL_UNLIKELY(NULL == rdma_bml)) {
+        if (recvreq->req_recv.req_base.req_convertor.flags & CONVERTOR_CUDA) {
+            /* Check to see if this is a CUDA get */
+            if (btl->btl_flags & MCA_BTL_FLAGS_CUDA_GET) {
+                rdma_bml = mca_bml_base_btl_array_find(&bml_endpoint->btl_send, btl);
+            }
+        } else {
+            /* Just default back to send and receive.  Must be mix of GPU and HOST memory. */
+            mca_pml_ob1_recv_request_ack(recvreq, &hdr->hdr_rndv, 0);
+            return;
+        }
+    }
+#endif /* OMPI_CUDA_SUPPORT */
+
+    if (OPAL_UNLIKELY(NULL == rdma_bml)) {
         opal_output(0, "[%s:%d] invalid bml for rdma get", __FILE__, __LINE__);
         orte_errmgr.abort(-1, NULL);
     }
-    frag->rdma_hdr.hdr_rget = *hdr;
-    frag->rdma_req = recvreq;
-    frag->rdma_ep = bml_endpoint;
-    frag->rdma_length = size;
-    frag->rdma_state = MCA_PML_OB1_RDMA_GET;
-    frag->reg = NULL;
 
-    mca_pml_ob1_recv_request_get_frag(frag);
-    return;
+    bytes_remaining = mca_pml_ob1_compute_segment_length_remote (btl->btl_seg_size, (void *)(hdr + 1),
+                                                                 hdr->hdr_seg_cnt, recvreq->req_recv.req_base.req_proc);
+
+    /* The while loop adds a fragmentation mechanism. The variable bytes_remaining holds the num
+     * of bytes left to be send. In each iteration we send the max possible bytes supported
+     * by the HCA. The field  frag->rdma_length holds the actual num of  bytes that were
+     * sent in each iteration. We subtract this number from bytes_remaining and continue to
+     * the next iteration with the updated size.
+     * Also - In each iteration we update the location in the buffer to be used for writing
+     * the message ,and the location to read from. This is done using the offset variable that
+     * accumulates the number of bytes that were sent so far. */
+    while (bytes_remaining > 0) {
+        /* allocate/initialize a fragment */
+        MCA_PML_OB1_RDMA_FRAG_ALLOC(frag);
+        if (OPAL_UNLIKELY(NULL == frag)) {
+            /* GLB - FIX */
+             ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
+             ompi_rte_abort(-1, NULL);
+        }
+
+        assert (btl->btl_seg_size * hdr->hdr_seg_cnt <= sizeof (frag->rdma_segs));
+
+        memcpy (frag->rdma_segs, hdr + 1, btl->btl_seg_size * hdr->hdr_seg_cnt);
+
+        /* update the read location -- NTH: note this will only work if there is exactly one
+           segment. TODO -- make this work with multiple segments */
+        r_segments = (mca_btl_base_segment_t *) frag->rdma_segs;
+        r_segments->seg_addr.lval += offset;
+
+        /* updating the write location */
+        OPAL_THREAD_LOCK(&recvreq->lock);
+        opal_convertor_set_position( &recvreq->req_recv.req_base.req_convertor, &offset);
+        OPAL_THREAD_UNLOCK(&recvreq->lock);
+
+        frag->rdma_bml = rdma_bml;
+
+        frag->rdma_hdr.hdr_rget = *hdr;
+        frag->retries     = 0;
+        frag->rdma_req    = recvreq;
+        frag->rdma_ep     = bml_endpoint;
+        frag->rdma_state  = MCA_PML_OB1_RDMA_GET;
+        frag->reg         = NULL;
+        frag->rdma_length = bytes_remaining;
+
+        /* NTH: TODO -- handle error conditions gracefully */
+        rc = mca_pml_ob1_recv_request_get_frag(frag);
+        if (OMPI_SUCCESS != rc) {
+            break;
+        }
+
+        prev_sent = frag->rdma_length;
+        bytes_remaining -= prev_sent;
+        offset += prev_sent;
+    }
 }
 
 /*
@@ -539,14 +738,13 @@ void mca_pml_ob1_recv_request_progress_rndv( mca_pml_ob1_recv_request_t* recvreq
                                              size_t num_segments )
 {
     size_t bytes_received = 0;
-    size_t bytes_delivered = 0;
+    size_t bytes_delivered __opal_attribute_unused__; /* is being set to zero in MCA_PML_OB1_RECV_REQUEST_UNPACK */
     size_t data_offset = 0;
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
 
-    MCA_PML_OB1_COMPUTE_SEGMENT_LENGTH( segments, num_segments,
-                                        0, bytes_received );
-    
-    bytes_received -= sizeof(mca_pml_ob1_rendezvous_hdr_t);
+    bytes_received = mca_pml_ob1_compute_segment_length_base (segments, num_segments,
+                                                              sizeof(mca_pml_ob1_rendezvous_hdr_t));
+
     recvreq->req_recv.req_bytes_packed = hdr->hdr_rndv.hdr_msg_length;
     recvreq->remote_req_send = hdr->hdr_rndv.hdr_src_req;
     recvreq->req_rdma_offset = bytes_received;
@@ -585,6 +783,17 @@ void mca_pml_ob1_recv_request_progress_rndv( mca_pml_ob1_recv_request_t* recvreq
         /* schedule additional rdma operations */
         mca_pml_ob1_recv_request_schedule(recvreq, NULL);
     }
+
+#if OMPI_CUDA_SUPPORT /* CUDA_ASYNC_RECV */
+    /* If BTL supports it and this is a CUDA buffer being received into,
+     * have all subsequent FRAGS copied in asynchronously. */
+    if ((recvreq->req_recv.req_base.req_convertor.flags & CONVERTOR_CUDA) &&
+        (btl->btl_flags & MCA_BTL_FLAGS_CUDA_COPY_ASYNC_RECV)) {
+        void *strm = mca_common_cuda_get_htod_stream();
+        opal_cuda_set_copy_function_async(&recvreq->req_recv.req_base.req_convertor, strm);
+    }
+#endif
+    
 }
 
 /*
@@ -596,12 +805,13 @@ void mca_pml_ob1_recv_request_progress_match( mca_pml_ob1_recv_request_t* recvre
                                               mca_btl_base_segment_t* segments,
                                               size_t num_segments )
 {
-    size_t bytes_received = 0, bytes_delivered = 0, data_offset = 0;
+    size_t bytes_received, data_offset = 0;
+    size_t bytes_delivered __opal_attribute_unused__; /* is being set to zero in MCA_PML_OB1_RECV_REQUEST_UNPACK */
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
 
-    MCA_PML_OB1_COMPUTE_SEGMENT_LENGTH( segments, num_segments,
-                                        0, bytes_received );
-    bytes_received -= OMPI_PML_OB1_MATCH_HDR_LEN;
+    bytes_received = mca_pml_ob1_compute_segment_length_base (segments, num_segments,
+                                                              OMPI_PML_OB1_MATCH_HDR_LEN);
+
     recvreq->req_recv.req_bytes_packed = bytes_received;
     
     MCA_PML_OB1_RECV_REQUEST_MATCHED(recvreq, &hdr->hdr_match);
@@ -654,12 +864,9 @@ void mca_pml_ob1_recv_request_matched_probe( mca_pml_ob1_recv_request_t* recvreq
 
     switch(hdr->hdr_common.hdr_type) {
         case MCA_PML_OB1_HDR_TYPE_MATCH:
-
-            MCA_PML_OB1_COMPUTE_SEGMENT_LENGTH( segments, num_segments,
-                                                OMPI_PML_OB1_MATCH_HDR_LEN,
-                                                bytes_packed );
+            bytes_packed = mca_pml_ob1_compute_segment_length_base (segments, num_segments,
+                                                                    OMPI_PML_OB1_MATCH_HDR_LEN);
             break;
-
         case MCA_PML_OB1_HDR_TYPE_RNDV:
         case MCA_PML_OB1_HDR_TYPE_RGET:
 
@@ -671,7 +878,8 @@ void mca_pml_ob1_recv_request_matched_probe( mca_pml_ob1_recv_request_t* recvreq
     recvreq->req_recv.req_base.req_ompi.req_status.MPI_TAG = hdr->hdr_match.hdr_tag;
     recvreq->req_recv.req_base.req_ompi.req_status.MPI_SOURCE = hdr->hdr_match.hdr_src;
     recvreq->req_bytes_received = bytes_packed;
-    recvreq->req_bytes_delivered = bytes_packed;
+    recvreq->req_bytes_expected = bytes_packed;
+
     recv_request_pml_complete(recvreq);
 }
 
@@ -704,8 +912,7 @@ int mca_pml_ob1_recv_request_schedule_once( mca_pml_ob1_recv_request_t* recvreq,
 
     while(bytes_remaining > 0 &&
            recvreq->req_pipeline_depth < mca_pml_ob1.recv_pipeline_depth) {
-        size_t hdr_size;
-        size_t size;
+        size_t size, seg_size;
         mca_pml_ob1_rdma_hdr_t* hdr;
         mca_btl_base_descriptor_t* dst;
         mca_btl_base_descriptor_t* ctl;
@@ -749,13 +956,14 @@ int mca_pml_ob1_recv_request_schedule_once( mca_pml_ob1_recv_request_t* recvreq,
         /* take lock to protect converter against concurrent access
          * from unpack */
         OPAL_THREAD_LOCK(&recvreq->lock);
-        ompi_convertor_set_position( &recvreq->req_recv.req_base.req_convertor,
+        opal_convertor_set_position( &recvreq->req_recv.req_base.req_convertor,
                                      &recvreq->req_rdma_offset );
 
         /* prepare a descriptor for RDMA */
         mca_bml_base_prepare_dst(bml_btl, reg, 
                                  &recvreq->req_recv.req_base.req_convertor,
-                                 MCA_BTL_NO_ORDER, 0, &size, MCA_BTL_DES_FLAGS_BTL_OWNERSHIP, &dst);
+                                 MCA_BTL_NO_ORDER, 0, &size, MCA_BTL_DES_FLAGS_BTL_OWNERSHIP |
+                                 MCA_BTL_DES_FLAGS_PUT, &dst);
         OPAL_THREAD_UNLOCK(&recvreq->lock);
 
         if(OPAL_UNLIKELY(dst == NULL)) {
@@ -765,14 +973,10 @@ int mca_pml_ob1_recv_request_schedule_once( mca_pml_ob1_recv_request_t* recvreq,
         dst->des_cbfunc = mca_pml_ob1_put_completion;
         dst->des_cbdata = recvreq;
 
-        /* prepare a descriptor for rdma control message */
-        hdr_size = sizeof(mca_pml_ob1_rdma_hdr_t);
-        if(dst->des_dst_cnt > 1) {
-            hdr_size += (sizeof(mca_btl_base_segment_t) *
-                    (dst->des_dst_cnt-1));
-        }
+        seg_size = btl->btl_seg_size * dst->des_dst_cnt;
 
-        mca_bml_base_alloc(bml_btl, &ctl, MCA_BTL_NO_ORDER, hdr_size,
+        /* prepare a descriptor for rdma control message */
+        mca_bml_base_alloc(bml_btl, &ctl, MCA_BTL_NO_ORDER, sizeof(mca_pml_ob1_rdma_hdr_t) + seg_size,
                            MCA_BTL_DES_FLAGS_PRIORITY | MCA_BTL_DES_FLAGS_BTL_OWNERSHIP | MCA_BTL_DES_SEND_ALWAYS_CALLBACK);
 
         if( OPAL_UNLIKELY(NULL == ctl) ) {
@@ -788,14 +992,12 @@ int mca_pml_ob1_recv_request_schedule_once( mca_pml_ob1_recv_request_t* recvreq,
             (!recvreq->req_ack_sent) ? MCA_PML_OB1_HDR_TYPE_ACK : 0;
         hdr->hdr_req = recvreq->remote_req_send;
         hdr->hdr_des.pval = dst;
+        hdr->hdr_recv_req.pval = recvreq;
         hdr->hdr_rdma_offset = recvreq->req_rdma_offset;
         hdr->hdr_seg_cnt = dst->des_dst_cnt;
 
-        for( i = 0; i < dst->des_dst_cnt; i++ ) {
-            hdr->hdr_segs[i].seg_addr.lval = ompi_ptr_ptol(dst->des_dst[i].seg_addr.pval);
-            hdr->hdr_segs[i].seg_len       = dst->des_dst[i].seg_len;
-            hdr->hdr_segs[i].seg_key.key64 = dst->des_dst[i].seg_key.key64;
-        }
+        /* copy segments */
+        memmove (hdr + 1, dst->des_dst, seg_size);
 
         if(!recvreq->req_ack_sent)
             recvreq->req_ack_sent = true;
@@ -824,12 +1026,18 @@ int mca_pml_ob1_recv_request_schedule_once( mca_pml_ob1_recv_request_t* recvreq,
 
 #define IS_PROB_REQ(R) \
     ((MCA_PML_REQUEST_IPROBE == (R)->req_recv.req_base.req_type) || \
-     (MCA_PML_REQUEST_PROBE == (R)->req_recv.req_base.req_type))
+     (MCA_PML_REQUEST_PROBE == (R)->req_recv.req_base.req_type) || \
+     (MCA_PML_REQUEST_IMPROBE == (R)->req_recv.req_base.req_type) || \
+     (MCA_PML_REQUEST_MPROBE == (R)->req_recv.req_base.req_type))
+#define IS_MPROB_REQ(R) \
+    ((MCA_PML_REQUEST_IMPROBE == (R)->req_recv.req_base.req_type) || \
+     (MCA_PML_REQUEST_MPROBE == (R)->req_recv.req_base.req_type))
 
 static inline void append_recv_req_to_queue(opal_list_t *queue,
         mca_pml_ob1_recv_request_t *req)
 {
-    if(OPAL_UNLIKELY(req->req_recv.req_base.req_type == MCA_PML_REQUEST_IPROBE))
+    if(OPAL_UNLIKELY(req->req_recv.req_base.req_type == MCA_PML_REQUEST_IPROBE ||
+                     req->req_recv.req_base.req_type == MCA_PML_REQUEST_IMPROBE))
         return;
 
     opal_list_append(queue, (opal_list_item_t*)req);
@@ -839,7 +1047,8 @@ static inline void append_recv_req_to_queue(opal_list_t *queue,
      * the compiler will optimize out the empty if loop in the case where PERUSE
      * support is not required by the user.
      */
-    if(req->req_recv.req_base.req_type != MCA_PML_REQUEST_PROBE) {
+    if(req->req_recv.req_base.req_type != MCA_PML_REQUEST_PROBE ||
+       req->req_recv.req_base.req_type != MCA_PML_REQUEST_MPROBE) {
         PERUSE_TRACE_COMM_EVENT(PERUSE_COMM_REQ_INSERT_IN_POSTED_Q,
                                 &(req->req_recv.req_base), PERUSE_RECV);
     }
@@ -894,20 +1103,35 @@ recv_req_match_wild( mca_pml_ob1_recv_request_t* req,
 {
     mca_pml_ob1_comm_t* comm = req->req_recv.req_base.req_comm->c_pml_comm;
     mca_pml_ob1_comm_proc_t* proc = comm->procs;
-    size_t proc_count = comm->num_procs, i;
+    size_t i;
 
     /*
      * Loop over all the outstanding messages to find one that matches.
      * There is an outer loop over lists of messages from each
      * process, then an inner loop over the messages from the
      * process.
+     *
+     * In order to avoid starvation do this in a round-robin fashion.
      */
-    for (i = 0; i < proc_count; i++) {
+    for (i = comm->last_probed + 1; i < comm->num_procs; i++) {
         mca_pml_ob1_recv_frag_t* frag;
 
         /* loop over messages from the current proc */
         if((frag = recv_req_match_specific_proc(req, &proc[i]))) {
             *p = &proc[i];
+            comm->last_probed = i;
+            req->req_recv.req_base.req_proc = proc[i].ompi_proc;
+            prepare_recv_req_converter(req);
+            return frag; /* match found */
+        }
+    }
+    for (i = 0; i <= comm->last_probed; i++) {
+        mca_pml_ob1_recv_frag_t* frag;
+
+        /* loop over messages from the current proc */
+        if((frag = recv_req_match_specific_proc(req, &proc[i]))) {
+            *p = &proc[i];
+            comm->last_probed = i;
             req->req_recv.req_base.req_proc = proc[i].ompi_proc;
             prepare_recv_req_converter(req);
             return frag; /* match found */
@@ -929,9 +1153,9 @@ void mca_pml_ob1_recv_req_start(mca_pml_ob1_recv_request_t *req)
 
     /* init/re-init the request */
     req->req_lock = 0;
-    req->req_pipeline_depth  = 0;
-    req->req_bytes_received  = 0;
-    req->req_bytes_delivered = 0;
+    req->req_pipeline_depth = 0;
+    req->req_bytes_received = 0;
+    req->req_bytes_expected = 0;
     /* What about req_rdma_cnt ? */
     req->req_rdma_idx = 0;
     req->req_pending = false;
@@ -954,7 +1178,7 @@ void mca_pml_ob1_recv_req_start(mca_pml_ob1_recv_request_t *req)
     if(req->req_recv.req_base.req_peer == OMPI_ANY_SOURCE) {
         frag = recv_req_match_wild(req, &proc);
         queue = &comm->wild_receives;
-#if !OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+#if !OPAL_ENABLE_HETEROGENEOUS_SUPPORT
         /* As we are in a homogeneous environment we know that all remote
          * architectures are exactly the same as the local one. Therefore,
          * we can safely construct the convertor based on the proc
@@ -964,7 +1188,7 @@ void mca_pml_ob1_recv_req_start(mca_pml_ob1_recv_request_t *req)
             req->req_recv.req_base.req_proc = ompi_proc_local_proc;
             prepare_recv_req_converter(req);
         }
-#endif  /* !OMPI_ENABLE_HETEROGENEOUS_SUPPORT */
+#endif  /* !OPAL_ENABLE_HETEROGENEOUS_SUPPORT */
     } else {
         proc = &comm->procs[req->req_recv.req_base.req_peer];
         req->req_recv.req_base.req_proc = proc->ompi_proc;
@@ -1020,6 +1244,21 @@ void mca_pml_ob1_recv_req_start(mca_pml_ob1_recv_request_t *req)
             
             MCA_PML_OB1_RECV_FRAG_RETURN(frag);
 
+        } else if (OPAL_UNLIKELY(IS_MPROB_REQ(req))) {
+            /* Remove the fragment from the match list, as it's now
+               matched.  Stash it somewhere in the request (which,
+               yes, is a complete hack), where it will be plucked out
+               during the end of mprobe.  The request will then be
+               "recreated" as a receive request, and the frag will be
+               restarted with this request during mrecv */
+            opal_list_remove_item(&proc->unexpected_frags,
+                                  (opal_list_item_t*)frag);
+            OPAL_THREAD_UNLOCK(&comm->matching_lock);
+
+            req->req_recv.req_base.req_addr = frag;
+            mca_pml_ob1_recv_request_matched_probe(req, frag->btl,
+                                                   frag->segments, frag->num_segments);
+
         } else {
             OPAL_THREAD_UNLOCK(&comm->matching_lock);
             mca_pml_ob1_recv_request_matched_probe(req, frag->btl,
