@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2012 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
@@ -10,6 +11,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -54,24 +57,29 @@ int MPI_Alltoall(void *sendbuf, int sendcount, MPI_Datatype sendtype,
 
     if (MPI_PARAM_CHECK) {
 
-      /* Unrooted operation -- same checks for all ranks on both
-         intracommunicators and intercommunicators */
+        /* Unrooted operation -- same checks for all ranks on both
+           intracommunicators and intercommunicators */
 
-      err = MPI_SUCCESS;
-      OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
-      if (ompi_comm_invalid(comm)) {
-	return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM, 
-                                     FUNC_NAME);
-      } else if (MPI_DATATYPE_NULL == recvtype || NULL == recvtype) {
-        err = MPI_ERR_TYPE;
-      } else if (recvcount < 0) {
-        err = MPI_ERR_COUNT;
-      } else if (MPI_IN_PLACE == sendbuf || MPI_IN_PLACE == recvbuf) {
-        err = MPI_ERR_ARG;
-      } else {
-        OMPI_CHECK_DATATYPE_FOR_SEND(err, sendtype, sendcount);
-      }
-      OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
+        if (MPI_IN_PLACE == sendbuf) {
+            sendcount = recvcount;
+            sendtype = recvtype;
+        }
+
+        err = MPI_SUCCESS;
+        OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
+        if (ompi_comm_invalid(comm)) {
+            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM, 
+                                          FUNC_NAME);
+        } else if (MPI_DATATYPE_NULL == recvtype || NULL == recvtype) {
+            err = MPI_ERR_TYPE;
+        } else if (recvcount < 0) {
+            err = MPI_ERR_COUNT;
+        } else if (MPI_IN_PLACE == recvbuf) {
+            err = MPI_ERR_ARG;
+        } else {
+            OMPI_CHECK_DATATYPE_FOR_SEND(err, sendtype, sendcount);
+        }
+        OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
     }
 
     /* Do we need to do anything? */
