@@ -21,12 +21,14 @@
 #include "ompi_config.h"
 
 #include "ompi/mpi/c/bindings.h"
+#include "ompi/runtime/params.h"
+#include "ompi/errhandler/errhandler.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/request/request.h"
 #include "ompi/memchecker.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Intercomm_create = PMPI_Intercomm_create
 #endif
 
@@ -179,7 +181,7 @@ int MPI_Intercomm_create(MPI_Comm local_comm, int local_leader,
                          NULL,                                         /* remote_procs */
                          NULL,                                         /* attrs */
                          local_comm->error_handler,                    /* error handler*/
-                         NULL,                                         /* topo mpodule */
+                         false,                                        /* dont copy the topo */
                          local_comm->c_local_group,                    /* local group */
                          new_group_pointer                             /* remote group */
                          );
@@ -210,7 +212,13 @@ int MPI_Intercomm_create(MPI_Comm local_comm, int local_leader,
     }
 
     /* activate comm and init coll-module */
-    rc = ompi_comm_activate ( &newcomp, 1 );
+    rc = ompi_comm_activate ( &newcomp, 
+                             local_comm,                  /* old comm */
+                             bridge_comm,                 /* bridge comm */
+                             &lleader,                    /* local leader */
+                             &rleader,                    /* remote_leader */
+                             OMPI_COMM_CID_INTRA_BRIDGE,  /* mode */
+                             -1 );                        /* send_first */
     if ( MPI_SUCCESS != rc ) {
         goto err_exit;
     }
