@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -20,10 +20,14 @@
 #include <stdio.h>
 
 #include "ompi/mpi/c/bindings.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/runtime/params.h"
+#include "ompi/communicator/communicator.h"
+#include "ompi/errhandler/errhandler.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/op/op.h"
+#include "ompi/memchecker.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Reduce = PMPI_Reduce
 #endif
 
@@ -39,6 +43,36 @@ int MPI_Reduce(void *sendbuf, void *recvbuf, int count,
 {
     int err;
 
+    MEMCHECKER(
+        memchecker_datatype(datatype);
+        memchecker_comm(comm);
+
+        if(OMPI_COMM_IS_INTRA(comm)) {
+            if(ompi_comm_rank(comm) == root) {
+                /* check whether root's send buffer is defined. */
+                if (MPI_IN_PLACE == sendbuf) {
+                    memchecker_call(&opal_memchecker_base_isdefined, recvbuf, count, datatype);
+                } else {
+                    memchecker_call(&opal_memchecker_base_isdefined, sendbuf, count, datatype);
+                }
+                
+                /* check whether root's receive buffer is addressable. */
+                memchecker_call(&opal_memchecker_base_isaddressable, recvbuf, count, datatype);
+            } else {
+                /* check whether send buffer is defined on other processes. */
+                memchecker_call(&opal_memchecker_base_isdefined, sendbuf, count, datatype);
+            }
+        } else {
+            if (MPI_ROOT == root) {
+                /* check whether root's receive buffer is addressable. */
+                memchecker_call(&opal_memchecker_base_isaddressable, recvbuf, count, datatype);
+            } else if (MPI_PROC_NULL != root) {               
+                /* check whether send buffer is defined. */
+                memchecker_call(&opal_memchecker_base_isdefined, sendbuf, count, datatype);
+            }
+        }
+    );
+
     if (MPI_PARAM_CHECK) {
         char *msg;
         err = MPI_SUCCESS;
@@ -50,27 +84,27 @@ int MPI_Reduce(void *sendbuf, void *recvbuf, int count,
 
         /* Checks for all ranks */
 	
-        else if (MPI_OP_NULL == op) {
-          err = MPI_ERR_OP;
+        else if (MPI_OP_NULL == op || NULL == op) {
+            err = MPI_ERR_OP;
         } else if (!ompi_op_is_valid(op, datatype, &msg, FUNC_NAME)) {
             int ret = OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_OP, msg);
             free(msg);
             return ret;
         } else if ((ompi_comm_rank(comm) != root && MPI_IN_PLACE == sendbuf) ||
-                   (ompi_comm_rank(comm) == root && MPI_IN_PLACE == recvbuf)) {
-          err = MPI_ERR_ARG;
+                   (ompi_comm_rank(comm) == root && ((MPI_IN_PLACE == recvbuf) || (sendbuf == recvbuf)))) {
+            err = MPI_ERR_ARG;
         } else {
-          OMPI_CHECK_DATATYPE_FOR_SEND(err, datatype, count);
+            OMPI_CHECK_DATATYPE_FOR_SEND(err, datatype, count);
         }
         OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
 
         /* Intercommunicator errors */
 
         if (!OMPI_COMM_IS_INTRA(comm)) {
-          if (! ((root >= 0 && root < ompi_comm_remote_size(comm)) ||
-                 MPI_ROOT == root || MPI_PROC_NULL == root)) {
-            return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_ROOT, FUNC_NAME);
-          }
+            if (! ((root >= 0 && root < ompi_comm_remote_size(comm)) ||
+                   MPI_ROOT == root || MPI_PROC_NULL == root)) {
+                return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_ROOT, FUNC_NAME);
+            }
         }
 
         /* Intracommunicator errors */
@@ -90,11 +124,14 @@ int MPI_Reduce(void *sendbuf, void *recvbuf, int count,
         return MPI_SUCCESS;
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
     /* Invoke the coll component to perform the back-end operation */
 
     OBJ_RETAIN(op);
     err = comm->c_coll.coll_reduce(sendbuf, recvbuf, count,
-                                   datatype, op, root, comm);
+                                   datatype, op, root, comm,
+                                   comm->c_coll.coll_reduce_module);
     OBJ_RELEASE(op);
     OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
 }
