@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2009 Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2012 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -60,7 +60,7 @@ int MPI_Type_create_f90_integer(int r, MPI_Datatype *newtype)
      */
 
     if      (r > 38) *newtype = &ompi_mpi_datatype_null.dt;
-#if OMPI_HAVE_F90_INTEGER16
+#if OMPI_HAVE_FORTRAN_INTEGER16
     else if (r > 18) *newtype = &ompi_mpi_long_long_int.dt;
 #else
     else if (r > 18) *newtype = &ompi_mpi_datatype_null.dt;
@@ -97,7 +97,10 @@ int MPI_Type_create_f90_integer(int r, MPI_Datatype *newtype)
         /* Make sure the user is not allowed to free this datatype as specified
          * in the MPI standard.
          */
-        datatype->super.flags |= OPAL_DATATYPE_FLAG_PREDEFINED;
+        datatype->super.flags |= OMPI_DATATYPE_FLAG_PREDEFINED;
+        /* Mark the datatype as a special F90 convenience type */
+        snprintf(datatype->name, MPI_MAX_OBJECT_NAME, "COMBINER %s",
+                 (*newtype)->name);
 
         a_i[0] = &r;
         ompi_datatype_set_args( datatype, 1, a_i, 0, NULL, 0, NULL, MPI_COMBINER_F90_INTEGER );
