@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
@@ -10,6 +10,9 @@
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2012      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
+ * Copyright (c) 2013      Intel, Inc. All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -20,24 +23,21 @@
 #include "ompi_config.h"
 
 #include <string.h>
+#include <strings.h>
 
 #include "ompi/constants.h"
 #include "opal/datatype/opal_convertor.h"
 #include "opal/threads/mutex.h"
 #include "opal/dss/dss.h"
+#include "opal/mca/db/db.h"
 #include "opal/util/arch.h"
-
-#include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/ess/ess.h"
-#include "orte/util/proc_info.h"
-#include "orte/util/name_fns.h"
-#include "orte/util/show_help.h"
-#include "orte/runtime/orte_globals.h"
+#include "opal/util/show_help.h"
 
 #include "ompi/proc/proc.h"
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/runtime/mpiruntime.h"
 #include "ompi/runtime/ompi_module_exchange.h"
+#include "ompi/runtime/params.h"
 
 static opal_list_t  ompi_proc_list;
 static opal_mutex_t ompi_proc_lock;
@@ -56,8 +56,7 @@ OBJ_CLASS_INSTANCE(
 
 void ompi_proc_construct(ompi_proc_t* proc)
 {
-    proc->proc_bml = NULL;
-    proc->proc_pml = NULL;
+    bzero(proc->proc_endpoints, sizeof(proc->proc_endpoints));
 
     /* By default all processors are supposedly having the same architecture as me. Thus,
      * by default we run in a homogeneous environment. Later, when the RTE can tell us
@@ -94,33 +93,33 @@ void ompi_proc_destruct(ompi_proc_t* proc)
 
 int ompi_proc_init(void)
 {
-    orte_vpid_t i;
+    ompi_vpid_t i;
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
     int ret;
+#endif
 
     OBJ_CONSTRUCT(&ompi_proc_list, opal_list_t);
     OBJ_CONSTRUCT(&ompi_proc_lock, opal_mutex_t);
 
     /* create proc structures and find self */
-    for( i = 0; i < orte_process_info.num_procs; i++ ) {
+    for( i = 0; i < ompi_process_info.num_procs; i++ ) {
         ompi_proc_t *proc = OBJ_NEW(ompi_proc_t);
         opal_list_append(&ompi_proc_list, (opal_list_item_t*)proc);
 
-        proc->proc_name.jobid = ORTE_PROC_MY_NAME->jobid;
+        proc->proc_name.jobid = OMPI_PROC_MY_NAME->jobid;
         proc->proc_name.vpid = i;
-        if (i == ORTE_PROC_MY_NAME->vpid) {
+
+        if (i == OMPI_PROC_MY_NAME->vpid) {
             ompi_proc_local_proc = proc;
             proc->proc_flags = OPAL_PROC_ALL_LOCAL;
-            proc->proc_hostname = orte_process_info.nodename;
+            proc->proc_hostname = ompi_process_info.nodename;
             proc->proc_arch = opal_local_arch;
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
             /* add our arch to the modex */
             if (OMPI_SUCCESS != (ret = ompi_modex_send_key_value("OMPI_ARCH", &proc->proc_arch, OPAL_UINT32))) {
                 return ret;
             }
-        } else {
-            /* get the locality information */
-            proc->proc_flags = orte_ess.proc_get_locality(&proc->proc_name);
-            /* get the name of the node it is on */
-            proc->proc_hostname = orte_ess.proc_get_hostname(&proc->proc_name);
+#endif
         }
     }
 
@@ -128,59 +127,83 @@ int ompi_proc_init(void)
 }
 
 
-/* in some cases, all MPI procs are required to do a modex so they
- * can (at the least) exchange their architecture. Since we cannot
- * know in advance if this was required, we provide a separate function
- * to set the arch (instead of doing it inside of ompi_proc_init) that
- * can be called after the modex completes in ompi_mpi_init. Thus, we
- * know that - regardless of how the arch is known, whether via modex
- * or dropped in from a local daemon - the arch can be set correctly
- * at this time
+/**
+ * The process creation is split into two steps. The second step
+ * is the important one, it sets the properties of the remote
+ * process, such as architecture, node name and locality flags.
+ *
+ * This function is to be called __only__ after the modex exchange
+ * has been performed, in order to allow the modex to carry the data
+ * instead of requiring the runtime to provide it.
  */
-int ompi_proc_set_arch(void)
+int ompi_proc_complete_init(void)
 {
     ompi_proc_t *proc = NULL;
     opal_list_item_t *item = NULL;
-    int ret;
-    
+    int ret, errcode = OMPI_SUCCESS;
+    opal_hwloc_locality_t *hwlocale;
+
     OPAL_THREAD_LOCK(&ompi_proc_lock);
-    
+
     for( item  = opal_list_get_first(&ompi_proc_list);
-        item != opal_list_get_end(&ompi_proc_list);
-        item  = opal_list_get_next(item)) {
+         item != opal_list_get_end(&ompi_proc_list);
+         item  = opal_list_get_next(item)) {
         proc = (ompi_proc_t*)item;
         
-        if (proc->proc_name.vpid != ORTE_PROC_MY_NAME->vpid) {
-            ret = ompi_modex_recv_key_value("OMPI_ARCH", proc, (void*)&(proc->proc_arch), OPAL_UINT32);
-            if (OMPI_SUCCESS == ret) {
-                /* if arch is different than mine, create a new convertor for this proc */
-                if (proc->proc_arch != opal_local_arch) {
-#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
-                    OBJ_RELEASE(proc->proc_convertor);
-                    proc->proc_convertor = opal_convertor_create(proc->proc_arch, 0);
-#else
-                    orte_show_help("help-mpi-runtime",
-                                   "heterogeneous-support-unavailable",
-                                   true, orte_process_info.nodename, 
-                                   proc->proc_hostname == NULL ? "<hostname unavailable>" :
-                                   proc->proc_hostname);
-                    OPAL_THREAD_UNLOCK(&ompi_proc_lock);
-                    return OMPI_ERR_NOT_SUPPORTED;
-#endif
-                }
-            } else if (OMPI_ERR_NOT_IMPLEMENTED == ret) {
-                proc->proc_arch = opal_local_arch;
-            } else {
-                OPAL_THREAD_UNLOCK(&ompi_proc_lock);
-                return ret;
+        if (proc->proc_name.vpid != OMPI_PROC_MY_NAME->vpid) {
+            /* get the locality information - do not use modex recv for
+             * this request as that will automatically cause the hostname
+             * to be loaded as well
+             */
+            hwlocale = &(proc->proc_flags);
+            ret = opal_db.fetch((opal_identifier_t*)&proc->proc_name, OMPI_DB_LOCALITY,
+                                (void**)&hwlocale, OPAL_HWLOC_LOCALITY_T);
+            if (OMPI_SUCCESS != ret) {
+                errcode = ret;
+                break;
             }
+
+            if (ompi_process_info.num_procs < ompi_hostname_cutoff) {
+                /* retrieve the hostname */
+                ret = ompi_modex_recv_string_pointer(OMPI_DB_HOSTNAME, proc, (void**)&(proc->proc_hostname), OPAL_STRING);
+                if (OMPI_SUCCESS != ret) {
+                    break;
+                }
+            } else {
+                /* just set the hostname to NULL for now - we'll fill it in
+                 * as modex_recv's are called for procs we will talk to
+                 */
+                proc->proc_hostname = NULL;
+            }
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
+            /* get the remote architecture */
+            {
+                uint32_t *ui32ptr;
+                ui32ptr = &(proc->proc_arch);
+                ret = ompi_modex_recv_key_value("OMPI_ARCH", proc, (void**)&ui32ptr, OPAL_UINT32);
+                if (OMPI_SUCCESS == ret) {
+                    /* if arch is different than mine, create a new convertor for this proc */
+                    if (proc->proc_arch != opal_local_arch) {
+                        OBJ_RELEASE(proc->proc_convertor);
+                        proc->proc_convertor = opal_convertor_create(proc->proc_arch, 0);
+                    }
+                } else if (OMPI_ERR_NOT_IMPLEMENTED == ret) {
+                    proc->proc_arch = opal_local_arch;
+                } else {
+                    errcode = ret;
+                    break;
+                }
+            }
+#else
+            /* must be same arch as my own */
+            proc->proc_arch = opal_local_arch;
+#endif
         }
     }
     OPAL_THREAD_UNLOCK(&ompi_proc_lock);
-    return OMPI_SUCCESS;
+    return errcode;
 }
 
-
 int ompi_proc_finalize (void)
 {
     opal_list_item_t *item;
@@ -219,14 +242,14 @@ ompi_proc_t** ompi_proc_world(size_t *size)
     ompi_proc_t **procs;
     ompi_proc_t *proc;
     size_t count = 0;
-    orte_ns_cmp_bitmask_t mask;
-    orte_process_name_t my_name;
+    ompi_rte_cmp_bitmask_t mask;
+    ompi_process_name_t my_name;
 
     /* check bozo case */
     if (NULL == ompi_proc_local_proc) {
         return NULL;
     }
-    mask = ORTE_NS_CMP_JOBID;
+    mask = OMPI_RTE_CMP_JOBID;
     my_name = ompi_proc_local_proc->proc_name;
 
     /* First count how many match this jobid */
@@ -234,7 +257,7 @@ ompi_proc_t** ompi_proc_world(size_t *size)
     for (proc =  (ompi_proc_t*)opal_list_get_first(&ompi_proc_list);
          proc != (ompi_proc_t*)opal_list_get_end(&ompi_proc_list);
          proc =  (ompi_proc_t*)opal_list_get_next(proc)) {
-        if (OPAL_EQUAL == orte_util_compare_name_fields(mask, &proc->proc_name, &my_name)) {
+        if (OPAL_EQUAL == ompi_rte_compare_name_fields(mask, &proc->proc_name, &my_name)) {
             ++count;
         }
     }
@@ -251,7 +274,7 @@ ompi_proc_t** ompi_proc_world(size_t *size)
     for (proc =  (ompi_proc_t*)opal_list_get_first(&ompi_proc_list);
          proc != (ompi_proc_t*)opal_list_get_end(&ompi_proc_list);
          proc =  (ompi_proc_t*)opal_list_get_next(proc)) {
-        if (OPAL_EQUAL == orte_util_compare_name_fields(mask, &proc->proc_name, &my_name)) {
+        if (OPAL_EQUAL == ompi_rte_compare_name_fields(mask, &proc->proc_name, &my_name)) {
             /* DO NOT RETAIN THIS OBJECT - the reference count on this
              * object will be adjusted by external callers. The intent
              * here is to allow the reference count to drop to zero if
@@ -324,18 +347,18 @@ ompi_proc_t** ompi_proc_self(size_t* size)
     return procs;
 }
 
-ompi_proc_t * ompi_proc_find ( const orte_process_name_t * name )
+ompi_proc_t * ompi_proc_find ( const ompi_process_name_t * name )
 {
     ompi_proc_t *proc, *rproc=NULL;
-    orte_ns_cmp_bitmask_t mask;
+    ompi_rte_cmp_bitmask_t mask;
 
     /* return the proc-struct which matches this jobid+process id */
-    mask = ORTE_NS_CMP_JOBID | ORTE_NS_CMP_VPID;
+    mask = OMPI_RTE_CMP_JOBID | OMPI_RTE_CMP_VPID;
     OPAL_THREAD_LOCK(&ompi_proc_lock);
     for(proc =  (ompi_proc_t*)opal_list_get_first(&ompi_proc_list);
         proc != (ompi_proc_t*)opal_list_get_end(&ompi_proc_list);
         proc =  (ompi_proc_t*)opal_list_get_next(proc)) {
-        if (OPAL_EQUAL == orte_util_compare_name_fields(mask, &proc->proc_name, name)) {
+        if (OPAL_EQUAL == ompi_rte_compare_name_fields(mask, &proc->proc_name, name)) {
             rproc = proc;
             break;
         }
@@ -349,7 +372,9 @@ ompi_proc_t * ompi_proc_find ( const orte_process_name_t * name )
 int ompi_proc_refresh(void) {
     ompi_proc_t *proc = NULL;
     opal_list_item_t *item = NULL;
-    orte_vpid_t i = 0;
+    ompi_vpid_t i = 0;
+    int ret=OMPI_SUCCESS;
+    opal_hwloc_locality_t *hwlocale;
 
     OPAL_THREAD_LOCK(&ompi_proc_lock);
 
@@ -359,40 +384,60 @@ int ompi_proc_refresh(void) {
         proc = (ompi_proc_t*)item;
 
         /* Does not change: proc->proc_name.vpid */
-        proc->proc_name.jobid = ORTE_PROC_MY_NAME->jobid;
+        proc->proc_name.jobid = OMPI_PROC_MY_NAME->jobid;
 
         /* Make sure to clear the local flag before we set it below */
         proc->proc_flags = 0;
 
-        if (i == ORTE_PROC_MY_NAME->vpid) {
+        if (i == OMPI_PROC_MY_NAME->vpid) {
             ompi_proc_local_proc = proc;
             proc->proc_flags = OPAL_PROC_ALL_LOCAL;
-            proc->proc_hostname = orte_process_info.nodename;
+            proc->proc_hostname = ompi_process_info.nodename;
             proc->proc_arch = opal_local_arch;
         } else {
-            proc->proc_flags = orte_ess.proc_get_locality(&proc->proc_name);
-            proc->proc_hostname = orte_ess.proc_get_hostname(&proc->proc_name);
-            /* if arch is different than mine, create a new convertor for this proc */
-            if (proc->proc_arch != opal_local_arch) {
+            /* get the locality information - do not use modex recv for
+             * this request as that will automatically cause the hostname
+             * to be loaded as well
+             */
+            hwlocale = &(proc->proc_flags);
+            ret = opal_db.fetch((opal_identifier_t*)&proc->proc_name, OMPI_DB_LOCALITY,
+                                (void**)&hwlocale, OPAL_HWLOC_LOCALITY_T);
+            if (OMPI_SUCCESS != ret) {
+                break;
+            }
+            if (ompi_process_info.num_procs < ompi_hostname_cutoff) {
+                /* retrieve the hostname */
+                ret = ompi_modex_recv_string_pointer(OMPI_DB_HOSTNAME, proc, (void**)&(proc->proc_hostname), OPAL_STRING);
+                if (OMPI_SUCCESS != ret) {
+                    break;
+                }
+            } else {
+                /* just set the hostname to NULL for now - we'll fill it in
+                 * as modex_recv's are called for procs we will talk to
+                 */
+                proc->proc_hostname = NULL;
+            }
 #if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
-                OBJ_RELEASE(proc->proc_convertor);
-                proc->proc_convertor = opal_convertor_create(proc->proc_arch, 0);
+            {
+                /* get the remote architecture */
+                uint32_t* uiptr = &(proc->proc_arch);
+                ret = ompi_modex_recv_key_value("OMPI_ARCH", proc, (void**)&uiptr, OPAL_UINT32);
+                /* if arch is different than mine, create a new convertor for this proc */
+                if (proc->proc_arch != opal_local_arch) {
+                    OBJ_RELEASE(proc->proc_convertor);
+                    proc->proc_convertor = opal_convertor_create(proc->proc_arch, 0);
+                }
+            }
 #else
-                orte_show_help("help-mpi-runtime",
-                               "heterogeneous-support-unavailable",
-                               true, orte_process_info.nodename, 
-                               proc->proc_hostname == NULL ? "<hostname unavailable>" :
-                               proc->proc_hostname);
-                OPAL_THREAD_UNLOCK(&ompi_proc_lock);
-                return OMPI_ERR_NOT_SUPPORTED;
+            /* must be same arch as my own */
+            proc->proc_arch = opal_local_arch;
 #endif
-            } 
         }
     }
 
     OPAL_THREAD_UNLOCK(&ompi_proc_lock);
 
-    return OMPI_SUCCESS;   
+    return ret;   
 }
 
 int
@@ -415,21 +460,21 @@ ompi_proc_pack(ompi_proc_t **proclist, int proclistsize, opal_buffer_t* buf)
      * can be sent.
      */
     for (i=0; i<proclistsize; i++) {
-        rc = opal_dss.pack(buf, &(proclist[i]->proc_name), 1, ORTE_NAME);
-        if(rc != ORTE_SUCCESS) {
-            ORTE_ERROR_LOG(rc);
+        rc = opal_dss.pack(buf, &(proclist[i]->proc_name), 1, OMPI_NAME);
+        if(rc != OPAL_SUCCESS) {
+            OMPI_ERROR_LOG(rc);
             OPAL_THREAD_UNLOCK(&ompi_proc_lock);
             return rc;
         }
         rc = opal_dss.pack(buf, &(proclist[i]->proc_arch), 1, OPAL_UINT32);
-        if(rc != ORTE_SUCCESS) {
-            ORTE_ERROR_LOG(rc);
+        if(rc != OPAL_SUCCESS) {
+            OMPI_ERROR_LOG(rc);
             OPAL_THREAD_UNLOCK(&ompi_proc_lock);
             return rc;
         }
         rc = opal_dss.pack(buf, &(proclist[i]->proc_hostname), 1, OPAL_STRING);
-        if(rc != ORTE_SUCCESS) {
-            ORTE_ERROR_LOG(rc);
+        if(rc != OPAL_SUCCESS) {
+            OMPI_ERROR_LOG(rc);
             OPAL_THREAD_UNLOCK(&ompi_proc_lock);
             return rc;
         }
@@ -439,18 +484,18 @@ ompi_proc_pack(ompi_proc_t **proclist, int proclistsize, opal_buffer_t* buf)
 }
 
 static ompi_proc_t *
-ompi_proc_find_and_add(const orte_process_name_t * name, bool* isnew)
+ompi_proc_find_and_add(const ompi_process_name_t * name, bool* isnew)
 {
     ompi_proc_t *proc, *rproc = NULL;
-    orte_ns_cmp_bitmask_t mask;
+    ompi_rte_cmp_bitmask_t mask;
     
     /* return the proc-struct which matches this jobid+process id */
-    mask = ORTE_NS_CMP_JOBID | ORTE_NS_CMP_VPID;
+    mask = OMPI_RTE_CMP_JOBID | OMPI_RTE_CMP_VPID;
     OPAL_THREAD_LOCK(&ompi_proc_lock);
     for(proc =  (ompi_proc_t*)opal_list_get_first(&ompi_proc_list);
         proc != (ompi_proc_t*)opal_list_get_end(&ompi_proc_list);
         proc =  (ompi_proc_t*)opal_list_get_next(proc)) {
-        if (OPAL_EQUAL == orte_util_compare_name_fields(mask, &proc->proc_name, name)) {
+        if (OPAL_EQUAL == ompi_rte_compare_name_fields(mask, &proc->proc_name, name)) {
             rproc = proc;
             *isnew = false;
             break;
@@ -503,30 +548,30 @@ ompi_proc_unpack(opal_buffer_t* buf,
      * their info - as packed by ompi_proc_pack
      */
     for ( i=0; i<proclistsize; i++ ){
-        orte_std_cntr_t count=1;
-        orte_process_name_t new_name;
+        int32_t count=1;
+        ompi_process_name_t new_name;
         uint32_t new_arch;
         char *new_hostname;
         bool isnew = false;
         int rc;
         
-        rc = opal_dss.unpack(buf, &new_name, &count, ORTE_NAME);
-        if (rc != ORTE_SUCCESS) {
-            ORTE_ERROR_LOG(rc);
+        rc = opal_dss.unpack(buf, &new_name, &count, OMPI_NAME);
+        if (rc != OPAL_SUCCESS) {
+            OMPI_ERROR_LOG(rc);
             free(plist);
             free(newprocs);
             return rc;
         }
         rc = opal_dss.unpack(buf, &new_arch, &count, OPAL_UINT32);
-        if (rc != ORTE_SUCCESS) {
-            ORTE_ERROR_LOG(rc);
+        if (rc != OPAL_SUCCESS) {
+            OMPI_ERROR_LOG(rc);
             free(plist);
             free(newprocs);
             return rc;
         }
         rc = opal_dss.unpack(buf, &new_hostname, &count, OPAL_STRING);
-        if (rc != ORTE_SUCCESS) {
-            ORTE_ERROR_LOG(rc);
+        if (rc != OPAL_SUCCESS) {
+            OMPI_ERROR_LOG(rc);
             free(plist);
             free(newprocs);
             return rc;
@@ -549,9 +594,9 @@ ompi_proc_unpack(opal_buffer_t* buf,
                 OBJ_RELEASE(plist[i]->proc_convertor);
                 plist[i]->proc_convertor = opal_convertor_create(plist[i]->proc_arch, 0);
 #else
-                orte_show_help("help-mpi-runtime",
+                opal_show_help("help-mpi-runtime",
                                "heterogeneous-support-unavailable",
-                               true, orte_process_info.nodename, 
+                               true, ompi_process_info.nodename, 
                                new_hostname == NULL ? "<hostname unavailable>" :
                                new_hostname);
                 free(plist);
