@@ -2,14 +2,15 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2008 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2011 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2011 Oak Ridge National Labs.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -33,11 +34,7 @@
 #endif
 
 #include "opal/mca/backtrace/backtrace.h"
-#include "orte/util/proc_info.h"
-#include "orte/runtime/runtime.h"
-#include "orte/runtime/orte_globals.h"
-#include "orte/util/name_fns.h"
-#include "orte/mca/errmgr/errmgr.h"
+
 #include "ompi/communicator/communicator.h"
 #include "ompi/runtime/mpiruntime.h"
 #include "ompi/runtime/params.h"
@@ -51,11 +48,11 @@ ompi_mpi_abort(struct ompi_communicator_t* comm,
                int errcode,
                bool kill_remote_of_intercomm)
 {
-    int count = 0, i;
+    int count = 0, i, ret;
     char *msg, *host, hostname[MAXHOSTNAMELEN];
     pid_t pid = 0;
-    orte_process_name_t *abort_procs;
-    orte_std_cntr_t nabort_procs;
+    ompi_process_name_t *abort_procs;
+    int32_t nabort_procs;
 
     /* Protection for recursive invocation */
     if (have_been_invoked) {
@@ -63,19 +60,18 @@ ompi_mpi_abort(struct ompi_communicator_t* comm,
     }
     have_been_invoked = true;
 
-    /* If ORTE is initialized, use its nodename.  Otherwise, call
+    /* If MPI is initialized, we know we have a runtime nodename, so use that.  Otherwise, call
        gethostname. */
-
-    if (orte_initialized) {
-        host = orte_process_info.nodename;
+    if (ompi_mpi_initialized) {
+        host = ompi_process_info.nodename;
     } else {
         gethostname(hostname, sizeof(hostname));
         host = hostname;
     }
     pid = getpid();
 
-    /* Should we print a stack trace? */
-
+    /* Should we print a stack trace?  Not aggregated because they
+       might be different on all processes. */
     if (ompi_mpi_abort_print_stack) {
         char **messages;
         int len, i;
@@ -129,12 +125,12 @@ ompi_mpi_abort(struct ompi_communicator_t* comm,
     }
 
     /* If OMPI isn't setup yet/any more, then don't even try killing
-       everyone.  Ditto for ORTE (e.g., ORTE may be initialized before
-       MPI_INIT is over, but ompi_initialized will be false because
-       communicators are not setup yet). Sorry, Charlie... */
+       everyone.  OMPI's initialized period covers all runtime
+       initialized period of time, so no need to check that here.
+       Sorry, Charlie... */
 
-    if (!orte_initialized || !ompi_mpi_initialized || ompi_mpi_finalized) {
-        fprintf(stderr, "[%s:%d] Abort %s completed successfully; not able to guarantee that all other processes were killed!\n",
+    if (!ompi_mpi_initialized || ompi_mpi_finalized) {
+        fprintf(stderr, "[%s:%d] Local abort %s completed successfully; not able to aggregate error messages, and not able to guarantee that all other processes were killed!\n",
                 host, (int) pid, ompi_mpi_finalized ? 
                 "after MPI_FINALIZE" : "before MPI_INIT");
         exit(errcode);
@@ -151,17 +147,17 @@ ompi_mpi_abort(struct ompi_communicator_t* comm,
         nabort_procs += ompi_comm_remote_size(comm);
     }
 
-    abort_procs = (orte_process_name_t*)malloc(sizeof(orte_process_name_t) * nabort_procs);
+    abort_procs = (ompi_process_name_t*)malloc(sizeof(ompi_process_name_t) * nabort_procs);
     if (NULL == abort_procs) {
         /* quick clean orte and get out */
-        orte_errmgr.abort(errcode, "Abort unable to malloc memory to kill procs");
+        ompi_rte_abort(errcode, "Abort unable to malloc memory to kill procs");
     }
 
     /* put all the local procs in the abort list */
     for (i = 0 ; i < ompi_comm_size(comm) ; ++i) {
-        if (OPAL_EQUAL != orte_util_compare_name_fields(ORTE_NS_CMP_ALL, 
+        if (OPAL_EQUAL != ompi_rte_compare_name_fields(OMPI_RTE_CMP_ALL, 
                                  &comm->c_local_group->grp_proc_pointers[i]->proc_name,
-                                 ORTE_PROC_MY_NAME)) {
+                                 OMPI_PROC_MY_NAME)) {
             assert(count <= nabort_procs);
             abort_procs[count++] = comm->c_local_group->grp_proc_pointers[i]->proc_name;
         } else {
@@ -173,9 +169,9 @@ ompi_mpi_abort(struct ompi_communicator_t* comm,
     /* if requested, kill off remote procs too */
     if (kill_remote_of_intercomm) {
         for (i = 0 ; i < ompi_comm_remote_size(comm) ; ++i) {
-            if (OPAL_EQUAL != orte_util_compare_name_fields(ORTE_NS_CMP_ALL, 
+            if (OPAL_EQUAL != ompi_rte_compare_name_fields(OMPI_RTE_CMP_ALL, 
                                      &comm->c_remote_group->grp_proc_pointers[i]->proc_name,
-                                     ORTE_PROC_MY_NAME)) {
+                                     OMPI_PROC_MY_NAME)) {
                 assert(count <= nabort_procs);
                 abort_procs[count++] =
                     comm->c_remote_group->grp_proc_pointers[i]->proc_name;
@@ -187,16 +183,24 @@ ompi_mpi_abort(struct ompi_communicator_t* comm,
     }
 
     if (nabort_procs > 0) {
-#if 0
-        int ret = orte_errmgr.abort_procs_request(abort_procs, nabort_procs);
-        if (OMPI_SUCCESS != ret) {
-            orte_errmgr.abort(ret, "Open MPI failed to abort procs as requested (%d). Exiting.", ret);
+        /* This must be implemented for MPI_Abort() to work according to the
+         * standard language for a 'high-quality' implementation.
+         * It would be nifty if we could differentiate between the
+         * abort scenarios:
+         *      - MPI_Abort()
+         *      - MPI_ERRORS_ARE_FATAL
+         *      - Victim of MPI_Abort()
+         */
+        /*
+         * Abort peers in this communicator group. Does not include self.
+         */
+        if( OMPI_SUCCESS != (ret = ompi_rte_abort_peers(abort_procs, nabort_procs)) ) {
+            ompi_rte_abort(ret, "Open MPI failed to abort all of the procs requested (%d).", ret);
         }
-#endif
     }
 
     /* now that we've aborted everyone else, gracefully die. */
-    orte_errmgr.abort(errcode, NULL);
+    ompi_rte_abort(errcode, NULL);
     
     return OMPI_SUCCESS;
 }
