@@ -1,19 +1,21 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
- * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2006      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2012 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * Copyright (c) 2006      University of Houston. All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
  *
  * $COPYRIGHT$
  * 
@@ -37,26 +39,20 @@
 #include <netdb.h>
 #endif
 
-#include "opal/event/event.h"
+#include "opal/mca/event/event.h"
+#include "opal/util/output.h"
 #include "opal/runtime/opal_progress.h"
-#include "opal/mca/maffinity/base/base.h"
 #include "opal/mca/base/base.h"
-#include "orte/util/show_help.h"
 #include "opal/sys/atomic.h"
-
-#include "orte/util/proc_info.h"
-#include "orte/mca/oob/base/base.h"
-#include "orte/mca/rml/rml.h"
-#include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/grpcomm/grpcomm.h"
-#include "orte/runtime/runtime.h"
+#include "opal/runtime/opal.h"
+#include "opal/util/show_help.h"
 
 #include "mpi.h"
 #include "ompi/constants.h"
-#include "ompi/group/group.h"
 #include "ompi/errhandler/errcode.h"
 #include "ompi/communicator/communicator.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/datatype/ompi_datatype.h"
+#include "ompi/message/message.h"
 #include "ompi/op/op.h"
 #include "ompi/file/file.h"
 #include "ompi/info/info.h"
@@ -65,31 +61,35 @@
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/pml/base/base.h"
 #include "ompi/mca/osc/base/base.h"
-#include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/base.h"
-#include "ompi/mca/topo/topo.h"
+#include "ompi/mca/rte/rte.h"
 #include "ompi/mca/topo/base/base.h"
 #include "ompi/mca/io/io.h"
 #include "ompi/mca/io/base/base.h"
 #include "ompi/mca/mpool/base/base.h"
+#include "ompi/mca/mpool/base/mpool_base_tree.h"
 #include "ompi/mca/rcache/base/base.h"
 #include "ompi/mca/pml/base/pml_base_bsend.h"
 #include "ompi/runtime/params.h"
-#include "ompi/mca/mpool/base/mpool_base_tree.h"
 #include "ompi/mca/dpm/base/base.h"
 #include "ompi/mca/pubsub/base/base.h"
+#include "ompi/mpiext/mpiext.h"
 
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
 #include "ompi/mca/crcp/crcp.h"
 #include "ompi/mca/crcp/base/base.h"
 #endif
 #include "ompi/runtime/ompi_cr.h"
 
+extern bool ompi_enable_timing;
+
 int ompi_mpi_finalize(void)
 {
     int ret;
     static int32_t finalize_has_already_started = 0;
     opal_list_item_t *item;
+    struct timeval ompistart, ompistop;
+    ompi_rte_collective_t *coll;
 
     /* Be a bit social if an erroneous program calls MPI_FINALIZE in
        two different threads, otherwise we may deadlock in
@@ -104,12 +104,19 @@ int ompi_mpi_finalize(void)
         pid_t pid = getpid();
         gethostname(hostname, sizeof(hostname));
 
-        orte_show_help("help-mpi-runtime.txt",
+        opal_show_help("help-mpi-runtime.txt",
                        "mpi_finalize:invoked_multiple_times",
                        true, hostname, pid);
         return MPI_ERR_OTHER;
     }
 
+    ompi_mpiext_fini();
+
+    /* As finalize is the last legal MPI call, we are allowed to force the release
+     * of the user buffer used for bsend, before going anywhere further.
+     */
+    (void)mca_pml_base_bsend_detach(NULL, NULL);
+
     /* Per MPI-2:4.8, we have to free MPI_COMM_SELF before doing
        anything else in MPI_FINALIZE (to include setting up such that
        MPI_FINALIZED will return true). */
@@ -126,33 +133,106 @@ int ompi_mpi_finalize(void)
     ompi_mpi_finalized = true;
 
 #if OMPI_ENABLE_PROGRESS_THREADS == 0
-    opal_progress_set_event_flag(OPAL_EVLOOP_ONELOOP);
+    opal_progress_set_event_flag(OPAL_EVLOOP_ONCE | OPAL_EVLOOP_NONBLOCK);
 #endif
 
     /* Redo ORTE calling opal_progress_event_users_increment() during
        MPI lifetime, to get better latency when not using TCP */
     opal_progress_event_users_increment();
 
-    /* If maffinity was setup, tear it down */
-    if (ompi_mpi_maffinity_setup) {
-        opal_maffinity_base_close();
+    /* check to see if we want timing information */
+    if (ompi_enable_timing != 0 && 0 == OMPI_PROC_MY_NAME->vpid) {
+        gettimeofday(&ompistart, NULL);
     }
 
+    /* NOTE: MPI-2.1 requires that MPI_FINALIZE is "collective" across
+       *all* connected processes.  This only means that all processes
+       have to call it.  It does *not* mean that all connected
+       processes need to synchronize (either directly or indirectly).  
+
+       For example, it is quite easy to construct complicated
+       scenarios where one job is "connected" to another job via
+       transitivity, but have no direct knowledge of each other.
+       Consider the following case: job A spawns job B, and job B
+       later spawns job C.  A "connectedness" graph looks something
+       like this:
+
+           A <--> B <--> C
+
+       So what are we *supposed* to do in this case?  If job A is
+       still connected to B when it calls FINALIZE, should it block
+       until jobs B and C also call FINALIZE?
+
+       After lengthy discussions many times over the course of this
+       project, the issue was finally decided at the Louisville Feb
+       2009 meeting: no.
+
+       Rationale:
+
+       - "Collective" does not mean synchronizing.  It only means that
+         every process call it.  Hence, in this scenario, every
+         process in A, B, and C must call FINALIZE.
+
+       - KEY POINT: if A calls FINALIZE, then it is erroneous for B or
+         C to try to communicate with A again.
+
+       - Hence, OMPI is *correct* to only effect a barrier across each
+         jobs' MPI_COMM_WORLD before exiting.  Specifically, if A
+         calls FINALIZE long before B or C, it's *correct* if A exits
+         at any time (and doesn't notify B or C that it is exiting).
+
+       - Arguably, if B or C do try to communicate with the now-gone
+         A, OMPI should try to print a nice error ("you tried to
+         communicate with a job that is already gone...") instead of
+         segv or other Badness.  However, that is an *extremely*
+         difficult problem -- sure, it's easy for A to tell B that it
+         is finalizing, but how can A tell C?  A doesn't even know
+         about C.  You'd need to construct a "connected" graph in a
+         distributed fashion, which is fraught with race conditions,
+         etc.
+
+      Hence, our conclusion is: OMPI is *correct* in its current
+      behavior (of only doing a barrier across its own COMM_WORLD)
+      before exiting.  Any problems that occur are as a result of
+      erroneous MPI applications.  We *could* tighten up the erroneous
+      cases and ensure that we print nice error messages / don't
+      crash, but that is such a difficult problem that we decided we
+      have many other, much higher priority issues to handle that deal
+      with non-erroneous cases. */
+
     /* wait for everyone to reach this point
        This is a grpcomm barrier instead of an MPI barrier because an
        MPI barrier doesn't ensure that all messages have been transmitted
        before exiting, so the possibility of a stranded message exists.
     */
-    if (OMPI_SUCCESS != (ret = orte_grpcomm.barrier())) {
-        ORTE_ERROR_LOG(ret);
+    coll = OBJ_NEW(ompi_rte_collective_t);
+    coll->id = ompi_process_info.peer_fini_barrier;
+    if (OMPI_SUCCESS != (ret = ompi_rte_barrier(coll))) {
+        OMPI_ERROR_LOG(ret);
         return ret;
     }
 
+    /* wait for barrier to complete */
+    while (coll->active) {
+        opal_progress();  /* block in progress pending events */
+    }
+    OBJ_RELEASE(coll);
+
+    /* check for timing request - get stop time and report elapsed
+     time if so */
+    if (ompi_enable_timing && 0 == OMPI_PROC_MY_NAME->vpid) {
+        gettimeofday(&ompistop, NULL);
+        opal_output(0, "ompi_mpi_finalize[%ld]: time to execute barrier %ld usec",
+                    (long)OMPI_PROC_MY_NAME->vpid,
+                    (long int)((ompistop.tv_sec - ompistart.tv_sec)*1000000 +
+                               (ompistop.tv_usec - ompistart.tv_usec)));
+    }
+
     /*
      * Shutdown the Checkpoint/Restart Mech.
      */
     if (OMPI_SUCCESS != (ret = ompi_cr_finalize())) {
-        ORTE_ERROR_LOG(ret);
+        OMPI_ERROR_LOG(ret);
     }
 
     /* Shut down any bindings-specific issues: C++, F77, F90 */
@@ -177,8 +257,6 @@ int ompi_mpi_finalize(void)
 
     /* Free communication objects */
 
-    /* free window resources */
-
     /* free file resources */
     if (OMPI_SUCCESS != (ret = ompi_file_finalize())) {
         return ret;
@@ -206,6 +284,10 @@ int ompi_mpi_finalize(void)
         return ret;
     }
 
+    if (OMPI_SUCCESS != (ret = ompi_message_finalize())) {
+        return ret;
+    }
+
     /* If requested, print out a list of memory allocated by ALLOC_MEM
        but not freed by FREE_MEM */
     if (0 != ompi_debug_show_mpi_alloc_mem_leaks) {
@@ -214,19 +296,20 @@ int ompi_mpi_finalize(void)
 
     /* Now that all MPI objects dealing with communications are gone,
        shut down MCA types having to do with communications */
-    if (OMPI_SUCCESS != (ret = mca_pml_base_close())) {
+    if (OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_pml_base_framework) ) ) {
+        OMPI_ERROR_LOG(ret);
         return ret;
     }
 
     /* shut down buffered send code */
     mca_pml_base_bsend_fini();
 
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
     /*
      * Shutdown the CRCP Framework, must happen after PML shutdown
      */
-    if (OMPI_SUCCESS != (ret = ompi_crcp_base_close() ) ) {
-        ORTE_ERROR_LOG(ret);
+    if (OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_crcp_base_framework) ) ) {
+        OMPI_ERROR_LOG(ret);
         return ret;
     }
 #endif
@@ -249,12 +332,12 @@ int ompi_mpi_finalize(void)
     }
     
     /* finalize the pubsub functions */
-    if ( OMPI_SUCCESS != (ret = ompi_pubsub_base_close())) {
+    if (OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_pubsub_base_framework) ) ) {
         return ret;
     }
     
     /* finalize the DPM framework */
-    if ( OMPI_SUCCESS != (ret = ompi_dpm_base_close())) {
+    if ( OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_dpm_base_framework))) {
         return ret;
     }
     
@@ -281,7 +364,7 @@ int ompi_mpi_finalize(void)
     }
 
     /* free ddt resources */
-    if (OMPI_SUCCESS != (ret = ompi_ddt_finalize())) {
+    if (OMPI_SUCCESS != (ret = ompi_datatype_finalize())) {
         return ret;
     }
 
@@ -294,32 +377,35 @@ int ompi_mpi_finalize(void)
 
     /* io is opened lazily, so it's only necessary to close it if it
        was actually opened */
+    if (0 < ompi_io_base_framework.framework_refcnt) {
+        /* May have been "opened" multiple times. We want it closed now */
+        ompi_io_base_framework.framework_refcnt = 1;
 
-    if (mca_io_base_components_opened_valid ||
-        mca_io_base_components_available_valid) {
-        if (OMPI_SUCCESS != (ret = mca_io_base_close())) {
+        if (OMPI_SUCCESS != mca_base_framework_close(&ompi_io_base_framework)) {
             return ret;
         }
     }
-    if (OMPI_SUCCESS != (ret = mca_topo_base_close())) {
+    (void) mca_base_framework_close(&ompi_topo_base_framework);
+    if (OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_osc_base_framework))) {
         return ret;
     }
-    if (OMPI_SUCCESS != (ret = ompi_osc_base_close())) {
+    if (OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_coll_base_framework))) {
         return ret;
     }
-    if (OMPI_SUCCESS != (ret = mca_coll_base_close())) {
+    if (OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_mpool_base_framework))) {
         return ret;
     }
-    if (OMPI_SUCCESS != (ret = mca_mpool_base_close())) {
+    if (OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_rcache_base_framework))) {
         return ret;
     }
-    if (OMPI_SUCCESS != (ret = mca_rcache_base_close())) { 
-        return ret;
-    }
-    
+
     /* Leave the RTE */
 
-    if (OMPI_SUCCESS != (ret = orte_finalize())) {
+    if (OMPI_SUCCESS != (ret = ompi_rte_finalize())) {
+        return ret;
+    }
+
+    if (OPAL_SUCCESS != (ret = opal_finalize_util())) {
         return ret;
     }
 
