@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2010 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
@@ -11,6 +11,7 @@
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
+ * Copyright (c) 2011      NVIDIA Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -37,6 +38,11 @@
 #include "opal/datatype/opal_datatype_checksum.h"
 #include "opal/datatype/opal_datatype_prototypes.h"
 #include "opal/datatype/opal_convertor_internal.h"
+#if OPAL_CUDA_SUPPORT
+#include "opal/datatype/opal_datatype_cuda.h"
+#define MEMCPY_CUDA( DST, SRC, BLENGTH, CONVERTOR ) \
+    CONVERTOR->cbmemcpy( (DST), (SRC), (BLENGTH), (CONVERTOR) )
+#endif
 
 extern int opal_convertor_create_stack_with_pos_general( opal_convertor_t* convertor,
                                                          int starting_point, const int* sizes );
@@ -48,6 +54,9 @@ static void opal_convertor_construct( opal_convertor_t* convertor )
     convertor->partial_length = 0;
     convertor->remoteArch     = opal_local_arch;
     convertor->flags          = OPAL_DATATYPE_FLAG_NO_GAPS | CONVERTOR_COMPLETED;
+#if OPAL_CUDA_SUPPORT
+    convertor->cbmemcpy       = &opal_cuda_memcpy;
+#endif
 }
 
 
@@ -235,7 +244,11 @@ int32_t opal_convertor_pack( opal_convertor_t* pConv,
             if( OPAL_LIKELY(NULL == iov[i].iov_base) )
                 iov[i].iov_base = (IOVBASE_TYPE *) base_pointer;
             else
+#if OPAL_CUDA_SUPPORT
+                MEMCPY_CUDA( iov[i].iov_base, base_pointer, iov[i].iov_len, pConv );
+#else
                 MEMCPY( iov[i].iov_base, base_pointer, iov[i].iov_len );
+#endif
             pending_length -= iov[i].iov_len;
             base_pointer += iov[i].iov_len;
         }
@@ -248,7 +261,11 @@ complete_contiguous_data_pack:
         if( OPAL_LIKELY(NULL == iov[i].iov_base) )
             iov[i].iov_base = (IOVBASE_TYPE *) base_pointer;
         else
+#if OPAL_CUDA_SUPPORT
+            MEMCPY_CUDA( iov[i].iov_base, base_pointer, iov[i].iov_len, pConv );
+#else
             MEMCPY( iov[i].iov_base, base_pointer, iov[i].iov_len );
+#endif
         pConv->bConverted = pConv->local_size;
         *out_size = i + 1;
         pConv->flags |= CONVERTOR_COMPLETED;
@@ -282,7 +299,11 @@ int32_t opal_convertor_unpack( opal_convertor_t* pConv,
             if( iov[i].iov_len >= pending_length ) {
                 goto complete_contiguous_data_unpack;
             }
+#if OPAL_CUDA_SUPPORT
+            MEMCPY_CUDA( base_pointer, iov[i].iov_base, iov[i].iov_len, pConv );
+#else
             MEMCPY( base_pointer, iov[i].iov_base, iov[i].iov_len );
+#endif
             pending_length -= iov[i].iov_len;
             base_pointer += iov[i].iov_len;
         }
@@ -292,7 +313,11 @@ int32_t opal_convertor_unpack( opal_convertor_t* pConv,
 
 complete_contiguous_data_unpack:
         iov[i].iov_len = pending_length;
+#if OPAL_CUDA_SUPPORT
+        MEMCPY_CUDA( base_pointer, iov[i].iov_base, iov[i].iov_len, pConv );
+#else
         MEMCPY( base_pointer, iov[i].iov_base, iov[i].iov_len );
+#endif
         pConv->bConverted = pConv->local_size;
         *out_size = i + 1;
         pConv->flags |= CONVERTOR_COMPLETED;
@@ -403,6 +428,17 @@ int32_t opal_convertor_set_position_nocheck( opal_convertor_t* convertor,
                                                           opal_datatype_local_sizes );
     } else {
         rc = opal_convertor_generic_simple_position( convertor, position );
+        /**
+         * If we have a non-contigous send convertor don't allow it move in the middle
+         * of a predefined datatype, it won't be able to copy out the left-overs
+         * anyway. Instead force the position to stay on predefined datatypes
+         * boundaries. As we allow partial predefined datatypes on the contiguous
+         * case, we should be accepted by any receiver convertor.
+         */
+        if( CONVERTOR_SEND & convertor->flags ) {
+            convertor->bConverted -= convertor->partial_length;
+            convertor->partial_length = 0;
+        }
     }
     *position = convertor->bConverted;
     return rc;
@@ -412,9 +448,10 @@ int32_t opal_convertor_set_position_nocheck( opal_convertor_t* convertor,
 /**
  * Compute the remote size.
  */
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
 #define OPAL_CONVERTOR_COMPUTE_REMOTE_SIZE(convertor, datatype, bdt_mask) \
 {                                                                         \
-    if( OPAL_UNLIKELY(0 != bdt_mask) ) {                                  \
+    if( OPAL_UNLIKELY(0 != (bdt_mask)) ) {                                \
         opal_convertor_master_t* master;                                  \
         int i;                                                            \
         uint32_t mask = datatype->bdt_used;                               \
@@ -432,6 +469,10 @@ int32_t opal_convertor_set_position_nocheck( opal_convertor_t* convertor,
         convertor->use_desc = &(datatype->desc);                          \
     }                                                                     \
 }
+#else
+#define OPAL_CONVERTOR_COMPUTE_REMOTE_SIZE(convertor, datatype, bdt_mask) \
+    assert(0 == (bdt_mask))
+#endif  /* OPAL_ENABLE_HETEROGENEOUS_SUPPORT */
 
 /**
  * This macro will initialize a convertor based on a previously created
@@ -514,6 +555,9 @@ int32_t opal_convertor_prepare_for_recv( opal_convertor_t* convertor,
     /* Here I should check that the data is not overlapping */
 
     convertor->flags |= CONVERTOR_RECV;
+#if OPAL_CUDA_SUPPORT
+    mca_cuda_convertor_init(convertor, pUserBuf);
+#endif
 
     OPAL_CONVERTOR_PREPARE( convertor, datatype, count, pUserBuf );
 
@@ -550,6 +594,9 @@ int32_t opal_convertor_prepare_for_send( opal_convertor_t* convertor,
                                          const void* pUserBuf )
 {
     convertor->flags |= CONVERTOR_SEND;
+#if OPAL_CUDA_SUPPORT
+    mca_cuda_convertor_init(convertor, pUserBuf);
+#endif
 
     OPAL_CONVERTOR_PREPARE( convertor, datatype, count, pUserBuf );
 
@@ -618,6 +665,9 @@ int opal_convertor_clone( const opal_convertor_t* source,
         destination->bConverted = source->bConverted;
         destination->stack_pos  = source->stack_pos;
     }
+#if OPAL_CUDA_SUPPORT
+    destination->cbmemcpy   = source->cbmemcpy;
+#endif
     return OPAL_SUCCESS;
 }
 
