@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2011 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -68,8 +68,6 @@
 
 /* Do we have posix or solaris thread lib */
 #define OPAL_HAVE_THREADS (OPAL_HAVE_POSIX_THREADS || OPAL_HAVE_SOLARIS_THREADS)
-/* Do we have thread support? */
-#define OPAL_HAVE_THREAD_SUPPORT (OPAL_ENABLE_MPI_THREADS || OPAL_ENABLE_PROGRESS_THREADS)
 
 /*
  * BEGIN_C_DECLS should be used at the beginning of your declarations,
@@ -240,7 +238,11 @@
 #      define OPAL_DECLSPEC        __declspec(dllexport)
 #      define OPAL_MODULE_DECLSPEC
 #    else
-#      define OPAL_DECLSPEC        __declspec(dllimport)
+#      if defined(OPAL_IMPORTS)
+#        define OPAL_DECLSPEC      __declspec(dllimport)
+#      else
+#        define OPAL_DECLSPEC
+#      endif  /*defined(OPAL_IMPORTS)*/
 #      if defined(OPAL_MODULE_EXPORTS)
 #        define OPAL_MODULE_DECLSPEC __declspec(dllexport)
 #      else
@@ -616,4 +618,23 @@ static inline uint16_t ntohs(uint16_t netvar) { return netvar; }
 #define restrict
 #endif
 
+#else
+
+/* For a similar reason to what is listed in opal_config_top.h, we
+   want to protect others from the autoconf/automake-generated
+   PACKAGE_<foo> macros in opal_config.h.  We can't put these undef's
+   directly in opal_config.h because they'll be turned into #defines'
+   via autoconf.  
+
+   So put them here in case any only else includes OMPI/ORTE/OPAL's
+   config.h files. */
+
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
+#undef PACKAGE_URL
+#undef HAVE_CONFIG_H
+
 #endif /* OMPI_BUILDING */
