@@ -379,6 +379,13 @@ static void ri_destructor(opal_object_t *obj)
   repository_item_t *ri = (repository_item_t *) obj;
   dependency_item_t *di;
   opal_list_item_t *item;
+  int group_id;
+
+  group_id = mca_base_var_group_find (NULL, ri->ri_type,
+                                      ri->ri_component_struct->mca_component_name);
+  if (0 <= group_id) {
+    mca_base_var_group_deregister (group_id);
+  }
 
   /* Close the component (and potentially unload it from memory */
   lt_dlclose(ri->ri_dlhandle);
