@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -10,6 +10,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
+ *                         All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -42,17 +44,36 @@
 #include <signal.h>
 #endif
 
+#include "opal/class/opal_list.h"
+#include "opal/class/opal_bitmap.h"
 #include "opal/mca/mca.h"
+#include "opal/mca/event/event.h"
 
 #include "orte/mca/iof/iof.h"
 #include "orte/runtime/orte_globals.h"
 
 BEGIN_C_DECLS
 
-ORTE_DECLSPEC int orte_iof_base_open(void);
+/*
+ * MCA framework
+ */
+ORTE_DECLSPEC extern mca_base_framework_t orte_iof_base_framework;
+
+/*
+ * Select an available component.
+ */
+ORTE_DECLSPEC int orte_iof_base_select(void);
 
 #if !ORTE_DISABLE_FULL_SUPPORT
 
+/* track xon/xoff of processes */
+typedef struct {
+    opal_object_t super;
+    orte_job_t *jdata;
+    opal_bitmap_t xoff;
+} orte_iof_job_t;
+ORTE_DECLSPEC OBJ_CLASS_DECLARATION(orte_iof_job_t);
+
 /*
  * Maximum size of single msg 
  */
@@ -64,7 +85,7 @@ ORTE_DECLSPEC int orte_iof_base_open(void);
 typedef struct {
     opal_list_item_t super;
     bool pending;
-    opal_event_t ev;
+    opal_event_t *ev;
     int fd;
     opal_list_t outputs;
 } orte_iof_write_event_t;
@@ -76,23 +97,17 @@ typedef struct {
     orte_process_name_t daemon;
     orte_iof_tag_t tag;
     orte_iof_write_event_t *wev;
-#if OMPI_ENABLE_DEBUG
-    char *file;
-    int line;
-#endif
+    bool xoff;
 } orte_iof_sink_t;
 ORTE_DECLSPEC OBJ_CLASS_DECLARATION(orte_iof_sink_t);
 
 typedef struct {
     opal_object_t super;
     orte_process_name_t name;
-    opal_event_t ev;
+    opal_event_t *ev;
+    int fd;
     orte_iof_tag_t tag;
     bool active;
-#if OMPI_ENABLE_DEBUG
-    char *file;
-    int line;
-#endif
 } orte_iof_read_event_t;
 ORTE_DECLSPEC OBJ_CLASS_DECLARATION(orte_iof_read_event_t);
 
@@ -102,6 +117,7 @@ typedef struct {
     orte_iof_read_event_t *revstdout;
     orte_iof_read_event_t *revstderr;
     orte_iof_read_event_t *revstddiag;
+    orte_iof_sink_t *sink;
 } orte_iof_proc_t;
 ORTE_DECLSPEC OBJ_CLASS_DECLARATION(orte_iof_proc_t);
 
@@ -114,8 +130,8 @@ ORTE_DECLSPEC OBJ_CLASS_DECLARATION(orte_iof_write_output_t);
 
 /* the iof globals struct */
 struct orte_iof_base_t {
-    int                     iof_output;
-    opal_list_t             iof_components_opened;
+    size_t                  output_limit;
+    char                    *input_files;
     opal_mutex_t            iof_write_output_lock;
     orte_iof_sink_t         *iof_write_stdout;
     orte_iof_sink_t         *iof_write_stderr;
@@ -123,12 +139,10 @@ struct orte_iof_base_t {
 typedef struct orte_iof_base_t orte_iof_base_t;
 
 
-#if OMPI_ENABLE_DEBUG
-
 #define ORTE_IOF_SINK_DEFINE(snk, nm, fid, tg, wrthndlr, eplist)    \
     do {                                                            \
         orte_iof_sink_t *ep;                                        \
-        OPAL_OUTPUT_VERBOSE((1, orte_iof_base.iof_output,           \
+        OPAL_OUTPUT_VERBOSE((1, orte_iof_base_framework.framework_output,           \
                             "defining endpt: file %s line %d fd %d",\
                             __FILE__, __LINE__, (fid)));            \
         ep = OBJ_NEW(orte_iof_sink_t);                              \
@@ -137,16 +151,16 @@ typedef struct orte_iof_base_t orte_iof_base_t;
         ep->tag = (tg);                                             \
         if (0 <= (fid)) {                                           \
             ep->wev->fd = (fid);                                    \
-            opal_event_set(&(ep->wev->ev), ep->wev->fd,             \
+            opal_event_set(orte_event_base,                         \
+                           ep->wev->ev, ep->wev->fd,                \
                            OPAL_EV_WRITE,                           \
                            wrthndlr, ep);                           \
+            opal_event_set_priority(ep->wev->ev, ORTE_MSG_PRI);     \
         }                                                           \
         if (NULL != (eplist)) {                                     \
             opal_list_append((eplist), &ep->super);                 \
         }                                                           \
         *(snk) = ep;                                                \
-        ep->file = strdup(__FILE__);                                \
-        ep->line = __LINE__;                                        \
     } while(0);
 
 /* add list of structs that has name of proc + orte_iof_tag_t - when
@@ -158,7 +172,7 @@ typedef struct orte_iof_base_t orte_iof_base_t;
 #define ORTE_IOF_READ_EVENT(rv, nm, fid, tg, cbfunc, actv)          \
     do {                                                            \
         orte_iof_read_event_t *rev;                                 \
-        OPAL_OUTPUT_VERBOSE((1, orte_iof_base.iof_output,           \
+        OPAL_OUTPUT_VERBOSE((1, orte_iof_base_framework.framework_output,           \
                             "%s defining read event for %s: %s %d", \
                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),     \
                             ORTE_NAME_PRINT((nm)),                  \
@@ -167,61 +181,20 @@ typedef struct orte_iof_base_t orte_iof_base_t;
         rev->name.jobid = (nm)->jobid;                              \
         rev->name.vpid = (nm)->vpid;                                \
         rev->tag = (tg);                                            \
+        rev->fd = (fid);                                            \
         *(rv) = rev;                                                \
-        rev->file = strdup(__FILE__);                               \
-        rev->line = __LINE__;                                       \
-        opal_event_set(&rev->ev, (fid),                             \
+        opal_event_set(orte_event_base,                             \
+                       rev->ev, (fid),                              \
                        OPAL_EV_READ,                                \
                        (cbfunc), rev);                              \
+        opal_event_set_priority(rev->ev, ORTE_MSG_PRI);             \
         if ((actv)) {                                               \
             rev->active = true;                                     \
-            opal_event_add(&rev->ev, 0);                            \
+            opal_event_add(rev->ev, 0);                             \
         }                                                           \
     } while(0);
 
 
-#else
-
-#define ORTE_IOF_SINK_DEFINE(snk, nm, fid, tg, wrthndlr, eplist)    \
-    do {                                                            \
-        orte_iof_sink_t *ep;                                        \
-        ep = OBJ_NEW(orte_iof_sink_t);                              \
-        ep->name.jobid = (nm)->jobid;                               \
-        ep->name.vpid = (nm)->vpid;                                 \
-        ep->tag = (tg);                                             \
-        if (0 <= (fid)) {                                           \
-            ep->wev->fd = (fid);                                    \
-            opal_event_set(&(ep->wev->ev), ep->wev->fd,             \
-                           OPAL_EV_WRITE,                           \
-                           wrthndlr, ep);                           \
-        }                                                           \
-        if (NULL != (eplist)) {                                     \
-            opal_list_append((eplist), &ep->super);                 \
-        }                                                           \
-        *(snk) = ep;                                                \
-    } while(0);
-
-#define ORTE_IOF_READ_EVENT(rv, nm, fid, tg, cbfunc, actv)          \
-    do {                                                            \
-        orte_iof_read_event_t *rev;                                 \
-        rev = OBJ_NEW(orte_iof_read_event_t);                       \
-        rev->name.jobid = (nm)->jobid;                              \
-        rev->name.vpid = (nm)->vpid;                                \
-        rev->tag = (tg);                                            \
-        *(rv) = rev;                                                \
-        opal_event_set(&rev->ev, (fid),                             \
-                       OPAL_EV_READ,                                \
-                       (cbfunc), rev);                              \
-        if ((actv)) {                                               \
-            rev->active = true;                                     \
-            opal_event_add(&rev->ev, 0);                            \
-        }                                                           \
-    } while(0);
-
-#endif
-
-ORTE_DECLSPEC int orte_iof_base_close(void);
-ORTE_DECLSPEC int orte_iof_base_select(void);
 ORTE_DECLSPEC int orte_iof_base_flush(void);
 
 ORTE_DECLSPEC extern orte_iof_base_t orte_iof_base;
