@@ -9,8 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2009 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2012 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * Copyright (c) 2008      Institut National de Recherche en Informatique
  *                         et Automatique. All rights reserved.
@@ -57,18 +57,15 @@
 
 #include "opal/mca/installdirs/installdirs.h"
 #include "opal/util/argv.h"
+#include "opal/util/output.h"
 #include "opal/util/opal_environ.h"
-#include "opal/util/path.h"
-#include "opal/util/basename.h"
-#include "opal/mca/base/mca_base_param.h"
 
 #include "orte/util/show_help.h"
-#include "orte/util/name_fns.h"
-#include "orte/runtime/runtime.h"
+#include "orte/runtime/orte_globals.h"
 #include "orte/runtime/orte_wait.h"
-#include "orte/mca/rml/rml.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/rmaps/rmaps.h"
+#include "orte/mca/state/state.h"
 
 #include "orte/mca/plm/plm.h"
 #include "orte/mca/plm/base/base.h"
@@ -81,7 +78,6 @@
  */
 static int plm_lsf_init(void);
 static int plm_lsf_launch_job(orte_job_t *jdata);
-static int plm_lsf_terminate_job(orte_jobid_t jobid);
 static int plm_lsf_terminate_orteds(void);
 static int plm_lsf_signal_job(orte_jobid_t jobid, int32_t signal);
 static int plm_lsf_finalize(void);
@@ -95,16 +91,14 @@ orte_plm_base_module_t orte_plm_lsf_module = {
     orte_plm_base_set_hnp_name,
     plm_lsf_launch_job,
     NULL,
-    plm_lsf_terminate_job,
+    orte_plm_base_orted_terminate_job,
     plm_lsf_terminate_orteds,
+    orte_plm_base_orted_kill_local_procs,
     plm_lsf_signal_job,
     plm_lsf_finalize
 };
 
-/*
- * Local variables
- */
-static orte_jobid_t active_job = ORTE_JOBID_INVALID;
+static void launch_daemons(int fd, short args, void *cbdata);
 
 /**
  * Init the module
@@ -116,6 +110,27 @@ int plm_lsf_init(void)
     if (ORTE_SUCCESS != (rc = orte_plm_base_comm_start())) {
         ORTE_ERROR_LOG(rc);
     }
+
+    if (orte_do_not_launch) {
+        /* must assign daemons as won't be launching them */
+        orte_plm_globals.daemon_nodes_assigned_at_launch = true;
+    } else {
+        /* we do NOT assign daemons to nodes at launch - we will
+         * determine that mapping when the daemon
+         * calls back. This is required because lsf does
+         * its own mapping of proc-to-node, and we cannot know
+         * in advance which daemon will wind up on which node
+         */
+        orte_plm_globals.daemon_nodes_assigned_at_launch = false;
+    }
+
+    /* point to our launch command */
+    if (ORTE_SUCCESS != (rc = orte_state.add_job_state(ORTE_JOB_STATE_LAUNCH_DAEMONS,
+                                                       launch_daemons, ORTE_SYS_PRI))) {
+        ORTE_ERROR_LOG(rc);
+        return rc;
+    }
+
     return rc;
 }
 
@@ -124,6 +139,18 @@ int plm_lsf_init(void)
  * the job can cleanly terminate
  */
 static int plm_lsf_launch_job(orte_job_t *jdata)
+{
+    if (ORTE_JOB_CONTROL_RESTART & jdata->controls) {
+        /* this is a restart situation - skip to the mapping stage */
+        ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_MAP);
+    } else {
+        /* new job - set it up */
+        ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_INIT);
+    }
+    return ORTE_SUCCESS;
+}
+
+static void launch_daemons(int fd, short args, void *cbdata)
 {
     orte_job_map_t *map;
     size_t num_nodes;
@@ -133,83 +160,92 @@ static int plm_lsf_launch_job(orte_job_t *jdata)
     int rc;
     char** env = NULL;
     char **nodelist_argv;
+    char *nodelist;
     int nodelist_argc;
     char *vpid_string;
     int i;
     char *cur_prefix;
-    struct timeval joblaunchstart, launchstart, launchstop;
     int proc_vpid_index = 0;
     bool failed_launch = true;
-    orte_app_context_t **apps;
-    orte_node_t **nodes;
+    orte_app_context_t *app;
+    orte_node_t *node;
     orte_std_cntr_t nnode;
-    orte_jobid_t failed_job;
-    
-    /* default to declaring the daemons failed*/
-    failed_job = ORTE_PROC_MY_NAME->jobid;
+    orte_job_t *daemons;
+    orte_state_caddy_t *state = (orte_state_caddy_t*)cbdata;
+    orte_job_t *jdata = state->jdata;
 
-    if (orte_timing) {
-        if (0 != gettimeofday(&joblaunchstart, NULL)) {
-            opal_output(0, "plm_lsf: could not obtain job start time");
-        }        
-    }
-    
-    /* create a jobid for this job */
-    if (ORTE_SUCCESS != (rc = orte_plm_base_create_jobid(&jdata->jobid))) {
+    /* start by setting up the virtual machine */
+    daemons = orte_get_job_data_object(ORTE_PROC_MY_NAME->jobid);
+    if (ORTE_SUCCESS != (rc = orte_plm_base_setup_virtual_machine(jdata))) {
         ORTE_ERROR_LOG(rc);
         goto cleanup;
     }
-    
-    OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                         "%s plm:lsf: launching job %s",
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                         ORTE_JOBID_PRINT(jdata->jobid)));
-    
-    /* setup the job */
-    if (ORTE_SUCCESS != (rc = orte_plm_base_setup_job(jdata))) {
-        ORTE_ERROR_LOG(rc);
-        goto cleanup;
+
+    /* if we don't want to launch, then don't attempt to
+     * launch the daemons - the user really wants to just
+     * look at the proposed process map
+     */
+    if (orte_do_not_launch) {
+        /* set the state to indicate the daemons reported - this
+         * will trigger the daemons_reported event and cause the
+         * job to move to the following step
+         */
+        state->jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
+        ORTE_ACTIVATE_JOB_STATE(state->jdata, ORTE_JOB_STATE_DAEMONS_REPORTED);
+        OBJ_RELEASE(state);
+        return;
     }
+
+    OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
+                         "%s plm:lsf: launching vm",
+                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
     
-    /* save the active jobid */
-    active_job = jdata->jobid;
     
     /* Get the map for this job */
-    if (NULL == (map = orte_rmaps.get_job_map(active_job))) {
+    if (NULL == (map = daemons->map)) {
         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
         rc = ORTE_ERR_NOT_FOUND;
         goto cleanup;
     }
-    apps = (orte_app_context_t**)jdata->apps->addr;
-    nodes = (orte_node_t**)map->nodes->addr;
     
     num_nodes = map->num_new_daemons;
-    if (num_nodes == 0) {
-        /* have all the daemons we need - launch app */
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+    if (0 == num_nodes) {
+        /* set the state to indicate the daemons reported - this
+         * will trigger the daemons_reported event and cause the
+         * job to move to the following step
+         */
+        OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                              "%s plm:lsf: no new daemons to launch",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-        goto launch_apps;
+        state->jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
+        if (ORTE_JOB_STATE_DAEMONS_REPORTED == daemons->state) {
+            ORTE_ACTIVATE_JOB_STATE(state->jdata, ORTE_JOB_STATE_DAEMONS_REPORTED);
+        }
+        OBJ_RELEASE(state);
+        return;
     }
 
     /* create nodelist */
     nodelist_argv = NULL;
     nodelist_argc = 0;
 
-    for (nnode=0; nnode < map->num_nodes; nnode++) {
+    for (nnode=0; nnode < map->nodes->size; nnode++) {
+        if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, nnode))) {
+            continue;
+        }
         /* if the daemon already exists on this node, then
          * don't include it
          */
-        if (nodes[nnode]->daemon_launched) {
+        if (node->daemon_launched) {
             continue;
         }
         
         /* otherwise, add it to the list of nodes upon which
          * we need to launch a daemon
          */
-        opal_argv_append(&nodelist_argc, &nodelist_argv, nodes[nnode]->name);
+        opal_argv_append(&nodelist_argc, &nodelist_argv, node->name);
     }
-
+    nodelist = opal_argv_join(nodelist_argv, ',');
 
     /*
      * start building argv array
@@ -228,7 +264,8 @@ static int plm_lsf_launch_job(orte_job_t *jdata)
     orte_plm_base_orted_append_basic_args(&argc, &argv,
                                           "lsf",
                                           &proc_vpid_index,
-                                          false);
+                                          nodelist);
+    free(nodelist);
 
     /* tell the new daemons the base of the name list so they can compute
      * their own name on the other end
@@ -242,7 +279,7 @@ static int plm_lsf_launch_job(orte_job_t *jdata)
     argv[proc_vpid_index] = strdup(vpid_string);
     free(vpid_string);
 
-    if (0 < opal_output_get_verbosity(orte_plm_globals.output)) {
+    if (0 < opal_output_get_verbosity(orte_plm_base_framework.framework_output)) {
         param = opal_argv_join(argv, ' ');
         if (NULL != param) {
             opal_output(0, "plm:lsf: final top-level argv:");
@@ -254,13 +291,17 @@ static int plm_lsf_launch_job(orte_job_t *jdata)
     /* Copy the prefix-directory specified in the
        corresponding app_context.  If there are multiple,
        different prefix's in the app context, complain (i.e., only
-       allow one --prefix option for the entire slurm run -- we
+       allow one --prefix option for the entire lsf run -- we
        don't support different --prefix'es for different nodes in
-       the SLURM plm) */
+       the LSF plm) */
     cur_prefix = NULL;
-    for (i=0; i < jdata->num_apps; i++) {
-        char * app_prefix_dir = apps[i]->prefix_dir;
-         /* Check for already set cur_prefix_dir -- if different,
+    for (i=0; i < jdata->apps->size; i++) {
+        char *app_prefix_dir;
+        if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, i))) {
+            continue;
+        }
+        app_prefix_dir = app->prefix_dir;
+        /* Check for already set cur_prefix_dir -- if different,
            complain */
         if (NULL != app_prefix_dir) {
             if (NULL != cur_prefix &&
@@ -275,7 +316,7 @@ static int plm_lsf_launch_job(orte_job_t *jdata)
                same anyway */
             if (NULL == cur_prefix) {
                 cur_prefix = strdup(app_prefix_dir);
-                OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+                OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                                      "%s plm:lsf: Set prefix:%s",
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), cur_prefix));
             }
@@ -285,12 +326,6 @@ static int plm_lsf_launch_job(orte_job_t *jdata)
     /* setup environment */
     env = opal_argv_copy(orte_launch_environ);
 
-    if (orte_timing) {
-        if (0 != gettimeofday(&launchstart, NULL)) {
-            opal_output(0, "plm_lsf: could not obtain start time");
-        }        
-    }
-    
     /* lsb_launch tampers with SIGCHLD.
      * After the call to lsb_launch, the signal handler for SIGCHLD is NULL.
      * So, we disable the SIGCHLD handler of libevent for the duration of 
@@ -304,7 +339,7 @@ static int plm_lsf_launch_job(orte_job_t *jdata)
      * orterun can do the rest of its stuff. Instead, we'll catch any
      * failures and deal with them elsewhere
      */
-    if (lsb_launch(nodelist_argv, argv, LSF_DJOB_NOWAIT, env) < 0) {
+    if (lsb_launch(nodelist_argv, argv, LSF_DJOB_REPLACE_ENV | LSF_DJOB_NOWAIT, env) < 0) {
         ORTE_ERROR_LOG(ORTE_ERR_FAILED_TO_START);
         opal_output(0, "lsb_launch failed: %d", rc);
         rc = ORTE_ERR_FAILED_TO_START;
@@ -313,49 +348,14 @@ static int plm_lsf_launch_job(orte_job_t *jdata)
     }
     orte_wait_enable();  /* re-enable our SIGCHLD handler */
     
-    /* wait for daemons to callback */
-    if (ORTE_SUCCESS != 
-        (rc = orte_plm_base_daemon_callback(map->num_new_daemons))) {
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                             "%s plm:lsf: daemon launch failed for job %s on error %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             ORTE_JOBID_PRINT(active_job), ORTE_ERROR_NAME(rc)));
-        goto cleanup;
-    }
+    /* indicate that the daemons for this job were launched */
+    state->jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
+    daemons->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
 
-launch_apps:
-    /* daemons succeeded - any failure now would be from apps */
-    failed_job = active_job;
-    if (ORTE_SUCCESS != (rc = orte_plm_base_launch_apps(active_job))) {
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                             "%s plm:lsf: launch of apps failed for job %s on error %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             ORTE_JOBID_PRINT(active_job), ORTE_ERROR_NAME(rc)));
-        goto cleanup;
-    }
-    
-    /* declare the launch a success */
+    /* flag that launch was successful, so far as we currently know */
     failed_launch = false;
-    
-    if (orte_timing) {
-        if (0 != gettimeofday(&launchstop, NULL)) {
-             opal_output(0, "plm_lsf: could not obtain stop time");
-         } else {
-             opal_output(0, "plm_lsf: daemon block launch time is %ld usec",
-                         (launchstop.tv_sec - launchstart.tv_sec)*1000000 + 
-                         (launchstop.tv_usec - launchstart.tv_usec));
-             opal_output(0, "plm_lsf: total job launch time is %ld usec",
-                         (launchstop.tv_sec - joblaunchstart.tv_sec)*1000000 + 
-                         (launchstop.tv_usec - joblaunchstart.tv_usec));
-         }
-    }
 
-    if (ORTE_SUCCESS != rc) {
-        opal_output(0, "plm:lsf: start_procs returned error %d", rc);
-        goto cleanup;
-    }
-
-cleanup:
+ cleanup:
     if (NULL != argv) {
         opal_argv_free(argv);
     }
@@ -363,26 +363,13 @@ cleanup:
         opal_argv_free(env);
     }
     
+    /* cleanup the caddy */
+    OBJ_RELEASE(state);
+
     /* check for failed launch - if so, force terminate */
     if (failed_launch) {
-        orte_plm_base_launch_failed(failed_job, -1, ORTE_ERROR_DEFAULT_EXIT_CODE, ORTE_JOB_STATE_FAILED_TO_START);
+        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
     }
-
-    return rc;
-}
-
-
-static int plm_lsf_terminate_job(orte_jobid_t jobid)
-{
-    int rc;
-    
-    /* order them to kill their local procs for this job */
-    if (ORTE_SUCCESS !=
-        (rc = orte_plm_base_orted_kill_local_procs(jobid))) {
-        ORTE_ERROR_LOG(rc);
-    }
-    
-    return rc;
 }
 
 
@@ -393,11 +380,24 @@ static int plm_lsf_terminate_orteds(void)
 {
     int rc;
     
-    /* tell them to die! */
-    if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_EXIT_WITH_REPLY_CMD))) {
-        ORTE_ERROR_LOG(rc);
+    /* now tell them to die */
+    if (orte_abnormal_term_ordered) {
+        /* cannot know if a daemon is able to
+         * tell us it died, so just ensure they
+         * all terminate
+         */
+        if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_HALT_VM_CMD))) {
+            ORTE_ERROR_LOG(rc);
+        }
+    } else {
+        /* we need them to "phone home", though,
+         * so we can know that they have exited
+         */
+        if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_EXIT_CMD))) {
+            ORTE_ERROR_LOG(rc);
+        }
     }
-    
+
     return rc;
 }
 
