@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2007-2012 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * $COPYRIGHT$
  * 
@@ -56,20 +56,19 @@
 #include <tm.h>
 
 #include "opal/mca/installdirs/installdirs.h"
-#include "opal/event/event.h"
+#include "opal/mca/event/event.h"
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
 #include "orte/util/show_help.h"
 #include "opal/util/opal_environ.h"
 #include "opal/util/basename.h"
-#include "opal/mca/base/mca_base_param.h"
-#include "opal/runtime/opal_progress.h"
 
 #include "orte/util/name_fns.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/runtime/orte_wait.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/rmaps/rmaps.h"
+#include "orte/mca/state/state.h"
 
 #include "orte/mca/plm/plm.h"
 #include "orte/mca/plm/base/plm_private.h"
@@ -78,7 +77,7 @@
 
 
 /*
- * Local functions
+ * API functions
  */
 static int plm_tm_init(void);
 static int plm_tm_launch_job(orte_job_t *jdata);
@@ -86,15 +85,11 @@ static int plm_tm_terminate_orteds(void);
 static int plm_tm_signal_job(orte_jobid_t jobid, int32_t signal);
 static int plm_tm_finalize(void);
 
-static int plm_tm_connect(void);
-static int plm_tm_disconnect(void);
-static void failed_start(int fd, short event, void *arg);
-
 /*
  * Local "global" variables
  */
-static opal_event_t *ev=NULL;
-static bool local_launch_available = false;
+static orte_std_cntr_t launched = 0;
+static bool connected = false;
 
 /*
  * Global variable
@@ -111,6 +106,13 @@ orte_plm_base_module_t orte_plm_tm_module = {
     plm_tm_finalize
 };
 
+/* Local functions */
+static int plm_tm_connect(void);
+static void failed_start(int fd, short event, void *arg);
+static void launch_daemons(int fd, short args, void *cbdata);
+static void poll_spawns(int fd, short args, void *cbdata);
+
+
 /**
 * Init the module
  */
@@ -122,23 +124,50 @@ static int plm_tm_init(void)
         ORTE_ERROR_LOG(rc);
     }
 
-    if (ORTE_SUCCESS == orte_plm_base_rsh_launch_agent_setup()) {
-        local_launch_available = true;
+    /* we assign daemon nodes at launch */
+    orte_plm_globals.daemon_nodes_assigned_at_launch = true;
+
+    /* point to our launch command */
+    if (ORTE_SUCCESS != (rc = orte_state.add_job_state(ORTE_JOB_STATE_LAUNCH_DAEMONS,
+                                                       launch_daemons, ORTE_SYS_PRI))) {
+        ORTE_ERROR_LOG(rc);
+        return rc;
     }
     
+    /* overwrite the daemons_launched state to point to
+     * our own local function
+     */
+    if (ORTE_SUCCESS != (rc = orte_state.set_job_state_callback(ORTE_JOB_STATE_DAEMONS_LAUNCHED,
+                                                                poll_spawns))) {
+        ORTE_ERROR_LOG(rc);
+        return rc;
+    }
+
     return rc;
 }
 
 
+static int plm_tm_launch_job(orte_job_t *jdata)
+{
+    if (ORTE_JOB_CONTROL_RESTART & jdata->controls) {
+        /* this is a restart situation - skip to the mapping stage */
+        ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_MAP);
+    } else {
+        /* new job - set it up */
+        ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_INIT);
+    }
+    return ORTE_SUCCESS;
+}
+
 /* When working in this function, ALWAYS jump to "cleanup" if
  * you encounter an error so that orterun will be woken up and
  * the job can cleanly terminate
  */
-static int plm_tm_launch_job(orte_job_t *jdata)
+static void launch_daemons(int fd, short args, void *cbdata)
 {
     orte_job_map_t *map = NULL;
-    orte_app_context_t **apps;
-    orte_node_t **nodes;
+    orte_app_context_t *app;
+    orte_node_t *node;
     int proc_vpid_index;
     char *param;
     char **env = NULL;
@@ -147,67 +176,75 @@ static int plm_tm_launch_job(orte_job_t *jdata)
     char **nodeargv;
     int argc = 0;
     int rc;
-    bool connected = false;
-    orte_std_cntr_t launched = 0, i; 
+    orte_std_cntr_t i; 
     char *bin_base = NULL, *lib_base = NULL;
     tm_event_t *tm_events = NULL;
     tm_task_id *tm_task_ids = NULL;
-    int local_err;
-    tm_event_t event;
     bool failed_launch = true;
     mode_t current_umask;
-    orte_jobid_t failed_job;
     char *nodelist;
-    
-    if (jdata->controls & ORTE_JOB_CONTROL_LOCAL_SLAVE) {
-        /* if this is a request to launch a local slave,
-         * then we will not be launching an orted - we will
-         * directly ssh the slave process itself. No mapping
-         * is performed to support this - the caller must
-         * provide all the info required to launch the job,
-         * including the target hosts
-         */
-        if (!local_launch_available) {
-            /* if we can't support this, then abort */
-            orte_show_help("help-plm-tm.txt", "no-local-slave-support", true);
-            return ORTE_ERR_FAILED_TO_START;
-        }
-        return orte_plm_base_local_slave_launch(jdata);
+    char* vpid_string;
+    orte_job_t *daemons, *jdata;
+    orte_state_caddy_t *state = (orte_state_caddy_t*)cbdata;
+
+    jdata = state->jdata;
+
+    /* if we are launching debugger daemons, then just go
+     * do it - no new daemons will be launched
+     */
+    if (ORTE_JOB_CONTROL_DEBUGGER_DAEMON & jdata->controls) {
+        jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
+        ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_DAEMONS_REPORTED);
+        OBJ_RELEASE(state);
+        return;
     }
 
-    /* if we are timing, record the start time */
-    if (orte_timing) {
-        gettimeofday(&orte_plm_globals.daemonlaunchstart, NULL);
-    }
-    
-    /* default to declaring the daemons as failed */
-    failed_job = ORTE_PROC_MY_NAME->jobid;
-    
-    /* setup the job */
-    if (ORTE_SUCCESS != (rc = orte_plm_base_setup_job(jdata))) {
+    /* setup the virtual machine */
+    daemons = orte_get_job_data_object(ORTE_PROC_MY_NAME->jobid);
+    if (ORTE_SUCCESS != (rc = orte_plm_base_setup_virtual_machine(jdata))) {
         ORTE_ERROR_LOG(rc);
         goto cleanup;
     }
-    
-    OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                         "%s plm:tm: launching job %s",
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                         ORTE_JOBID_PRINT(jdata->jobid)));
+
+    /* if we don't want to launch, then don't attempt to
+     * launch the daemons - the user really wants to just
+     * look at the proposed process map
+     */
+    if (orte_do_not_launch) {
+        /* set the state to indicate the daemons reported - this
+         * will trigger the daemons_reported event and cause the
+         * job to move to the following step
+         */
+        jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
+        ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_DAEMONS_REPORTED);
+        OBJ_RELEASE(state);
+        return;
+    }
     
     /* Get the map for this job */
-    if (NULL == (map = orte_rmaps.get_job_map(jdata->jobid))) {
+    if (NULL == (map = daemons->map)) {
         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
         rc = ORTE_ERR_NOT_FOUND;
         goto cleanup;
     }
-    apps = (orte_app_context_t**)jdata->apps->addr;
-    nodes = (orte_node_t**)map->nodes->addr;
 
     if (0 == map->num_new_daemons) {
-        /* have all the daemons we need - launch app */
-        goto launch_apps;
+        /* set the state to indicate the daemons reported - this
+         * will trigger the daemons_reported event and cause the
+         * job to move to the following step
+         */
+        jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
+        if (ORTE_JOB_STATE_DAEMONS_REPORTED == daemons->state) {
+            ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_DAEMONS_REPORTED);
+        }
+        OBJ_RELEASE(state);
+        return;
     }
     
+    OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
+                         "%s plm:tm: launching vm",
+                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
+    
     /* Allocate a bunch of TM events to use for tm_spawn()ing */
     tm_events = malloc(sizeof(tm_event_t) * map->num_new_daemons);
     if (NULL == tm_events) {
@@ -227,8 +264,10 @@ static int plm_tm_launch_job(orte_job_t *jdata)
 
     /* create a list of nodes in this launch */
     nodeargv = NULL;
-    for (i = 0; i < map->num_nodes; i++) {
-        orte_node_t* node = nodes[i];
+    for (i = 0; i < map->nodes->size; i++) {
+        if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
+            continue;
+        }
         
         /* if this daemon already exists, don't launch it! */
         if (node->daemon_launched) {
@@ -244,23 +283,24 @@ static int plm_tm_launch_job(orte_job_t *jdata)
     /* Add basic orted command line options */
     orte_plm_base_orted_append_basic_args(&argc, &argv, "tm",
                                           &proc_vpid_index,
-                                          true, nodelist);
+                                          nodelist);
     free(nodelist);
     
-    if (0 < opal_output_get_verbosity(orte_plm_globals.output)) {
+    if (0 < opal_output_get_verbosity(orte_plm_base_framework.framework_output)) {
         param = opal_argv_join(argv, ' ');
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+        OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                              "%s plm:tm: final top-level argv:\n\t%s",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              (NULL == param) ? "NULL" : param));
         if (NULL != param) free(param);
     }
 
-    rc = plm_tm_connect();
-    if (ORTE_SUCCESS != rc) {
-        goto cleanup;
+    if (!connected) {
+        if (ORTE_SUCCESS != plm_tm_connect()) {
+            goto cleanup;
+        }
+        connected = true;
     }
-    connected = true;
 
     /* Figure out the basenames for the libdir and bindir.  There is a
        lengthy comment about this in plm_rsh_module.c explaining all
@@ -272,7 +312,7 @@ static int plm_tm_launch_job(orte_job_t *jdata)
     env = opal_argv_copy(orte_launch_environ);
 
     /* enable local launch by the orteds */
-    var = mca_base_param_environ_variable("plm", NULL, NULL);
+    (void) mca_base_var_env_name ("plm", &var);
     opal_setenv(var, "rsh", true, &env);
     free(var);
     
@@ -289,15 +329,16 @@ static int plm_tm_launch_job(orte_job_t *jdata)
         always be at least one app_context, we take it from
         there
     */
-    if (NULL != apps[0]->prefix_dir) {
+    app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, 0);
+    if (NULL != app->prefix_dir) {
         char *newenv;
         
         for (i = 0; NULL != env && NULL != env[i]; ++i) {
             /* Reset PATH */
             if (0 == strncmp("PATH=", env[i], 5)) {
                 asprintf(&newenv, "%s/%s:%s", 
-                            apps[0]->prefix_dir, bin_base, env[i] + 5);
-                OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+                            app->prefix_dir, bin_base, env[i] + 5);
+                OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                                      "%s plm:tm: resetting PATH: %s",
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                      newenv));
@@ -308,8 +349,8 @@ static int plm_tm_launch_job(orte_job_t *jdata)
             /* Reset LD_LIBRARY_PATH */
             else if (0 == strncmp("LD_LIBRARY_PATH=", env[i], 16)) {
                 asprintf(&newenv, "%s/%s:%s", 
-                            apps[0]->prefix_dir, lib_base, env[i] + 16);
-                OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+                            app->prefix_dir, lib_base, env[i] + 16);
+                OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                                      "%s plm:tm: resetting LD_LIBRARY_PATH: %s",
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                      newenv));
@@ -322,22 +363,22 @@ static int plm_tm_launch_job(orte_job_t *jdata)
     /* Iterate through each of the nodes and spin
      * up a daemon.
      */
-    for (i = 0; i < map->num_nodes; i++) {
-        orte_node_t* node = nodes[i];
-        char* vpid_string;
-        
+    for (i = 0; i < map->nodes->size; i++) {
+        if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
+            continue;
+        }
         /* if this daemon already exists, don't launch it! */
         if (node->daemon_launched) {
             continue;
         }
  
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+        OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                              "%s plm:tm: launching on node %s",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              node->name));
         
         /* setup process name */
-        rc = orte_util_convert_vpid_to_string(&vpid_string, nodes[i]->daemon->name.vpid);
+        rc = orte_util_convert_vpid_to_string(&vpid_string, node->daemon->name.vpid);
         if (ORTE_SUCCESS != rc) {
             opal_output(0, "plm:tm: unable to get daemon vpid as string");
             exit(-1);
@@ -347,9 +388,9 @@ static int plm_tm_launch_job(orte_job_t *jdata)
         free(vpid_string);
         
         /* exec the daemon */
-        if (0 < opal_output_get_verbosity(orte_plm_globals.output)) {
+        if (0 < opal_output_get_verbosity(orte_plm_base_framework.framework_output)) {
             param = opal_argv_join(argv, ' ');
-            OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+            OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                                  "%s plm:tm: executing:\n\t%s",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                  (NULL == param) ? "NULL" : param));
@@ -365,117 +406,74 @@ static int plm_tm_launch_job(orte_job_t *jdata)
         }
         
         launched++;
-
-        /* Allow some progress to occur */
-        opal_event_loop(OPAL_EVLOOP_NONBLOCK);
     }
 
-    OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+    /* indicate that the daemons for this job were launched */
+    state->jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
+    daemons->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
+
+    /* flag that launch was successful, so far as we currently know */
+    failed_launch = false;
+
+    OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                          "%s plm:tm:launch: finished spawning orteds",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
 
+ cleanup:
+    /* cleanup */
+    OBJ_RELEASE(state);
+
+    /* check for failed launch - if so, force terminate */
+    if (failed_launch) {
+        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+    }
+}
+
+static void poll_spawns(int fd, short args, void *cbdata)
+{
+    orte_state_caddy_t *state = (orte_state_caddy_t*)cbdata;
+    int i, rc;
+    bool failed_launch = true;
+    int local_err;
+    tm_event_t event;
+
     /* TM poll for all the spawns */
     for (i = 0; i < launched; ++i) {
         rc = tm_poll(TM_NULL_EVENT, &event, 1, &local_err);
         if (TM_SUCCESS != rc) {
-            errno = local_err;
             opal_output(0, "plm:tm: failed to poll for a spawned daemon, return status = %d", rc);
             goto cleanup;
         }
         if (TM_SUCCESS != local_err) {
-            errno = local_err;
-            opal_output(0, "plm:tm: failed to spawn daemon, error code = %d", errno );
+            opal_output(0, "plm:tm: failed to spawn daemon, error code = %d", local_err );
             goto cleanup;
         }
     }
-    
+    failed_launch = false;
+
+#if 0
     /* set a timer to tell us if one or more daemon's fails to start - use the
      * millisec/daemon timeout provided by the user to compute time
      */
     if (0 < orte_startup_timeout) {
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+        OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                              "%s plm:tm: setting startup timer for %d milliseconds",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              orte_startup_timeout));
-        ORTE_DETECT_TIMEOUT(&ev, map->num_new_daemons,
+        ORTE_DETECT_TIMEOUT(map->num_new_daemons,
                             orte_startup_timeout*1000,
-                            -1, failed_start);
+                            -1, failed_start, state->jdata);
     }
-    
-    /* wait for daemons to callback */
-    if (ORTE_SUCCESS != (rc = orte_plm_base_daemon_callback(map->num_new_daemons))) {
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                             "%s plm:tm: daemon launch failed for job %s on error %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             ORTE_JOBID_PRINT(jdata->jobid), ORTE_ERROR_NAME(rc)));
-        goto cleanup;
-    }
-    
-    /* if issued, cancel the failed-to-start timer */
-    if (NULL != ev) {
-        opal_event_del(ev);
-    }
-    
-launch_apps:
-    /* since the daemons have launched, any failures now will be for the
-     * application job
-     */
-    failed_job = jdata->jobid;
-    if (ORTE_SUCCESS != (rc = orte_plm_base_launch_apps(jdata->jobid))) {
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                             "%s plm:tm: launch of apps failed for job %s on error %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             ORTE_JOBID_PRINT(jdata->jobid), ORTE_ERROR_NAME(rc)));
-        goto cleanup;
-    }
-    
-    /* if we get here, then everything launched okay - record that fact */
-    failed_launch = false;
-    
+#endif
     
  cleanup:
-    if (NULL != argv) {
-        opal_argv_free(argv);
-    }
-    if (NULL != env) {
-        opal_argv_free(env);
-    }
-    
-    if (connected) {
-        plm_tm_disconnect();
-    }
-    if (NULL != tm_events) {
-        free(tm_events);
-    }
-    if (NULL != tm_task_ids) {
-        free(tm_task_ids);
-    }
-    
-    if (NULL != lib_base) {
-        free(lib_base);
-    }
-    if (NULL != bin_base) {
-        free(bin_base);
-    }
+    /* cleanup */
+    OBJ_RELEASE(state);
 
     /* check for failed launch - if so, force terminate */
     if (failed_launch) {
-        orte_plm_base_launch_failed(failed_job, -1, ORTE_ERROR_DEFAULT_EXIT_CODE, ORTE_JOB_STATE_FAILED_TO_START);
+        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
     }
-        
-    /* setup a "heartbeat" timer to periodically check on
-     * the state-of-health of the orteds, if requested AND
-     * we actually launched some daemons!
-     */
-    if (0 < map->num_new_daemons) {
-        orte_plm_base_start_heart();
-    }
-    
-    OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                         "%s plm:tm:launch: finished",
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-
-    return rc;
 }
 
 
@@ -486,9 +484,22 @@ int plm_tm_terminate_orteds(void)
 {
     int rc;
     
-    /* now tell them to die! */
-    if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_EXIT_CMD))) {
-        ORTE_ERROR_LOG(rc);
+    /* now tell them to die */
+    if (orte_abnormal_term_ordered) {
+        /* cannot know if a daemon is able to
+         * tell us it died, so just ensure they
+         * all terminate
+         */
+        if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_HALT_VM_CMD))) {
+            ORTE_ERROR_LOG(rc);
+        }
+    } else {
+        /* we need them to "phone home", though,
+         * so we can know that they have exited
+         */
+        if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_EXIT_CMD))) {
+            ORTE_ERROR_LOG(rc);
+        }
     }
     
     return rc;
@@ -519,6 +530,11 @@ static int plm_tm_finalize(void)
         ORTE_ERROR_LOG(rc);
     }
 
+    if (connected) {
+        tm_finalize();
+        connected = false;
+    }
+
     return ORTE_SUCCESS;
 }
 
@@ -527,7 +543,8 @@ static int plm_tm_connect(void)
 {
     int ret;
     struct tm_roots tm_root;
-    int count, progress;
+    int count;
+    struct timespec tp = {0, 100};
 
     /* try a couple times to connect - might get busy signals every
        now and then */
@@ -537,42 +554,14 @@ static int plm_tm_connect(void)
             return ORTE_SUCCESS;
         }
 
-        for (progress = 0 ; progress < 10 ; ++progress) {
-            opal_progress();
+        /* provide a very short quiet period so we
+         * don't hammer the cpu while we wait
+         */
+        nanosleep(&tp, NULL);
 #if HAVE_SCHED_YIELD
-            sched_yield();
+        sched_yield();
 #endif
-        }
     }
-
+    
     return ORTE_ERR_RESOURCE_BUSY;
 }
-
-
-static int plm_tm_disconnect(void)
-{
-    tm_finalize();
-
-    return ORTE_SUCCESS;
-}
-
-/* call this function if the timer fires indicating that one
- * or more daemons failed to start
- */
-static void failed_start(int fd, short dummy, void *arg)
-{
-    OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                         "%s plm:tm:failed_start",
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-    
-    /* if we are aborting, ignore this */
-    if (orte_abnormal_term_ordered) {
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                             "%s plm:tm:failed_start - abnormal term in progress",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-        return;
-    }
-    
-    orte_plm_base_launch_failed(ORTE_PROC_MY_NAME->jobid, -1,
-                                ORTE_ERROR_DEFAULT_EXIT_CODE, ORTE_JOB_STATE_FAILED_TO_START);  
-}
