@@ -9,6 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2011-2012 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -29,10 +31,11 @@
 #include "opal/mca/base/base.h"
 #include "opal/class/opal_list.h"
 #include "opal/util/output.h"
+#include "opal/dss/dss.h"
 
 #include "orte/util/show_help.h"
-#include "opal/dss/dss.h"
 #include "orte/mca/errmgr/errmgr.h"
+#include "orte/mca/rmaps/base/base.h"
 #include "orte/util/name_fns.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/runtime/orte_wait.h"
@@ -40,6 +43,8 @@
 #include "orte/util/dash_host/dash_host.h"
 #include "orte/util/proc_info.h"
 #include "orte/util/comm/comm.h"
+#include "orte/mca/state/state.h"
+#include "orte/runtime/orte_quit.h"
 
 #include "orte/mca/ras/base/ras_private.h"
 
@@ -47,7 +52,7 @@
 static void display_alloc(void)
 {
     char *tmp=NULL, *tmp2, *tmp3, *pfx=NULL;
-    int i;
+    int i, istart;
     orte_node_t *alloc;
     
     if (orte_xml_output) {
@@ -56,7 +61,12 @@ static void display_alloc(void)
     } else {
         asprintf(&tmp, "\n======================   ALLOCATED NODES   ======================\n");
     }
-    for (i=0; i < orte_node_pool->size; i++) {
+    if (orte_hnp_is_allocated) {
+            istart = 0;
+    } else {
+        istart = 1;
+    }
+    for (i=istart; i < orte_node_pool->size; i++) {
         if (NULL == (alloc = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, i))) {
             continue;
         }
@@ -83,38 +93,36 @@ static void display_alloc(void)
  * Function for selecting one component from all those that are
  * available.
  */
-int orte_ras_base_allocate(orte_job_t *jdata)
+void orte_ras_base_allocate(int fd, short args, void *cbdata)
 {
     int rc;
+    orte_job_t *jdata;
     opal_list_t nodes;
     orte_node_t *node;
     orte_std_cntr_t i;
-    bool override_oversubscribed;
     orte_app_context_t *app;
+    orte_state_caddy_t *caddy = (orte_state_caddy_t*)cbdata;
     bool default_hostfile_used;
 
-    OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+    OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                          "%s ras:base:allocate",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
     
+    /* convenience */
+    jdata = caddy->jdata;
+
     /* if we already did this, don't do it again - the pool of
      * global resources is set. 
      */
     if (orte_ras_base.allocation_read) {
         
-        OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+        OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                              "%s ras:base:allocate allocation already read",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-        
-        /* loop through the global node pool and set the
-         * number of allocated slots to the difference
-         * between slots and slots_in_use. Note that
-         * oversubscription will still allow procs to
-         * be mapped up to slots_max
-         */
-        return ORTE_SUCCESS;
+        goto next_state;
     }
-    
+    orte_ras_base.allocation_read = true;
+
     /* Otherwise, we have to create
      * the initial set of resources that will delineate all
      * further operations serviced by this HNP. This list will
@@ -124,11 +132,6 @@ int orte_ras_base_allocate(orte_job_t *jdata)
      * no job launched by this HNP will be able to utilize it.
      */
     
-    /* note that the allocation has been read so we don't
-     * come in here again!
-     */
-    orte_ras_base.allocation_read = true;
-
     /* construct a list to hold the results */
     OBJ_CONSTRUCT(&nodes, opal_list_t);
 
@@ -147,20 +150,28 @@ int orte_ras_base_allocate(orte_job_t *jdata)
             }
             ORTE_ERROR_LOG(rc);
             OBJ_DESTRUCT(&nodes);
-            return rc;
+            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            OBJ_RELEASE(caddy);
+            return;
         }
     } 
     /* If something came back, save it and we are done */
     if (!opal_list_is_empty(&nodes)) {
         /* store the results in the global resource pool - this removes the
-        * list items
-        */
+         * list items
+         */
         if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
             ORTE_ERROR_LOG(rc);
             OBJ_DESTRUCT(&nodes);
-            return rc;
+            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            OBJ_RELEASE(caddy);
+            return;
         }
         OBJ_DESTRUCT(&nodes);
+        /* default to no-oversubscribe-allowed for managed systems */
+        if (!(ORTE_MAPPING_SUBSCRIBE_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping))) {
+            ORTE_SET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_NO_OVERSUBSCRIBE);
+        }
         /* flag that the allocation is managed */
         orte_managed_allocation = true;
         goto DISPLAY;
@@ -170,14 +181,12 @@ int orte_ras_base_allocate(orte_job_t *jdata)
          */
         OBJ_DESTRUCT(&nodes);
         orte_show_help("help-ras-base.txt", "ras-base:no-allocation", true);
-        ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
-        orte_trigger_event(&orte_exit);
-        return ORTE_ERROR;
+        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+        OBJ_RELEASE(caddy);
+        return;
     }
     
-    
-    
-    OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+    OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                          "%s ras:base:allocate nothing found in module - proceeding to hostfile",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
     
@@ -209,56 +218,55 @@ int orte_ras_base_allocate(orte_job_t *jdata)
             continue;
         }
         if (NULL != app->hostfile) {
-            OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+            OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                                  "%s ras:base:allocate adding hostfile %s",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                  app->hostfile));
             
             /* hostfile was specified - parse it and add it to the list */
             if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
-                                                                   &override_oversubscribed,
                                                                    app->hostfile))) {
                 ORTE_ERROR_LOG(rc);
                 OBJ_DESTRUCT(&nodes);
-                return rc;
+                /* set an error event */
+                ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                OBJ_RELEASE(caddy);
+                return;
             }
         } else if (NULL != app->dash_host) {
-            OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+            OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                                  "%s ras:base:allocate adding dash_hosts",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
             if (ORTE_SUCCESS != (rc = orte_util_add_dash_host_nodes(&nodes,
-                                                                    &override_oversubscribed,
                                                                     app->dash_host))) {
                 ORTE_ERROR_LOG(rc);
                 OBJ_DESTRUCT(&nodes);
-                return rc;
+                ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                OBJ_RELEASE(caddy);
+                return;
             }
         } else if (!default_hostfile_used) {
             if (NULL != orte_default_hostfile) {
-                OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+                OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                                      "%s ras:base:allocate parsing default hostfile %s",
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                      orte_default_hostfile));
         
                 /* a default hostfile was provided - parse it */
                 if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
-                                                                       &override_oversubscribed,
                                                                        orte_default_hostfile))) {
                     ORTE_ERROR_LOG(rc);
                     OBJ_DESTRUCT(&nodes);
-                    return rc;
+                    ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                    OBJ_RELEASE(caddy);
+                    return;
                 }
             }
             /* only look at it once */
             default_hostfile_used = true;
         }
-        /* update the jdata object with override_oversubscribed flag, but don't
-         * overwrite a prior setting to true
-         */
-        if (!jdata->oversubscribe_override) {
-            jdata->oversubscribe_override = override_oversubscribed;
-        }
     }
+
     /* if something was found in the hostfile(s), we use that as our global
      * pool - set it and we are done
      */
@@ -268,16 +276,17 @@ int orte_ras_base_allocate(orte_job_t *jdata)
          */
         if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
             ORTE_ERROR_LOG(rc);
-            OBJ_DESTRUCT(&nodes);
-            return rc;
+            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            OBJ_RELEASE(caddy);
+            return;
         }
         /* cleanup */
         OBJ_DESTRUCT(&nodes);
         goto DISPLAY;
     }
-
-    OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
-                         "%s ras:base:allocate nothing found in hostfiles or dash-host - checking for rankfile",
+    
+    OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
+                         "%s ras:base:allocate nothing found in hostfiles - checking for rankfile",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
     
     /* Our next option is to look for a rankfile - if one was provided, we
@@ -286,11 +295,12 @@ int orte_ras_base_allocate(orte_job_t *jdata)
     if (NULL != orte_rankfile) {
         /* check the rankfile for node information */
         if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
-                                                               &override_oversubscribed,
                                                                orte_rankfile))) {
             ORTE_ERROR_LOG(rc);
             OBJ_DESTRUCT(&nodes);
-            return rc;
+            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            OBJ_RELEASE(caddy);
+            return ;
         }
     }
     /* if something was found in rankfile, we use that as our global
@@ -302,20 +312,25 @@ int orte_ras_base_allocate(orte_job_t *jdata)
          */
         if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
             ORTE_ERROR_LOG(rc);
+            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            OBJ_RELEASE(caddy);
+            return;
+        }
+        /* rankfile is considered equivalent to an RM allocation */
+        if (!(ORTE_MAPPING_SUBSCRIBE_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping))) {
+            ORTE_SET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_NO_OVERSUBSCRIBE);
         }
-        /* update the jdata object with override_oversubscribed flag */
-        jdata->oversubscribe_override = false;
         /* cleanup */
         OBJ_DESTRUCT(&nodes);
         goto DISPLAY;
     }
     
     
-    OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+    OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                          "%s ras:base:allocate nothing found in rankfile - inserting current node",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
     
-addlocal:
+ addlocal:
     /* if nothing was found by any of the above methods, then we have no
      * earthly idea what to do - so just add the local host
      */
@@ -323,7 +338,9 @@ addlocal:
     if (NULL == node) {
         ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
         OBJ_DESTRUCT(&nodes);
-        return ORTE_ERR_OUT_OF_RESOURCE;
+        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+        OBJ_RELEASE(caddy);
+        return;
     }
     /* use the same name we got in orte_process_info so we avoid confusion in
      * the session directories
@@ -333,8 +350,6 @@ addlocal:
     node->slots_inuse = 0;
     node->slots_max = 0;
     node->slots = 1;
-    /* indicate that we don't know anything about over_subscribing */
-    jdata->oversubscribe_override = true;
     opal_list_append(&nodes, &node->super);
     
     /* store the results in the global resource pool - this removes the
@@ -343,30 +358,42 @@ addlocal:
     if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
         ORTE_ERROR_LOG(rc);
         OBJ_DESTRUCT(&nodes);
-        return rc;
+        ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+        OBJ_RELEASE(caddy);
+        return;
     }
     OBJ_DESTRUCT(&nodes);
 
-DISPLAY:
+ DISPLAY:
+    /* shall we display the results? */
+    if (4 < opal_output_get_verbosity(orte_ras_base_framework.framework_output) || orte_ras_base.display_alloc) {
+        display_alloc();
+    }
+
+ next_state:
     /* are we to report this event? */
     if (orte_report_events) {
         if (ORTE_SUCCESS != (rc = orte_util_comm_report_event(ORTE_COMM_EVENT_ALLOCATE))) {
             ORTE_ERROR_LOG(rc);
+            ORTE_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+            OBJ_RELEASE(caddy);
         }
     }
-    /* shall we display the results? */
-    if (orte_ras_base.display_alloc) {
-        display_alloc();
-    }
     
-    return rc;
+    /* set total slots alloc */
+    jdata->total_slots_alloc = orte_ras_base.total_slots_alloc;
+
+    /* set the job state to the next position */
+    ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_ALLOCATION_COMPLETE);
+
+    /* cleanup */
+    OBJ_RELEASE(caddy);
 }
 
 int orte_ras_base_add_hosts(orte_job_t *jdata)
 {
     int rc;
     opal_list_t nodes;
-    bool override_oversubscribed;
     int i;
     orte_app_context_t *app;
 
@@ -391,14 +418,13 @@ int orte_ras_base_add_hosts(orte_job_t *jdata)
             continue;
         }
         if (NULL != app->add_hostfile) {
-            OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+            OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                                  "%s ras:base:add_hosts checking add-hostfile %s",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                  app->add_hostfile));
             
             /* hostfile was specified - parse it and add it to the list */
             if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
-                                                                   &override_oversubscribed,
                                                                    app->add_hostfile))) {
                 ORTE_ERROR_LOG(rc);
                 OBJ_DESTRUCT(&nodes);
@@ -422,7 +448,6 @@ int orte_ras_base_add_hosts(orte_job_t *jdata)
         }
         if (NULL != app->add_host) {
             if (ORTE_SUCCESS != (rc = orte_util_add_dash_host_nodes(&nodes,
-                                                                    &override_oversubscribed,
                                                                     app->add_host))) {
                 ORTE_ERROR_LOG(rc);
                 OBJ_DESTRUCT(&nodes);
@@ -439,14 +464,12 @@ int orte_ras_base_add_hosts(orte_job_t *jdata)
         if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
             ORTE_ERROR_LOG(rc);
         }
-        /* update the jdata object with override_oversubscribed flag */
-        jdata->oversubscribe_override = override_oversubscribed;
         /* cleanup */
         OBJ_DESTRUCT(&nodes);
     }
     
     /* shall we display the results? */
-    if (orte_ras_base.display_alloc) {
+    if (0 < opal_output_get_verbosity(orte_ras_base_framework.framework_output) || orte_ras_base.display_alloc) {
         display_alloc();
     }
     
