@@ -9,6 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2011-2012 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,7 +23,6 @@
 
 #include <string.h>
 
-#include "orte/util/show_help.h"
 #include "opal/util/argv.h"
 #include "opal/util/if.h"
 
@@ -66,6 +67,7 @@ int orte_ras_base_node_insert(opal_list_t* nodes, orte_job_t *jdata)
     orte_std_cntr_t num_nodes;
     int rc, i;
     orte_node_t *node, *hnp_node;
+    char *ptr;
 
     /* get the number of nodes */
     num_nodes = (orte_std_cntr_t)opal_list_get_size(nodes);
@@ -73,7 +75,7 @@ int orte_ras_base_node_insert(opal_list_t* nodes, orte_job_t *jdata)
         return ORTE_SUCCESS;  /* nothing to do */
     }
     
-    OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+    OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                          "%s ras:base:node_insert inserting %ld nodes",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          (long)num_nodes));
@@ -87,7 +89,7 @@ int orte_ras_base_node_insert(opal_list_t* nodes, orte_job_t *jdata)
     }
     
     /* get the hnp node's info */
-    hnp_node = (orte_node_t*)(orte_node_pool->addr[0]);
+    hnp_node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, 0);
     
     /* cycle through the list */
     while (NULL != (item = opal_list_remove_first(nodes))) {
@@ -97,16 +99,21 @@ int orte_ras_base_node_insert(opal_list_t* nodes, orte_job_t *jdata)
          * first position since it is the first one entered. We need to check to see
          * if this node is the same as the HNP's node so we don't double-enter it
          */
-        if (opal_ifislocal(node->name)) {
-            OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+        if (NULL != hnp_node &&
+            (0 == strcmp(node->name, hnp_node->name) || opal_ifislocal(node->name))) {
+            OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                                  "%s ras:base:node_insert updating HNP info to %ld slots",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                  (long)node->slots));
             
             /* flag that hnp has been allocated */
             orte_hnp_is_allocated = true;
-            /* adjust the total slots in the job */
-            jdata->total_slots_alloc -= hnp_node->slots;
+            /* adjust the total slots in the job - note
+             * that the HNP ess module will not have entered
+             * a value for its slots, so that will be zero.
+             * Thus, this only has impact if we re-allocate
+             */
+            orte_ras_base.total_slots_alloc -= hnp_node->slots;
             /* copy the allocation data to that node's info */
             hnp_node->slots = node->slots;
             hnp_node->slots_max = node->slots_max;
@@ -123,22 +130,22 @@ int orte_ras_base_node_insert(opal_list_t* nodes, orte_job_t *jdata)
                 /* if the node name is different, store it as an alias */
                 if (0 != strcmp(node->name, hnp_node->name)) {
                     /* add to list of aliases for this node - only add if unique */
-                    opal_argv_append_unique_nosize(&hnp_node->alias, node->name);
+                    opal_argv_append_unique_nosize(&hnp_node->alias, node->name, false);
                 }
                 if (NULL != node->alias) {
                     /* now copy over any aliases that are unique */
                     for (i=0; NULL != node->alias[i]; i++) {
-                        opal_argv_append_unique_nosize(&hnp_node->alias, node->alias[i]);
+                        opal_argv_append_unique_nosize(&hnp_node->alias, node->alias[i], false);
                     }
                 }
             }
             /* update the total slots in the job */
-            jdata->total_slots_alloc += hnp_node->slots;
+            orte_ras_base.total_slots_alloc += hnp_node->slots;
             /* don't keep duplicate copy */
             OBJ_RELEASE(node);
         } else {
             /* insert the object onto the orte_nodes global array */
-            OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+            OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                                  "%s ras:base:node_insert node %s",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                  (NULL == node->name) ? "NULL" : node->name));
@@ -153,7 +160,21 @@ int orte_ras_base_node_insert(opal_list_t* nodes, orte_job_t *jdata)
                 return rc;
             }
             /* update the total slots in the job */
-            jdata->total_slots_alloc += node->slots;
+            orte_ras_base.total_slots_alloc += node->slots;
+            /* check if we have fqdn names in the allocation */
+            if (NULL != strchr(node->name, '.')) {
+                orte_have_fqdn_allocation = true;
+            }
+        }
+    }
+
+    /* if we didn't find any fqdn names in the allocation, then
+     * ensure we don't have any domain info in the node record
+     * for the hnp
+     */
+    if (!orte_have_fqdn_allocation) {
+        if (NULL != (ptr = strchr(hnp_node->name, '.'))) {
+            *ptr = '\0';
         }
     }
     
