@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2008 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -9,6 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2011      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -100,9 +102,6 @@
  * beginning at node 512, then the RAS can derive the nodenames from
  * this information).
  * 
- * - cellid: the id assigned to the cell within which this node
- * exists. This id is provided in the ORTE_RESOURCE_SEGMENT.
- * 
  * For each node, the RAS stores the following information on the segment:
  * 
  * - number of cpus allocated from this node to the user. This will
@@ -160,32 +159,33 @@
 #define ORTE_MCA_RAS_H
 
 #include "orte_config.h"
-#include "orte/orte_constants.h"
+#include "orte/constants.h"
+#include "orte/types.h"
 
 #include "opal/mca/mca.h"
+#include "opal/mca/event/event.h"
 #include "opal/class/opal_list.h"
 
-#include "orte/mca/ns/ns_types.h"
+#include "orte/runtime/orte_globals.h"
 
 #include "ras_types.h"
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
+
+/* allocation event - the event one activates to schedule resource
+ * allocation for pending jobs
+ */
+ORTE_DECLSPEC extern opal_event_t orte_allocate_event;
 
 /*
- * ras module functions
+ * ras module functions - these are not accessible to the outside world,
+ * but are defined here by convention
  */
 
 /**
  * Allocate resources to a job.
  */
-typedef int (*orte_ras_base_module_allocate_fn_t)(orte_jobid_t jobid, opal_list_t *attributes);
-
-/**
- * Deallocate resources from a job
- */
-typedef int (*orte_ras_base_module_deallocate_fn_t)(orte_jobid_t jobid);
+typedef int (*orte_ras_base_module_allocate_fn_t)(opal_list_t *nodes);
 
 /**
  * Cleanup module resources.
@@ -193,48 +193,18 @@ typedef int (*orte_ras_base_module_deallocate_fn_t)(orte_jobid_t jobid);
 typedef int (*orte_ras_base_module_finalize_fn_t)(void);
 
 /**
- * Add a node to the current allocation
+ * ras module
  */
-typedef int (*orte_ras_base_module_node_insert_fn_t)(opal_list_t *);
-
-/**
- * Query for a list of nodes in current allocation
- */
-typedef int (*orte_ras_base_module_node_query_fn_t)(opal_list_t *);
-
-/**
-    * Query the registry for all nodes allocated to a specific job
- */
-typedef int (*orte_ras_base_module_node_query_alloc_fn_t)(opal_list_t*, orte_jobid_t);
-
-/*
- * Query the registry for a specific node 
- */
-typedef orte_ras_node_t* (*orte_ras_base_module_node_lookup_fn_t)(orte_cellid_t, const char* nodename);
-
-/**
- * ras module version 1.3.0
- */
-struct orte_ras_base_module_1_3_0_t {
+struct orte_ras_base_module_2_0_0_t {
     /** Allocation function pointer */
-    orte_ras_base_module_allocate_fn_t              allocate_job;
-    /** Node Insertion function pointer */
-    orte_ras_base_module_node_insert_fn_t           node_insert;
-    /** Node Query function pointer */
-    orte_ras_base_module_node_query_fn_t            node_query;
-    /* node query allocate function pointer */
-    orte_ras_base_module_node_query_alloc_fn_t      node_query_alloc;
-    /* node lookup */
-    orte_ras_base_module_node_lookup_fn_t           node_lookup;
-    /** Deallocation function pointer */
-    orte_ras_base_module_deallocate_fn_t            deallocate_job;
+    orte_ras_base_module_allocate_fn_t              allocate;
     /** Finalization function pointer */
     orte_ras_base_module_finalize_fn_t              finalize;
 };
 /** Convenience typedef */
-typedef struct orte_ras_base_module_1_3_0_t orte_ras_base_module_1_3_0_t;
+typedef struct orte_ras_base_module_2_0_0_t orte_ras_base_module_2_0_0_t;
 /** Convenience typedef */
-typedef orte_ras_base_module_1_3_0_t orte_ras_base_module_t;
+typedef orte_ras_base_module_2_0_0_t orte_ras_base_module_t;
 
 /*
  * ras component
@@ -242,44 +212,29 @@ typedef orte_ras_base_module_1_3_0_t orte_ras_base_module_t;
 
 /**
  * Component init / selection
+ * ras component
  */
-typedef orte_ras_base_module_t* (*orte_ras_base_component_init_fn_t)(int* priority);
-
- 
-/**
- * ras component version 1.0.0
- */
-struct orte_ras_base_component_1_3_0_t {
+struct orte_ras_base_component_2_0_0_t {
     /** Base MCA structure */
-    mca_base_component_t ras_version;
+    mca_base_component_t base_version;
     /** Base MCA data */
-    mca_base_component_data_1_0_0_t ras_data;
-    /** Initialization / selection function pointer */
-    orte_ras_base_component_init_fn_t ras_init;
+    mca_base_component_data_t base_data;
 };
 /** Convenience typedef */
-typedef struct orte_ras_base_component_1_3_0_t orte_ras_base_component_1_3_0_t;
+typedef struct orte_ras_base_component_2_0_0_t orte_ras_base_component_2_0_0_t;
 /** Convenience typedef */
-typedef orte_ras_base_component_1_3_0_t orte_ras_base_component_t;
+typedef orte_ras_base_component_2_0_0_t orte_ras_base_component_t;
 
 
 /**
- * Macro for use in components that are of type ras v1.0.0
+ * Macro for use in components that are of type ras
  */
-#define ORTE_RAS_BASE_VERSION_1_3_0 \
-  /* ras v1.3 is chained to MCA v1.0 */ \
-  MCA_BASE_VERSION_1_0_0, \
-  /* ras v1.3 */ \
-  "ras", 1, 3, 0
+#define ORTE_RAS_BASE_VERSION_2_0_0 \
+  MCA_BASE_VERSION_2_0_0, \
+  "ras", 2, 0, 0
 
-/*
- * global module that holds function pointers
- */
-ORTE_DECLSPEC extern orte_ras_base_module_t orte_ras;
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 
 #endif
 
