@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2011      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * $COPYRIGHT$
  * 
@@ -120,20 +120,22 @@ static int orte_ras_slurm_allocate(opal_list_t *nodes)
     free(regexp);
     free(node_tasks);
     if (ORTE_SUCCESS != ret) {
-        OPAL_OUTPUT_VERBOSE((1, orte_ras_base.ras_output,
+        OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                              "%s ras:slurm:allocate: discover failed!",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
         return ret;
     }
+    /* record the number of allocated nodes */
+    orte_num_allocated_nodes = opal_list_get_size(nodes);
 
     /* All done */
 
     if (ORTE_SUCCESS == ret) {
-        OPAL_OUTPUT_VERBOSE((1, orte_ras_base.ras_output,
+        OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                              "%s ras:slurm:allocate: success",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
     } else {
-        OPAL_OUTPUT_VERBOSE((1, orte_ras_base.ras_output,
+        OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                              "%s ras:slurm:allocate: failure (base_allocate_nodes=%d)",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), ret));
     }
@@ -146,7 +148,7 @@ static int orte_ras_slurm_allocate(opal_list_t *nodes)
 static int orte_ras_slurm_finalize(void)
 {
     
-    OPAL_OUTPUT_VERBOSE((1, orte_ras_base.ras_output,
+    OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                          "%s ras:slurm:finalize: success (nothing to do)",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
     return ORTE_SUCCESS;
@@ -184,7 +186,7 @@ static int orte_ras_slurm_discover(char *regexp, char *tasks_per_node,
         return ORTE_ERR_OUT_OF_RESOURCE;
     }
     
-    OPAL_OUTPUT_VERBOSE((1, orte_ras_base.ras_output,
+    OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                          "%s ras:slurm:allocate:discover: checking nodelist: %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          regexp));
@@ -256,7 +258,7 @@ static int orte_ras_slurm_discover(char *regexp, char *tasks_per_node,
         } else {
             /* If we didn't find a range, just add the node */
             
-            OPAL_OUTPUT_VERBOSE((1, orte_ras_base.ras_output,
+            OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                                  "%s ras:slurm:allocate:discover: found node %s",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                  base));
@@ -340,7 +342,7 @@ static int orte_ras_slurm_discover(char *regexp, char *tasks_per_node,
     for (i = 0; NULL != names && NULL != names[i]; ++i) {
         orte_node_t *node;
         
-        OPAL_OUTPUT_VERBOSE((1, orte_ras_base.ras_output,
+        OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                              "%s ras:slurm:allocate:discover: adding node %s (%d slot%s)",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              names[i], slots[i], (1 == slots[i]) ? "" : "s"));
@@ -398,7 +400,7 @@ static int orte_ras_slurm_parse_ranges(char *base, char *ranges, char ***names)
 
     if (start < orig + len) {
         
-        OPAL_OUTPUT_VERBOSE((1, orte_ras_base.ras_output,
+        OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                              "%s ras:slurm:allocate:discover: parse range %s (2)",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              start));
@@ -428,7 +430,6 @@ static int orte_ras_slurm_parse_range(char *base, char *range, char ***names)
     char *str, temp1[BUFSIZ];
     size_t i, j, start, end;
     size_t base_len, len, num_len;
-    size_t str_start, str_end;
     size_t num_str_len;
     bool found;
     int ret;
@@ -444,7 +445,6 @@ static int orte_ras_slurm_parse_range(char *base, char *range, char ***names)
     for (found = false, i = 0; i < len; ++i) {
         if (isdigit((int) range[i])) {
             if (!found) {
-                str_start = i;
                 start = atoi(range + i);
                 found = true;
                 break;
@@ -467,7 +467,6 @@ static int orte_ras_slurm_parse_range(char *base, char *range, char ***names)
     /* Was there no range, just a single number? */
     
     if (i >= len) {
-        str_end = len;
         end = start;
         found = true;
     }
@@ -476,7 +475,6 @@ static int orte_ras_slurm_parse_range(char *base, char *range, char ***names)
        number */
     
     else {
-        str_end = i - 1;
         for (; i < len; ++i) {
             if (isdigit((int) range[i])) {
                 end = atoi(range + i);
