@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
- * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -10,6 +11,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2011-2013 Los Alamos National Security, LLC.
+ *                         All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -20,12 +23,20 @@
 #include "orte_config.h"
 #include "orte/constants.h"
 
-#include "orte/util/show_help.h"
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
+
 #include "opal/mca/base/base.h"
-#include "opal/mca/base/mca_base_param.h"
+#include "opal/util/output.h"
 #include "opal/mca/backtrace/backtrace.h"
+#include "opal/mca/event/event.h"
 
 #include "orte/mca/rml/base/base.h"
+#include "orte/mca/rml/rml_types.h"
 #include "orte/mca/routed/routed.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/util/name_fns.h"
@@ -78,7 +89,6 @@ orte_rml_oob_module_t orte_rml_oob_module = {
         orte_rml_oob_get_uri,
         orte_rml_oob_set_uri,
 
-        orte_rml_oob_get_new_name,
         orte_rml_oob_ping,
 
         orte_rml_oob_send,
@@ -95,19 +105,22 @@ orte_rml_oob_module_t orte_rml_oob_module = {
         orte_rml_oob_add_exception,
         orte_rml_oob_del_exception,
 
-        orte_rml_oob_ft_event
+        orte_rml_oob_ft_event,
+        
+        orte_rml_oob_purge
     }
 };
 
+/* Local variables */
+static bool init_done = false;
 
 static int
 rml_oob_open(void)
 {
     int rc;
 
-    if (ORTE_SUCCESS != (rc = mca_oob_base_open())) {
+    if (ORTE_SUCCESS != (rc = mca_base_framework_open(&orte_oob_base_framework, 0))) {
         ORTE_ERROR_LOG(rc);
-        return rc;
     }
 
     return rc;
@@ -117,20 +130,22 @@ rml_oob_open(void)
 static int
 rml_oob_close(void)
 {
-    int rc;
-
-    if (ORTE_SUCCESS != (rc = mca_oob_base_close())) {
-        return rc;
-    }
-
-    return rc;
+    return mca_base_framework_close(&orte_oob_base_framework);
 }
 
 static orte_rml_module_t*
 rml_oob_init(int* priority)
 {
-    if (mca_oob_base_init() != ORTE_SUCCESS)
+    if (init_done) {
+        *priority = 1;
+        return &orte_rml_oob_module.super;
+    }
+    
+    if (mca_oob_base_init() != ORTE_SUCCESS) {
+        *priority = -1;
         return NULL;
+    }
+
     *priority = 1;
     
     OBJ_CONSTRUCT(&orte_rml_oob_module.exceptions, opal_list_t);
@@ -140,17 +155,19 @@ rml_oob_init(int* priority)
     /* Set default timeout for queued messages to be 1/2 second */
     orte_rml_oob_module.timeout.tv_sec = 0;
     orte_rml_oob_module.timeout.tv_usec = 500000;
-    orte_rml_oob_module.timer_event = (opal_event_t *) malloc(sizeof(opal_event_t));
+    orte_rml_oob_module.timer_event =  (opal_event_t *) malloc(sizeof(opal_event_t));
     if (NULL == orte_rml_oob_module.timer_event) {
         return NULL;
     }
-    opal_evtimer_set(orte_rml_oob_module.timer_event, rml_oob_queued_progress,
-                     NULL);
+    opal_event_evtimer_set(orte_event_base, orte_rml_oob_module.timer_event,
+                           rml_oob_queued_progress,
+                           NULL);
 
     orte_rml_oob_module.active_oob = &mca_oob;
     orte_rml_oob_module.active_oob->oob_exception_callback = 
         orte_rml_oob_exception_callback;
-
+    
+    init_done = true;
     return &orte_rml_oob_module.super;
 }
 
@@ -238,13 +255,9 @@ orte_rml_oob_ft_event(int state) {
         ;
     }
     else if(OPAL_CRS_RESTART == state) {
-        if( ORTE_SUCCESS != (ret = mca_oob_base_close())) {
-            ORTE_ERROR_LOG(ret);
-            exit_status = ret;
-            goto cleanup;
-        }
+        (void) mca_base_framework_close(&orte_oob_base_framework);
 
-        if( ORTE_SUCCESS != (ret = mca_oob_base_open())) {
+        if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_oob_base_framework, 0))) {
             ORTE_ERROR_LOG(ret);
             exit_status = ret;
             goto cleanup;
@@ -367,7 +380,7 @@ rml_oob_queued_progress(int fd, short event, void *arg)
             real_tag = ORTE_RML_TAG_RML_ROUTE;
         }
 
-        OPAL_OUTPUT_VERBOSE((1, orte_rml_base_output,
+        OPAL_OUTPUT_VERBOSE((1, orte_rml_base_framework.framework_output,
                              "%s routing message from %s for %s to %s (tag: %d)",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              ORTE_NAME_PRINT(&hdr->origin),
@@ -394,8 +407,8 @@ rml_oob_queued_progress(int fd, short event, void *arg)
                 opal_list_append(&orte_rml_oob_module.queued_routing_messages,
                                  &qmsg->super);
                 if (1 == opal_list_get_size(&orte_rml_oob_module.queued_routing_messages)) {
-                    opal_evtimer_add(orte_rml_oob_module.timer_event,
-                                     &orte_rml_oob_module.timeout);
+                    opal_event_evtimer_add(orte_rml_oob_module.timer_event,
+                                           &orte_rml_oob_module.timeout);
                 }
                 OPAL_THREAD_UNLOCK(&orte_rml_oob_module.queued_lock);
             } else {
@@ -421,6 +434,8 @@ rml_oob_recv_route_send_callback(int status,
                                  orte_rml_tag_t tag,
                                  void* cbdata)
 {
+    /* NTH: free the iovec allocated by rml_oob_recv_route_callback */
+    free (iov);
 }
 
 
@@ -437,6 +452,7 @@ rml_oob_recv_route_callback(int status,
     int real_tag;
     int ret;
     orte_process_name_t next, origin;
+    struct iovec *new_iov;
 
     /* BWB -- propogate errors here... */
     assert(status >= 0);
@@ -454,6 +470,7 @@ rml_oob_recv_route_callback(int status,
                     hdr->tag);
         opal_backtrace_print(stderr);
         orte_errmgr.abort(ORTE_ERROR_DEFAULT_EXIT_CODE, NULL);
+        return;
     }
 
     if (OPAL_EQUAL == orte_util_compare_name_fields(ORTE_NS_CMP_ALL, &next, ORTE_PROC_MY_NAME)) {
@@ -472,7 +489,7 @@ rml_oob_recv_route_callback(int status,
         real_tag = ORTE_RML_TAG_RML_ROUTE;
     }
 
-    OPAL_OUTPUT_VERBOSE((1, orte_rml_base_output,
+    OPAL_OUTPUT_VERBOSE((1, orte_rml_base_framework.framework_output,
                          "%s routing message from %s for %s to %s (tag: %d)",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          ORTE_NAME_PRINT(&hdr->origin),
@@ -482,9 +499,20 @@ rml_oob_recv_route_callback(int status,
 
     ORTE_RML_OOB_MSG_HEADER_HTON(*hdr);
 
+    /* NTH: fix potential race condition. oob may modify iov before
+       the oob send completes */
+    new_iov = (struct iovec*) calloc (count, sizeof (struct iovec));
+    if (NULL == new_iov) {
+	opal_output (0, "%s:route_callback malloc error!", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+        opal_backtrace_print(stderr);
+        orte_errmgr.abort(ORTE_ERROR_DEFAULT_EXIT_CODE, NULL);    
+    }
+
+    memcpy (new_iov, iov, count * sizeof (struct iovec));
+
     ret = orte_rml_oob_module.active_oob->oob_send_nb(&next,
                                                       &origin,
-                                                      iov,
+                                                      new_iov,
                                                       count,
                                                       real_tag,
                                                       0,
@@ -495,12 +523,12 @@ rml_oob_recv_route_callback(int status,
         if (ORTE_ERR_ADDRESSEE_UNKNOWN == ret) {
             /* no route -- queue and hope we find a route */
             orte_rml_oob_queued_msg_t *qmsg = OBJ_NEW(orte_rml_oob_queued_msg_t);
-            OPAL_OUTPUT_VERBOSE((1, orte_rml_base_output,
+            OPAL_OUTPUT_VERBOSE((1, orte_rml_base_framework.framework_output,
                                  "%s: no OOB information for %s.  Queuing for later.",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                  ORTE_NAME_PRINT(&next)));
             ORTE_RML_OOB_MSG_HEADER_NTOH(*hdr);
-            qmsg->payload[0].iov_base = (char *) malloc(iov[0].iov_len);
+            qmsg->payload[0].iov_base = (IOVBASE_TYPE*) malloc(iov[0].iov_len);
             if (NULL == qmsg->payload[0].iov_base) abort();
             qmsg->payload[0].iov_len = iov[0].iov_len;
             memcpy(qmsg->payload[0].iov_base, iov[0].iov_base, iov[0].iov_len);
@@ -508,8 +536,8 @@ rml_oob_recv_route_callback(int status,
             opal_list_append(&orte_rml_oob_module.queued_routing_messages,
                              &qmsg->super);
             if (1 == opal_list_get_size(&orte_rml_oob_module.queued_routing_messages)) {
-                opal_evtimer_add(orte_rml_oob_module.timer_event,
-                                 &orte_rml_oob_module.timeout);
+                opal_event_evtimer_add(orte_rml_oob_module.timer_event,
+                                       &orte_rml_oob_module.timeout);
             }
             OPAL_THREAD_UNLOCK(&orte_rml_oob_module.queued_lock);
         } else {
