@@ -2,13 +2,16 @@
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2011 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2011      Los Alamos National Security, LLC.
+ *                         All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -22,6 +25,9 @@
 #include <sys/types.h>
 
 #include "opal/util/argv.h"
+#include "opal/dss/dss.h"
+#include "opal/dss/dss_internal.h"
+#include "opal/mca/hwloc/base/base.h"
 #include "opal/class/opal_pointer_array.h"
 
 #include "orte/mca/errmgr/errmgr.h"
@@ -97,7 +103,7 @@ int orte_dt_pack_name(opal_buffer_t *buffer, const void *src,
         return rc;
     }
     free(vpid);
-    
+
     return ORTE_SUCCESS;
 }
 
@@ -147,9 +153,8 @@ int orte_dt_pack_job(opal_buffer_t *buffer, const void *src,
                      int32_t num_vals, opal_data_type_t type)
 {
     int rc;
-    int32_t i, j, np;
+    int32_t i, j;
     orte_job_t **jobs;
-    orte_proc_t *proc;
     orte_app_context_t *app;
     
     /* array of pointers to orte_job_t objects - need to pack the objects a set of fields at a time */
@@ -165,7 +170,7 @@ int orte_dt_pack_job(opal_buffer_t *buffer, const void *src,
 
         /* pack the number of apps */
         if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
-                         (void*)(&(jobs[i]->num_apps)), 1, OPAL_INT32))) {
+                         (void*)(&(jobs[i]->num_apps)), 1, ORTE_APP_IDX))) {
             ORTE_ERROR_LOG(rc);
             return rc;
         }
@@ -204,41 +209,6 @@ int orte_dt_pack_job(opal_buffer_t *buffer, const void *src,
             return rc;
         }
         
-        /* pack the number of procs for the job */
-        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
-                         (void*)(&(jobs[i]->num_procs)), 1, ORTE_VPID))) {
-            ORTE_ERROR_LOG(rc);
-            return rc;
-        }
-        
-        /* there might actually not be any procs in the array, so we
-         * need to count them first
-         */
-        np = 0;
-        for (j=0; j < jobs[i]->procs->size; j++) {
-            if (NULL != opal_pointer_array_get_item(jobs[i]->procs, j)) {
-                np++;
-            }
-        }
-        /* now pack that number */
-        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, (void*)&np, 1, OPAL_INT32))) {
-            ORTE_ERROR_LOG(rc);
-            return rc;
-        }
-
-        if (0 < np) {
-            for (j=0; j < jobs[i]->procs->size; j++) {
-                if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(jobs[i]->procs, j))) {
-                    continue;
-                }
-                if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
-                                    (void*)&proc, 1, ORTE_PROC))) {
-                    ORTE_ERROR_LOG(rc);
-                    return rc;
-                }
-            }
-        }
-        
         /* if the map is NULL, then we cannot pack it as there is
          * nothing to pack. However, we have to flag whether or not
          * the map is included so the unpacking routine can know
@@ -305,6 +275,20 @@ int orte_dt_pack_job(opal_buffer_t *buffer, const void *src,
             return rc;
         }
 
+        /* pack the recovery policy defined flag */
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
+                         (void*)(&(jobs[i]->recovery_defined)), 1, OPAL_BOOL))) {
+            ORTE_ERROR_LOG(rc);
+            return rc;
+        }
+
+        /* pack the recovery flag */
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
+                         (void*)(&(jobs[i]->enable_recovery)), 1, OPAL_BOOL))) {
+            ORTE_ERROR_LOG(rc);
+            return rc;
+        }
+
 #if OPAL_ENABLE_FT_CR == 1
         /* pack the ckpt state */
         if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
@@ -407,8 +391,6 @@ int orte_dt_pack_node(opal_buffer_t *buffer, const void *src,
             return rc;
         }
         
-        /* do not pack the local board, socket, and core info */
-        
         /* do not pack the username */
     }
     return ORTE_SUCCESS;
@@ -435,13 +417,6 @@ int orte_dt_pack_proc(opal_buffer_t *buffer, const void *src,
             return rc;
         }
         
-        /* pack the pid */
-        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
-                         (void*)(&(procs[i]->pid)), 1, OPAL_PID))) {
-            ORTE_ERROR_LOG(rc);
-            return rc;
-        }
-        
         /* pack the local rank */
         if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
                          (void*)(&(procs[i]->local_rank)), 1, ORTE_LOCAL_RANK))) {
@@ -456,6 +431,14 @@ int orte_dt_pack_proc(opal_buffer_t *buffer, const void *src,
             return rc;
         }
         
+#if OPAL_HAVE_HWLOC
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
+                          (void*)(&procs[i]->cpu_bitmap), 1, OPAL_STRING))) {
+        ORTE_ERROR_LOG(rc);
+        return rc;
+    }
+#endif
+
         /* pack the state */
         if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
                          (void*)(&(procs[i]->state)), 1, ORTE_PROC_STATE))) {
@@ -470,13 +453,6 @@ int orte_dt_pack_proc(opal_buffer_t *buffer, const void *src,
             return rc;
         }
         
-        /* pack the name of the node where this proc is executing */
-        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
-                         (void*)(&(procs[i]->nodename)), 1, OPAL_STRING))) {
-            ORTE_ERROR_LOG(rc);
-            return rc;
-        }
-        
         /* pack the number of restarts */
         if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
                          (void*)&(procs[i]->restarts), 1, OPAL_INT32))) {
@@ -528,7 +504,7 @@ int orte_dt_pack_app_context(opal_buffer_t *buffer, const void *src,
     for (i=0; i < num_vals; i++) {
         /* pack the application index (for multiapp jobs) */
         if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
-                        (void*)(&(app_context[i]->idx)), 1, OPAL_INT32))) {
+                        (void*)(&(app_context[i]->idx)), 1, ORTE_STD_CNTR))) {
             ORTE_ERROR_LOG(rc);
             return rc;
         }
@@ -547,6 +523,13 @@ int orte_dt_pack_app_context(opal_buffer_t *buffer, const void *src,
             return rc;
         }
 
+        /* pack the first rank for this app */
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
+                         (void*)(&(app_context[i]->first_rank)), 1, ORTE_VPID))) {
+            ORTE_ERROR_LOG(rc);
+            return rc;
+        }
+
         /* pack the number of entries in the argv array */
         count = opal_argv_count(app_context[i]->argv);
         if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, (void*)(&count), 1, ORTE_STD_CNTR))) {
@@ -741,6 +724,42 @@ int orte_dt_pack_app_context(opal_buffer_t *buffer, const void *src,
                 return rc;
             }
         }
+
+        /* pack the recovery defined flag */
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
+                                    (void*)(&(app_context[i]->recovery_defined)), 1, OPAL_BOOL))) {
+            ORTE_ERROR_LOG(rc);
+            return rc;
+        }
+        /* pack the restart limit */
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
+                                                       (void*)(&(app_context[i]->max_restarts)), 1, OPAL_INT32))) {
+            ORTE_ERROR_LOG(rc);
+            return rc;
+        }
+
+#if OPAL_ENABLE_FT_CR == 1
+        /* Pack the preload_files_src_dir if we have one */
+        if (NULL != app_context[i]->sstore_load) {
+            have_preload_files_dest_dir = 1;
+        } else {
+            have_preload_files_dest_dir = 0;
+        }
+        
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
+                                                       (void*)(&have_preload_files_dest_dir), 1, OPAL_INT8))) {
+            ORTE_ERROR_LOG(rc);
+            return rc;
+        }
+        
+        if( have_preload_files_dest_dir) {
+            if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer,
+                                                           (void*)(&(app_context[i]->sstore_load)), 1, OPAL_STRING))) {
+                ORTE_ERROR_LOG(rc);
+                return rc;
+            }
+        }
+#endif
     }
     
     return ORTE_SUCCESS;
@@ -822,20 +841,29 @@ int orte_dt_pack_map(opal_buffer_t *buffer, const void *src,
     maps = (orte_job_map_t**) src;
     
     for (i=0; i < num_vals; i++) {
-        /* pack the policy used to generate it */
-        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->policy), 1, ORTE_MAPPING_POLICY))) {
+        /* pack the requested mapper */
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->req_mapper), 1, OPAL_STRING))) {
             ORTE_ERROR_LOG(rc);
             return rc;
         }
-        
-        /* pack the #procs/node */
-        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->npernode), 1, ORTE_STD_CNTR))) {
+
+        /* pack the policies */
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->mapping), 1, ORTE_MAPPING_POLICY))) {
             ORTE_ERROR_LOG(rc);
             return rc;
         }
-        
-        /* pack the oversubscribe flag */
-        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->oversubscribe), 1, OPAL_BOOL))) {
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->ranking), 1, ORTE_RANKING_POLICY))) {
+            ORTE_ERROR_LOG(rc);
+            return rc;
+        }
+#if OPAL_HAVE_HWLOC
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->binding), 1, OPAL_BINDING_POLICY))) {
+            ORTE_ERROR_LOG(rc);
+            return rc;
+        }
+#endif
+        /* pack any ppr */
+        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->ppr), 1, OPAL_STRING))) {
             ORTE_ERROR_LOG(rc);
             return rc;
         }
@@ -845,24 +873,6 @@ int orte_dt_pack_map(opal_buffer_t *buffer, const void *src,
             ORTE_ERROR_LOG(rc);
             return rc;
         }
-        
-        /* pack the number of new daemons */
-        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->num_new_daemons), 1, ORTE_STD_CNTR))) {
-            ORTE_ERROR_LOG(rc);
-            return rc;
-        }
-        
-        /* pack the daemon starting vpid */
-        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->daemon_vpid_start), 1, ORTE_VPID))) {
-            ORTE_ERROR_LOG(rc);
-            return rc;
-        }
-        
-        /* pack the number of nodes */
-        if (ORTE_SUCCESS != (rc = opal_dss_pack_buffer(buffer, &(maps[i]->num_nodes), 1, ORTE_STD_CNTR))) {
-            ORTE_ERROR_LOG(rc);
-            return rc;
-        }
     }
     
     return ORTE_SUCCESS;
@@ -900,22 +910,6 @@ int orte_dt_pack_daemon_cmd(opal_buffer_t *buffer, const void *src, int32_t num_
     return ret;
 }
 
-/*
- * ORTE_GRPCOMM_MODE
- */
-int orte_dt_pack_grpcomm_mode(opal_buffer_t *buffer, const void *src, int32_t num_vals,
-                            opal_data_type_t type)
-{
-    int ret;
-    
-    /* Turn around and pack the real type */
-    if (ORTE_SUCCESS != (ret = opal_dss_pack_buffer(buffer, src, num_vals, ORTE_GRPCOMM_MODE_T))) {
-        ORTE_ERROR_LOG(ret);
-    }
-    
-    return ret;
-}
-
 /*
  * ORTE_IOF_TAG
  */
