@@ -2,17 +2,18 @@
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2009 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2009-2011 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c)      2011 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ * Copyright (c) 2007-2011 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
+ *                         All rights reserved.
+ * Copyright (c) 2013      Intel, Inc. All rights reserved
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -28,15 +29,17 @@
 #include <sys/time.h>
 #endif
 
-#include "opal/mca/base/mca_base_param.h"
-#include "opal/mca/paffinity/paffinity.h"
+#include "opal/mca/db/db.h"
+#include "opal/mca/hwloc/hwloc.h"
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
 #include "opal/class/opal_pointer_array.h"
 #include "opal/class/opal_value_array.h"
 #include "opal/dss/dss.h"
+#include "opal/threads/threads.h"
 
 #include "orte/mca/errmgr/errmgr.h"
+#include "orte/mca/rml/rml.h"
 #include "orte/util/proc_info.h"
 #include "orte/util/name_fns.h"
 
@@ -47,6 +50,13 @@
 /* need the data type support functions here */
 #include "orte/runtime/data_type_support/orte_dt_support.h"
 
+/* State Machine */
+opal_list_t orte_job_states;
+opal_list_t orte_proc_states;
+
+/* a clean output channel without prefix */
+int orte_clean_output = -1;
+
 #if !ORTE_DISABLE_FULL_SUPPORT
 
 /* globals used by RTE */
@@ -57,9 +67,9 @@ bool orte_debug_daemons_file_flag = false;
 bool orte_leave_session_attached;
 bool orte_do_not_launch = false;
 bool orted_spin_flag = false;
-bool orte_daemon_bootstrap = false;
 char *orte_local_cpu_type = NULL;
 char *orte_local_cpu_model = NULL;
+char *orte_basename = NULL;
 
 /* ORTE OOB port flags */
 bool orte_static_ports = false;
@@ -67,38 +77,45 @@ char *orte_oob_static_ports = NULL;
 bool orte_standalone_operation = false;
 
 bool orte_keep_fqdn_hostnames = false;
+bool orte_have_fqdn_allocation = false;
 bool orte_show_resolved_nodenames;
+bool orte_retain_aliases;
+int orte_use_hostname_alias;
+
 int orted_debug_failure;
 int orted_debug_failure_delay;
 bool orte_homogeneous_nodes = false;
 bool orte_hetero_apps = false;
+bool orte_hetero_nodes = false;
 bool orte_never_launched = false;
 bool orte_devel_level_output = false;
+bool orte_display_topo_with_map = false;
+bool orte_display_diffable_output = false;
 
 char **orte_launch_environ;
 
 bool orte_hnp_is_allocated = false;
 bool orte_allocation_required;
+bool orte_managed_allocation = false;
 
+/* launch agents */
 char *orte_launch_agent = NULL;
 char **orted_cmd_line=NULL;
+char **orte_fork_agent=NULL;
 
 /* debugger job */
-orte_job_t *orte_debugger_daemon=NULL;
 bool orte_debugger_dump_proctable;
 char *orte_debugger_test_daemon;
 bool orte_debugger_test_attach;
-bool orte_debugger_enable_fifo_attach;
 int orte_debugger_check_rate;
 
-/* exit triggers and flags */ 
-orte_trigger_event_t orte_exit, orteds_exit;
+/* exit flags */
 int orte_exit_status = 0;
 bool orte_abnormal_term_ordered = false;
-bool orte_routing_is_enabled = false;
+bool orte_routing_is_enabled = true;
 bool orte_job_term_ordered = false;
+bool orte_orteds_term_ordered = false;
 
-int orte_heartbeat_rate;
 int orte_startup_timeout;
 
 int orte_timeout_usec_per_proc;
@@ -109,23 +126,9 @@ opal_buffer_t *orte_tree_launch_cmd = NULL;
 /* global arrays for data storage */
 opal_pointer_array_t *orte_job_data;
 opal_pointer_array_t *orte_node_pool;
-
-/* a clean output channel without prefix */
-int orte_clean_output = -1;
-
-/* RHC: FLAG TO SELECT WHETHER OR NOT TO SEND PROFILE FILE IN NIDMAP */
-bool orte_send_profile;
-
-/* Nidmap and job maps */
-opal_pointer_array_t orte_nidmap;
-opal_pointer_array_t orte_jobmap;
-bool orte_use_regexp;
-char *orted_launch_cmd = NULL;
-
-/* list of local children on a daemon */
-opal_list_t orte_local_children;
-/* list of job data for local children on a daemon */
-opal_list_t orte_local_jobdata;
+opal_pointer_array_t *orte_node_topologies;
+opal_pointer_array_t *orte_local_children;
+uint16_t orte_num_jobs = 0;
 
 /* IOF controls */
 bool orte_tag_output;
@@ -137,30 +140,18 @@ char *orte_xterm;
 /* whether or not to forward SIGTSTP and SIGCONT signals */
 bool orte_forward_job_control;
 
-/* rsh support */
-char *orte_rsh_agent = NULL;
-bool orte_assume_same_shell = true;
-
-/* orted exit with barrier */
-bool orte_orted_exit_with_barrier = true;
-
 /* report launch progress */
 bool orte_report_launch_progress = false;
 
-/* cluster hardware info */
-uint8_t orte_default_num_boards;
-uint8_t orte_default_num_sockets_per_board;
-uint8_t orte_default_num_cores_per_socket;
-
 /* allocation specification */
 char *orte_default_hostfile = NULL;
-char *orte_rankfile;
+bool orte_default_hostfile_given = false;
+char *orte_rankfile = NULL;
 #ifdef __WINDOWS__
 char *orte_ccp_headnode;
 #endif
-
-/* default rank assigment and binding policy */
-orte_mapping_policy_t orte_default_mapping_policy = 0;
+int orte_num_allocated_nodes = 0;
+char *orte_node_regex = NULL;
 
 /* tool communication controls */
 bool orte_report_events = false;
@@ -172,15 +163,54 @@ bool orte_report_bindings = false;
 /* barrier control */
 bool orte_do_not_barrier = false;
 
+/* process recovery */
+bool orte_enable_recovery;
+int32_t orte_max_restarts;
+
+/* exit status reporting */
+bool orte_report_child_jobs_separately;
+struct timeval orte_child_time_to_exit;
+bool orte_abort_non_zero_exit;
+
+/* length of stat history to keep */
+int orte_stat_history_size;
+
+/* envars to forward */
+char *orte_forward_envars = NULL;
+
+/* preload binaries */
+bool orte_preload_binaries = false;
+
+/* map-reduce mode */
+bool orte_map_reduce = false;
+
+/* map stddiag output to stderr so it isn't forwarded to mpirun */
+bool orte_map_stddiag_to_stderr = false;
+
+/* maximum size of virtual machine - used to subdivide allocation */
+int orte_max_vm_size = -1;
+
+/* progress thread */
+#if ORTE_ENABLE_PROGRESS_THREADS
+opal_thread_t orte_progress_thread;
+opal_event_t orte_finalize_event;
+#endif
+
+char *orte_selected_oob_component = NULL;
+
 #endif /* !ORTE_DISABLE_FULL_RTE */
+/* user debugger */
+char *orte_base_user_debugger = NULL;
 
 int orte_debug_output = -1;
 bool orte_debug_daemons_flag = false;
 bool orte_xml_output = false;
 FILE *orte_xml_fp = NULL;
 char *orte_job_ident = NULL;
+bool orte_execute_quiet = false;
+bool orte_report_silent_errors = false;
 
-/* See comment in orte/tools/orterun/orterun.c about this MCA
+/* See comment in orte/tools/orterun/debuggers.c about this MCA
    param */
 bool orte_in_parallel_debugger = false;
 
@@ -209,9 +239,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_std_cntr,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_std_cntr,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_std_cntr,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
                                                      (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "ORTE_STD_CNTR", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -222,9 +250,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_name,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_name,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_name,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
                                                      (opal_dss_print_fn_t)orte_dt_print_name,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "ORTE_NAME", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -236,9 +262,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_vpid,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_vpid,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_vpid,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
                                                      (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "ORTE_VPID", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -250,9 +274,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_jobid,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_jobid,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_jobid,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
                                                      (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "ORTE_JOBID", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -260,24 +282,12 @@ int orte_dt_init(void)
     }
 
 #if !ORTE_DISABLE_FULL_SUPPORT
-    /* get a clean output channel too */
-    {
-        opal_output_stream_t lds;
-        OBJ_CONSTRUCT(&lds, opal_output_stream_t);
-        lds.lds_want_stdout = true;
-        orte_clean_output = opal_output_open(&lds);
-        OBJ_DESTRUCT(&lds);
-        
-    }
-
     tmp = ORTE_JOB;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_job,
                                                      orte_dt_unpack_job,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_job,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_job,
-                                                     (opal_dss_size_fn_t)orte_dt_size_job,
                                                      (opal_dss_print_fn_t)orte_dt_print_job,
-                                                     (opal_dss_release_fn_t)orte_dt_std_obj_release,
                                                      OPAL_DSS_STRUCTURED,
                                                      "ORTE_JOB", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -289,9 +299,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_node,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_node,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_node,
-                                                     (opal_dss_size_fn_t)orte_dt_size_node,
                                                      (opal_dss_print_fn_t)orte_dt_print_node,
-                                                     (opal_dss_release_fn_t)orte_dt_std_obj_release,
                                                      OPAL_DSS_STRUCTURED,
                                                      "ORTE_NODE", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -303,9 +311,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_proc,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_proc,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_proc,
-                                                     (opal_dss_size_fn_t)orte_dt_size_proc,
                                                      (opal_dss_print_fn_t)orte_dt_print_proc,
-                                                     (opal_dss_release_fn_t)orte_dt_std_obj_release,
                                                      OPAL_DSS_STRUCTURED,
                                                      "ORTE_PROC", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -317,9 +323,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_app_context,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_app_context,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_app_context,
-                                                     (opal_dss_size_fn_t)orte_dt_size_app_context,
                                                      (opal_dss_print_fn_t)orte_dt_print_app_context,
-                                                     (opal_dss_release_fn_t)orte_dt_std_obj_release,
                                                      OPAL_DSS_STRUCTURED,
                                                      "ORTE_APP_CONTEXT", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -331,9 +335,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_node_state,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_node_state,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_node_state,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
                                                      (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "ORTE_NODE_STATE", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -345,9 +347,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_proc_state,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_proc_state,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_proc_state,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
                                                      (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "ORTE_PROC_STATE", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -359,9 +359,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_job_state,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_job_state,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_job_state,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
                                                      (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "ORTE_JOB_STATE", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -373,9 +371,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_exit_code,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_exit_code,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_exit_code,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
                                                      (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "ORTE_EXIT_CODE", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -387,9 +383,7 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_map,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_map,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_map,
-                                                     (opal_dss_size_fn_t)orte_dt_size_map,
                                                      (opal_dss_print_fn_t)orte_dt_print_map,
-                                                     (opal_dss_release_fn_t)orte_dt_std_obj_release,
                                                      OPAL_DSS_STRUCTURED,
                                                      "ORTE_JOB_MAP", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -401,9 +395,7 @@ int orte_dt_init(void)
                                                       orte_dt_unpack_tag,
                                                       (opal_dss_copy_fn_t)orte_dt_copy_tag,
                                                       (opal_dss_compare_fn_t)orte_dt_compare_tags,
-                                                      (opal_dss_size_fn_t)orte_dt_std_size,
                                                       (opal_dss_print_fn_t)orte_dt_std_print,
-                                                      (opal_dss_release_fn_t)orte_dt_std_release,
                                                       OPAL_DSS_UNSTRUCTURED,
                                                       "ORTE_RML_TAG", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -415,37 +407,19 @@ int orte_dt_init(void)
                                                      orte_dt_unpack_daemon_cmd,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_daemon_cmd,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_daemon_cmd,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
                                                      (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "ORTE_DAEMON_CMD", &tmp))) {
         ORTE_ERROR_LOG(rc);
         return rc;
     }
 
-    tmp = ORTE_GRPCOMM_MODE;
-    if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_grpcomm_mode,
-                                                     orte_dt_unpack_grpcomm_mode,
-                                                     (opal_dss_copy_fn_t)orte_dt_copy_grpcomm_mode,
-                                                     (opal_dss_compare_fn_t)orte_dt_compare_grpcomm_mode,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
-                                                     (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
-                                                     OPAL_DSS_UNSTRUCTURED,
-                                                     "ORTE_GRPCOMM_MODE", &tmp))) {
-        ORTE_ERROR_LOG(rc);
-        return rc;
-    }
-
     tmp = ORTE_IOF_TAG;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_iof_tag,
                                                      orte_dt_unpack_iof_tag,
                                                      (opal_dss_copy_fn_t)orte_dt_copy_iof_tag,
                                                      (opal_dss_compare_fn_t)orte_dt_compare_iof_tag,
-                                                     (opal_dss_size_fn_t)orte_dt_std_size,
                                                      (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     (opal_dss_release_fn_t)orte_dt_std_release,
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "ORTE_IOF_TAG", &tmp))) {
         ORTE_ERROR_LOG(rc);
@@ -462,8 +436,8 @@ orte_job_t* orte_get_job_data_object(orte_jobid_t job)
 {
     int32_t ljob;
     
-    /* if I am not an HNP, I cannot provide this object */
-    if (!ORTE_PROC_IS_HNP && !ORTE_PROC_IS_CM) {
+    /* if the job data wasn't setup, we cannot provide the data */
+    if (NULL == orte_job_data) {
         return NULL;
     }
     
@@ -478,6 +452,119 @@ orte_job_t* orte_get_job_data_object(orte_jobid_t job)
     return (orte_job_t*)opal_pointer_array_get_item(orte_job_data, ljob);
 }
 
+orte_proc_t* orte_get_proc_object(orte_process_name_t *proc)
+{
+    orte_job_t *jdata;
+    orte_proc_t *proct;
+
+    if (NULL == (jdata = orte_get_job_data_object(proc->jobid))) {
+        return NULL;
+    }
+    proct = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, proc->vpid);
+    return proct;
+}
+
+orte_vpid_t orte_get_proc_daemon_vpid(orte_process_name_t *proc)
+{
+    orte_job_t *jdata;
+    orte_proc_t *proct;
+
+    if (NULL == (jdata = orte_get_job_data_object(proc->jobid))) {
+        return ORTE_VPID_INVALID;
+    }
+    if (NULL == (proct = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, proc->vpid))) {
+        return ORTE_VPID_INVALID;
+    }
+    if (NULL == proct->node || NULL == proct->node->daemon) {
+        return ORTE_VPID_INVALID;
+    }
+    return proct->node->daemon->name.vpid;
+}
+
+char* orte_get_proc_hostname(orte_process_name_t *proc)
+{
+    orte_proc_t *proct;
+    char *hostname;
+    int rc;
+
+    if (ORTE_PROC_IS_DAEMON || ORTE_PROC_IS_HNP) {
+        /* look it up on our arrays */
+        if (NULL == (proct = orte_get_proc_object(proc))) {
+            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+            return NULL;
+        }
+        if (NULL == proct->node || NULL == proct->node->name) {
+            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+            return NULL;
+        }
+        return proct->node->name;
+    }
+
+    /* if we are an app, get the pointer from the modex db */
+    if (ORTE_SUCCESS != (rc = opal_db.fetch_pointer((opal_identifier_t*)proc, ORTE_DB_HOSTNAME,
+                                                    (void**)&hostname, OPAL_STRING))) {
+        ORTE_ERROR_LOG(rc);
+        return NULL;
+    }
+    return hostname;
+}
+
+orte_node_rank_t orte_get_proc_node_rank(orte_process_name_t *proc)
+{
+    orte_proc_t *proct;
+    orte_node_rank_t noderank, *nr;
+    int rc;
+
+    if (ORTE_PROC_IS_DAEMON || ORTE_PROC_IS_HNP) {
+        /* look it up on our arrays */
+        if (NULL == (proct = orte_get_proc_object(proc))) {
+            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+            return ORTE_NODE_RANK_INVALID;
+        }
+        return proct->node_rank;
+    }
+
+    /* if we are an app, get the value from the modex db */
+    nr = &noderank;
+    if (ORTE_SUCCESS != (rc = opal_db.fetch_pointer((opal_identifier_t*)proc, ORTE_DB_NODERANK,
+                                                    (void**)&nr, ORTE_NODE_RANK))) {
+        ORTE_ERROR_LOG(rc);
+        return ORTE_NODE_RANK_INVALID;
+    }
+    return noderank;
+}
+
+orte_vpid_t orte_get_lowest_vpid_alive(orte_jobid_t job)
+{
+    int i;
+    orte_job_t *jdata;
+    orte_proc_t *proc;
+
+    if (NULL == (jdata = orte_get_job_data_object(job))) {
+        return ORTE_VPID_INVALID;
+    }
+
+    if (ORTE_PROC_IS_DAEMON &&
+        ORTE_PROC_MY_NAME->jobid == job &&
+        NULL != orte_process_info.my_hnp_uri) {
+        /* if we were started by an HNP, then the lowest vpid
+         * is always 1
+         */
+        return 1;
+    }
+
+    for (i=0; i < jdata->procs->size; i++) {
+        if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, i))) {
+            continue;
+        }
+        if (proc->state == ORTE_PROC_STATE_RUNNING) {
+            /* must be lowest one alive */
+            return proc->name.vpid;
+        }
+    }
+    /* only get here if no live proc found */
+    return ORTE_VPID_INVALID;
+}
 
 /*
  * CONSTRUCTORS, DESTRUCTORS, AND CLASS INSTANTIATIONS
@@ -489,6 +576,7 @@ static void orte_app_context_construct(orte_app_context_t* app_context)
     app_context->idx=0;
     app_context->app=NULL;
     app_context->num_procs=0;
+    app_context->first_rank = 0;
     app_context->argv=NULL;
     app_context->env=NULL;
     app_context->cwd=NULL;
@@ -497,16 +585,25 @@ static void orte_app_context_construct(orte_app_context_t* app_context)
     app_context->add_hostfile=NULL;
     app_context->add_host = NULL;
     app_context->dash_host = NULL;
+    OBJ_CONSTRUCT(&app_context->resource_constraints, opal_list_t);
     app_context->prefix_dir = NULL;
     app_context->preload_binary = false;
     app_context->preload_files  = NULL;
     app_context->preload_files_dest_dir  = NULL;
     app_context->preload_files_src_dir  = NULL;
     app_context->used_on_node = false;
+
+#if OPAL_ENABLE_FT_CR == 1
+    app_context->sstore_load = NULL;
+#endif
+    app_context->recovery_defined = false;
+    app_context->max_restarts = -1000;
 }
 
 static void orte_app_context_destructor(orte_app_context_t* app_context)
 {
+    opal_list_item_t *item;
+
     if (NULL != app_context->app) {
         free (app_context->app);
         app_context->app = NULL;
@@ -548,6 +645,11 @@ static void orte_app_context_destructor(orte_app_context_t* app_context)
         app_context->dash_host = NULL;
     }
     
+    while (NULL != (item = opal_list_remove_first(&app_context->resource_constraints))) {
+        OBJ_RELEASE(item);
+    }
+    OBJ_DESTRUCT(&app_context->resource_constraints);
+
     if (NULL != app_context->prefix_dir) {
         free(app_context->prefix_dir);
         app_context->prefix_dir = NULL;
@@ -570,6 +672,13 @@ static void orte_app_context_destructor(orte_app_context_t* app_context)
         free(app_context->preload_files_src_dir);
         app_context->preload_files_src_dir = NULL;
     }
+
+#if OPAL_ENABLE_FT_CR == 1
+    if( NULL != app_context->sstore_load ) {
+        free(app_context->sstore_load);
+        app_context->sstore_load = NULL;
+    }
+#endif
 }
 
 OBJ_CLASS_INSTANCE(orte_app_context_t,
@@ -588,6 +697,7 @@ static void orte_job_construct(orte_job_t* job)
     job->num_apps = 0;
     job->controls = ORTE_JOB_CONTROL_FORWARD_OUTPUT;
     job->stdin_target = ORTE_VPID_INVALID;
+    job->stdout_target = ORTE_JOBID_INVALID;
     job->total_slots_alloc = 0;
     job->num_procs = 0;
     job->procs = OBJ_NEW(opal_pointer_array_t);
@@ -598,20 +708,26 @@ static void orte_job_construct(orte_job_t* job)
     
     job->map = NULL;
     job->bookmark = NULL;
-    job->oversubscribe_override = false;
     job->state = ORTE_JOB_STATE_UNDEF;
+    job->restart = false;
 
     job->num_launched = 0;
     job->num_reported = 0;
     job->num_terminated = 0;
+    job->num_daemons_reported = 0;
+    job->num_non_zero_exit = 0;
     job->abort = false;
     job->aborted_proc = NULL;
     
-    job->err_cbfunc = NULL;
-    job->err_cbstates = ORTE_PROC_STATE_UNDEF;
-    job->err_cbdata = NULL;
-    job->max_restarts = INT32_MAX;
-    
+    job->originator.jobid = ORTE_JOBID_INVALID;
+    job->originator.vpid = ORTE_VPID_INVALID;
+
+    job->recovery_defined = false;
+    job->enable_recovery = false;
+    job->num_local_procs = 0;
+
+    job->pmap = NULL;
+
 #if OPAL_ENABLE_FT_CR == 1
     job->ckpt_state = 0;
     job->ckpt_snapshot_ref = NULL;
@@ -625,7 +741,7 @@ static void orte_job_destruct(orte_job_t* job)
     orte_app_context_t *app;
     orte_job_t *jdata;
     int n;
-    
+
     if (NULL == job) {
         /* probably just a race condition - just return */
         return;
@@ -657,6 +773,13 @@ static void orte_job_destruct(orte_job_t* job)
     }
     OBJ_RELEASE(job->procs);
     
+    if (NULL != job->pmap) {
+        if (NULL != job->pmap->bytes) {
+            free(job->pmap->bytes);
+        }
+        free(job->pmap);
+    }
+
 #if OPAL_ENABLE_FT_CR == 1
     if (NULL != job->ckpt_snapshot_ref) {
         free(job->ckpt_snapshot_ref);
@@ -696,6 +819,7 @@ static void orte_node_construct(orte_node_t* node)
     node->index = -1;
     node->daemon = NULL;
     node->daemon_launched = false;
+    node->location_verified = false;
     node->launch_id = -1;
 
     node->num_procs = 0;
@@ -713,20 +837,22 @@ static void orte_node_construct(orte_node_t* node)
     node->slots_alloc = 0;
     node->slots_max = 0;
     
-    node->boards = orte_default_num_boards;
-    node->sockets_per_board = orte_default_num_sockets_per_board;
-    node->cores_per_socket = orte_default_num_cores_per_socket;
-    
     node->username = NULL;
     
-    OBJ_CONSTRUCT(&node->resources, opal_list_t);
+#if OPAL_HAVE_HWLOC
+    node->topology = NULL;
+#endif
+
+    OBJ_CONSTRUCT(&node->stats, opal_ring_buffer_t);
+    opal_ring_buffer_init(&node->stats, orte_stat_history_size);
 }
 
 static void orte_node_destruct(orte_node_t* node)
 {
     int i;
-    opal_list_item_t *item;
-    
+    opal_node_stats_t *stats;
+    orte_proc_t *proc;
+
     if (NULL != node->name) {
         free(node->name);
         node->name = NULL;
@@ -744,23 +870,26 @@ static void orte_node_destruct(orte_node_t* node)
     }
     
     for (i=0; i < node->procs->size; i++) {
-        if (NULL != node->procs->addr[i]) {
-            ((orte_proc_t*)(node->procs->addr[i]))->node = NULL;
-            OBJ_RELEASE(node->procs->addr[i]);
-            node->procs->addr[i] = NULL;
+        if (NULL != (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, i))) {
+            opal_pointer_array_set_item(node->procs, i, NULL);
+            OBJ_RELEASE(proc);
         }
     }
     OBJ_RELEASE(node->procs);
     
+    /* we release the topology elsewhere */
+
     if (NULL != node->username) {
         free(node->username);
         node->username = NULL;
     }
     
-    while (NULL != (item = opal_list_remove_first(&node->resources))) {
-        OBJ_RELEASE(item);
+    /* do NOT destroy the topology */
+
+    while (NULL != (stats = (opal_node_stats_t*)opal_ring_buffer_pop(&node->stats))) {
+        OBJ_RELEASE(stats);
     }
-    OBJ_DESTRUCT(&node->resources);
+    OBJ_DESTRUCT(&node->stats);
 }
 
 
@@ -777,14 +906,36 @@ static void orte_proc_construct(orte_proc_t* proc)
     proc->pid = 0;
     proc->local_rank = ORTE_LOCAL_RANK_INVALID;
     proc->node_rank = ORTE_NODE_RANK_INVALID;
+    proc->app_rank = -1;
+    proc->last_errmgr_state = ORTE_PROC_STATE_UNDEF;
     proc->state = ORTE_PROC_STATE_UNDEF;
-    proc->app_idx = -1;
-    proc->slot_list = NULL;
+    proc->alive = false;
+    proc->aborted = false;
+    proc->app_idx = 0;
+#if OPAL_HAVE_HWLOC
+    proc->locale = NULL;
+    proc->bind_idx = 0;
+    proc->cpu_bitmap = NULL;
+#endif
     proc->node = NULL;
+    proc->local_proc = false;
+    proc->do_not_barrier = false;
+    proc->prior_node = NULL;
     proc->nodename = NULL;
+    proc->exit_code = 0;      /* Assume we won't fail unless otherwise notified */
     proc->rml_uri = NULL;
-    proc->beat = 0;
     proc->restarts = 0;
+    proc->fast_failures = 0;
+    proc->last_failure.tv_sec = 0;
+    proc->last_failure.tv_usec = 0;
+    proc->reported = false;
+    proc->beat = 0;
+    OBJ_CONSTRUCT(&proc->stats, opal_ring_buffer_t);
+    opal_ring_buffer_init(&proc->stats, orte_stat_history_size);
+    proc->registered = false;
+    proc->deregistered = false;
+    proc->iof_complete = false;
+    proc->waitpid_recvd = false;
 #if OPAL_ENABLE_FT_CR == 1
     proc->ckpt_state = 0;
     proc->ckpt_snapshot_ref = NULL;
@@ -794,16 +945,18 @@ static void orte_proc_construct(orte_proc_t* proc)
 
 static void orte_proc_destruct(orte_proc_t* proc)
 {
+    opal_pstats_t *stats;
+
     /* do NOT free the nodename field as this is
      * simply a pointer to a field in the
      * associated node object - the node object
      * will free it
      */
-    
-    if (NULL != proc->slot_list) {
-        free(proc->slot_list);
-        proc->slot_list = NULL;
+#if OPAL_HAVE_HWLOC
+    if (NULL != proc->cpu_bitmap) {
+        free(proc->cpu_bitmap);
     }
+#endif
 
     if (NULL != proc->node) {
         OBJ_RELEASE(proc->node);
@@ -814,7 +967,12 @@ static void orte_proc_destruct(orte_proc_t* proc)
         free(proc->rml_uri);
         proc->rml_uri = NULL;
     }
-    
+
+    while (NULL != (stats = (opal_pstats_t*)opal_ring_buffer_pop(&proc->stats))) {
+        OBJ_RELEASE(stats);
+    }
+    OBJ_DESTRUCT(&proc->stats);
+
 #if OPAL_ENABLE_FT_CR == 1
     if (NULL != proc->ckpt_snapshot_ref) {
         free(proc->ckpt_snapshot_ref);
@@ -832,111 +990,19 @@ OBJ_CLASS_INSTANCE(orte_proc_t,
                    orte_proc_construct,
                    orte_proc_destruct);
 
-static void orte_attr_construct(orte_attr_t *ptr)
-{
-    ptr->name = NULL;
-    ptr->size = 0;
-    ptr->bytes = NULL;
-}
-
-static void orte_attr_destruct(orte_attr_t *ptr)
-{
-    if (NULL != ptr->name) {
-        free(ptr->name);
-        ptr->name = NULL;
-    }
-    if (NULL != ptr->bytes) {
-        free(ptr->bytes);
-        ptr->bytes = NULL;
-    }
-}
-
-OBJ_CLASS_INSTANCE(orte_attr_t,
-                   opal_list_item_t,
-                   orte_attr_construct,
-                   orte_attr_destruct);
-
-static void orte_nid_construct(orte_nid_t *ptr)
-{
-    ptr->name = NULL;
-    ptr->daemon = ORTE_VPID_INVALID;
-    ptr->oversubscribed = false;
-    OBJ_CONSTRUCT(&ptr->attrs, opal_list_t);
-}
-
-static void orte_nid_destruct(orte_nid_t *ptr)
-{
-    opal_list_item_t *item;
-    
-    if (NULL != ptr->name) {
-        free(ptr->name);
-        ptr->name = NULL;
-    }
-    while (NULL != (item = opal_list_remove_first(&ptr->attrs))) {
-        OBJ_RELEASE(item);
-    }
-    OBJ_DESTRUCT(&ptr->attrs);
-}
-
-OBJ_CLASS_INSTANCE(orte_nid_t,
-                   opal_object_t,
-                   orte_nid_construct,
-                   orte_nid_destruct);
-
-static void orte_pmap_construct(orte_pmap_t *ptr)
-{
-    ptr->node = -1;
-    ptr->local_rank = ORTE_LOCAL_RANK_INVALID;
-    ptr->node_rank = ORTE_NODE_RANK_INVALID;
-}
-
-OBJ_CLASS_INSTANCE(orte_pmap_t,
-                   opal_object_t,
-                   orte_pmap_construct,
-                   NULL);
-
-
-static void orte_jmap_construct(orte_jmap_t *ptr)
-{
-    ptr->job = ORTE_JOBID_INVALID;
-    ptr->num_procs = 0;
-    OBJ_CONSTRUCT(&ptr->pmap, opal_pointer_array_t);
-    opal_pointer_array_init(&ptr->pmap,
-                            ORTE_GLOBAL_ARRAY_BLOCK_SIZE,
-                            ORTE_GLOBAL_ARRAY_MAX_SIZE,
-                            ORTE_GLOBAL_ARRAY_BLOCK_SIZE);
-}
-
-static void orte_jmap_destruct(orte_jmap_t *ptr)
-{
-    orte_pmap_t **pmaps;
-    int i;
-    
-    pmaps = (orte_pmap_t**)ptr->pmap.addr;
-    for (i=0; i < ptr->pmap.size && NULL != pmaps[i]; i++) {
-        OBJ_RELEASE(pmaps[i]);
-    }
-    OBJ_DESTRUCT(&ptr->pmap);
-}
-
-OBJ_CLASS_INSTANCE(orte_jmap_t,
-                   opal_object_t,
-                   orte_jmap_construct,
-                   orte_jmap_destruct);
-
-
-
 static void orte_job_map_construct(orte_job_map_t* map)
 {
-    map->policy = 0;
-    map->npernode = 0;
-    map->nperboard = 0;
-    map->npersocket = 0;
+    map->req_mapper = NULL;
+    map->last_mapper = NULL;
+    map->mapping = 0;
+    map->ranking = 0;
+#if OPAL_HAVE_HWLOC
+    map->binding = 0;
+    map->bind_level = OPAL_HWLOC_NODE_LEVEL;
+#endif
+    map->ppr = NULL;
     map->cpus_per_rank = 1;
-    map->stride = 1;
-    map->oversubscribe = true;  /* default to allowing oversubscribe */
     map->display_map = false;
-    map->cpu_lists = false;
     map->num_new_daemons = 0;
     map->daemon_vpid_start = ORTE_VPID_INVALID;
     map->num_nodes = 0;
@@ -950,11 +1016,21 @@ static void orte_job_map_construct(orte_job_map_t* map)
 static void orte_job_map_destruct(orte_job_map_t* map)
 {
     orte_std_cntr_t i;
-    
+    orte_node_t *node;
+
+    if (NULL != map->req_mapper) {
+        free(map->req_mapper);
+    }
+    if (NULL != map->last_mapper) {
+        free(map->last_mapper);
+    }
+    if (NULL != map->ppr) {
+        free(map->ppr);
+    }
     for (i=0; i < map->nodes->size; i++) {
-        if (NULL != map->nodes->addr[i]) {
-            OBJ_RELEASE(map->nodes->addr[i]);
-            map->nodes->addr[i] = NULL;
+        if (NULL != (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
+            OBJ_RELEASE(node);
+            opal_pointer_array_set_item(map->nodes, i, NULL);
         }
     }
     OBJ_RELEASE(map->nodes);
@@ -965,38 +1041,4 @@ OBJ_CLASS_INSTANCE(orte_job_map_t,
                    orte_job_map_construct,
                    orte_job_map_destruct);
 
-static void orte_regex_node_construct(orte_regex_node_t *ptr)
-{
-    ptr->prefix = NULL;
-    OBJ_CONSTRUCT(&ptr->suffix, opal_value_array_t);
-    opal_value_array_init(&ptr->suffix, sizeof(char));
-    OBJ_CONSTRUCT(&ptr->nodes, opal_value_array_t);
-    opal_value_array_init(&ptr->nodes, sizeof(int32_t));
-    OBJ_CONSTRUCT(&ptr->cnt, opal_value_array_t);
-    opal_value_array_init(&ptr->cnt, sizeof(int32_t));
-    OBJ_CONSTRUCT(&ptr->starting_vpid, opal_value_array_t);
-    opal_value_array_init(&ptr->starting_vpid, sizeof(orte_vpid_t));
-    OBJ_CONSTRUCT(&ptr->ppn, opal_value_array_t);
-    opal_value_array_init(&ptr->ppn, sizeof(int32_t));
-    OBJ_CONSTRUCT(&ptr->nrank, opal_value_array_t);
-    opal_value_array_init(&ptr->nrank, sizeof(orte_node_rank_t));
-}
-static void orte_regex_node_destruct(orte_regex_node_t *ptr)
-{
-    if (NULL != ptr->prefix) {
-        free(ptr->prefix);
-        ptr->prefix = NULL;
-    }
-    OBJ_DESTRUCT(&ptr->suffix);
-    OBJ_DESTRUCT(&ptr->nodes);
-    OBJ_DESTRUCT(&ptr->cnt);
-    OBJ_DESTRUCT(&ptr->starting_vpid);
-    OBJ_DESTRUCT(&ptr->ppn);
-    OBJ_DESTRUCT(&ptr->nrank);
-}
-OBJ_CLASS_INSTANCE(orte_regex_node_t,
-                   opal_list_item_t,
-                   orte_regex_node_construct,
-                   orte_regex_node_destruct);
-
 #endif
