@@ -1,18 +1,19 @@
 /*
- * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved. 
+ * Copyright (c) 2006-2013 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
  *                         reserved. 
+ * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -53,30 +54,27 @@
 #include <dirent.h>
 #endif  /* HAVE_DIRENT_H */
 
+#include "opal/util/basename.h"
 #include "opal/util/cmd_line.h"
 #include "opal/util/output.h"
-#include "opal/util/argv.h"
 #include "opal/util/opal_environ.h"
-#include "opal/util/os_path.h"
+#include "opal/util/show_help.h"
 #include "opal/mca/base/base.h"
-#include "opal/mca/base/mca_base_param.h"
 #include "opal/runtime/opal.h"
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
 #include "opal/runtime/opal_cr.h"
 #endif
 
-#include "opal/dss/dss.h"
 #include "orte/runtime/runtime.h"
-#include "orte/util/proc_info.h"
-#include "opal/util/os_path.h"
-#include "orte/util/session_dir.h"
+#include "orte/util/error_strings.h"
 #include "orte/util/hnp_contact.h"
 #include "orte/util/name_fns.h"
 #include "orte/util/show_help.h"
+#include "orte/util/proc_info.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/util/comm/comm.h"
 #include "orte/mca/ras/ras_types.h"
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
 #include "orte/mca/snapc/base/base.h"
 #endif
 #include "orte/runtime/orte_globals.h"
@@ -87,7 +85,7 @@ struct orte_ps_mpirun_info_t {
 
     /* HNP info */
     orte_hnp_contact_t *hnp;
-    
+
     /* array of jobs */
     orte_std_cntr_t num_jobs;
     orte_job_t **jobs;
@@ -145,10 +143,11 @@ static int pretty_print(orte_ps_mpirun_info_t *hnpinfo);
 static int pretty_print_nodes(orte_node_t **nodes, orte_std_cntr_t num_nodes);
 static int pretty_print_jobs(orte_job_t **jobs, orte_std_cntr_t num_jobs);
 static int pretty_print_vpids(orte_job_t *job);
+static void pretty_print_dashed_line(int len);
 
 static char *pretty_node_state(orte_node_state_t state);
-static char *pretty_job_state(orte_job_state_t state);
-static char *pretty_vpid_state(orte_proc_state_t state);
+
+static int parseable_print(orte_ps_mpirun_info_t *hnpinfo);
 
 /*****************************************
  * Global Vars for Command line Arguments
@@ -156,54 +155,61 @@ static char *pretty_vpid_state(orte_proc_state_t state);
 typedef struct {
     bool help;
     bool verbose;
-    int  jobid;
-    int  vpid;
+    bool parseable;
+    orte_jobid_t jobid;
     bool nodes;
     bool daemons;
     int  output;
+    pid_t pid;
 } orte_ps_globals_t;
 
 orte_ps_globals_t orte_ps_globals;
 
 opal_cmd_line_init_t cmd_line_opts[] = {
-    { NULL, NULL, NULL, 
+    { NULL,
       'h', NULL, "help", 
       0,
       &orte_ps_globals.help, OPAL_CMD_LINE_TYPE_BOOL,
       "This help message" },
 
-    { NULL, NULL, NULL, 
+    { NULL,
       'v', NULL, "verbose", 
       0,
       &orte_ps_globals.verbose, OPAL_CMD_LINE_TYPE_BOOL,
       "Be Verbose" },
 
-    { NULL, NULL, NULL, 
+    { NULL,
+      '\0', NULL, "parseable", 
+      0,
+      &orte_ps_globals.parseable, OPAL_CMD_LINE_TYPE_BOOL,
+      "Provide parseable output" },
+
+    { NULL,
       '\0', NULL, "daemons", 
       0,
      &orte_ps_globals.daemons, OPAL_CMD_LINE_TYPE_INT,
       "Display daemon job information" },
 
-    { NULL, NULL, NULL, 
+    { NULL,
       'j', NULL, "jobid", 
       1,
       &orte_ps_globals.jobid, OPAL_CMD_LINE_TYPE_INT,
-      "Specify a specific jobid" },
+      "Specify a local jobid for the given mpirun - a value from 0 to N" },
 
-    { NULL, NULL, NULL, 
-      'p', NULL, "vpid", 
+    { NULL,
+      'p', NULL, "pid", 
       1,
-      &orte_ps_globals.vpid, OPAL_CMD_LINE_TYPE_INT,
-      "Specify a specific vpid. Must specify a --jobid as well" },
+      &orte_ps_globals.pid, OPAL_CMD_LINE_TYPE_INT,
+      "Specify mpirun pid" },
 
-    { NULL, NULL, NULL, 
+    { NULL,
       'n', NULL, "nodes", 
       0,
       &orte_ps_globals.nodes, OPAL_CMD_LINE_TYPE_INT,
       "Display Node Information" },
 
     /* End of list */
-    { NULL, NULL, NULL, 
+    { NULL,
       '\0', NULL, NULL, 
       0,
       NULL, OPAL_CMD_LINE_TYPE_NULL,
@@ -239,6 +245,10 @@ main(int argc, char *argv[])
         goto cleanup;
     }
 
+    opal_output_verbose(10, orte_ps_globals.output,
+                        "orte_ps: Found %d HNPs\n",
+                        (int)opal_list_get_size(&hnp_list));
+
     /*
      * For each hnp in the listing
      */
@@ -246,6 +256,15 @@ main(int argc, char *argv[])
         orte_hnp_contact_t *hnp = (orte_hnp_contact_t*)item;
         hnpinfo.hnp = hnp;
 
+        opal_output_verbose(10, orte_ps_globals.output,
+                            "orte_ps: Processing HNP %lu\n",
+                            (unsigned long)hnpinfo.hnp->pid);
+
+        if (0 < orte_ps_globals.pid &&
+            hnpinfo.hnp->pid != orte_ps_globals.pid) {
+            continue;
+        }
+
         /*
          * Gather the information
          */
@@ -255,16 +274,28 @@ main(int argc, char *argv[])
                             hnpinfo.hnp->pid);
         
         if( ORTE_SUCCESS != (ret = gather_information(&hnpinfo)) ) {
-            exit_status = ret;
+            /* this could be due to a stale session directory - if so,
+             * just skip this entry, but don't abort
+             */
+            if (ORTE_ERR_SILENT == ret) {
+                orte_show_help("help-orte-ps.txt", "stale-hnp", true,
+                               ORTE_NAME_PRINT(&(hnpinfo.hnp->name)));
+                continue;
+            }
             goto cleanup;
         }
 
-        /*
-         * Print the information
-         */
-        if(ORTE_SUCCESS != (ret = pretty_print(&hnpinfo)) ) {
-            exit_status = ret;
-            goto cleanup;
+        /* Print the information */
+        if (orte_ps_globals.parseable) {
+            if (ORTE_SUCCESS != (ret = parseable_print(&hnpinfo))) {
+                exit_status = ret;
+                goto cleanup;
+            }
+        } else {
+            if(ORTE_SUCCESS != (ret = pretty_print(&hnpinfo)) ) {
+                exit_status = ret;
+                goto cleanup;
+            }
         }
     }
 
@@ -280,13 +311,14 @@ main(int argc, char *argv[])
 static int parse_args(int argc, char *argv[]) {
     int ret;
     opal_cmd_line_t cmd_line;
-    orte_ps_globals_t tmp = { false, 
-                              false, 
-                              -1, 
-                              -1, 
-                              false,
-                              false,
-                              -1};
+    orte_ps_globals_t tmp = { false,                    /* help */
+                              false,                    /* verbose */
+                              false,                    /* parseable */
+                              ORTE_JOBID_WILDCARD,      /* jobid */
+                              false,                    /* nodes */
+                              false,                    /* daemons */
+                              -1,                       /* output */
+                              0};                       /* pid */
 
     orte_ps_globals = tmp;
 
@@ -295,30 +327,39 @@ static int parse_args(int argc, char *argv[]) {
     
     mca_base_open();
     mca_base_cmd_line_setup(&cmd_line);
-    ret = opal_cmd_line_parse(&cmd_line, true, argc, argv);
+    ret = opal_cmd_line_parse(&cmd_line, false, argc, argv);
     
+    if (OPAL_SUCCESS != ret) {
+        if (OPAL_ERR_SILENT != ret) {
+            fprintf(stderr, "%s: command line error (%s)\n", argv[0],
+                    opal_strerror(ret));
+        }
+        return ret;
+    }
+
     /**
      * Now start parsing our specific arguments
      */
-    if (OPAL_SUCCESS != ret || 
-        orte_ps_globals.help) {
-        char *args = NULL;
+    if (orte_ps_globals.help) {
+        char *str, *args = NULL;
         args = opal_cmd_line_get_usage_msg(&cmd_line);
-        orte_show_help("help-orte-ps.txt", "usage", true,
-                       args);
+        str = opal_show_help_string("help-orte-ps.txt", "usage", true,
+                                    args);
+        if (NULL != str) {
+            printf("%s", str);
+            free(str);
+        }
         free(args);
-        return ORTE_ERROR;
+        /* If we show the help message, that should be all we do */
+        exit(0);
     }
 
-    /*
-     * If they specify a vpid, they must specify a jobid
-     */
-    if( 0 <= orte_ps_globals.vpid) {
-        if( 0 > orte_ps_globals.jobid) {
-            orte_show_help("help-orte-ps.txt", "vpid-usage", true,
-                           orte_ps_globals.vpid);
-            return ORTE_ERROR;
-        }
+    /* if the jobid is given, then we need a pid */
+    if (ORTE_JOBID_WILDCARD != orte_ps_globals.jobid &&
+        0 == orte_ps_globals.pid) {
+        orte_show_help("help-orte-ps.txt", "need-vpid", true,
+                       orte_ps_globals.jobid);
+        return ORTE_ERROR;
     }
 
     return ORTE_SUCCESS;
@@ -326,14 +367,16 @@ static int parse_args(int argc, char *argv[]) {
 
 static int orte_ps_init(int argc, char *argv[]) {
     int ret;
+#if OPAL_ENABLE_FT_CR == 1
     char * tmp_env_var = NULL;
+#endif
 
     /*
      * Make sure to init util before parse_args
      * to ensure installdirs is setup properly
      * before calling mca_base_open();
      */
-    if( ORTE_SUCCESS != (ret = opal_init_util()) ) {
+    if( ORTE_SUCCESS != (ret = opal_init_util(&argc, &argv)) ) {
         return ret;
     }
 
@@ -354,7 +397,7 @@ static int orte_ps_init(int argc, char *argv[]) {
         orte_ps_globals.output = 0; /* Default=STDERR */
     }
 
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
     /* Disable the checkpoint notification routine for this
      * tool. As we will never need to checkpoint this tool.
      * Note: This must happen before opal_init().
@@ -362,44 +405,40 @@ static int orte_ps_init(int argc, char *argv[]) {
     opal_cr_set_enabled(false);
 
     /* Select the none component, since we don't actually use a checkpointer */
-    tmp_env_var = mca_base_param_env_var("crs");
+    (void) mca_base_var_env_name("crs", &tmp_env_var);
     opal_setenv(tmp_env_var,
                 "none",
                 true, &environ);
     free(tmp_env_var);
     tmp_env_var = NULL;
 
-    tmp_env_var = mca_base_param_env_var("opal_cr_is_tool");
+    (void) mca_base_var_env_name("opal_cr_is_tool", &tmp_env_var);
     opal_setenv(tmp_env_var,
                 "1",
                 true, &environ);
     free(tmp_env_var);
 #endif
-    tmp_env_var = NULL; /* Silence compiler warning */
 
     /***************************
      * We need all of OPAL and the TOOL portion of ORTE
      ***************************/
-    ret = orte_init(ORTE_TOOL);
+    ret = orte_init(&argc, &argv, ORTE_PROC_TOOL);
 
     return ret;
 }
 
 static int pretty_print(orte_ps_mpirun_info_t *hnpinfo) {
     char *header;
-    int len_hdr, i;
+    int len_hdr;
     
     /*
-     * Print header
+     * Print header and remember header length
      */
-    asprintf(&header, "\n\nInformation from mpirun %s", ORTE_JOBID_PRINT(hnpinfo->hnp->name.jobid));
-    len_hdr = strlen(header);
+    len_hdr = asprintf(&header, "Information from mpirun %s", ORTE_JOBID_PRINT(hnpinfo->hnp->name.jobid));
     
-    printf("%s\n", header);
-    for (i=0; i < len_hdr; i++) {
-        printf("%c", '-');
-    }
-    printf("\n");
+    printf("\n\n%s\n", header);
+    free(header);
+    pretty_print_dashed_line(len_hdr);
     
     /*
      * Print Node Information
@@ -418,7 +457,6 @@ static int pretty_print(orte_ps_mpirun_info_t *hnpinfo) {
 static int pretty_print_nodes(orte_node_t **nodes, orte_std_cntr_t num_nodes) {
     int line_len;
     int len_name    = 0,
-        len_arch    = 0,
         len_state   = 0,
         len_slots   = 0,
         len_slots_i = 0,
@@ -430,7 +468,6 @@ static int pretty_print_nodes(orte_node_t **nodes, orte_std_cntr_t num_nodes) {
      * Caculate segment lengths
      */
     len_name    = (int) strlen("Node Name");
-    len_arch    = (int) strlen("Arch");
     len_state   = (int) strlen("State");
     len_slots   = (int) strlen("Slots");
     len_slots_i = (int) strlen("Slots In Use");
@@ -448,7 +485,6 @@ static int pretty_print_nodes(orte_node_t **nodes, orte_std_cntr_t num_nodes) {
     }
     
     line_len = (len_name    + 3 +
-                len_arch    + 3 +
                 len_state   + 3 +
                 len_slots   + 3 +
                 len_slots_i + 3 +
@@ -458,17 +494,13 @@ static int pretty_print_nodes(orte_node_t **nodes, orte_std_cntr_t num_nodes) {
      * Print the header
      */
     printf("%*s | ", len_name,    "Node Name");
-    printf("%*s | ", len_arch,    "Arch");
     printf("%*s | ", len_state,   "State");
     printf("%*s | ", len_slots,   "Slots");
     printf("%*s | ", len_slots_m, "Slots Max");
     printf("%*s | ", len_slots_i, "Slots In Use");
     printf("\n");
 
-    for(i = 0; i < line_len; ++i) {
-        printf("-");
-    }
-    printf("\n");
+    pretty_print_dashed_line(line_len);
     
     /*
      * Print Info
@@ -477,7 +509,6 @@ static int pretty_print_nodes(orte_node_t **nodes, orte_std_cntr_t num_nodes) {
         node = nodes[i];
         
         printf("%*s | ", len_name,    node->name);
-        printf("%*x | ", len_arch,    node->arch);
         printf("%*s | ", len_state,   pretty_node_state(node->state));
         printf("%*d | ", len_slots,   (uint)node->slots);
         printf("%*d | ", len_slots_m, (uint)node->slots_max);
@@ -499,9 +530,12 @@ static int pretty_print_jobs(orte_job_t **jobs, orte_std_cntr_t num_jobs) {
         len_ckpt_l = 0;
     int line_len;
     orte_job_t *job;
-    orte_std_cntr_t i, j;
+    orte_std_cntr_t i;
     char *jobstr;
     orte_jobid_t mask=0x0000ffff;
+#if OPAL_ENABLE_FT_CR == 1
+    char * state_str = NULL;
+#endif
 
     for(i=0; i < num_jobs; i++) {
         job = jobs[i];
@@ -518,15 +552,16 @@ static int pretty_print_jobs(orte_job_t **jobs, orte_std_cntr_t num_jobs) {
          * Caculate segment lengths
          */
         len_jobid  = strlen(jobstr);;
-        len_state  = (int) (strlen(pretty_job_state(job->state)) < strlen("State") ?
+        len_state  = (int) (strlen(orte_job_state_to_str(job->state)) < strlen("State") ?
                             strlen("State") :
-                            strlen(pretty_job_state(job->state)));
+                            strlen(orte_job_state_to_str(job->state)));
         len_slots  = 6;
         len_vpid_r = (int) strlen("Num Procs");
-#if OPAL_ENABLE_FT == 1
-        len_ckpt_s = (int) (strlen(orte_snapc_ckpt_state_str(job->ckpt_state)) < strlen("Ckpt State") ?
+#if OPAL_ENABLE_FT_CR == 1
+        orte_snapc_ckpt_state_str(&state_str, job->ckpt_state);
+        len_ckpt_s = (int) (strlen(state_str) < strlen("Ckpt State") ?
                             strlen("Ckpt State") :
-                            strlen(orte_snapc_ckpt_state_str(job->ckpt_state)) );
+                            strlen(state_str) );
         len_ckpt_r = (int) (NULL == job->ckpt_snapshot_ref ? strlen("Ckpt Ref") :
                             (strlen(job->ckpt_snapshot_ref) < strlen("Ckpt Ref") ?
                              strlen("Ckpt Ref") :
@@ -558,27 +593,24 @@ static int pretty_print_jobs(orte_job_t **jobs, orte_std_cntr_t num_jobs) {
         printf("%*s | ", len_state  , "State");
         printf("%*s | ", len_slots  , "Slots");
         printf("%*s | ", len_vpid_r , "Num Procs");
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
         printf("%*s | ", len_ckpt_s , "Ckpt State");
         printf("%*s | ", len_ckpt_r , "Ckpt Ref");
         printf("%*s |",  len_ckpt_l , "Ckpt Loc");
 #endif
         printf("\n");
 
-        for(j = 0; j < line_len; ++j) {
-            printf("-");
-        }
-        printf("\n");
+        pretty_print_dashed_line(line_len);
 
         /*
          * Print Info
          */        
         printf("%*s | ",  len_jobid ,  ORTE_JOBID_PRINT(job->jobid));
-        printf("%*s | ",  len_state ,  pretty_job_state(job->state));
+        printf("%*s | ",  len_state ,  orte_job_state_to_str(job->state));
         printf("%*d | ",  len_slots ,  (uint)job->total_slots_alloc);
         printf("%*d | ",  len_vpid_r,  job->num_procs);
-#if OPAL_ENABLE_FT == 1
-        printf("%*s | ",  len_ckpt_s,  orte_snapc_ckpt_state_str(job->ckpt_state));
+#if OPAL_ENABLE_FT_CR == 1
+        printf("%*s | ",  len_ckpt_s,  state_str);
         printf("%*s | ",  len_ckpt_r,  (NULL == job->ckpt_snapshot_ref ? 
                                         "" :
                                         job->ckpt_snapshot_ref) );
@@ -605,12 +637,15 @@ static int pretty_print_vpids(orte_job_t *job) {
         len_node        = 0,
         len_ckpt_s      = 0,
         len_ckpt_r      = 0,
-        len_ckpt_l      = 0;
+        len_ckpt_l      = 0; 
     int i, line_len;
     orte_vpid_t v;
     orte_proc_t *vpid;
     orte_app_context_t *app;
     char *o_proc_name;
+#if OPAL_ENABLE_FT_CR == 1
+    char *state_str = NULL;
+#endif
 
     /*
      * Caculate segment lengths
@@ -621,7 +656,7 @@ static int pretty_print_vpids(orte_job_t *job) {
     len_pid         = 6;
     len_state       = 0;
     len_node        = 0;
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
     len_ckpt_s      = strlen("Ckpt State");
     len_ckpt_r      = strlen("Ckpt Ref");
     len_ckpt_l      = strlen("Ckpt Loc");
@@ -638,7 +673,6 @@ static int pretty_print_vpids(orte_job_t *job) {
         /*
          * Find my app context
          */
-        len_proc_name = len_proc_name;
         if( 0 >= (int)job->num_apps ) {
             if( 0 == vpid->name.vpid ) {
                 if( (int)strlen("orterun") > len_proc_name)
@@ -673,12 +707,13 @@ static int pretty_print_vpids(orte_job_t *job) {
             len_node = strlen("Unknown");
         }
 
-        if( (int)strlen(pretty_vpid_state(vpid->state)) > len_state)
-            len_state = strlen(pretty_vpid_state(vpid->state));
+        if( (int)strlen(orte_proc_state_to_str(vpid->state)) > len_state)
+            len_state = strlen(orte_proc_state_to_str(vpid->state));
         
-#if OPAL_ENABLE_FT == 1
-        if( (int)strlen(orte_snapc_ckpt_state_str(vpid->ckpt_state)) > len_ckpt_s)
-            len_ckpt_s = strlen(orte_snapc_ckpt_state_str(vpid->ckpt_state));
+#if OPAL_ENABLE_FT_CR == 1
+        orte_snapc_ckpt_state_str(&state_str, vpid->ckpt_state);
+        if( (int)strlen(state_str) > len_ckpt_s)
+            len_ckpt_s = strlen(state_str);
         
         if( NULL != vpid->ckpt_snapshot_ref &&
             (int)strlen(vpid->ckpt_snapshot_ref) > len_ckpt_r) 
@@ -711,7 +746,7 @@ static int pretty_print_vpids(orte_job_t *job) {
     printf("%*s | ", len_pid         , "PID");
     printf("%*s | ", len_node        , "Node");
     printf("%*s | ", len_state       , "State");
-#if OPAL_ENABLE_FT == 1
+#if OPAL_ENABLE_FT_CR == 1
     printf("%*s | ", len_ckpt_s      , "Ckpt State");
     printf("%*s | ", len_ckpt_r      , "Ckpt Ref");
     printf("%*s |",  len_ckpt_l      , "Ckpt Loc");
@@ -719,10 +754,7 @@ static int pretty_print_vpids(orte_job_t *job) {
     printf("\n");
     
     printf("\t");
-    for(i = 0; i < line_len; ++i) {
-        printf("-");
-    }
-    printf("\n");
+    pretty_print_dashed_line(line_len);
     
     /*
      * Print Info
@@ -753,10 +785,10 @@ static int pretty_print_vpids(orte_job_t *job) {
         printf("%*u | ",  len_rank       , (uint)vpid->local_rank);
         printf("%*d | ",  len_pid        , vpid->pid);
         printf("%*s | ",  len_node       , (NULL == vpid->nodename) ? "Unknown" : vpid->nodename);
-        printf("%*s | ",  len_state      , pretty_vpid_state(vpid->state));
+        printf("%*s | ",  len_state      , orte_proc_state_to_str(vpid->state));
         
-#if OPAL_ENABLE_FT == 1
-        printf("%*s | ",  len_ckpt_s, orte_snapc_ckpt_state_str(vpid->ckpt_state));
+#if OPAL_ENABLE_FT_CR == 1
+        printf("%*s | ",  len_ckpt_s, state_str);
         printf("%*s | ",  len_ckpt_r, (NULL == vpid->ckpt_snapshot_ref ? 
                                        "" : 
                                        vpid->ckpt_snapshot_ref));
@@ -771,6 +803,16 @@ static int pretty_print_vpids(orte_job_t *job) {
     return ORTE_SUCCESS;
 }
 
+static void pretty_print_dashed_line(int len) {
+    static const char dashes[9] = "--------";
+
+    while (len >= 8) {
+        printf("%8.8s", dashes);
+        len -= 8;
+    }
+    printf("%*.*s\n", len, len, dashes);
+}
+
 static int gather_information(orte_ps_mpirun_info_t *hnpinfo) {
     int ret;
     
@@ -793,7 +835,7 @@ static int gather_information(orte_ps_mpirun_info_t *hnpinfo) {
 static int gather_active_jobs(orte_ps_mpirun_info_t *hnpinfo) {
     int ret;
     
-    if (ORTE_SUCCESS != (ret = orte_util_comm_query_job_info(&(hnpinfo->hnp->name), ORTE_JOBID_WILDCARD,
+    if (ORTE_SUCCESS != (ret = orte_util_comm_query_job_info(&(hnpinfo->hnp->name), orte_ps_globals.jobid,
                                                              &hnpinfo->num_jobs, &hnpinfo->jobs))) {
         ORTE_ERROR_LOG(ret);
     }
@@ -834,8 +876,11 @@ static int gather_vpid_info(orte_ps_mpirun_info_t *hnpinfo) {
         }
 
         /* query the HNP for info on the procs in this job */
-        if (ORTE_SUCCESS != (ret = orte_util_comm_query_proc_info(&(hnpinfo->hnp->name), job->jobid,
-                                                                  ORTE_VPID_WILDCARD, &cnt, &procs))) {
+        if (ORTE_SUCCESS != (ret = orte_util_comm_query_proc_info(&(hnpinfo->hnp->name), 
+                                                                  job->jobid,
+                                                                  ORTE_VPID_WILDCARD, 
+                                                                  &cnt, 
+                                                                  &procs))) {
             ORTE_ERROR_LOG(ret);
         }
         job->procs->addr = (void**)procs;
@@ -846,75 +891,6 @@ static int gather_vpid_info(orte_ps_mpirun_info_t *hnpinfo) {
     return ORTE_SUCCESS;
 }
 
-static char *pretty_job_state(orte_job_state_t state) {
-    switch(state) {
-        case ORTE_JOB_STATE_UNDEF:
-            return strdup("Undef");
-            break;
-        case ORTE_JOB_STATE_INIT:
-            return strdup("Init");
-            break;
-        case ORTE_JOB_STATE_LAUNCHED:
-            return strdup("Launched");
-            break;
-        case ORTE_JOB_STATE_RUNNING:
-            return strdup("Running");
-            break;
-        case ORTE_JOB_STATE_TERMINATED:
-            return strdup("Terminated");
-            break;
-        case ORTE_JOB_STATE_ABORTED:
-            return strdup("Aborted");
-            break;
-        case ORTE_JOB_STATE_FAILED_TO_START:
-            return strdup("Failed to start");
-            break;
-        case ORTE_JOB_STATE_ABORTED_BY_SIG:
-            return strdup("Aborted by signal");
-            break;
-        case ORTE_JOB_STATE_ABORT_ORDERED:
-            return strdup("Aborted ordered");
-            break;
-        default:
-            break;
-    }
-    
-    return strdup("");
-}
-
-static char *pretty_vpid_state(orte_proc_state_t state) {
-    switch(state) {
-        case ORTE_PROC_STATE_UNDEF:
-            return strdup("Undef");
-            break;
-        case ORTE_PROC_STATE_INIT:
-            return strdup("Init");
-            break;
-        case ORTE_PROC_STATE_LAUNCHED:
-            return strdup("Launched");
-            break;
-        case ORTE_PROC_STATE_RUNNING:
-            return strdup("Running");
-            break;
-        case ORTE_PROC_STATE_TERMINATED:
-            return strdup("Terminated");
-            break;
-        case ORTE_PROC_STATE_ABORTED:
-            return strdup("Aborted");
-            break;
-        case ORTE_PROC_STATE_FAILED_TO_START:
-            return strdup("Failed to start");
-            break;
-        case ORTE_PROC_STATE_ABORTED_BY_SIG:
-            return strdup("Aborted by signal");
-            break;
-        default:
-            break;
-    }
-    
-    return strdup("");
-}
-
 static char *pretty_node_state(orte_node_state_t state) {
     switch(state) {
     case ORTE_NODE_STATE_DOWN:
@@ -932,3 +908,56 @@ static char *pretty_node_state(orte_node_state_t state) {
         break;
     }
 }
+
+static int parseable_print(orte_ps_mpirun_info_t *hnpinfo)
+{
+    orte_job_t **jobs;
+    orte_node_t **nodes;
+    orte_proc_t *proc;
+    orte_app_context_t *app;
+    char *appname;
+    int i, j;
+
+    /* don't include the daemon job in the number of jobs reported */
+    printf("mpirun:%lu:num nodes:%d:num jobs:%d\n",
+           (unsigned long)hnpinfo->hnp->pid, hnpinfo->num_nodes, hnpinfo->num_jobs-1);
+
+    if (orte_ps_globals.nodes) {
+        nodes = hnpinfo->nodes;
+        for (i=0; i < hnpinfo->num_nodes; i++) {
+            printf("node:%s:state:%s:slots:%d:in use:%d\n",
+                   nodes[i]->name, pretty_node_state(nodes[i]->state),
+                   nodes[i]->slots, nodes[i]->slots_inuse);
+        }
+    }
+
+    jobs = hnpinfo->jobs;
+    /* skip job=0 as that's the daemon job */
+    for (i=1; i < hnpinfo->num_jobs; i++) {
+        printf("jobid:%d:state:%s:slots:%d:num procs:%d\n",
+               ORTE_LOCAL_JOBID(jobs[i]->jobid),
+               orte_job_state_to_str(jobs[i]->state),
+               jobs[i]->total_slots_alloc,
+               jobs[i]->num_procs);
+        /* print the proc info */
+        for (j=0; j < jobs[i]->procs->size; j++) {
+            if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(jobs[i]->procs, j))) {
+                continue;
+            }
+            app = (orte_app_context_t*)opal_pointer_array_get_item(jobs[i]->apps, proc->app_idx);
+            if (NULL == app) {
+                appname = strdup("NULL");
+            } else {
+                appname = opal_basename(app->app);
+            }
+            printf("process:%s:rank:%s:pid:%lu:node:%s:state:%s\n",
+                   appname, ORTE_VPID_PRINT(proc->name.vpid),
+                   (unsigned long)proc->pid,
+                   (NULL == proc->nodename) ? "unknown" : proc->nodename,
+                   orte_proc_state_to_str(proc->state));
+            free(appname);
+        }
+    }
+
+    return ORTE_SUCCESS;
+}
