@@ -11,6 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
  *                         reserved. 
+ * Copyright (c) 2011 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -247,6 +248,9 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
         /* store this for later processing */
         node = OBJ_NEW(orte_node_t);
         node->name = strdup(orte_util_hostfile_value.sval);
+        if (NULL != username) {
+            node->username = strdup(username);
+        }
     } else if (ORTE_HOSTFILE_RANK == token) {
         /* we can ignore the rank, but we need to extract the node name. we
          * first need to shift over to the other side of the equal sign as
@@ -329,42 +333,6 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
             node->username = hostfile_parse_string();
             break;
 
-        case ORTE_HOSTFILE_BOARDS:
-            rc = hostfile_parse_int();
-            if (rc < 0) {
-                orte_show_help("help-hostfile.txt", "boards",
-                               true,
-                               cur_hostfile_name, rc);
-                OBJ_RELEASE(node);
-                return ORTE_ERROR;
-            }
-            node->boards = rc;
-            break;
-
-        case ORTE_HOSTFILE_SOCKETS_PER_BOARD:
-            rc = hostfile_parse_int();
-            if (rc < 0) {
-                orte_show_help("help-hostfile.txt", "sockets",
-                               true,
-                               cur_hostfile_name, rc);
-                OBJ_RELEASE(node);
-                return ORTE_ERROR;
-            }
-            node->sockets_per_board = rc;
-            break;
-
-        case ORTE_HOSTFILE_CORES_PER_SOCKET:
-            rc = hostfile_parse_int();
-            if (rc < 0) {
-                orte_show_help("help-hostfile.txt", "cores",
-                               true,
-                               cur_hostfile_name, rc);
-                OBJ_RELEASE(node);
-                return ORTE_ERROR;
-            }
-            node->cores_per_socket = rc;
-            break;
-
         case ORTE_HOSTFILE_COUNT:
         case ORTE_HOSTFILE_CPU:
         case ORTE_HOSTFILE_SLOTS:
@@ -451,8 +419,25 @@ static int hostfile_parse(const char *hostfile, opal_list_t* updates, opal_list_
     orte_util_hostfile_done = false;
     orte_util_hostfile_in = fopen(hostfile, "r");
     if (NULL == orte_util_hostfile_in) {
-        orte_show_help("help-hostfile.txt", "no-hostfile", true, hostfile);
-        rc = ORTE_ERR_NOT_FOUND;
+        if (NULL == orte_default_hostfile ||
+            0 != strcmp(orte_default_hostfile, hostfile)) {
+            /* not the default hostfile, so not finding it
+             * is an error
+             */
+            orte_show_help("help-hostfile.txt", "no-hostfile", true, hostfile);
+            rc = ORTE_ERR_SILENT;
+            goto unlock;
+        }
+        /* if this is the default hostfile and it was given,
+         * then it's an error
+         */
+        if (orte_default_hostfile_given) {
+            orte_show_help("help-hostfile.txt", "no-hostfile", true, hostfile);
+            rc = ORTE_ERR_NOT_FOUND;
+            goto unlock;
+        }
+        /* otherwise, not finding it is okay */
+        rc = ORTE_SUCCESS;
         goto unlock;
     }
 
@@ -506,7 +491,6 @@ unlock:
  */
 
 int orte_util_add_hostfile_nodes(opal_list_t *nodes,
-                                 bool *override_oversubscribed,
                                  char *hostfile)
 {
     opal_list_t exclude;
@@ -557,15 +541,6 @@ int orte_util_add_hostfile_nodes(opal_list_t *nodes,
         OBJ_RELEASE(item);
     }
     
-    /* indicate that ORTE should override any oversubscribed conditions
-     * based on local hardware limits since the user (a) might not have
-     * provided us any info on the #slots for a node, and (b) the user
-     * might have been wrong! If we don't check the number of local physical
-     * processors, then we could be too aggressive on our sched_yield setting
-     * and cause performance problems.
-     */
-    *override_oversubscribed = true;
-
 cleanup:
     OBJ_DESTRUCT(&exclude);
 
@@ -577,7 +552,8 @@ cleanup:
  * are not found in the hostfile
  */
 int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
-                                    char *hostfile)
+                                    char *hostfile,
+                                    bool remove)
 {
     opal_list_t newnodes, exclude;
     opal_list_item_t *item1, *item2, *next, *item3;
@@ -587,7 +563,8 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
     int num_empty, nodeidx;
     bool want_all_empty = false;
     opal_list_t keep;
-    
+    bool found;
+
     OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
                         "%s hostfile: filtering nodes through hostfile %s",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), hostfile));
@@ -605,7 +582,8 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
     if (0 == opal_list_get_size(&newnodes)) {
         OBJ_DESTRUCT(&newnodes);
         OBJ_DESTRUCT(&exclude);
-        return ORTE_SUCCESS;
+        /* indicate that the hostfile was empty */
+        return ORTE_ERR_TAKE_NEXT_OPTION;
     }
 
     /* remove from the list of newnodes those that are in the exclude list
@@ -627,7 +605,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
         OBJ_RELEASE(item1);
     }
     
-    /* now check our nodes and keep those that match. We can
+    /* now check our nodes and keep or mark those that match. We can
      * destruct our hostfile list as we go since this won't be needed
      */
     OBJ_CONSTRUCT(&keep, opal_list_t);
@@ -673,10 +651,15 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                                 goto skipnode;
                             }
                         }
-                        /* remove item from list */
-                        opal_list_remove_item(nodes, item1);
-                        /* xfer to keep list */
-                        opal_list_append(&keep, item1);
+                        if (remove) {
+                            /* remove item from list */
+                            opal_list_remove_item(nodes, item1);
+                            /* xfer to keep list */
+                            opal_list_append(&keep, item1);
+                        } else {
+                            /* mark as included */
+                            node_from_list->mapped = true;
+                        }
                         --num_empty;
                     }
                 skipnode:
@@ -708,10 +691,15 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                      item1 = opal_list_get_next(nodes)) {
                     node_from_list = (orte_node_t*)item1;
                     if (0 == strcmp(node_from_list->name, node_from_pool->name)) {
-                        /* match - remove item from list */
-                        opal_list_remove_item(nodes, item1);
-                        /* xfer to keep list */
-                        opal_list_append(&keep, item1);
+                        if (remove) {
+                            /* match - remove item from list */
+                            opal_list_remove_item(nodes, item1);
+                            /* xfer to keep list */
+                            opal_list_append(&keep, item1);
+                        } else {
+                            /* mark as included */
+                            node_from_list->mapped = true;
+                        }
                         break;
                     }
                 }
@@ -727,6 +715,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
              * search the provided list of nodes to see if this
              * one is found
              */
+            found = false;
             for (item1 = opal_list_get_first(nodes);
                  item1 != opal_list_get_end(nodes);
                  item1 = opal_list_get_next(item1)) {
@@ -746,13 +735,29 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                     if (node_from_file->slots < node_from_list->slots) {
                         node_from_list->slots_alloc = node_from_file->slots;
                     }
-                    /* remove the node from the list */
-                    opal_list_remove_item(nodes, item1);
-                    /* xfer it to keep list */
-                    opal_list_append(&keep, item1);
+                    if (remove) {
+                        /* remove the node from the list */
+                        opal_list_remove_item(nodes, item1);
+                        /* xfer it to keep list */
+                        opal_list_append(&keep, item1);
+                    } else {
+                        /* mark as included */
+                        node_from_list->mapped = true;
+                    }
+                    found = true;
                     break;
                 }
             }
+            /* if the host in the newnode list wasn't found, 
+             * then that is an error we need to report to the 
+             * user and abort 
+             */ 
+            if (!found) { 
+                orte_show_help("help-hostfile.txt", "hostfile:extra-node-not-found", 
+                               true, hostfile, node_from_file->name); 
+                rc = ORTE_ERR_SILENT; 
+                goto cleanup; 
+            } 
         }
         /* cleanup the newnode list */
         OBJ_RELEASE(item2);
@@ -772,6 +777,12 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
         return ORTE_ERR_SILENT;
     }
 
+    if (!remove) {
+        /* all done */
+        OBJ_DESTRUCT(&newnodes);
+        return ORTE_SUCCESS;
+    }
+
     /* clear the rest of the nodes list */
     while (NULL != (item1 = opal_list_remove_first(nodes))) {
         OBJ_RELEASE(item1);
