@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2011 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -27,6 +27,7 @@
 #include "opal/util/show_help.h"
 #include "opal/util/output.h"
 #include "opal/dss/dss.h"
+#include "opal/mca/event/event.h"
 
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/rml/rml.h"
@@ -38,7 +39,24 @@
 
 #include "orte/util/show_help.h"
 
-bool orte_help_want_aggregate;
+bool orte_help_want_aggregate = false;
+
+/*
+ * Local variable to know whether aggregated show_help is available or
+ * not 
+ */
+static bool ready = false;
+
+/*
+ * Same for systems with or without full ORTE support
+ */
+bool orte_show_help_is_available(void)
+{
+    /* This is a function only to give us forward flexibility in case
+       we need a more complicated check someday. */
+
+    return ready;
+}
 
 /************************************************************************/
 
@@ -48,11 +66,13 @@ bool orte_help_want_aggregate;
 
 int orte_show_help_init(void)
 {
+    ready = true;
     return ORTE_SUCCESS;
 }
 
 void orte_show_help_finalize(void)
 {
+    ready = false;
     return;
 }
 
@@ -62,6 +82,10 @@ int orte_show_help(const char *filename, const char *topic,
     va_list arglist;
     char *output;
     
+    if (orte_execute_quiet) {
+        return ORTE_SUCCESS;
+    }
+    
     va_start(arglist, want_error_header);
     output = opal_show_help_vstring(filename, topic, want_error_header, 
                                     arglist);
@@ -72,10 +96,14 @@ int orte_show_help(const char *filename, const char *topic,
         return ORTE_SUCCESS;
     }
     
-    opal_output(0, output);
+    opal_output(0, "%s", output);
     return ORTE_SUCCESS;
 }
 
+int orte_show_help_suppress(const char *filename, const char *topic)
+{
+    return ORTE_ERR_NOT_SUPPORTED;
+}
 
 #else
 
@@ -97,6 +125,8 @@ typedef struct {
     /* Count of processes since last display (i.e., "new" processes
        that have showed this message that have not yet been output) */
     int tli_count_since_last_display;
+    /* Do we want to display these? */
+    bool tli_display;
 } tuple_list_item_t;
 
 static void tuple_list_item_constructor(tuple_list_item_t *obj);
@@ -116,7 +146,6 @@ static struct timeval show_help_interval = { 5, 0 };
 static time_t show_help_time_last_displayed = 0;
 static bool show_help_timer_set = false;
 static opal_event_t show_help_timer_event;
-static bool ready;
 
 static opal_show_help_fn_t save_help = NULL;
 
@@ -127,6 +156,7 @@ static void tuple_list_item_constructor(tuple_list_item_t *obj)
     OBJ_CONSTRUCT(&(obj->tli_processes), opal_list_t);
     obj->tli_time_displayed = time(NULL);
     obj->tli_count_since_last_display = 0;
+    obj->tli_display = true;
 }
 
 static void tuple_list_item_destructor(tuple_list_item_t *obj)
@@ -259,6 +289,53 @@ error:
 }
 
 
+/*
+ * Returns ORTE_SUCCESS if the strings match; ORTE_ERROR otherwise.
+ */
+static int match(const char *a, const char *b)
+{
+    int rc = ORTE_ERROR;
+    char *p1, *p2, *tmp1 = NULL, *tmp2 = NULL;
+    size_t min;
+
+    /* Check straight string match first */
+    if (0 == strcmp(a, b)) return ORTE_SUCCESS;
+
+    if (NULL != strchr(a, '*') || NULL != strchr(b, '*')) {
+        tmp1 = strdup(a);
+        if (NULL == tmp1) {
+            return ORTE_ERR_OUT_OF_RESOURCE;
+        }
+        tmp2 = strdup(b);
+        if (NULL == tmp2) {
+            free(tmp1);
+            return ORTE_ERR_OUT_OF_RESOURCE;
+        }
+        p1 = strchr(tmp1, '*');
+        p2 = strchr(tmp2, '*');
+
+        if (NULL != p1) {
+            *p1 = '\0';
+        }
+        if (NULL != p2) {
+            *p2 = '\0';
+        }
+        min = strlen(tmp1);
+        if (strlen(tmp2) < min) {
+            min = strlen(tmp2);
+        }
+        if (0 == min || 0 == strncmp(tmp1, tmp2, min)) {
+            rc = ORTE_SUCCESS;
+        }
+        free(tmp1);
+        free(tmp2);
+        return rc;
+    }
+
+    /* No match */
+    return ORTE_ERROR;
+}
+
 /*
  * Check to see if a given (filename, topic) tuple has been displayed
  * already.  Return ORTE_SUCCESS if so, or ORTE_ERR_NOT_FOUND if not.
@@ -283,8 +360,8 @@ static int get_tli(const char *filename, const char *topic,
          opal_list_get_end(&abd_tuples) != item;
          item = opal_list_get_next(item)) {
         (*tli) = (tuple_list_item_t*) item;
-        if (0 == strcmp((*tli)->tli_filename, filename) &&
-            0 == strcmp((*tli)->tli_topic, topic)) {
+        if (ORTE_SUCCESS == match((*tli)->tli_filename, filename) &&
+            ORTE_SUCCESS == match((*tli)->tli_topic, topic)) {
             return ORTE_SUCCESS;
         }
     }
@@ -315,7 +392,8 @@ static void show_accumulated_duplicates(int fd, short event, void *context)
          opal_list_get_end(&abd_tuples) != item;
          item = opal_list_get_next(item)) {
         tli = (tuple_list_item_t*) item;
-        if (tli->tli_count_since_last_display > 0) {
+        if (tli->tli_display && 
+            tli->tli_count_since_last_display > 0) {
             static bool first = true;
             if (orte_xml_output) {
                 asprintf(&tmp, "%d more process%s sent help message %s / %s",
@@ -366,6 +444,13 @@ static int show_help(const char *filename, const char *topic,
         rc = ORTE_ERR_NOT_FOUND;
     }
 
+    /* If there's no output string (i.e., this is a control message
+       asking us to suppress), then skip to the end. */
+    if (NULL == output) {
+        tli->tli_display = false;
+        goto after_output;
+    }
+
     /* Was it already displayed? */
     if (ORTE_SUCCESS == rc) {
         /* Yes.  But do we want to print anything?  That's complicated.
@@ -397,9 +482,9 @@ static int show_help(const char *filename, const char *topic,
         if (now > show_help_time_last_displayed + 5 && !show_help_timer_set) {
             show_accumulated_duplicates(0, 0, NULL);
         } else if (!show_help_timer_set) {
-            opal_evtimer_set(&show_help_timer_event,
-                             show_accumulated_duplicates, NULL);
-            opal_evtimer_add(&show_help_timer_event, &show_help_interval);
+            opal_event_evtimer_set(orte_event_base, &show_help_timer_event,
+                                   show_accumulated_duplicates, NULL);
+            opal_event_evtimer_add(&show_help_timer_event, &show_help_interval);
             show_help_timer_set = true;
         }
     } 
@@ -424,6 +509,7 @@ static int show_help(const char *filename, const char *topic,
         return rc;
     }
 
+ after_output:
     /* If we're aggregating, add this process name to the list */
     if (orte_help_want_aggregate) {
         pnli = OBJ_NEW(orte_namelist_t);
@@ -433,7 +519,7 @@ static int show_help(const char *filename, const char *topic,
             return rc;
         }
         pnli->name = *sender;
-        opal_list_append(&(tli->tli_processes), &(pnli->item));
+        opal_list_append(&(tli->tli_processes), &(pnli->super));
     }
     return ORTE_SUCCESS;
 }
@@ -448,6 +534,7 @@ void orte_show_help_recv(int status, orte_process_name_t* sender,
     char *output=NULL;
     char *filename=NULL, *topic=NULL;
     int32_t n;
+    int8_t have_output;
     int rc;
     
     OPAL_OUTPUT_VERBOSE((5, orte_debug_output,
@@ -467,13 +554,22 @@ void orte_show_help_recv(int status, orte_process_name_t* sender,
         ORTE_ERROR_LOG(rc);
         goto cleanup;
     }
-    /* unpack the resulting string */
+    /* unpack the flag */
     n = 1;
-    if (ORTE_SUCCESS != (rc = opal_dss.unpack(buffer, &output, &n, OPAL_STRING))) {
+    if (ORTE_SUCCESS != (rc = opal_dss.unpack(buffer, &have_output, &n, OPAL_INT8))) {
         ORTE_ERROR_LOG(rc);
         goto cleanup;
     }
     
+    /* If we have an output string, unpack it */
+    if (have_output) {
+        n = 1;
+        if (ORTE_SUCCESS != (rc = opal_dss.unpack(buffer, &output, &n, OPAL_STRING))) {
+            ORTE_ERROR_LOG(rc);
+            goto cleanup;
+        }
+    }
+    
     /* Send it to show_help */
     rc = show_help(filename, topic, output, sender);
     
@@ -487,13 +583,7 @@ cleanup:
     if (NULL != topic) {
         free(topic);
     }
-    /* reissue the recv */
-    rc = orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, ORTE_RML_TAG_SHOW_HELP,
-                                 ORTE_RML_NON_PERSISTENT, orte_show_help_recv, NULL);
-    if (rc != ORTE_SUCCESS && rc != ORTE_ERR_NOT_IMPLEMENTED) {
-        ORTE_ERROR_LOG(rc);
-    }
-}
+ }
 
 int orte_show_help_init(void)
 {
@@ -503,12 +593,12 @@ int orte_show_help_init(void)
     if (ready) {
         return ORTE_SUCCESS;
     }
-    ready = true;
 
     OBJ_CONSTRUCT(&abd_tuples, opal_list_t);
     
     save_help = opal_show_help;
     opal_show_help = orte_show_help;
+    ready = true;
 
     return ORTE_SUCCESS;
 }
@@ -528,7 +618,7 @@ void orte_show_help_finalize(void)
         show_accumulated_duplicates(0, 0, NULL);
         OBJ_DESTRUCT(&abd_tuples);
         if (show_help_timer_set) {
-            opal_evtimer_del(&show_help_timer_event);
+            opal_event_evtimer_del(&show_help_timer_event);
         }
         
         /* cancel the recv */
@@ -544,6 +634,10 @@ int orte_show_help(const char *filename, const char *topic,
     va_list arglist;
     char *output;
     
+    if (orte_execute_quiet) {
+        return ORTE_SUCCESS;
+    }
+    
     va_start(arglist, want_error_header);
     output = opal_show_help_vstring(filename, topic, want_error_header, 
                                     arglist);
@@ -554,6 +648,17 @@ int orte_show_help(const char *filename, const char *topic,
         return ORTE_SUCCESS;
     }
 
+    rc = orte_show_help_norender(filename, topic, want_error_header, output);
+    free(output);
+    return rc;
+}
+
+int orte_show_help_norender(const char *filename, const char *topic, 
+                            bool want_error_header, const char *output)
+{
+    int rc = ORTE_SUCCESS;
+    int8_t have_output = 1;
+
     if (!ready) {
         /* if we are finalizing, then we have no way to process
          * this through the orte_show_help system - just drop it to
@@ -602,9 +707,75 @@ int orte_show_help(const char *filename, const char *topic,
             opal_dss.pack(&buf, &filename, 1, OPAL_STRING);
             /* pack the topic tag */
             opal_dss.pack(&buf, &topic, 1, OPAL_STRING);
+            /* pack the flag that we have a string */
+            opal_dss.pack(&buf, &have_output, 1, OPAL_INT8);
             /* pack the resulting string */
             opal_dss.pack(&buf, &output, 1, OPAL_STRING);
             /* send it to the HNP */
+            if (0 > (rc = orte_rml.send_buffer(ORTE_PROC_MY_HNP, &buf, ORTE_RML_TAG_SHOW_HELP, 0))) {
+                ORTE_ERROR_LOG(rc);
+            } else {
+                rc = ORTE_SUCCESS;
+            }
+            OBJ_DESTRUCT(&buf);
+            am_inside = false;
+        }
+    }
+    
+CLEANUP:
+    return rc;
+}
+
+int orte_show_help_suppress(const char *filename, const char *topic)
+{
+    int rc = ORTE_SUCCESS;
+    int8_t have_output = 0;
+    
+    if (orte_execute_quiet) {
+        return ORTE_SUCCESS;
+    }
+    
+    if (!ready) {
+        /* If we are finalizing, then we have no way to process this
+           through the orte_show_help system - just drop it. */
+        return ORTE_SUCCESS;
+    }
+    
+    /* If we are the HNP, or the RML has not yet been setup, or ROUTED
+       has not been setup, or we weren't given an HNP, then all we can
+       do is process this locally. */
+    if (ORTE_PROC_IS_HNP ||
+        NULL == orte_rml.send_buffer ||
+        NULL == orte_routed.get_route ||
+        NULL == orte_process_info.my_hnp_uri) {
+        rc = show_help(filename, topic, NULL, ORTE_PROC_MY_NAME);
+    }
+    
+    /* otherwise, we relay the output message to
+     * the HNP for processing
+     */
+    else {
+        opal_buffer_t buf;
+        static bool am_inside = false;
+
+        /* JMS Note that we *may* have a recursion situation here where
+           the RML could call show_help.  Need to think about this
+           properly, but put a safeguard in here for sure for the time
+           being. */
+        if (am_inside) {
+            rc = show_help(filename, topic, NULL, ORTE_PROC_MY_NAME);
+        } else {
+            am_inside = true;
+        
+            /* build the message to the HNP */
+            OBJ_CONSTRUCT(&buf, opal_buffer_t);
+            /* pack the filename of the show_help text file */
+            opal_dss.pack(&buf, &filename, 1, OPAL_STRING);
+            /* pack the topic tag */
+            opal_dss.pack(&buf, &topic, 1, OPAL_STRING);
+            /* pack the flag that we DO NOT have a string */
+            opal_dss.pack(&buf, &have_output, 1, OPAL_INT8);
+            /* send it to the HNP */
             if (0 > (rc = orte_rml.send_buffer(ORTE_PROC_MY_HNP, &buf, ORTE_RML_TAG_SHOW_HELP, 0))) {
                 ORTE_ERROR_LOG(rc);
             }
@@ -613,9 +784,7 @@ int orte_show_help(const char *filename, const char *topic,
         }
     }
     
-CLEANUP:
-    free(output);
-    return rc;
+    return ORTE_SUCCESS;
 }
 
 #endif /* ORTE_DISABLE_FULL_SUPPORT */
